
#ifdef UNIXMODE
#include <errno.h>
#include <fcntl.h> 
#include <sys/types.h>
#include <sys/stat.h> 
#include <stropts.h> 
#ifdef USLSVR4
#include <unistd.h>
#endif
#else  /* OS2MODE */
#include <process.h>
#endif

/* Standard C include files */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>

typedef int BOOL ;
#define FALSE 0
#define TRUE  1

/* ME/X - ME/2 include files */
#include "me2def.h"
#include "me2.h"
#include "me2err.h"

/* DSP include file */
#include "dsp.h"

/* SwitchPath include file */
#include "swpath.h"

#ifdef UNIXMODE
/* VBX device driver API include file */
#include "vbxdrv.h"
#ifndef USLSVR4
/* External function prototypes */
int close () ;
int fork () ;
int ioctl () ;
int open () ;
#endif
#endif

/*-----------------------------------------
   Miscellaneous defines used in this file
  -----------------------------------------*/

/* Inter-process hangup Event */
#define CALLER_GONE (ME2INT) 0x8020

/* Number of digits in directory number for outgoing call */
#define NUM_OUTGOING_DN 4

/* Global variable definitions */
BOOL  CallerHungUp          = FALSE ;
BOOL  DestHungUp            = FALSE ;
BOOL  VBXInOGTMode          = FALSE ;
int   TrunkOvlyParm         = -1 ;
int   VBXLineNum            = -1 ;
int   VBXSlot               = -1 ;
short FirstDigitalVBXPort   = -1 ;
short NumDNISDigitsExpected = -1 ;
ME2INT Vmsg = -1;           /* Voice file handles.*/
ME2INT Vrec = -1;
short  Maxrec = 10;	    /*  maximum voice recording sec. */
char Vmsgfil[20];           /* Name of voice file containing prompts */
char Vrecfil[80];           /* Name of voice file to record usr message to. */


/* Local functions declarations (in alphabetical order) */
int  CallOut () ;
void ChangeToIncomingMode () ;
void ChangeToOutgoingMode () ;
void ConnectVBXToInTrunk () ;
void ConnectVBXToOutTrunk () ;
ME2INT vox_waitdone();
void flushttqueue ();
void setparm ();
void voxexit ();
ME2INT play ();
ME2INT recplay ();
int  CollectTouchTones () ;
void ConnectTrunks () ;
void DisconnectOutTrunk () ;
void DoExit () ;
void FlushEventQueue () ;
void ForceVBXSignal () ;
int  GetVBX () ;
void HangupIncomingCall () ;
void InitializeGlobalData () ;
void InitializeDSP ();
void InitializeSwitching () ;
int  InteractWithCaller ()  ;
#ifdef UNIXMODE
 void MonitorSupervision () ;
#else /* OS2 */
 #if defined (__BORLANDC__)
  void _USERENTRY MonitorSupervision (void *) ;
 #else /* MSC 6.0 or IBM C/Set */
  void MonitorSupervision (void *) ;
 #endif
#endif
int  SendEventToParent () ;
void ShowEvent () ;
void SilenceInTrunk () ;
void StartSupervision () ;
void StopSupervision () ;
int  WaitDone () ;
void WaitForDisconnect () ;
void WaitForIncomingCall () ;
int  WaitTrunkEvent () ;

/*----------------------------------------------------------------------------
  main

  Perform initialization, wait for incoming call, and interact with
  incoming caller.
  ---------------------------------------------------------------------------*/

int main (argc,argv)
     int argc;
     char *argv[];
{
  char outgoingDN[32] ;
  int  inT1Trunk, outT1Trunk ;

#ifdef UNIXMODE
  /* In UNIX environment, expect one argument, the line number to use */
  if (argc < 2) {
    puts ("usage: spcp <line number>") ;
    exit (0) ;
  }
  else
    VBXLineNum = atoi (argv[1]) ;
#else
  argc = argc ;  argv = argv ;    /*xx- NOT USED-xx*/
#endif

   /* Get ME/X - ME/2 channel */
  Me2RequestChannel (99, "SPCP") ;

        /* Initialize SwitchPath library */
  InitializeSwitching () ;

        /* Initialize some global data */
  InitializeGlobalData () ;

	/* Initialize DSP parameters */
  InitializeDSP ();

  inT1Trunk = GetVBX () ;

  /*--------------------------------------------------------------------
     NOTE: This demo uses same trunk number for the outgoing T1 trunk,
          as the incoming trunk, but in the other T1 span
   --------------------------------------------------------------------*/
  outT1Trunk = inT1Trunk ;

  /*---------------------------------------------------------------
     Before processing the first call, place the outgoing T1 trunk
     into an idle state.
   ---------------------------------------------------------------*/

  DisconnectOutTrunk (outT1Trunk) ;

        /* Enter a loop to process calls */
  for (;;) {
    /* Wait for an incoming call */
    WaitForIncomingCall () ;

    /* Voice announcement of prompt */
    play (Vmsg, MWELCOME);
    play (Vmsg, MMENU);
    
    /* Request the directory number for outgoing call from the caller */
    if (InteractWithCaller (outgoingDN) == 1)  {
      if (outT1Trunk != -1) { /* ALWAYS TRUE */
          /* Monitor incoming T1 trunk for hangup */
	StartSupervision (inT1Trunk) ;
          /* Prepare VBX port for outgoing call */
	ChangeToOutgoingMode () ;
          /* Connect VBX port to outgoing T1 trunk */
	ConnectVBXToOutTrunk (outT1Trunk) ;
	  /* Break the talk path to incoming T1 trunk */
	SilenceInTrunk (inT1Trunk) ;
	  /* Dial outgoing call and perform call progress */
	if (CallOut (outgoingDN) == 1) {
	  ConnectTrunks (inT1Trunk, outT1Trunk) ;
	  puts (" *** CONNECTED ***") ;
	   /* Wait for either end to hangup */
	  WaitForDisconnect () ;
	}

/*----------------------------------------------------
   Hang up the outgoing T1 trunk and disconnect VBX
   port from it.
 ----------------------------------------------------*/

	DisconnectOutTrunk (outT1Trunk) ;

	if (!CallerHungUp)
	  /* Stop monitoring incoming T1 trunk for hangup */
	  StopSupervision () ;

	ChangeToIncomingMode () ;
	ConnectVBXToInTrunk (inT1Trunk) ;
      }

      else {   /* No free outgoing T1 trunk */

	/* Restart Trunk Manager */
	HangupIncomingCall () ;
      }
    }
    else {  /* Don't need outgoing T1 trunk, caller hung up */
      /* Restart Trunk Manager */
      HangupIncomingCall () ;
    }

  } /* Process calls forever */
}	
/* main */

/*----------------------------------------------------------------------------
   InitializeGlobalData

   Initializes some globals from values of ME/X - ME/2 parameters.
   --------------------------------------------------------------------------*/

void InitializeGlobalData ()
{
  int rc ;

  /* Load specific parameters for this program from an ASCII .PRO file */

  if ((rc = Me2ParmLoad ("SPCP.PRO")) != SUCCESS) {
    printf ("Error %d loading profile file SPCP.PRO\n", rc) ;
    DoExit (1) ;
  }

  /* Get the number of DNIS digits */
  rc = Me2ParmByName ("TCPDID.NUM_DIGITS", (void *) &NumDNISDigitsExpected,
		      GETPARM) ;
  if (rc != SUCCESS) {
    if (rc == ME2_CATEGORY_NOT_FOUND)
      printf ("Missing TCPDID parameter file\n") ;
    else
      printf ("Error %d getting DID parameter \"TCPDID.NUM_DIGITS\"\n", rc) ;

    DoExit (1) ;
  }

  /*------------------------------------------------------------------
    Get the ME/X - ME/2 line number of the first digital port.  This
    will normally be zero, but doesn't have to be.  For example, it
    could be 1, if port 0 is a VBX 100 board.
    ------------------------------------------------------------------*/

  if((rc = Me2ParmByName("TRKDEMO.1ST_VBX_PORT",(void *)&FirstDigitalVBXPort,
			 GETPARM))  != SUCCESS) {
    printf ("Error %d reading parameter \"TRKDEMO.1ST_VBX_PORT\"\n", rc) ;
    DoExit (1) ;
  }

  return ;
}
/* InitializeGlobalData */

/*************************************************************************
  GetVBX

  Allocates a VBX port for the process and initializes it to handle
  the incoming DNIS trunk protocol.  Then connects it to an incoming
  T1 trunk.
**************************************************************************/
int GetVBX ()
{
  int   inT1Trunk, rc ;
  short relaySelect ;
#ifndef UNIXMODE
  int event ;
#endif

  /*------------------------------------------------------------------------
    This routine performs the initial allocation of a VBX port for this
    process.  The same port will be used for the life of the process.  The
    VBX port will be initialized for incoming DNIS service.

    NOTE:  The TRUNK category contains two parameters that must be set
    (they don't have default values).  These parameters define the
    type of trunk and type of hardware.  The trunk type is also the
    name of a category of parameters specific to that trunk
    protocol.

    If any of the trunk protocol parameters are to be changed from their
    defaults, this must be done before calling Me2GrabTrunkLine (UNIX) or
    Me2RequestLine (... , LINETRUNK, ...).

    In this example, TRUNK parameters are set by the program in its C
    language statements.  The same result could be obtained by adding the
    settings to ME2.PRO or to a user profile that the program loads with
    Me2ParmLoad.
    ------------------------------------------------------------------------*/

  /*------------------------------------------------------------------
    Set the name of the initial Trunk Control Program and its
    associated parameter category.  In this case the name is "TCPDID".
    ------------------------------------------------------------------*/

  rc = Me2ParmByName("TRUNK.OVERLAY", "TCPDID", SETPARM) ;

  if (rc != SUCCESS) {
    if (rc == ME2_CATEGORY_NOT_FOUND)
      printf("Missing TRUNK parameter file\n");
    else
      printf ("Error %d setting parameter TRUNK.Overlay\n", rc) ;
    DoExit (1) ;
  }

  /* Set the parameter that defines hardware type */
  relaySelect = 3 ;  /* VBX 1200 or AG */
  rc = Me2ParmByName ("TRUNK.SIGNAL_TYPE", (void *) &relaySelect, SETPARM) ;

  if (rc != SUCCESS) {
    printf ("Error %d setting parameter TRUNK.Signal_Type\n", rc) ;
    DoExit (1) ;
  }

#ifdef UNIXMODE
  /*--------------------------------------------------------------------
    (UNIX) Now get a VBX port.  Me2GrabTrunkLine causes the Trunk
    Control Program (TCP) specified in TRUNK.OVERLAY to be downloaded
    to the VBX port and initialized.
    --------------------------------------------------------------------*/

  if ((rc = Me2GrabTrunkLine (VBXLineNum)) == SUCCESS)
    printf("ME/X trunk line request succeeded. Got VBX line %d\n", VBXLineNum);
  else {
    printf ("Me2GrabTrunkLine returned error %d\n", rc) ;
    DoExit (1) ;
  }
#else  /* OS2MODE */
  /*---------------------------------------------------------------------
    (OS2) Use ME/2 line arbitration to select any free VBX port.
    The LINETRUNK argument causes the Trunk Control Program (TCP)
    specified in TRUNK.OVERLAY to be downloaded to the VBX port and
    initialized.

    Note: Group 0 is a pre-defined group that contains all VBX devices.
    You may choose to define and use a different group.
    ---------------------------------------------------------------------*/

  if ((rc = Me2RequestLine (0 /* group */, LINETRUNK, 0L)) < SUCCESS) {
    printf ("Me2RequestLine returned error %d\n", rc) ;
    DoExit (1) ;
  }
  if ((event = WaitDone ()) == LINRQFAIL) {
    printf ("Line request failed. Me2GetLineError returned %d\n",
	    Me2GetLineError ()) ;
    DoExit (1) ;
  }
  else if (event != LINRQDONE) {
    printf ("Unexpected completion event from line request: %d (0x%x)\n",
	    event, event) ;
    DoExit (1) ;
  }
  else {
    VBXLineNum = Me2GetLine () ;
    printf("ME/2 Line request succeeded. Got VBX line %d\n", Me2GetLine ());
  }
#endif

  /* Convert VBX line number to SwitchPath slot */
  VBXSlot = VBXLineNum - FirstDigitalVBXPort ;

  /*-------------------------------------------------------------------
    NOTE: This demo assumes a one-to-one correspondence of incoming
    T1 trunks to VBX ports.
    -------------------------------------------------------------------*/

  inT1Trunk = VBXSlot ;

  /*-----------------------------------------------------------------
    Connect VBX to incoming T1 trunk before starting execution of
    trunk control program so that the trunk control program doesn't
    detect a false seizure.
    -----------------------------------------------------------------*/

  ConnectVBXToInTrunk (inT1Trunk) ;

  /* Start the trunk control program at its initial entry point */
  if ((rc = Me2TrunkStart (0)) != SUCCESS) {
    printf ("Me2TrunkStart returned error %d\n", rc) ;
    DoExit (1) ;
  }

  printf ("Trunk Control Program started, %d digits are expected\n",
	  NumDNISDigitsExpected) ;

  return inT1Trunk ;
}
/* GetVBX */

/*************************************************************************
   WaitForIncomingCall

   Wait for the trunk event that signifies an incoming call has arrived.
   All other trunk events, including incomplete calls, can be ignored.
**************************************************************************/
void WaitForIncomingCall ()
{
  int  event, i ;
  char dialedNumber[32] ;

  /* Loop until a call is successfully answered */
  for (;;) {
    while ((event = WaitTrunkEvent ()) != TRUNKCALLRECD)
      if (event == TRUNKCALLRECD)
	/* Internal protocol error */
	DoExit (1) ;

    /*----------------------------------------------------------------
      Got a call.  Get the dialed number from the Touch Tone queue.
      Note that the Touch Tone queue (a.k.a. digit queue) will be
      automatically flushed by the ME/X - ME/2 Trunk Manager between
      calls.
      ----------------------------------------------------------------*/

    for (i = 0; i < NumDNISDigitsExpected; i++)
      if (Me2GetTTC (&dialedNumber[i], QREMOVE) != SUCCESS) {
	printf ("(warning) Missing digit(s)\n") ;
	break ;
      }

    /* Form a null terminated string */
    dialedNumber[i] = '\0' ;
    printf ("Call received on DNIS # %s\n", dialedNumber) ;

    /* Generate a 1 second ringing tone and then answer */
    if ((i = Me2TrunkAnswer (100)) != SUCCESS) {
      printf ("Error %d sending response to DID control program\n", i) ;
      DoExit (1) ;
    }

    printf ("Trunk Ring+Answer request successful\n") ;

    /*--------------------------------------------------------------------
      The normal case is that after the ring time period, the Trunk
      Control Program answers the call and the Trunk Manager exits.  A
      TRUNKDONE event will be received by the application.

      If the caller hangs up before the Trunk Control Program answers, then
      the ring tone stops and a TRUNKHANGUP event will be received.  The
      Trunk Manager stays active.

      If the application fails to respond to the TRUNKCALLRECD event in
      a timely manner (e.g. the system is overburdened), the trunk
      control program will time out and start playing a default tone on
      its own.  When the trunk control program times out, it places a
      TRUNKTIMEOUT event in the application's ME/X - ME/2 event queue.
      Once the TRUNKTIMEOUT event has been enqueued, an application
      request to answer the trunk will not be processed.  The application
      must wait for the caller to hangup (TRUNKHANGUP event).
      --------------------------------------------------------------------*/

    /* Wait for the Trunk Control Program to exit after the answer */
    while ((event = WaitTrunkEvent ()) != TRUNKDONE) {
      if (event == TRUNKFAIL)
	/* Internal protocol error */
	DoExit (1) ;

      if (event == TRUNKHANGUP)
	break ;
    }

    if (event == TRUNKHANGUP)
      printf ("Caller abandoned\n") ;
    else
      return ;

    /*--------------------------------------------------------------------
      If a "done" event came, then the Trunk Control Program has stopped
      executing.  Hangup detection is now being done by a DSP supervision
      detector that is independent of the TCP.  From this point on,
      as long as the VBX port is connected to the trunk, a remote hangup
      is reported with a LPCUROFF event from the VBX port.
      --------------------------------------------------------------------*/
  }
}
/* WaitForIncomingCall */

/************************************************************************
  InteractWithCaller

  Application specific code to interact with a caller
  on the incoming T1 trunk.  This demonstration program simply attempts
  to get the directory number to which the incoming caller
  wants to be transferred.  It does this by collecting digits from the
  caller.  This function returns one of the following values:
  0 = Caller hung up
  1 = Need to place outgoing call
**************************************************************************/
int InteractWithCaller (digitBuf)
     char *digitBuf;
{
  int rc ;

  /*--------------------------------------------------------------------
    The 'caller hung up' flag is set in the WaitDone routine if a
    hangup event is received.
   --------------------------------------------------------------------*/

  CallerHungUp = FALSE ;

  /* Collect the digits from the caller */
  rc = CollectTouchTones (digitBuf, NUM_OUTGOING_DN) ;

  /* Timed out or caller hung up during collection of digits */
  if (rc == 0)
    return 0 ;
  else {
    /* Provide some token feedback to the caller */
    Me2Beep (2) ;
    WaitDone () ;

    if (CallerHungUp)
      return 0 ;
    else
      /* Caller requested an outgoing T1 trunk */
      return 1 ;
  }
}
/* InteractWithCaller */


/***************************************************************
  CollectTouchTones

  Uses Me2GetTTS to get a string of Touch Tones.
  This function returns one of the following values:
  0 = Time out or caller hung up
  1 = Success
****************************************************************/
int CollectTouchTones (ttBuf, ttMax)
     char *ttBuf;
     int ttMax;
{
  int rc ;
  ME2INT ttResult ;

  printf ("Waiting for %d digits.\n", NUM_OUTGOING_DN) ;
  rc = Me2GetTTS (NULL, NULL, ttBuf, &ttResult, ttMax) ;

  if (rc != SUCCESS) {
    printf ("Me2GetTTS returned %d\n", rc) ;
    DoExit (1) ;
  }

  WaitDone ()  ;

  if (CallerHungUp)
    return 0 ;

    if (ttResult == 'T')  {
      printf ("Timed out waiting for digits\n") ;
      return 0 ;
    }

  if (ttResult == 'M') {
    printf ("Received digits %s\n", ttBuf) ;
    return 1 ;
  }

  else {
    printf ("Unexpected result '%c' from Me2GetTTs\n", ttResult) ;
    return 0 ;
  }

}
/* CollectTouchTones */


/************************************************************************
  ChangeToOutgoingMode

  Set VBX port to control an outgoing T1 trunk instead of an incoming
  one.
*************************************************************************/
void ChangeToOutgoingMode ()
{
  int rc ;

        /* Switch to outgoing Trunk Control Program */
  rc = Me2TrunkLoad ("TCPOGT") ;

  if (rc != SUCCESS) {
    printf ("Me2TrunkLoad returned error code %d\n", rc) ;
    DoExit (1) ;
  }

  /*--------------------------------------------------------------------
    Start the (outgoing) Trunk Control Program at its initial entry
    point.  This is an idle state, where it is ready to receive a DIAL
    command.
    --------------------------------------------------------------------*/

  if ((rc = Me2TrunkStart (0)) != SUCCESS) {
    printf ("Me2TrunkStart returned error %d\n", rc) ;
    DoExit (1) ;
  }

  /*--------------------------------------------------------------------
    Remove all events from the event queue.  If the incoming call
    hung up, then the global 'CallerHungUp' will be set to TRUE.
    --------------------------------------------------------------------*/

  FlushEventQueue () ;

  printf ("Trunk Control Program re-started in outgoing mode\n") ;
  VBXInOGTMode = TRUE ;
  return ;
}
/* ChangeToOutgoingMode */

/*----------------------------------------------------------------------------
   ChangeToIncomingMode

   Re-initialize VBX port for incoming trunk protocol.  Note that the
   protocol should be started after re-connecting to the incoming trunk,
   in order to avoid incorrectly detecting caller gone or a new incoming
   call.
  ---------------------------------------------------------------------------*/

void ChangeToIncomingMode ()
{
  int rc ;

  /*---------------------------------------------------------------
    Force correct signal towards the VBX to avoid false hangup or
    seizure when the trunk control program is started.
    ---------------------------------------------------------------*/

  ForceVBXSignal ((BOOL) (CallerHungUp ? FALSE : TRUE)) ;
  
  /* Remove all events from the event queue */
  FlushEventQueue () ;

  /* Switch to incoming Trunk Control Program */
  rc = Me2TrunkLoad ("TCPDID") ;

  if (rc != SUCCESS) {
    printf ("Me2TrunkLoad returned error code %d\n", rc) ;
    DoExit (1) ;
  }

  if ((rc = Me2TrunkStart (CallerHungUp ? 0 : 1)) != SUCCESS) {
    printf ("Me2TrunkStart returned error %d\n", rc) ;
    DoExit (1) ;
  }

  printf ("Initialized Incoming Trunk Control Program\n") ;
  VBXInOGTMode = FALSE ;
  return ;
}
/* ChangeToIncomingMode */

/*****************************************************************
   CallOut

   Dial the outgoing call and wait for call progress indication.
   Return code indicates call disposition.
******************************************************************/
int CallOut (dn)
     char *dn;
{
  BOOL dialFailed = FALSE ;
  int  event, rc ;

  /*-------------------------------------------------------------------
    Need to check if hangup was detected.  It could have been
    detected while the queue was being emptied in
    ChangeToOutgoingMode ().
    -----------------------------------------------------------------*/

  if (CallerHungUp)
    /* Stop the outgoing Trunk Control Program */
    Me2Stop () ;
  else {
    printf ("Dialing %s\n", dn) ;

    if ((rc = Me2TrunkDial (dn)) != SUCCESS) {
      printf ("Me2TrunkDial returned error %d\n", rc) ;
      DoExit (1) ;
    }
    Me2TrunkAnswer (100000);
  }

  /*-------------------------------------------------------------------
    Wait for dial completion (the Trunk Control Program will
    automatically exit).

    The TRUNKDONE event indicates that the Trunk Control Program
    running on the VBX or AG board has stopped and that answer
    detection is now being done by the DSP supervision detector.

    TRUNKSTOP may be received instead, if Me2Stop was called.
    -------------------------------------------------------------------*/

  while ((event = WaitTrunkEvent ()) != TRUNKDONE) {
    if (event == TRUNKFAIL)
      /* Internal protocol error */
      DoExit (1) ;

    if (event == TRUNKWINKTIMEOUT) {
      printf ("FAILED: Timed out waiting for wink\n") ;
      dialFailed = TRUE ;
    }
    else if (event == TRUNKSTUCKSENDER) {
      printf ("FAILED: Timed out waiting for end of wink\n") ;
      dialFailed = TRUE ;
    }
    else if (event == TRUNKSTOP)
      break ;
  }

  if (dialFailed || CallerHungUp)
    return 0 ;
  else
    return 1 ;
}
/* CallOut */

/*********************************************************
   FlushEventQueue

   Remove all events from the ME/X - ME/2 event queue.
**********************************************************/
void FlushEventQueue ()
{
  ME2INT event ;

  /*--------------------------------------------------------------------
    Pull all events from the event queue to find any lurking
    disconnect events.  This is needed to discover if the caller has
    released and also to ensure that a leftover event doesn't cause
    problems later.
    --------------------------------------------------------------------*/

  while (Me2GetEvent (&event, QREMOVE) == SUCCESS)  {
    if (event == CALLER_GONE) {
      /* Hangup detected by SwitchPath */
#ifdef UNIXMODE
      printf ("Received on-hook event from trunk monitor process\n") ;
#else  /* OS2MODE */
      printf ("Received on-hook event from trunk monitor thread\n") ;
#endif

      CallerHungUp = TRUE ;
    }
    else if (event == LPCUROFF && !VBXInOGTMode) {
      /* Hangup detected by VBX */
      printf ("Caller hangup detected\n") ;
      CallerHungUp = TRUE ;
    }
  }

  return ;
}
/* FlushEventQueue */

/****************************************************************
   HangupIncomingCall

   Send a disconnect to the incoming trunk.
*****************************************************************/
void HangupIncomingCall ()
{
  int rc ;

  printf ("Hanging up the incoming call\n") ;

  /*--------------------------------------------------------------------
    Now start the Trunk Control Program at a specified entry point to
    release the current call.  If the call has not been released by
    the caller, then to initiate the disconnect sequence and wait for
    caller to release, use entry point 1.  If the call has already
    been released, then treat an incoming off-hook as a new call
    (entry point 0).

    This function will also disable the VMOS supervision signal
    monitor because signal detection will be taken over by the Trunk
    Control Program.

    If caller already hung up, start the trunk protocol at entry point
    0 to watch for new incoming call.  Otherwise start at '1' to wait
    for the caller to hangup.
    --------------------------------------------------------------------*/

  if ((rc = Me2TrunkStart (CallerHungUp == TRUE ? 0 : 1)) != SUCCESS) {
    printf ("Me2TrunkStart returned error %d\n", rc) ;
    DoExit (1) ;
  }

  printf ("Incoming Trunk Control Program restarted\n") ;
  return ;
}
/* HangupIncomingCall */


/*****************************************************************
   WaitForDisconnect

   Wait for either end of the call to release.
******************************************************************/
void WaitForDisconnect ()
{
  ME2INT event;
  int rc ;

  /*--------------------------------------------------------------------
    Wait for either end to hangup.  Called party hangup will be
    indicated by a LPCUROFF event from the VBX port.  Caller hangup
    will be received as the locally defined "caller gone" event from
    the monitor process (UNIX) or thread (OS/2).
    --------------------------------------------------------------------*/

  puts ("Waiting for disconnect event...") ;

  for (;;) {
    /* Wait for an event to come through the ME/X - ME/2 event queue */
    rc = Me2WaitEvent (20000L) ;

    if (rc == ME2_QUEUE_TIMEOUT) {
      puts ("still waiting for disconnect event...") ;
      continue ;
    }
    else if (rc != SUCCESS) {
      printf ("Me2WaitEvent returned error %d\n", rc) ;
      DoExit (1) ;
    }

    if ((rc = Me2GetEvent (&event, QREMOVE)) != SUCCESS) {
      printf ("Me2GetEvent returned error %d\n", rc) ;
      DoExit (1) ;
    }

    if (event == LPCURON) {
      /* VBX is monitoring the outgoing call */
      printf ("CALLED PARTY ANSWER DETECTED\n") ;
      DestHungUp = FALSE ;
      continue ;
    }

    if (event == LPCUROFF) {
      /* VBX is monitoring the outgoing call */
      printf ("CALLED PARTY HANGUP DETECTED\n") ;
      DestHungUp = TRUE ;
      break ;
    }

    if (event == CALLER_GONE) {
#ifdef UNIXMODE
      printf ("Received on-hook event from trunk monitor process\n") ;
#else  /* OS2MODE */
      printf ("Received on-hook event from trunk monitor thread\n") ;
#endif
      CallerHungUp = TRUE ;
      break  ;
    }
  }
}
/* WaitForDisconnect */

/***********************************************************************
  WaitDone

  Local subroutine to wait for completion of a background function. If
  a hangup event is detected, this routine will stop the background
  function, wait for the done event, and return a hangup indication.
************************************************************************/
int WaitDone ()
{
  ME2INT event ;
  int rc ;

  for (;;) {
    /* Wait for an event to come through the event queue */
    rc = Me2WaitEvent (-1L) ;

    if (rc != SUCCESS) {
      printf ("Me2WaitEvent returned error %d\n", rc) ;
      DoExit (1) ;
    }

    if ((rc = Me2GetEvent (&event, QREMOVE)) != SUCCESS) {
      printf ("Me2GetEvent returned error %d\n", rc) ;
      DoExit (1) ;
    }

/*---------------------------------------------------------------
  If caller hangs up, abort the active ME/X - ME/2 function.
  Note that the CALLER_GONE event is sent by monitor
  process/thread.
 ---------------------------------------------------------------*/

    if (event == LPCUROFF && !VBXInOGTMode || event == CALLER_GONE) {
      printf ("Caller HANGUP detected\n") ;
      Me2Stop () ;
      CallerHungUp = TRUE ;
    }
    else if (event == LPCUROFF && VBXInOGTMode) {
      printf ("CALLED PARTY HANGUP DETECTED\n") ;
      Me2Stop () ;
      DestHungUp = TRUE ;
    }
    /* Check for done event */
    else if (EXITCODE <= (unsigned)event && (unsigned)event <= EXITCMAX)
      return event ;
  }
}
/* WaitDone */

/*************************************************************************
  WaitTrunkEvent

  Local subroutine to wait for an event from the trunk handler process/
  thread.  This routine looks for events in the TRUNKCODEMIN -
  TRUNKCODEMAX range and also for any exit code.
**************************************************************************/
int WaitTrunkEvent ()
{
  ME2INT event ;
  int rc ;

  for (;;) {
    /*----------------------------------------------------------------
      Wait for an event to come through the ME/X - ME/2 event queue.
      A timeout is used to print a periodic message for debugging
      purposes.
    ----------------------------------------------------------------*/

    rc = Me2WaitEvent (10000L) ;
    
    if (rc == ME2_QUEUE_TIMEOUT) {
      puts ("still waiting for trunk event...") ;
      continue ;
    }

    if (rc != SUCCESS) {
      printf ("Me2WaitEvent returned error %d\n", rc) ;
      DoExit (1) ;
    }

    if ((rc = Me2GetEvent (&event, QREMOVE)) != SUCCESS) {
      printf ("Me2GetEvent returned error %d\n", rc) ;
      DoExit (1) ;
    }
    /* Check for done code */
    else if (EXITCODE <= (unsigned)event && (unsigned)event <= EXITCMAX){
      if (event == TRUNKFAIL)
	printf ("TRUNKFAIL received\n") ;
      else if (event == TRUNKDONE)
	printf ("TRUNKDONE received\n") ;
      else if (event == TRUNKSTOP)
	printf ("TRUNKSTOP received\n") ;

      return event ;
    }
    /* Check for Trunk Manager event */
    else if (TRUNKCODEMIN <= event && event <= TRUNKCODEMAX) {
      ShowEvent (event) ;
      return event ;
    }

    /*----------------------------------------------------------------
      If a hangup event is received from the incoming call monitor
      process (UNIX) or thread (OS2) during outgoing call set up,
      then stop the Trunk Manager.  If a stray event arrives while
      the VBX is connected to the incoming trunk, ignore it.
      ----------------------------------------------------------------*/

    if (VBXInOGTMode && event == CALLER_GONE) {
      printf ("Received on-hook event from trunk monitor process\n") ;
      Me2Stop () ;
      CallerHungUp = TRUE ;
      /* Don't return yet, wait for TRUNKSTOP */
    }
  }
}
/* WaitTrunkEvent */

/******************************************************************
  ShowEvent

  Local subroutine to display a Trunk Manager event.
*******************************************************************/
void ShowEvent (event)
     int event;
{
  char        buf[80] ;
  char        *eventText ;
  static char *trunkEvent[] = {
                                "TRUNKBEG        ",
                                "TRUNKCALLRECD   ",
                                "TRUNKHANGUP     ",
                                "TRUNKTIMEOUT    ",
                                "TRUNKSEIZED     ",
                                "TRUNKNODIGITS   ",
                                "TRUNKPARTIALDIAL",
                                "TRUNKPERMANENT  ",
                                "TRUNKWINKTIMEOUT",
                                "TRUNKSTUCKSENDER",
                                "TRUNKENDOFDIAL  ",
                                "TRUNKCODEMAX    "  };

  if (TRUNKCODEMIN <= event && event <= TRUNKCODEMAX)
    eventText = trunkEvent[event - TRUNKCODEMIN] ;
  else {
    sprintf (buf, "0x%x", event) ;
    eventText = buf ;
  }
  printf ("Received event: %s\n", eventText) ;
  return ;
}
/* ShowEvent */

#ifdef UNIXMODE
/**********************************************************************
  SendEventToParent  (UNIX)
  
  Use vbxsndmsg pseudo device to enqueue an event onto the parent's
  ME/X event queue.
***********************************************************************/
int SendEventToParent (destDevice, event)
     int destDevice;
     int event;
{
  static int dh  = 0;

  if (Me2IsAStream()) {
    VBXS_ioctl_t vbxioctl;
    struct strioctl strioctl;

    if (dh == 0)
          /* Open the pseudo device */
      if ((dh = open ("/dev/vbxesndmsg", O_RDONLY)) == -1) {
	printf ("Can't open %s.\n", dh);
	return -1;
      }

    vbxioctl.hdr.nparm = 2; /* 2 word parameters for SNDMSG command */
    vbxioctl.hdr.dir = 0; /* No data            */

    vbxioctl.parm[0] = (short) destDevice;
    vbxioctl.parm[1] = (short) event;

    strioctl.ic_cmd = SNDMSG;
    strioctl.ic_timout = -1;
    strioctl.ic_len = sizeof (PARMHDR) + vbxioctl.hdr.nparm *
      sizeof (vbxioctl.parm[0]);
    strioctl.ic_dp = (char *) &vbxioctl;

    if (ioctl (dh,I_STR,&strioctl) == -1) {
      printf("Error issuing IOCTL command = %d.\n",errno);
      return -1;
    }

    return SUCCESS;
  }
  else {
    PARMBLOCK  pb ;

    if (dh == 0) {
      /* Open the pseudo device */
      if ((dh = open ("/dev/vbxsndmsg", O_RDONLY)) == -1) {
	printf ("Can't open %s.\n", dh) ;
	return -1 ;
      }
    }

    pb.hdr.nparm = 2 ;  /* 2 word parameters for SNDMSG command */
    pb.hdr.dir   = 0 ;  /* No data                              */

    pb.parm[0] = (short) destDevice ;
    pb.parm[1] = (short) event ;

    if (ioctl (dh, SNDMSG, &pb) == -1) {
      printf ("Error issuing IOCTL command = %d.\n", errno) ;
      return -1 ;
    }

    return SUCCESS ;
  }
}
/* Sendeventtoparent */
#endif

/*====================================================================*/
/*---------------------------------------------------------------------
  Switching Routines
  
  The following routines interface with SwitchPath to perform all
  voice path switching and to control and monitor signaling.
----------------------------------------------------------------------*/

/* Globals used by switching routines */

SWPINT DTISwitch = -1 ;  /* switch handle */
SWPINT T1AVoice  = -1 ;  /* group handle  */
SWPINT T1BVoice  = -1 ;  /* group handle  */
SWPINT T1ASignal = -1 ;  /* group handle  */
SWPINT T1BSignal = -1 ;  /* group handle  */
SWPINT VBXVoice  = -1 ;  /* group handle  */
SWPINT VBXSignal = -1 ;  /* group handle  */

long SupvId   = 0 ;   /* ID used for CxWaitSupv and CxStopSupv */

/********************************************************************
*  InitializeSwitching
*
*  Initialize SwitchPath and get handles.
*********************************************************************/
void InitializeSwitching ()
{
  /*-------------------------------------------------------------------
    The various ASCII switch and group names are assigned in the
    configuration file.
  -------------------------------------------------------------------*/

  /*-------------------------------------------------------------------
    Macro to check for error while getting handle.  An error implies that
    a name in the configuration file is not consistent with the program.
    ------------------------------------------------------------------*/

#define CHECKHANDLE(hdl)                                       \
  if (hdl < 0) {                                               \
    printf ("Error %d getting handle for \" #hdl \" \n", hdl) ;   \
    DoExit (1) ;                                               \
  }                                                            

  /* Access SwitchPath services. */
  CxInitialize (INITTYPE_DEFAULT) ;

  /* Get handles */
  DTISwitch = CxGetSwitchHd ("DT0") ;
  CHECKHANDLE(DTISwitch) ; 

  T1AVoice = CxGetGroupHd ("T1AVoice") ;
  CHECKHANDLE (T1AVoice) ;

  T1ASignal = CxGetGroupHd ("T1ASignal") ;
  CHECKHANDLE (T1ASignal) ;

  T1BVoice = CxGetGroupHd ("T1BVoice") ;
  CHECKHANDLE (T1BVoice) ;

  T1BSignal = CxGetGroupHd ("T1BSignal") ;
  CHECKHANDLE (T1BSignal) ;

  VBXVoice = CxGetGroupHd ("VBXVoice") ;
  CHECKHANDLE (VBXVoice) ;

  VBXSignal = CxGetGroupHd ("VBXSignal") ;
  CHECKHANDLE (VBXSignal) ;

  return ;
}
/* InitializeSwitching */

/********************************************************
  ConnectVBXToInTrunk

  Connects the VBX port to an incoming T1 trunk.
*********************************************************/
void ConnectVBXToInTrunk (inT1Trunk)
     int inT1Trunk;
{
  int rc ;

  /* Make the voice connection */
  rc = CxConnect (DTISwitch,    /* switch handle              */
		  VBXVoice,     /* source group handle        */
		  VBXSlot,      /* source group member        */
		  T1AVoice,     /* destination group handle   */
		  inT1Trunk,    /* destination group member   */
		  DUPLEX) ;     /* connect in both directions */

  if (rc != 0) {
    printf ("CxConnect returned %d\n", rc) ;
    DoExit (1) ;
  }

  /* Make the signaling connection */
  rc = CxConnect (DTISwitch,  /* switch handle              */
		  VBXSignal,  /* source group handle        */
		  VBXSlot,    /* source group member        */
		  T1ASignal,  /* destination group handle   */
		  inT1Trunk,  /* destination group member   */
		  DUPLEX) ;   /* connect in both directions */

  if (rc != 0) {
    printf ("CxConnect returned %d\n", rc) ;
    DoExit (1) ;
  }

  return ;
}
/* ConnectVBXToInTrunk */

/*********************************************************
  ConnectVBXToOutTrunk

  Connects the VBX port to an outgoing T1 trunk.
**********************************************************/
void ConnectVBXToOutTrunk (outT1Trunk)
     int outT1Trunk;
{
  int rc ;

  /* Make the voice connection */
  rc = CxConnect (DTISwitch,   /* switch handle              */
		  VBXVoice,    /* source group handle        */
		  VBXSlot,     /* source group member        */
		  T1BVoice,    /* destination group handle   */
		  outT1Trunk,  /* destination group member   */
		  DUPLEX) ;    /* connect in both directions */

  if (rc != 0) {
    printf ("CxConnect returned %d\n", rc) ;
    DoExit (1) ;
  }

  /* Make the signaling connection */
  rc = CxConnect (DTISwitch,   /* switch handle              */
		  VBXSignal,   /* source group handle        */
		  VBXSlot,     /* source group member        */
		  T1BSignal,   /* dest group handle          */
		  outT1Trunk,  /* dest member                */
		  DUPLEX) ;       /* connect in both directions */

  if (rc != 0) {
    printf ("CxConnect returned %d\n", rc) ;
    DoExit (1) ;
  }

  return ;
}
/* ConnectVBXToOutTrunk */


/*********************************************************************
  SilenceInTrunk

  Break the talk path to the caller so that she/he doesn't hear the
  DTMF used to dial the outgoing call.
**********************************************************************/
void SilenceInTrunk (inTrunk)
     int inTrunk;
{
  int rc ;

  /* Disconnect the voice path by sending a silence pattern to it */
  rc = CxSendPattern (DTISwitch,  /* switch handle            */
		      0x7F,       /* digital silence          */
		      T1AVoice,   /* destination group handle */
		      inTrunk) ;  /* destination group member */

  if (rc != 0) {
    printf ("CxSendPattern returned %d\n", rc) ;
    DoExit (1) ;
  }

  return ;
}
/* SilenceInTrunk */

/*************************************************************************
  ConnectTrunks

  Connects the incoming and outgoing trunks' voice paths together. Note
  that their signaling paths are not connected together.
**************************************************************************/
void ConnectTrunks (inT1Trunk, outT1Trunk)
     int inT1Trunk;
     int outT1Trunk;
{
  int rc ;

  rc = CxConnect (DTISwitch,   /* switch handle              */
		  T1AVoice,    /* source group handle        */
		  inT1Trunk,   /* source group member        */
		  T1BVoice,    /* destination group handle   */
		  outT1Trunk,  /* destination group member   */
		  DUPLEX) ;    /* connect in both directions */

  if (rc != 0) {
    printf ("CxConnect returned %d\n", rc) ;
    DoExit (1) ;
  }

  return ;
}
/* ConnectTrunks */

/************************************************************
   DisconnectOutTrunk

   Break the connections to the outgoing T1 trunk.
*************************************************************/
void DisconnectOutTrunk (outT1Trunk)
     int outT1Trunk;
{
  int rc ;

  /* Disconnect the voice path by sending a silence pattern to it. */
  rc = CxSendPattern (DTISwitch,     /* switch handle            */
		      0x7F,          /* digital silence          */
		      T1BVoice,      /* destination group handle */
		      outT1Trunk) ;  /* destination member       */

  if (rc != 0) {
    printf ("CxSendPattern returned %d\n", rc) ;
    DoExit (1) ;
  }

  /* Disconnect the signaling path by sending an on-hook pattern to it */
  rc = CxSendPattern (DTISwitch,     /* switch handle            */
		      0x00,          /* on-hook                  */
		      T1BSignal,     /* destination group handle */
		      outT1Trunk) ;  /* destination member       */

  if (rc != 0) {
    printf ("CxSendPattern returned %d\n", rc) ;
    DoExit (1) ;
  }

  return ;
}
/* DisconnectOutTrunk */

/***************************************************
  ForceVBXSignal

  Jam a signal into the VBX port.
****************************************************/
void ForceVBXSignal (offHook)
     BOOL offHook;
{
  int rc ;

  rc = CxSendPattern (DTISwitch,              /* switch handle            */
		      offHook ? 0x08 : 0x00,  /* set or clear A bit       */
		      VBXSignal,              /* destination group handle */
		      VBXSlot) ;              /* destination member       */


  if (rc != SUCCESS) {
    printf ("CxSendPattern returned %d\n", rc) ;
    DoExit (1) ;
  }

  return ;
}
/* ForceVBXSignal */

/********************************************************************
  StartSupervision

  Start a child process (UNIX) or thread (OS/2) to monitor the
  incoming call for hangup.
*********************************************************************/
void StartSupervision (t1Trunk)
     int t1Trunk;
{
  SUPVMON s ;
  int rc ;

  /* Control the signaling by sending an off-hook pattern to it */
  rc = CxSendPattern (DTISwitch,  /* switch handle              */
		      0x0C,       /* off-hook (A + B)           */
		      T1ASignal,  /* calling party group handle */
		      t1Trunk) ;  /* calling party member       */

  if (rc != 0) {
    printf ("CxSendPattern returned %d\n", rc) ;
    DoExit (1) ;
  }

  /* Start the background supervision monitoring */
  s.target  = A_BIT_OFF;             /* signal to watch for        */
  s.qualify = 100 ;                  /* time (ms) to qualify       */
  s.assert  = A_BIT_OFF + B_BIT_OFF; /* signal to assert on detect */

  rc = CxStartSupv (DTISwitch,  /* switch handle */
		    T1ASignal,  /* group         */
		    t1Trunk,    /* member        */
		    &s) ;       /* misc parms    */

  if (rc != SUCCESS) {
    printf ("Error %d returned by CxStartSupv\n", rc) ;
    DoExit (1) ;
  }

  /*------------------------------------------------------------------
    Save supervision ID in a global, for later passing to CxWaitSupv
    and CxStopSupv.
    ------------------------------------------------------------------*/

  SupvId = s.supv_id ;

#ifdef UNIXMODE
  /* Create a child process to monitor the incoming call */
  if (fork () == 0)
    /* Child process */
    MonitorSupervision () ;
  else
    ;    /* Parent process */

#else  /* OS2MODE */
  /* OS.2: Start a thread to monitor the incoming call supervision */
#if defined (__BORLANDC__)
  if (_beginthread (MonitorSupervision, 8192, NULL)
#else  /* Microsoft 6.0 or IBM C/Set */
  if (_beginthread (MonitorSupervision, NULL, 8192, NULL)
#endif
      == -1) {
    perror ("Unable to start monitor thread") ;
    DoExit (1) ;
  }
#endif
  return ;
}
/* StartSupervision */

/********************************************************
   StopSupervision

   Stop monitoring the T1 trunk for hangup.
*********************************************************/
void StopSupervision ()
{
  int rc ;

  rc = CxStopSupv (DTISwitch, SupvId) ;

  /*---------------------------------------------------------------
    CxStopSupv may return the error DRV_INVALID_SUPV_ID because
    of a race condition over which end hung up first.  Therefore,
    this error is ignored.  A CALLER_GONE event sent by the supervision
    monitor thread/process may be waiting on the event queue.
    ---------------------------------------------------------------*/

  if (rc != SUCCESS && rc != DRV_INVALID_SUPV_ID) {
    printf ("Error %d returned by CxStopSupv\n", rc) ;
    DoExit (1) ;
  }

  return ;
}
/* StopSupervision */

/***********************************************************************
  MonitorSupervision

  Function called by child process (UNIX) or thread (OS/2) to monitor
  for caller hangup in parallel with other activities.  If
  hangup is detected, it sends a locally defined event onto the main
  process's/thread's event queue.

  The monitor function will automatically drop answer when it detects
  the hangup.
*************************************************************************/
#ifdef UNIXMODE
void MonitorSupervision ()
#else /* OS2 */
#if defined (__BORLANDC__)
     void _USERENTRY MonitorSupervision (void *dummy)
#else /* MSC 6.0  or IBM C*/
     void MonitorSupervision (void *dummy)
#endif
#endif
{
  int     rc ;

  rc = CxWaitSupv  (DTISwitch,
		    SupvId,     /* supervision ID */
		    -1L) ;      /* No time limit on wait */

  if (rc == SUCCESS)
#ifdef UNIXMODE
    SendEventToParent (VBXLineNum, CALLER_GONE) ;
#else  /* OS2MODE */
  Me2SendEvent (Me2GetChannel (), CALLER_GONE) ;
#endif

    else if (rc != DRV_SUPV_MONITOR_STOPPED) {
      printf ("Error %d returned by CxWaitSupv\n", rc) ;
      exit (1) ;
    }

#ifdef UNIXMODE
  exit (0) ;
#else  /* OS2MODE */
  _endthread () ;
  dummy = dummy ;
#endif
}
/* MonitorSupervision */

/***********************************************************************
  DoExit ()

  This function is used to abort cleanly from the application if a
  failure occurs.
************************************************************************/
void DoExit (errorCode)
     int errorCode;
{
  /* Terminate use of SwitchPath */
  CxTerminate () ;

  /* Release the line and channel before exiting */
  Me2ReleaseLine () ;
  Me2ReleaseChannel () ;

  exit (errorCode) ;
}
/* DoExit */


/*****************************************************************************
PLAY:
   This routine plays the specified message from a particular voice file.
   It waits for the completion of play before returning.  While playing special
   event processing occurs to detect local phone hangup, <ESC> key press, etc.

   Argument List:    ME2INT vsel    => Voice file handle.
                     ME2INT msg     => Message number to be played.

   Return Value:          value from waitdone()
*****************************************************************************/
ME2INT play( vsel, msg )
     ME2INT vsel, msg;  
{
  ME2INT i;

  i = ME2VOXPLAYMSG( vsel, msg );
  if( i != SUCCESS ) {               /* Display error msg if play fails. */
    puts( "\tProblem with prompt voice file.");
    voxexit( i );
  }
  vox_waitdone(MAXWAIT); 
  return ;
}


/****************************************************************************
RECPLAY:
   This function records a test message and plays it
   back when the record is done executing.

   Return Value:        value from waitdone()
*****************************************************************************/
ME2INT recplay()
{
  ME2INT i ;
  
  if ((i=play( Vmsg, MPMPT1 )) != 0)  /* Play prompt for recording test msg */
    return i;

  ME2BEEP( 1 );                          /* Beep tone for start of record. */
  if ((i=vox_waitdone( MAXWAIT )) != 0)
    return i;

  i = ME2VOXRECORDMSG(Vrec, MRECORD, Maxrec); /* Record up to Maxrec secs */
  if( i != SUCCESS ){                /* Display error message if */
                                     /*  record failed.          */
    puts("\tError recording speech! Bad file handle or no available disk space." );
    voxexit( i );
  }
  if ((i=vox_waitdone( MAXWAIT )) != 0)
    return i;

  flushttqueue();              /* Empty the touch-tone queue.  */

  if ((i=play( Vmsg, MPMPT2 )) != 0)  /* Play prompt that prepares caller         */
    return i;                /* to hear what was recorded.   */

  i = ME2VOXPLAYMSG( Vrec, MRECORD ); /* Play msg that contains their voice.*/
  if( i != SUCCESS ){  /* Abort if problem with that voice file. */
    puts( "\tError playing message back." );
    voxexit( i );
  }
  return( vox_waitdone( MAXWAIT ) );
}


/*******************************************************************
* setparm - subroutine to call Me2ParmByName and test the return code.
******************************************************************/
void setparm(parm, val)
     char *parm;
     void *val;
{
  ME2INT i=ME2PARMBYNAME(parm, val, SETPARM) ;
  if (i != SUCCESS) {
    if (i==ME2_CATEGORY_NOT_FOUND)
      printf("Category not found for %s\n", parm);
    else
      printf("Error %d setting parameter %s\n", i, parm) ;
    ME2RELEASELINE();
    ME2RELEASECHANNEL();
    exit (1) ;
  }
  return ;
}


/*******************************************************************************
VOXEXIT:
   This function is used to abort cleanly from the application if a failure
   occurs.  Error descriptions can found in ME2ERR.H.
   Argument List:    ME2INT n       =>    Error code.
   Return Value:     No return
*******************************************************************************/
void voxexit( n )
    ME2INT n;
{
  if( Vmsg >= 0 )            /* Close both voice files if opened. */
    ME2VOXCLOSE( Vmsg );
  if( Vrec >= 0 )
    ME2VOXCLOSE( Vrec );

  ME2RELEASELINE();          /* Release the line                  */
  ME2RELEASECHANNEL();       /* and channel before exiting.       */

  strcat( Vrecfil, ".vox");
#ifdef __STDC__
  remove( Vrecfil );
#else
  unlink( Vrecfil );
#endif

  if( n != SUCCESS ) {
    printf( "ME/2 error code: %d, process aborted.\n", n );
    printf( "Refer to 'me2err.h' and your manual for a list of errors.\n" );
  }

  exit( n );
}


/*******************************************************************
* InitializeDSP ()   - set ME2 parameters
*
* For this example, parameters are set by the program.  The same result
* could be obtained by adding the settings to ME2.PRO, or to a user profile
* that is loaded with Me2ParmLoad.  Note that setting of trunk-related
* parameters must be done before getting a port with Me2RequestLine.
********************************************************************/
void InitializeDSP ()
{
  short val ;
  char  *parm ;
  ME2INT prompt_handle ;
  ME2INT vh,
         buff[20] ;
  short id=0 ;
  short linenum;
  char  *trunk_type_text ;
  ME2INT i ;

  /*xx- Set the parameter that defines hardware type,  -xx*/
  /*xx- unless already set in ME2.PRO.                 -xx*/

 i = ME2PARMBYNAME( "SPCP.MAX_RECORD", &Maxrec, GETPARM);
 if(i != SUCCESS){ /* See if call to ME/2 function is successful */
    puts( "\tError reading SPCP.MAX_RECORD parameter." );
    voxexit(i);   /* If not, process routine to provide error msg. */
 }

 i = ME2PARMBYNAME( "SPCP.VOX_MSG_FILE", Vmsgfil, GETPARM);
 if(i != SUCCESS){ /* See if call to ME/2 function is successful */
    puts( "\tError reading SPCP.VOX_MSG_FILE parameter." );
    voxexit(i);   /* If not, process routine to provide error msg. */
 }

  /*  Here we are appending our channel number to the end of the test
      voice file name in order to make sure it is unique from the test voice
      file used by any other channels.  It is because of this appending
      process that we only allow six characters in in the test voice file
      name, instead of eight (see parameter file).
    */
 i = ME2PARMBYNAME( "SPCP.VOX_REC_FILE", Vrecfil, GETPARM);

 if(i != SUCCESS){ /* See if call to ME/2 function is successful */
    puts( "\tError reading SPCP.VOX_REC_FILE parameter." );
    voxexit(i);   /* If not, process routine to provide error msg. */
 }

 strcat( Vrecfil, buff );             /* Append to end of name.  */

 i = ME2VOXOPEN(Vmsgfil, &Vmsg, PLAYONLY);
 if(i != SUCCESS){          /* See if call to ME/2 function is successful */
    puts( "\tSPCP.vox file could not be found in the correct subdirectory!" );
    voxexit(i);           /* If not, process routine to provide error msg. */
 }

 i = ME2VOXOPEN( Vrecfil, &Vrec, PLAYREC);
 if(i != SUCCESS){         /* See if call to ME/2 function is successful */
    puts( "\tError creating/opening the test file." );
    voxexit(i);          /* If not, process routine to provide error msg. */
 }
}



/*****************************************************************************
VOX_WAITDONE:
   This function waits for a 'DONE' code to appear on the event queue
   while checking any other events coming in to see if they need special
   handling. All other events are thrown away.  Note that even though Touch-
   Tone events are thrown away here, ME/2 maintains a copy of them in the TT
   queue for Me2GetTTS() and Me2GetTTC() to process.

   All ME/2 functions which take more than a fraction of a second to
   complete are designed to return immediately upon startup and then
   report a 'DONE' event code when the action is totally finished.  A
   function of this type generally has three valid 'DONE' codes to
   indicate one of the following:  finished normally, stopped by
   Me2Stop(), or the function failed and was aborted.

   All events which indicate the termination of a ME/2 function have been
   placed in a special range of values which can be easily checked for.
   vox_waitdone () returns when an event code in the 'DONE' code value range
   appears on the event queue.

   Return Value:        1   A hangup has been detected. Stop currently
                            active function and return.
                        0   No special event has been found.
                       <0   Timeout
*******************************************************************************/
ME2INT vox_waitdone (timeout)
  long timeout;
{
  ME2INT ev ;
  ME2INT i;

  while (1) {
    i = ME2WAITEVENT (timeout);   /* Wait for an event to come through */
    /* the event queue.                  */
    if (i == ME2_QUEUE_TIMEOUT){  /* Check and see if a timeout.  */
      ME2STOP();                  /* If timeout, stop everything  */
      return( i );                /* and abort.                   */
    }

    ME2GETEVENT( &ev, QREMOVE );   /* Get the event off the queue*/
    if (ev == LPCUROFF){     /* Check and see if a hangup occurred*/
      ME2STOP();                 /* If so, stop everything and   */
      return( 1 );               /* return.                       */
    }

          /* Check for done code.  */
    if( EXITCODE <= (unsigned)ev && (unsigned)ev <= EXITCMAX )
      return( 0 );
  }
}



/******************************************************************************
FLUSHTTQUEUE:
   This function removes any existing touchtones from the touch-tone queue.
   Return Value:     none
******************************************************************************/
void flushttqueue()
{
  char c;

  while( ME2GETTTC( &c, QREMOVE ) == 0 ) ;
  return ;
}



/*--------- End of spcp.c Single-Processing CallProcs  Program ------------*/


