/****************************************************************************
FILE:     beep.c

****!!!! THIS MODULE FOR CUSTOMER USE !!!!****
AUTHOR:   Michael Hoffman
CREATED:  880420

   $$History
*   1 BEEP.C 11-Aug-89,18:09:50,`GS' Placed under revision control
*   2 BEEP.C 15-Mar-90,19:51:44,`GS' If requestfunction fails, pass its error.
*   3 BEEP.C 19-Mar-90,16:24:32,`GS' Added TT Abort parm and global
*        if ME2HDLRATTACH returns -1 (tt abort), then detach.
*        Made all globals static (local to file);  replaced Vox... with Beep...
*   4 BEEP.C 26-Mar-90,13:31:24,`GS' Was absorbing Touch Tones
*   5 BEEP.C 6-Apr-90,17:30:22,`GS' Added special case '2' of TTAbort.
*   6 BEEP.C 26-Apr-90,13:53:54,`GS' Use if-else with HDLRDETACH, for DOS
*   7 BEEP.C 6-Jun-91,16:32:58,`GS' (RGP) added error checks
*   8 BEEP.C 14-Aug-91,10:57:22,`GS' Fixed parens around requestfunction call
*   9 BEEP.C 3-Sep-91,11:39:52,`GS' On FAILFUNC, stop function, flush bufs
*   10 BEEP.C 23-Sep-91,17:24:22,`GS' Use Me2HdlrDetach for startup failure
*   11 BEEP.C 1-Sep-92,17:15:46,`RP' Removed cdecl from declarations and
*        definitions of beepmon () and beeploop ().
*   12 BEEP.C 19-Jan-94,15:30:36,`GS' Calls Me2HdlrTimeLimit
*   13 BEEP.C 4-Mar-94,18:18:18,`GS' Load Category globals only once.
   $$History

****!!!! EXCISE ABOVE FOR CUSTOMER USE !!!!****

DESCRIPTION:

The ME2BEEP() function allows a specified number of beeps to be
generated.  This command follows the general program structure necessary
for any 'wait-for-done-code' operation to work properly in the ME/2
environment.

Copyright (c) 1987-94 Natural Microsystems Corporation.  All rights reserved.
****************************************************************************/
/* $$Revision "&(#)%n r%v" */
static char Revision[] = "&(#)BEEP.C r13";

#include "me2beep.h"         /* include all necessary definitions */

static int   BeepStarted = 0;
static int   BeepStopped = 0;
static short TTAbort = 0;
static int   BeepCat = -1 ;

static XFERPARM BeepPrm= {0};
static int   VoxMsg = -1;

static void far beepmon();
static void far beeploop();


/******************************************************************
ME2BEEP - generates 'cnt' beeps on the telephone.  If 'cnt' is
        0, then the value of the parameter BEEP.COUNT will be used
        instead for the number of beeps.  The following parameters
        will control the characteristics of the beeps heard:


             Name           Default  Description
             -----------------------------------------------------
             BEEP.FREQUENCY    3     Tone frequency.  Range:
                                       0 - 250 Hz.     4 - 1250 Hz.
                                       1 - 500 Hz.     5 - 1500 Hz.
                                       2 - 750 Hz.     6 - 1750 Hz.
                                       3 - 1000 Hz.    7 - 2000 Hz.
             BEEP.AMPLITUDE    0     0:nominal, -:softer +:louder
             BEEP.COUNT        1     Number of beeps.  This will
                                     be overridden by the function
                                     argument if argument is > 0.
             BEEP.TONE_ON      10    Tone ON time in 1/100 seconds
             BEEP.TONE_OFF     10    Tone OFF time in 1/100 seconds

*/


int far pascal ME2BEEP(cnt)
    int cnt;
    {
    int i;
    int tonectrl ;

        /***
            Determine if the application currently owns a line.  If not
            we cannot beep, since the beep is designed to beep a phone
            line and actually runs as microcode on the TMS320.
        ***/

    if ( ME2GETLINE() < SUCCESS )  return (ME2_NO_LINE);

        /***
            Make sure nothing is using the line we own as a resource by
            registering this function as the currently active function.
            If some other function, such as play or record, is already
            running, this will let us know we cannot run.  Otherwise,
            we will become registered as the active function to lock
            out other attempts to run functions which require the boards
            resources.
        ***/

    if ((i=ME2REQUESTFUNCTION( FNCBEEP )) < SUCCESS )  return i ;

        /***
            Get the integer category number for the BEEP category
            of parameters.  This category contains the values to
            use for the 5 parameters expected by the beep microcode
            running on the TMS320.
        ***/

    if (BeepCat == -1)
        BeepCat = ME2PARMCATEGORYNUM( "BEEP" );

        /***
            Get the parameter that selects whether to abort
            the beep function when a Touch-Tone is received.
        ***/

    ME2PARMBYNUM( BeepCat, PRMTTBEEPSTOP, &TTAbort, GETPARM );


        /***
            Use the Overlay Manager to download and run the overlay (usually
            VOX.OVL) which contains the microcode necessary to perform the beep
            function.  Note that the term RUN here means to run the overlay
            initialization code which installs itself in the command slots
            which will be used to actually perform the desired functions
            later.
        ***/

    if ( (i = ME2OVLYRUN( ME2PARMMSG( BeepCat, PRMBEEPOVLY))) < SUCCESS )
        {
        ME2RELEASEFUNCTION();
        return( i );
        }

        /***
            Get the integer category number assigned to the VOX_MSG
            category of parameters.  This will be used for generating
            the appropriate multilanguage supported text in status
            monitor events.
        ***/

    if (VoxMsg == -1)                                        /*xx- 940304-xx*/
        VoxMsg = ME2PARMCATEGORYNUM ("VOX_MSG" );

        /***
            Set up the parameter data structure necessary to send
            the 5 beep parameters down to the beep overlay.  This is
            done in a global XFERPARM structure which will be used
            by the beep monitor thread when the beep command is executed.
        ***/

    ME2SETPARMHDR( &BeepPrm, 5, BUFFTOBRD, 0, 0, 0L );

        /***
            Using the category number and the parameter numbers 0 to 4,
            move the current values of the 5 BEEP parameters to the
            parameter structure expected by the overlay.
        ***/


    for ( i=0; i<5; i++ )
        ME2PARMBYNUM( BeepCat, i, &BeepPrm.dat[i], GETPARM );

        /***
            Since the overlay expects the tone on and off times to be
            in 20 ms time units, it must be converted from 1/100 sec (10ms)
            to 20 ms by dividing by 2.
        ***/

    BeepPrm.dat[3] /= 2;       /* divide 20ms value by 2 to get 1/100 sec */
    BeepPrm.dat[4] /= 2;

        /***
            If a non zero count was passed to this function, use it
            instead of the value provided by the BEEP.COUNT parameter.
        ***/

    if ( cnt != 0 )  BeepPrm.dat[2] = (DSPWORD)cnt;
                                    /* override parameter if specified */

        /***
            Call the ME2SHOWOVLYPARMS debug routine to show on the screen
            the 5 parameters which will be passed to the overlay upon
            invocation.  This function will only do something if the
            ME2CHANNEL.TST_LVL bit for this info has been enabled.
        ***/

    ME2SHOWOVLYPARMS( "BEEP", &BeepPrm, 5 );

        /***
            Select behavior of starting and running the function in the
            presence of Touch Tones, based on the TT_BEEP_STOP parameter:
            In all cases, delay starting the function until a Touch-tone in
            progress has ended.  If the parameter is "1", then abort the
            function if a touch tone is received, and also if one is
            present already.  If the parameter is "2", then allow the
            function to start even when there are digits already queued.
        ***/

    switch (TTAbort)
        {
        case 0:
        default:
            tonectrl = TTWAIT ;
            break ;
        case 1:
            tonectrl = TTWAIT | TTABORT | TTCANCEL;
            break ;
        case 2:                              /*xx- Option to not abort if TT-xx*/
            tonectrl = TTWAIT | TTABORT ;    /*xx- already there-xx*/
            break ;
        }


        /***
            Call the Utility Manager to start the beep monitor function
            as a separate handler thread.

            This process will suspend until the monitor thread is up
            and running and has called the ME2HDLRREADY() function.
            So here we go!  Once the monitor thread is happy we
            simply return to the caller to continue processing.  It
            should then wait for BEEPDONE, BEEPSTOP or BEEPFAIL to
            appear on the event queue.
        ***/

    return  ME2HDLRSTART( beepmon, tonectrl ) ;
    }



/****************************************************************
beepmon  -  this is the beep handler function.  This function
    demonstrates the typical structure of a handler thread minus
    the complexities of large data management algorithms such as
    those necessary for managing a voice message file structure.

    This function is not executed directly.  It should ONLY be
    initiated as a separate thread by the ME2HDLRSTART function.
    Note that because it is a separate thread, this routine should
    not use functions which are not re-entrant.
*/

static void far beepmon()
    {
    int qsel,i;
                                /* to allow a stop command to be deferred. */
                                /* during the startup process.             */

        /***
            Acknowledge that we are alive and well by communicating
            with the utility manager.  The utility manager will give
            us a link on the event queue to use when processing events.
            It will also send up a BEEPBEG event on the event queue and
            on the status monitor queue, using text from the specified
            category (VoxMsg) and parameter (MSGBEEPBEG).  This should
            ALWAYS be the very first instruction in the monitor thread.
        ***/

    qsel = ME2HDLRATTACH( BEEPBEG, VoxMsg, MSGBEEPBEG );


        /***
            If there was a touch tone on the queue or in progress, then
            -1 is returned, indicating that the function was aborted.
            Exit the thread now, with ME2HDLRDETACH.
            (Under DOS, fall to the end of this routine)
        ***/

    if ((int)qsel == -1)
        {
        ME2HDLRDETACH( qsel, BEEPSTOP, VoxMsg, MSGBEEPSTOP );
        }
    else
        {
            /***
                Make sure that no old buffers are hanging around in
                the driver.  This is just a safety measure to make
                sure that previous functions have no nasty side affects.
            ***/

        ME2DRVCMD( BFLUSH, 0L, 0L );

            /***
                Now that we are official, we will command to overlay to
                beep.  The overlay has installed itself above to respond
                to the BEEPBG command.  This command will expect a parameter
                packet of type XFERPARM containing the following 5 integer
                parameters:  frequency, amplitude, count, tone on, tone off.
                Since no additional data buffers are being passed, the driver
                will inform us that it has quit buffer transfers by sending
                a BUFDONQ event.
            ***/

        i = ME2OSCMDWAIT( qsel, BEEPBG, &BeepPrm, NULL, BUFDONQ, 0, 0 ) ;

            /***
                Tell the Utility Manager that we are on our way and
                that it can wake up the main thread again.
            ***/

        ME2HDLRREADY();

        if (i != SUCCESS)
            {
                /***
                    If there was a failure starting the function, then
                    release the handler resource, push a failure event
                    onto the queue and exit the thread.
                ***/

            ME2HDLRDETACH (0, BEEPFAIL, VoxMsg, MSGBEEPFAIL) ;
            }
        else
            {
            long hdlrtimelimit ;

            BeepStopped = 0;          /* Not stopped.     */
            BeepStarted = 0;          /* Not started yet. */

                /***
                   Set the maximum time to wait for an event from the board
                   before the handler sends a FAILFUNC event.
                   milliseconds = 5000 + (count+1) * (timeoff + timeon) * 20
                ***/


            hdlrtimelimit = 5000L +
                    (BeepPrm.dat[2]+1) * (BeepPrm.dat[3]+BeepPrm.dat[4]) * 20 ;
            Me2HdlrTimeLimit (hdlrtimelimit) ;                     /*xx- 940119-xx*/

            ME2HDLRLOOP( beeploop );  /* Have Handler mgr call event function */
            }
        }
    }

/****************************************************************/
static void far beeploop( qsel, e )
    int qsel;
    int e;
    {
    /***
        Process the event if we know how, otherwise pass it
        on to the next higher queue which is usually the
        user event queue which is emptied by the main process
        using the ME2GETEVENT() function.
    ***/

    /***
        If Touch Tone abort is enabled and event is a Touch Tone, then
        treat it as a STOP request.
    ***/

    if ( (TTAbort != 0)
      && ((TTSTAR <= e && e <= TT9) || (EXTTA <= e && e <= EXTTD)) )
        {
               /* Pass Touch Tone event on to event queue       */
        ME2QUEUEPUSH( qsel, (char far *)&e, NEXTQUEUE );
        e = STOPFUNC ;
        }

    switch( e )
        {
    case BEEPBG:        /* Beep overlay has started beeping okay */
        BeepStarted = 1;    /* We have now started successfully.     */
            /***
                If a STOPFUNC has come through we can stop now that
                it is safe to do a clean shutdown of the overlay
            ***/
        if ( BeepStopped )
            if (ME2DRVCMD (BEEPST, 0L, 0L) < SUCCESS)
                {
                ME2DRVCMD (BFLUSH, 0L, 0L) ;
                ME2HDLRDETACH (qsel, BEEPFAIL, VoxMsg, MSGBEEPFAIL) ;
                }

        break;

    case BEEPST:
            /***
                The overlay has acknowledged a BEEPST command and
                is now shutdown.
            ***/
        ME2HDLRDETACH( qsel, BEEPSTOP, VoxMsg, MSGBEEPSTOP );
        break;

    case VCMDERR:                  /* dialing already in progress */
            /***
                The overlay received an unexpected command.  This
                is actually possible under normal operation if
                a stop command is sent to the overlay immediately
                after the beeping ended naturally.
            ***/
        if ( BeepStopped )      /* if it was stopped, ignore it      */
            ME2HDLRDETACH( qsel, BEEPSTOP, VoxMsg, MSGBEEPSTOP );
        else                /* if not stopped, it's a real error */
            ME2HDLRDETACH( qsel, BEEPFAIL, VoxMsg, MSGBEEPFAIL );
        break;

    case VLVLERR:           /* wrong speech environment failure  */
        ME2HDLRDETACH( qsel, BEEPFAIL, VoxMsg, MSGBEEPFAIL );
        break;

    case BEEPDN:
            /***
                If no BEEPST command was issued prior to now, this
                is a normal and natural ending of the beep process
                and we can use the Utility Manager to shutdown
                this thread, unlink us from the event queue and
                send both a BEEPDONE event and status monitor event.

                If a BEEPST has just been issued before realizing
                that beeping completed, then we can expect a
                VCMDERR to come up very shortly to tell us that
                the BEEPST command was ignored since beeping ended
                normally just before the stop command was received.
            ***/

        if ( !BeepStopped )
            ME2HDLRDETACH( qsel, BEEPDONE, VoxMsg, MSGBEEPDONE );
        break;

    case STOPFUNC:                  /* From ME2STOP or touch tone abort */
            /***
                The ME2STOP() function was called by the main
                process!  This is our cue to shut down quick!
                But, we have to be responsible and only do it
                when it is safe.  If the overlay has not started
                up, we have to defer the stop process until later.
                Also, if we have already received a STOPFUNC, we
                should ignore another one.
            ***/

        if ( BeepStarted && !BeepStopped )
            {
               /* Stop the overlay, dying with failure if that fails */
            if (ME2DRVCMD (BEEPST, 0L, 0L) < SUCCESS)
                {
                ME2DRVCMD (BFLUSH, 0L, 0L) ;
                ME2HDLRDETACH (qsel, BEEPFAIL, VoxMsg, MSGBEEPFAIL) ;
                }
            }
        BeepStopped = 1;    /* Tell the rest of the loop we are stopping */
        break;

    case FAILFUNC:
        /* A timeout occurred or an error was returned by the driver */
        /* Blindly stop function, clean up and exit with a failure event. */
        ME2DRVCMD (BEEPST, NULL, NULL) ;
        ME2DRVCMD (BFLUSH, NULL, NULL) ;
        ME2HDLRDETACH( qsel, BEEPFAIL, VoxMsg, MSGBEEPFAIL);
        break;

    default:
               /* If we don't understand the event, pass it on       */
        ME2QUEUEPUSH( qsel, (char far *)&e, NEXTQUEUE );
        break;
        }
    return;
    }
