/******************************************************************************
FILENAME:        vbxdrv.h

DESCRIPTION:   Header file for driver level programming; also used by 'C'
               portions of the driver.

               Defines driver and DSP commands and events and driver data
               structures.

               Note: Commands and events for overlays, such as DIAL or VOX,
               which are downloaded, are in a separate include file for the
               respective overlay.

   $$History
*   1 VBXDRV.H 28-Feb-91,16:17:08,`GS' Placed under revision control
*   2 VBXDRV.H 9-Apr-91,12:35:40,`GS' Renumbered UNIX driver events
*        CFG_UNIT was CFG_INIT; added CFG_BLKSIZ
*        Added fields to LINEDT for dynamic buffer allocation; added parmbuf
*   3 VBXDRV.H 10-Apr-91,11:03:14,`GS' Changed LINEDT to LINEDATA
*   4 VBXDRV.H 6-Jun-91,15:54:28,`GS' Added Bank codes for ISDN VMOS
*   5 VBXDRV.H 23-Sep-91,12:05:04,`GS' Increase event Q to 118
*   6 VBXDRV.H 28-Jan-92,10:39:18,`GS' Now used for/with OS2 driver.
*   7 VBXDRV.H 30-Jan-92,13:40:08,`GS' Added CFG error code
*   8 VBXDRV.H 28-Jun-92,14:46:34,`RP'  comments changed to ANSI C
*        comments.
*   9 VBXDRV.H 29-Jun-92,13:38:14,`RP' Fixed comment.
*   10 VBXDRV.H 14-Jul-92,11:46:40,`GS' Fixed missing close-comments
*   11 VBXDRV.H 11-Jan-93,13:28:10,`GS' (OS2) renamed some fields in LINEDATA
*   12 VBXDRV.H 18-Jan-93,17:30:56,`GS' changed struct fields to unsigned short
*   13 VBXDRV.H 25-Jan-93,17:49:46,`GS' Added lockinfo fields for 2.0 locking.
*   14 VBXDRV.H 17-Jun-93,16:02:28,`GS' Added VBXS_ioctl_t struct for STREAMS
*   15 VBXDRV.H 25-Jul-94,17:39:00,`MEL' added support for Interactive Unix
   $$History

Copyright 1991-93 by Natural Microsystems Corp.  All rights reserved.
******************************************************************************/

#ifdef unix
#define __32BIT__ 1
#endif

#ifndef FAR

#ifdef __32BIT__
#define FAR
#define NEAR
#else          /* 16-bit OS/2 */
#define FAR  _far
#define NEAR  _near
#endif

#endif

/*     **********    VMOS OPERATING SYSTEM COMMANDS    **********     */

#define HARDRST   0x01
#define SOFTRST   0x02
#define LOADMEM   0x03    /* Load to/from Program RAM                        */
#define INSTALL   0x04    /* Start overlay                                   */
#define DEINSTL   0x05    /* Stop overlay                                    */
#define PEEKMEM   0x06    /* Read data RAM                                   */
#define POKEMEM   0x07    /* Write data RAM                                  */
#define C2BANK    0x08    /* Select bank (except VBX 100)                    */
#define C2BANK0   0x08    /* Select bank 0 (VBX 100)                         */
#define C2BANK1   0x09    /* Select bank 1 (VBX 100)                         */
#define C2BANK2   0x0A    /* Select bank 2 (VBX 100)                         */
#define C2BANK3   0x0B    /* Select bank 3 (VBX 100)                         */
#define PEEKPRT   0x0E    /* Read TMS320 port registers                      */
#define POKEPRT   0x0F    /* Modify bit in port 2 or 3                       */

#define LINEONH   0x10    /* Place outside line on hook                      */
#define LINEOFH   0x11    /* Take outside line off hook                      */

#define MUTEOFF   0x0C    /* VBX 100: Turn MUTE off (local to outside line)  */
#define MUTEON    0x0D    /* VBX 100: Turn MUTE on (local phone to internal) */
#define ATTNOFF   0x12    /* VBX 100: Do not attenuate to muted local phone  */
#define ATTNON    0x13    /* VBX 100: Attenuate to muted local phone         */

#define AUDOFF    0x14    /* VBX 100-PS/2: Turn off the audio monitor        */
#define AUDON     0x15    /* VBX 100-PS/2: Turn on the audio monitor         */


/*     **********     DRIVER COMMANDS     **********     */
/* Commands that are interpreted by the driver */

#define NEXTBUF   0x18    /* Submit a follow on buffer                 */
#define DRVSTAT   0x19    /* Retrieve driver static data for port      */
#define QREAD     0x1a    /* Read an event from the event queue        */
#define SNDMSG    0x1b    /* Pass an event to another port             */
#ifndef unix
  #define QWAKE   0x1b    /* Queue a RDWAKE event (wake a read)        */
#endif
#define QFLUSH    0x1c    /* Flush the event queue                     */
#define BFLUSH    0x1d    /* Flush the buffer queue                    */
#define SOFTCLS   0x1e    /* Flag subsequent close to leave port alive */

#ifdef unix

#define GETBUF  0x1f    /* Get a buffer                              */
#define INTCMDB 0x18    /* Start of internal command range           */
#define INTCMDE 0x1f    /* End of internal command range (incl)      */

#endif


/*     **********    VMOS SPEECH ENVIRONMENT COMMANDS     **********     */

#define SEV1UP    0x20    /*  Move from level 0 to level 1       */
#define SEV1DWN   0x21    /*  Move from level 1 to level 0       */
#define SEV2UP    0x22    /*  Move from level 1 to level 2       */
#define SEV2DWN   0x23    /*  Move from level 2 to level 1       */
#define SEV3UP    0x24    /*  Move from level 2 to level 3       */
#define SEV3DWN   0x25    /*  Move from level 3 to level 2       */
#define STRTTIM   0x26    /*  Start the interval timer           */
#define KILLTIM   0x27    /*  Kill interval timer                */
#define SLPDET    0x28    /*  Start ring or supervision detector */


/**************************** EVENTS ********************************/

/* Vmos operating system events */

#define RSTDN     0x01    /* board just finished reset (command 1 or 2)    */
#define FRMREQ    0x03    /* Frame transfer request (absorbed by driver)   */
#define BLKTAK    0x04    /* Ready for block transfer (absorbed by driver) */
#define CMNDDN    0x06    /* command 3-7 or 0xE,0xF done                   */
#define TELCDN    0x07    /* command 0xC,0xD,0x10-0x13 done                */
#define NOSBK     0x08    /* (Except VBX 100) Just went to new OS bank     */
#define NOSBK0    0x08    /* (VBX 100) just went to new OS bank 0          */
#define NOSBK1    0x09    /* (VBX 100) just went to new OS bank 1          */
#define NOSBK2    0x0A    /* (VBX 100) just went to new OS bank 2          */
#define NOSBK3    0x0B    /* (VBX 100) just went to new OS bank 3          */

/* The following are VMOS error events */

#define CBKERR    0x0E    /* change OS bank error (overlay or bkg active) */
#define INSERR    0x0F    /* install or deinstall error (when already)    */
#define PRMERR    0x10    /* parameter get error (no frame ready)         */
#define BKOERR    0x11    /* too many bkgnd tasks assigned (3rd waiting)  */
#define BSWERR    0x12    /* bank error in polling lev bank switch        */
#define FOVERR    0x13    /* frame overrun error (last not yet xferred)   */
#define BRQERR    0x14    /* block request error (no frame ready)         */
#define POLOER    0x15    /* polling level overflow error                 */
#define EQOERR    0x16    /* event queue overflow error                   */
#define CQOERR    0x17    /* command queue overflow error                 */

/* Events generated in the driver */

#define EMPTY     0       /* No events are waiting in the queue     */
#define BUFDONC   0x18    /* Buffer done; next started              */
#define BUFDONQ   0x19    /* Last buffer done                       */
#define READTO    0x1A    /* Event read sleep timeout occurred      */
#define READINT   0x1B    /* (OS/2) interrupt woke a sleeping read  */
#define READWK    0x1C    /* (OS/2) QWAKE call woke a sleeping read */
#define CLOSED    0x1D    /* (obsolete)                             */
#define QFULL     0x1E    /* The event queue overflowed             */
#define BXERR     0x1F    /* Block transfer failure in driver.      */


/*  VMOS speech environment events */

#define SELEV0    0x20    /* at new spc env level 0                           */
#define SELEV1    0x21    /* at new spc env level 1                           */
#define SELEV2    0x22    /* at new spc env level 2                           */
#define SELEV3    0x23    /* at new spc env level 3                           */

#define LPCSRT    0x24    /* Detector started                                 */

#define TIMEDN    0x25    /* timer timeout                                    */
#define TIMEST    0x26    /* timer succesfully started                        */
#define TIMEKL    0x27    /* timer succesfully killed                         */

#define LPCRON    0x28    /* loop current detected  / E-lead dropped          */
#define LPCROF    0x29    /* loop current lost   / E-lead seized              */
#define RNGBEG    0x2A    /* beginning of ring                                */
#define RNGEND    0x2B    /* end of ring                                      */
#define LOCONH    0x2C    /* (VBX 100) local phone went on hook               */
#define LOCOFH    0x2D    /* (VBX 100) local phone went off hook              */

#define ENDTON    0x3A    /* End of touch tone                                */

#define TIMERR    0x3D    /* time start or kill error (already going or dead) */
#define TIMEIP    0x3E    /* attempt to bring down level 1 with timer going   */
#define LEVERR    0x3F    /* attempt to bring up or down from wrong level     */

/* The following digit codes are produced all boards except the VBX-100 */

#define EXTT1     0xf0    /* '1' (except VBX 100) */
#define EXTT2     0xf1    /* '2' (except VBX 100) */
#define EXTT3     0xf2    /* '3' (except VBX 100) */
#define EXTTA     0xf3    /* 'A' (except VBX 100) */
#define EXTT4     0xf4    /* '4' (except VBX 100) */
#define EXTT5     0xf5    /* '5' (except VBX 100) */
#define EXTT6     0xf6    /* '6' (except VBX 100) */
#define EXTTB     0xf7    /* 'B' (except VBX 100) */
#define EXTT7     0xf8    /* '7' (except VBX 100) */
#define EXTT8     0xf9    /* '8' (except VBX 100) */
#define EXTT9     0xfa    /* '9' (except VBX 100) */
#define EXTTC     0xfb    /* 'C' (except VBX 100) */
#define EXTTS     0xfc    /* '*' (except VBX 100) */
#define EXTT0     0xfd    /* '0' (except VBX 100) */
#define EXTTP     0xfe    /* '#' (except VBX 100) */
#define EXTTD     0xff    /* 'D' (except VBX 100) */

/* The following codes are produced by VBX-100 boards (but not VBX 100 MC) */

#define ASCIIS    0x2E    /* '*' (VBX 100) */
#define ASCIIP    0x2F    /* '#' (VBX 100) */
#define ASCII0    0x30    /* '0' (VBX 100) */
#define ASCII1    0x31    /* '1' (VBX 100) */
#define ASCII2    0x32    /* '2' (VBX 100) */
#define ASCII3    0x33    /* '3' (VBX 100) */
#define ASCII4    0x34    /* '4' (VBX 100) */
#define ASCII5    0x35    /* '5' (VBX 100) */
#define ASCII6    0x36    /* '6' (VBX 100) */
#define ASCII7    0x37    /* '7' (VBX 100) */
#define ASCII8    0x38    /* '8' (VBX 100) */
#define ASCII9    0x39    /* '9' (VBX 100) */



/* MISC DEFINITIONS FOR VMOS COMMANDS */

/* Bank codes for LOADMEM (1st parameter) and C2BANK */
#define BANKC     0x0000  /* Current Bank                 */
#define YBANK0    0x02CE  /* Bank 0                       */
#define YBANK1    0x02DE  /* Bank 1                       */
#define YBANK2    0x12de  /* Bank 2                       */
#define YBANK3    0x03ce  /* Bank 3                       */
#define YBANK4    0x03de  /* Bank 4                       */
#define YBANK5    0x13de  /* Bank 5                       */
#define YBANK6    0x02cf  /* Bank 6                       */
#define YBANK7    0x02df  /* Bank 7                       */
#define YBANK8    0x03cf  /* Bank 8                       */
#define YBANK9    0x03df  /* Bank 9                       */
#define YBANK10   0x12cf  /* Bank 10                      */
#define YBANK11   0x12df  /* Bank 11                      */
#define YBANK12   0x13cf  /* Bank 12                      */
#define YBANK13   0x13df  /* Bank 13                      */
#define YBANK14   0x12ce  /* Bank 14                      */
#define YBANKVMOS 0x13ce  /* VBX RAM Bank containing VMOS */

/* Banks for ROM-based VMOS (1-port) */
#define BANK0     0xCE    /* Bank 0 (VBX 100) */
#define BANK1     0xDE    /* Bank 1 (VBX 100) */
#define BANK2     0xCF    /* Bank 2 (VBX 100) */
#define BANK3     0xDF    /* Bank 3 (VBX 100) */

/* Banks for RAM-VMOS, ISDN version */
#define     V4BANK0   0xC0
#define     V4BANK1   0xD0
#define     V4BANK2   0xD1

/************************* DRIVER INTERFACE ******************************/
#ifdef unix
  /* SPECIAL UNIT NUMBERS */

#define MAXVBX_UNIT 253
#define SNDMSG_UNIT 254           /* special device for message sending */
#define CONFIG_UNIT 255           /* special device for configuration   */


  /* Commands to the "config" device (unit 255) */
#define CFG_UNIT    1
#define CFG_GETSTAT 2
#define CFG_BLKSIZ  3

  /* A data structure for returning device initialization results */
  struct vbx_init_status {
      short baseio ;
      short boardid ;
      short status ;
      short spare ;
      } ;
#endif

/* Configuration result codes */

#define VBX_CFG_NO_INIT            0  /* No initialization attempted           */
#define VBX_CFG_SUCCESS            1  /* Successfully initialized VBX port     */
#define VBX_CFG_NO_RESPONSE        2  /* Not found (no response)               */
#define VBX_CFG_BAD_RESPONSE       3  /* Not found (invalid response to reset) */
#define VBX_CFG_PROBE_DWNLD_FAILED 4  /* Failure to download probe routine     */
#define VBX_CFG_PROBE_RSP_FAILED   5  /* Failure to probe VBX port config      */
#define VBX_CFG_REBOOT_FAILED      6  /* Failure to reboot VBX port            */
#define VBX_CFG_VMOS_DWNLD_FAILED  7  /* Failure to download probe routine     */
#define VBX_CFG_BAD_VMOS_RESP      8  /* Wrong response to VBX OS reset        */
#define VBX_CFG_TMO_RESET          9  /* Timeout waiting for interrupt         */
#define VBX_CFG_TMO_IDXFER        10  /* Timeout on VBX/100 ID transfer        */
#define VBX_CFG_NO_VMOS_RESP      11  /* No Response to VBX OS reset    920130 */
#ifndef unix
  #define VBX_CFG_NO_GDT          12  /* Unable to allocate GDT selectors      */
#endif

#ifdef unix
  /* Error codes of our own definition returned by the driver. */
  /* These will be set in errno.  */

#define VBXERRBASE 200
#define STNYOP     201   /* Device not yet open; ioctl.             */
#define STCMTO     202   /* Timeout sending command to board.       */
#define STBSYV     203   /* Buffer channel busy on cmd with bufs.   */
#define STBOVR     204   /* Buffer channel full on NEXTBUF.         */
#define STRDLK     205   /* Read attempt while previous in progress.*/
#define STGIOP     206   /* IOCTL parameter error.                  */
#define STGBMT     207   /* No buffers allocated.                   */
#define STKBSZ     208   /* Kernel buffer size exceeded.            */
#define STINIT     209   /* Device has not been initialized.        */

#else /* OS2 */

   /* The following error codes are used internally in the OS2 driver.
      If the error is retuned on a DosDevIOCTL call, OS/2 will convert the
      upper byte to 0xFF.
   */
  #define STGENF    0x800C    /* General command failure.             */
  #define STCMTO    0xC084    /* Timeout issuing command to hardware. */
  #define STBSYV    0xC085    /* Busy violation - buffers queued.     */
  #define STBOVR    0xC086    /* Buffer queue overflow (NEXTBUF).     */
  #define STRDLK    0xC087    /* Read lock error - already reading.   */
  #define STGIOP    0xC088    /* General IOCTL parameter error.       */
#endif

/* Data structure definitions */

/* The following is the parameter block which is pointed to by the
"data" argument of an ioctl() call.  The first 5 elements are the same under
OS/2, DOS and UNIX, so are put in their own structure.

  Following the common header is a pointer to the data; if nfrms is nonzero,
pbdbuf must point to a user data buffer.  Note that under DOS-OS/2 the data
pointer is a separate argument to the ioctl call.

  Following the pointer are zero or more 16-bit parameters; the first member
of the header, nparm, indicates the number.
*/

typedef struct
    {
    unsigned short nparm;        /* # of word sized parameters after hdr */
    unsigned short dir;          /* Direction bits - both if transcode */
    unsigned short frmsz;        /* Size of each frame in words */
    unsigned short nfrms;        /* Number of frames left in buffer */
    unsigned long  frmct;        /* Current frame count */
    }
PARMHDR ;

#define MAXNPRMS    40

typedef struct
    {
    PARMHDR hdr ;
#ifdef unix
    void *pbdbuf;                 /* Address of data buffer in user space */
#endif
    unsigned short parm[MAXNPRMS];
    }
PARMBLOCK ;

#ifdef unix
  /* Use the following IOCtl parms and data descriptor for the Unix STREAMS
    driver ioctl() interface.  The data, if any must follow contiguously. */

#define VBXS_IOCTL_MAXPARM 16
  typedef struct
      {
      PARMHDR hdr ;
      unsigned short parm[VBXS_IOCTL_MAXPARM];
      }
  VBXS_ioctl_t ;
#endif


#define DOWNMSK     1               /* Buffer direction bit for down (play) */
#define UPMSK       2               /* Buffer direction bit for up (record) */

/* The buffer control structure below contains info for one of the two queued
buffers on its way to the board.  This structure only exists within the overall
static data for each device, which follows.  The .kvad member holds the pointer to
one of the two kernel buffers from open of a port until close.  A completely
clear buffer is defined as one with both .bdir and .bnfrms =0.  A buffer need
only have .bnfrms =0 to be considered clear for submitting a command or a follow
on buffer with NEXTBUF.  A buffer to be retrieved with GETBUF must have the
UPMSK bit set in .bdir and if it is in the "next" buffer position, must also
have .bnfrms = 0.  The .bfrmct member is bumped but not used in the driver -
it is provided as a convenience for upper level code and must be interrogated
using DRVSTAT.  */


  struct bctl
    {
#ifdef unix
    void *kvad;                 /* Kernel buffer address */
    void *ckvad;                /* Address of current frame in buffer */
#else /* OS/2 */
    long b_lockh;               /* Buffer lock handle (2.0: VA of lockinfo) */
    void FAR *kvad;             /* Kernel buffer address */
#endif
    unsigned short bdir;        /* Direction bits - both if transcode */
    unsigned short bfrmsz;      /* Size of each frame in words */
    unsigned short bnfrms;      /* Number of frames left in buffer */
#ifdef unix
    short allocsize;            /* Size in pages of buffer if allocated */
#endif
    unsigned long bfrmct;       /* Current frame count */
    };



/* The following structure is static data for each port.  Contains all useful
info except for interrupt search tables which must survive from one driver call
or one interrupt to the next.  The event queue is a ring with pointers equal
when empty.  Attempt to stuff with .lnquwp pointing just before .lnqurp
results in previous two entries being clobbered with QFULL followed by the
event which caused the overflow. */

#ifdef unix

#define OQSIZ      118              /* Size of driver event queue */
  /* note: this size results in a LINEDATA size of 384 bytes */

#else
  #define OQSIZ       50             /* Size of driver event queue */
#endif

typedef struct
    {
    unsigned short physln;          /* Physical port # = unit # */
    unsigned short lnioad;          /* Base I/O address of this port */
    unsigned short brdid;           /* Firmware version # */
    unsigned char opnflg;           /* Nonzero when board properly opened */
    unsigned char softcls;          /* Set by SOFTCLS; prevents reset on close */
    unsigned short lnpid ;          /* PID of owner process when open (OS2) */
    unsigned short bdstat;          /* Status read from board at BDSTAT */
    struct bctl cb;                 /* Current buffer control structure */
    struct bctl nb;                 /* Next buffer control structure */
#ifdef unix
    unsigned short lnqueue[OQSIZ];  /* Event queue */
    unsigned short *lnqurp;         /* Event queue read ptr */
    unsigned short *lnquwp;         /* Event queue write ptr */
    short rdlock;                   /* =1 while reading to prevent another */
    short rdtoflg;                  /* =1 when read wakes if timeout; else =0*/
    short init_stat ;               /* Initialization result */
    short need2free;                /* Set by int serv if buffers done */
    unsigned short parmbuf[MAXNPRMS] ; /* static buffer to hold parms */
#else /* OS2 */
    unsigned char  lnqueue[OQSIZ];  /* Event queue */
    unsigned char NEAR *lnqurp;     /* Event queue read ptr */
    unsigned char NEAR *lnquwp;     /* Event queue write ptr */
    long  lnqpad;                   /* Physical address of queue for BLOCK-RUN*/
    unsigned short rdlock;          /* =1 while reading to prevent another */
    unsigned short gtwait;          /* =1 when sleeping; saves unneed wakeup*/
    unsigned short tfoff ;          /* Temporary transfer offset address */
    unsigned short tfcnt ;          /* Temporary transfer word count */
    long ulhand1 ;                  /* Handle ready to be unlocked */
    long ulhand2 ;                  /* 2nd Handle ready to be unlocked */
  #ifdef TS
    long lntsq [OQSIZ] ;            /* Time stamps */
  #endif
    unsigned long lockinfo[2][3] ;  /* (OS2 2.0) 2 12-byte lock handles */
#endif
    }
LINEDATA;


/* (OS/2) NOTE: the above structure is congruent to STRUCT LINEDT
  in VBXOS2.INC */



