/*************************************************************************
FILE:  DSP.c

DESCRIPTION: DSP function for telephony services.

This demonstration program illustrates various ME/2 functions in action.
It is designed to be used for demonstrating the capabilities of the ME/2
system, as well as being an example of how to use the ME/2 functions to
create other applications.

Using Compiler : K & R compiler.

UPDATES: Bo-kyun Na bkna@cat.syr.edu

****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#if (defined( unix ) || defined( DOSMODE ))
#define DOSUNIXMODE 1
#endif

#ifndef DOSUNIXMODE /* OS/2 version, need prototype for DosSetMaxFH */
 #define INCL_BASE
 #define INCL_NOPM
 #include <os2.h>
#endif

#include "me2def.h"        /* Standard ME/2 header file.                */
#include "me2err.h"        /* Standard ME/2 error header file.          */

                           /* Function Prototypes                       */

void flushttqueue();
void doexit();
int  waitdone();
int  recplay();
int  play();
void docall();
void main();

ME2INT Vmsg = -1;       /* Voice file handles.*/
ME2INT Vtst = -1;

    /* Note - 'short' type used to hold the values of 16 bit ME2 parameters
     * passed by reference to Me2ParmByName and Me2ParmByNum */
short Maxrec = 10 ;    /* Max recording seconds */

char Vmsgfil[20];    /* Name of voice file containing prompts used by demo.*/
char Vtstfil[80];    /* Name of voice file to record test message to. */

#define MAXWAIT   600000L

#define FOREVER   (-1L)

/* Message Numbers */

#define MWELCOME        1
#define MGOODBYE        2
#define MMENU           3
#define MSEL1           4
#define MSEL2           5
#define MSEL3           6
#define MINVALID        7
#define MPMPT1          8
#define MPMPT2          9
#define MRECORD         1


void main( void )
    {
    ME2INT  i;
    char  buff[20];           /* Array used to make ascii channel number. */
    ME2INT ev ;

    /* Note - 'short' type used to hold the values of 16 bit ME2 parameters
     * passed by reference to Me2ParmByName and Me2ParmByNum */
    short id;
#ifdef unix
    short linenum;
#else
    short line_group;
#endif

    puts( "VBX ME/2 by Natural MicroSystems Corporation" );
    puts( "Demonstration.  Source is 'demo.c'.\n" );

   /*
     When you are running the demo on a system with many lines, you are
     required to increase the number of file handles available to this
     process.
   */
#ifndef DOSUNIXMODE
     DosSetMaxFH( 50 );
#endif

    /*
      First the "DEMO" profile is loaded to process any changes made to
      the default parameter values, which were found in demo.prm.
    */
    i = Me2ParmLoad( "DEMO" );

    if( i != SUCCESS )     /* See if call to ME/2 function is successful */
        {
        puts( "\tError processing .PRO file information." );
        doexit( i );       /* If not, process routine to provide error msg. */
        }

    /*
      Next we will get the application ID assigned to the applicaton to use
      when requesting a channel, and any other values which will be used
      throughout the demo.  It is recommended practice to put the
      application ID in a parameter so that if an end user happens to
      purchase two applications with the same ID, they will need only to
      change one of the ID's in the appropriate PROFILE.  Then we will get
      any other values which will be used throughout the demo.
    */

    i = Me2ParmByName( "DEMO.APPL_ID", &id, GETPARM );
    if( i != SUCCESS )     /* See if call to ME/2 function is successful */
        {
        puts( "\tError reading DEMO parameter." );
        doexit( i );       /* If not, process routine to provide error msg. */
        }

    i = Me2ParmByName( "DEMO.MAX_RECORD", &Maxrec, GETPARM);
    if( i != SUCCESS )     /* See if call to ME/2 function is successful */
        {
        puts( "\tError reading DEMO parameter." );
        doexit( i );       /* If not, process routine to provide error msg. */
        }
#ifdef unix
    i = Me2ParmByName( "DEMO.LINENUM", &linenum, GETPARM);
    if( i != SUCCESS )     /* See if call to ME/2 function is successful */
        {
        puts( "\tError reading DEMO parameter." );
        doexit( i );       /* If not, process routine to provide error msg. */
        }
#else
    i = Me2ParmByName( "DEMO.GROUPNUM", &line_group, GETPARM);
    if( i != SUCCESS )     /* See if call to ME/2 function is successful */
        {
        puts( "\tError reading DEMO parameter." );
        doexit( i );       /* If not, process routine to provide error msg. */
        }
#endif
    i = Me2ParmByName( "DEMO.VOX_MSG_FILE", Vmsgfil, GETPARM);
    if( i != SUCCESS )     /* See if call to ME/2 function is successful */
        {
        puts( "\tError reading DEMO parameter." );
        doexit( i );       /* If not, process routine to provide error msg. */
        }


    /*
      Because a channel is needed for most all ME2 functions, we will
      request one now, before we forget.
    */
    i = Me2RequestChannel( id, "DEMO" );
    if( i != SUCCESS )     /* Abort the startup of the demo if no */
        {                  /*  free channels are available. */
        puts( "\tError requesting a channel." );
        doexit( i );
        }


    /*
      Here we are appending our channel number to the end of the test
      voice file name in order to make sure it is unique from the test voice
      file used by any other channels.  It is because of this appending
      process that we only allow six characters in in the test voice file
      name, instead of eight (see parameter file).
    */
    i = Me2ParmByName( "DEMO.VOX_TST_FILE", Vtstfil, GETPARM);

    if( i != SUCCESS )     /* See if call to ME/2 function is successful */
        {
        puts( "\tError reading DEMO parameter." );
        doexit( i );       /* If not, process routine to provide error msg. */
        }

#ifdef unix
    sprintf (buff, "%d", linenum );          /* Turn line # into ascii. */
#else
    sprintf (buff, "%d", Me2GetChannel() );  /* Turn channel # into ascii. */
#endif
    strcat( Vtstfil, buff );                 /* Append to end of name.       */

    /*
      Open the voice file to use for all messages played during the
      demo.  Then open the file that will be used for voice recordings
      during the demonstration.  The function Me2VoxOpen searches the path
      specified by the ME2LOCAL variable for a voice file that is opened for
      PLAYONLY.  (If a voice file is opened for PLAYREC and cannot be found
      through this path, it is created in the current, local subdirectory.)

      It is also recommended practice not to specify extensions on voice
      file names when creating the source code.  This allows the flexibility
      of using other voice modules to support another file format or
      compression algorithim.  The replacement module will append the
      appropriate extension for it, as ME/2 does with the '.VOX' extension.
      NOTE:  This also applies to .PRO files and DATABASES.
    */
    i = Me2VoxOpen(Vmsgfil, &Vmsg, PLAYONLY);
    if( i != SUCCESS )     /* See if call to ME/2 function is successful */
        {
        puts( "\tDEMO.vox file could not be found in the correct subdirectory!" );
        doexit( i );       /* If not, process routine to provide error msg. */
        }

    i = Me2VoxOpen( Vtstfil, &Vtst, PLAYREC);
    if( i != SUCCESS )     /* See if call to ME/2 function is successful */
        {
        puts( "\tError creating/opening the test file." );
        doexit( i );       /* If not, process routine to provide error msg. */
        }
#ifdef unix
    /* Next, grab a line on which we will wait for an incoming call */
    i = Me2GrabLine( linenum );
#else
    /*
      Next, a ringing line is requested from group 0.  Group 0
      means wildcard and will match a line of any group.
    */
    i = Me2RequestLine( line_group, LINERING, -1L );
#endif
    if( i != SUCCESS )     /* Abort if there are no lines */
        {
        puts( "\tError requesting a line." );
        doexit( i );
        }

    puts("\tPLEASE DIAL THE DEMONSTRATION" );
    puts("\t (or press CTL-C to abort)\n" );
#ifdef unix
    while (1)    /* Wait for a call to come into the system. */
        {
        i = Me2WaitEvent( -1 );  /* Wait for an event to come through */
                                 /* the event queue.                  */

  
        Me2GetEvent(&ev, QREMOVE );  /* Get the event off the queue */

        if (ev == RINGBEG)           /* Wait for an incoming call */
          {
          break ;
          }
        }
#else
    waitdone( FOREVER );    /* Wait for a call to come into the system. */

    if( i != SUCCESS )      /* Check to see if call to ME/2 function returns */
        {                   /* successfully.                                 */
        puts( "\tCall not received." );
        doexit( i );        /* If not, process routine to provide error msg. */
        }
#endif

    /* Answer the call */
    i = Me2SetLineMode( PICKUP ) ;
    if( i != SUCCESS )      /* Check to see if call to ME/2 function returns */
        {                   /* successfully.                                 */
        doexit( i );        /* If not, process routine to provide error msg. */
        }


    /*
      Remove any existing events from the event queue, in order
      to start with an empty queue.
    */
    while( ( Me2GetEvent( &ev, QREMOVE ) ) != ME2_QUEUE_EMPTY )
        ;

    docall();               /* Function that actually starts demo. */

    puts( "\tDEMO COMPLETED.\n" );

    /*
      The exit routine will cleans up the system and closes all the files
      before exiting back to the operating system, once the demo is
      concluded.
    */
    doexit( 0 );                /* Exit back to the operating system    */
}


/*******************************************************************************
 DOCALL:
      This routine starts the interactive segments of the demonstration.

       Return Value:    None
******************************************************************************/
void docall( void )
    {
    char sel[5];     /* Character array for storing touchtones. */
    ME2INT result;    /* Return code from Me2GetTTS. */

    puts("\tSTART OF DEMO");
    Me2Delay( 40L );
    waitdone( MAXWAIT );
    flushttqueue();                            /* Empty the touch-tone queue.*/

    if( play( Vmsg, MWELCOME ) ) return;     /* Play the WELCOME message   */

    for( ;; )
        {
        if( play(Vmsg, MMENU ) ) return;     /* Play the main menu message.*/

        /* This function collects one touchtone digit */
        Me2GetTTS( "X", "", sel, &result, 1 );

        if( waitdone( MAXWAIT ) ) return;    /* Abort if no touch-tones */
                                             /* entered.                */

        /* See what touchtone was entered */

        if ( result == 'T' || sel[0] == '0' )   /* If timeout occurred or */
            {                                   /* '0' entered, play      */
            play( Vmsg, MGOODBYE );             /* GOODBYE message        */
            return;
            }
        else if( sel[0]>='1' && sel[0] <= '3' ) /* If touchtones '1'-'3'  */
            {                                   /* entered, play MENU     */
            if( play( Vmsg, (sel[0]-'0')+3) )   /* selection message.     */
                return;
            }
        else if( sel[0] == '4' )                /* If touchtone '4' play */
            {                                   /* prompt for recording  */
            if( recplay() )                     /* and playing voice.    */
                return;
            }
        else if( play( Vmsg, MINVALID ) )       /* If anything other than */
            return;
                                                /* '1-4' or '0' play      */
                                                /* invalid selection msg. */

        flushttqueue();                        /* Empty the touch-tone queue.*/
        }
    }


/*****************************************************************************
PLAY:
   This routine plays the specified message from a particular voice file.
   It waits for the completion of play before returning.  While playing special
   event processing occurs to detect local phone hangup, <ESC> key press, etc.

   Argument List:    ME2INT vsel    => Voice file handle.

                     ME2INT msg     => Message number to be played.

   Return Value:          value from waitdone()
*****************************************************************************/
ME2INT play( ME2INT vsel, ME2INT msg )
    {
    ME2INT i;

    i = Me2VoxPlayMsg( vsel, msg );
    if( i != SUCCESS )                 /* Display error msg if play fails.   */
        {
        puts( "\tProblem with demo voice file.  Reinstall demo to correct." );
        doexit( i );
        }
    return( waitdone( MAXWAIT ) );
    }


/****************************************************************************
RECPLAY:
   This function records a test message and plays it
   back when the record is done executing.


   Return Value:        value from waitdone()
*****************************************************************************/
ME2INT recplay( void )
    {
    ME2INT i ;

    if ((i=play( Vmsg, MPMPT1 )) != 0)  /* Play prompt for recording test msg */
        return i;

    Me2Beep( 1 );                          /* Beep tone for start of record. */
    if ((i=waitdone( MAXWAIT )) != 0)
        return i;

    i = Me2VoxRecordMsg( Vtst, MRECORD, Maxrec ); /* Record up to Maxrec secs */
    if( i != SUCCESS )                            /* Display error message if */
        {                                         /*  record failed.          */
        puts( "\tError recording speech! Bad file handle or no available disk space." );
        doexit( i );
        }
    if ((i=waitdone( MAXWAIT )) != 0)
        return i;

    flushttqueue();                       /*Empty the touch-tone queue.         */

    if ((i=play( Vmsg, MPMPT2 )) != 0)  /* Play prompt that prepares caller         */
        return i;                       /* to hear what was recorded.        */

    i = Me2VoxPlayMsg( Vtst, MRECORD ); /* Play msg that contains their voice.*/
    if( i != SUCCESS )                  /* Abort if problem with that voice file. */
        {
        puts( "\tError playing message back." );
        doexit( i );
        }
    return( waitdone( MAXWAIT ) );
    }


/*****************************************************************************
WAITDONE:
   This function waits for a 'DONE' code to appear on the event queue
   while checking any other events coming in to see if they need special
   handling. All other events are thrown away.  Note that even though Touch-
   Tone events are thrown away here, ME/2 maintains a copy of them in the TT
   queue for Me2GetTTS() and Me2GetTTC() to process.

   All ME/2 functions which take more than a fraction of a second to
   complete are designed to return immediately upon startup and then
   report a 'DONE' event code when the action is totally finished.  A
   function of this type generally has three valid 'DONE' codes to
   indicate one of the following:  finished normally, stopped by
   Me2Stop(), or the function failed and was aborted.

   All events which indicate the termination of a ME/2 function have been
   placed in a special range of values which can be easily checked for.
   Waitdone() returns when an event code in the 'DONE' code value range
   appears on the event queue.


   Return Value:        1   A hangup has been detected. Stop currently
                            active function and return.
                        0   No special event has been found.
                       <0   Timeout
*******************************************************************************/
ME2INT waitdone( long timeout )
    {
    ME2INT ev ;
    ME2INT i;

    while (1)
        {
        i = Me2WaitEvent( timeout );   /* Wait for an event to come through */
                                       /* the event queue.                  */
        if ( i == ME2_QUEUE_TIMEOUT )  /* Check and see if a timeout.       */
            {
            Me2Stop();                 /* If timeout, stop everything and   */
            return( i );               /* abort.                            */
            }

        Me2GetEvent( &ev, QREMOVE );   /* Get the event off the queue*/
        if ( ev == LPCUROFF )          /* Check and see if a hangup occurred*/
            {
            Me2Stop();                 /* If so, stop everything and   */
            return( 1 );               /* return.                       */
            }

          /* Check for done code.  */
        if( EXITCODE <= (unsigned)ev && (unsigned)ev <= EXITCMAX )
             return( 0 );
        }
    }



/*******************************************************************************
DOEXIT:
   This function is used to abort cleanly from the application if a failure
   occurs.  Error descriptions can found in ME2ERR.H.
   Argument List:    ME2INT n       =>    Error code.
   Return Value:     No return
*******************************************************************************/
void doexit( ME2INT n )
    {
    if( Vmsg >= 0 )                 /* Close both voice files if opened. */
        Me2VoxClose( Vmsg );
    if( Vtst >= 0 )
        Me2VoxClose( Vtst );

    Me2ReleaseLine();               /* Release the line                  */
    Me2ReleaseChannel();            /* and channel before exiting.       */

    strcat( Vtstfil, ".vox");
#ifdef __STDC__
    remove( Vtstfil );
#else
    unlink( Vtstfil );
#endif

    if( n != SUCCESS )
        {
        printf( "ME/2 error code: %d, process aborted.\n", n );
        printf( "Refer to 'me2err.h' and your manual for a list of errors.\n" );
        }

    exit( n );
    }


/******************************************************************************
FLUSHTTQUEUE:
   This function removes any existing touchtones from the touch-tone queue.
   Return Value:     none
******************************************************************************/
void flushttqueue( void )
    {
    char c;

    while( Me2GetTTC( &c, QREMOVE ) == 0 )
        ;
    return ;
    }
