/****************************************************************************
FILE:     dial.h
****!!!! THIS MODULE FOR CUSTOMER USE !!!!****
AUTHOR:   Michael Hoffman
CREATED:  871124
UPDATES:  880606    WEB     Dial.h split off from me2.h
****!!!! EXCISE ABOVE FOR CUSTOMER USE !!!!****

DESCRIPTION:
    The following mnemonic definitions are used to provide access to
the dial/call progress overlay functions when making IOCTL calls to
the device driver.  The parameters are accessed as offsets into the
dat[] array of the XFERPARM structure defined in ME2.H.

Copyright (c) 1987,88 Natural Microsystems Corporation.  All rights reserved.
****************************************************************************/

#define TRITNBG     0x40        /* Command & event: tri-tone begin
          No parameters.

          Data: direction down, multiple frames of 8 words, framecount
       may be used.  The words in a frame are interpreted as follows:
          Length of frame in 20mS periods
          Telephone control - up to four 4 bit nibbles written to port 2
               MUTE (6,7)  LINEOFH (A,B)  ATTN (C,D) or 0 for do nothing
               See section 4.4 where command = nibble +6
          Frequency of tone 1 in Hz - 0-4000Hz
          Amplitude of tone 1 in Hz - 1-1023; 32 nominal; 0= tone off
          Frequency of tone 2 in Hz - 0-4000Hz
          Amplitude of tone 2 in Hz - 1-1023; 32 nominal; 0= tone off
          Frequency of tone 3 in Hz - 0-4000Hz
          Amplitude of tone 3 in Hz - 1-1023; 32 nominal; 0= tone off */

#define TRITNST     0x41        /* Command & event: tri-tone stopped */

#define TRITNDN     0x42        /* Event: tri-tone done              */



#define DIALBG      0x44        /* Command & event: dial begin
          Parameters: eleven parameters in eleven words.  The first
       three are individual, but the last eight all control character
       durations in 20mS periods.  For all eleven, the default is the
       value assumed if the parameter passed is zero.  The parameters in
       order are:
          Tone if 0 (default) or pulse if nonzero (overridden by " or $)
          Amplitude for tones from 1-1023 (default 1023)
          Noise level on dial tone detect from 1-32 (default 8)
          Tone on time (default 4 = 80mS)
          Tone off time (default 4 = 80mS)
          Pulse dial interdigit time (default 40 = 800mS)
          Wait time for ',' (default 100 = 2 sec)
          Wait time for '.' (default 400 = 8 sec)
          Time to wait for dial tone (default 250 = 5 sec)
          Duration of tone considered dial tone (default 50 = 1 sec)
          Duration of switchhook (default 25 = 500mS)

          Data: direction down, one frame of up to 32 words (64
       characters), null framecount.  Consists of a single null
       terminated string.  The characters recognized are listed below.
       All others are ignored.
          0123456789   Digits to be dialled tone or pulse
          *#           Dialled as tones (ignored during pulse dial)
          "            Directive to dial rest of string as pulse
          $            Directive to dial rest of string as tone
          ,.           Proceed on delay time
          ;            Proceed on either dial tone or delay time
          !            Switchhook                           */

#define DIALST      0x45        /* Command & event: dial stop       */

#define DIALDN      0x46        /* Event: dial done                 */
#define DLPRMERR    0x47        /* Event: missing dial parameters   */
#define DIALTMO     0x48        /* Event: timed out waiting for tone*/
                                /*  (Alliance Generation only)      */


#define CPBG        0x4c        /* Command & event: call progress begin
          Parameters: twelve parameters in twelve words.  The first
       parameter is a set of flags, one for debug mode and the rest to
       indicate whether to quit after certain intermediate events or to
       continue call progress looking for some later event.  The
       following three parameters are individual, but the last eight all
       select various timing conditions placed on call progress
       decisions.  All twelve have default values that are used if the
       parameters are passed as zero.  The parameters in order are:
          Flags        Bit     0=1     1=1     2=1     3=1     4=1
             Means quit after  CPRNG   CPRNGQT CPVC    CPVCMD  CPVCLNG
             Bit 5=1 means send debug frames
          Noise level - 1-32 (default 8)
          Number of busys until report and quit (default 4)
          Number of rings until report and quit (default 7)
          Period separating fast/slow busy (default 35 = .7 sec)
          Period separating busy/ring cycle (default 75 = 1.5 sec)
          Tone length separating ring/dial tones (default 150 = 3 sec)
          Maximum ring period before CPRNGQT (default 300 = 6 sec)
          Time after CPVC until CPVCMD (default 125 = 2.5 sec)
          Time after CPVC until CPVCLNG (default 250 = 5 sec)
          Silence period after voice until CPVCEND (default 50 = 1 sec)
          Timeout to CPNORSP - no call progress (default 750 = 15 sec)

          Data: direction up, multiple frames of 16 words, framecount
       may be used.  This data buffer is only used if bit 5 of the first
       parameter is set to 1 and has no effect on the call progress
       decisions and events.  Each frame consists of four 4 word
       subframes representing information every 5mS.  Subsequent buffers
       may be submitted or not.  Call progress is not affected by
       running out of debug frame buffer space, and no event indicates
       so.  The framecount may be useful after call progress is complete
       to decide how much of the buffer to save.                */

#define CPST        0x4d        /* Command & event: call progress stop  */

#define CPDN        0x4e        /* Event: call progress done            */
#define CPPRMERR    0x4f        /* Event: missing call prog paramters   */



#define CPDLTN      0x50        /* Event: dial tone detect         */
#define CPBSY       0x51        /* Event: busy detect              */
#define CPFBSY      0x52        /* Event: fast busy detect         */
#define CPRNG       0x53        /* Event: ringing detect           */
#define CPRNGQT     0x54        /* Event: ringing quit             */
#define CPNOANS     0x55        /* Event: no answer                */
#define CPSIT       0x56        /* Event: special information tone */
#define CPVC        0x57        /* Event: voice detect             */
#define CPVCMD      0x58        /* Event: voice medium length      */
#define CPVCLNG     0x59        /* Event: voice long lentgh        */
#define CPVCEND     0x5a        /* Event: voice ended              */
#define CPNORSP     0x5b        /* Event: no response              */

#define DLTIMERR    0x5c        /* Event: out of exe. time error  */
#define DLFRMOV     0x5d        /* Event: frame overrun error     */
#define DLCMDERR    0x5e        /* Event: command error           */
#define DLLVLERR    0x5f        /* Event: speech env level error  */
