/***********************************************************************
*  File -  GTP.h
*
*  Description - Messages for Trunk Protocols
*
****!!!! THIS MODULE FOR CUSTOMER USE !!!!****
*  $$History
*   1 GTP.H 13-Jul-90,14:24:14,`GS' Placed under revision control
*   2 GTP.H 13-Dec-90,18:00:08,`GS' Added CmdDIAL, chg EvnTMHG to EvnDIAL
*   3 GTP.H 21-May-92,10:43:32,`JP' Added MF events.
*   4 GTP.H 1-Jul-92,18:38:28,`RP'  comments changed to ANSI C comments.
*   5 GTP.H 14-Jul-92,15:46:16,`GS' Added MFMIN and MFMAX
*   6 GTP.H 16-Mar-93,19:26:04,`GS' Added CmdLOADTCP
*   7 GTP.H 23-Mar-93,18:24:58,`GS' Added customer use comments
*   8 GTP.H 17-Jun-93,17:33:56,`GS' ANSI comments
*   9 GTP.H 29-Sep-93,10:08:50,`GS' Added cmd 86, comments
*  $$History
****!!!! EXCISE ABOVE FOR CUSTOMER USE !!!!****
*
*    Copyright (c) 1993  Natural MicroSystems Corp.  All rights reserved.
***********************************************************************/

/*--  Commands to the Engine --*/

#define CmdGTP_MIN     0x80
#define CmdBEG         0x80   /* Start the Trunk Control Program */
#define CmdEND         0x81   /* Stop the Trunk Control Program  */
#define CmdTST         0x82   /* Report test results             */
#define CmdARG         0x83   /* Setup GTP arguments             */
#define CmdPAR         0x84   /* Set up the parameter area       */
#define CmdTON         0x85   /* Generate a tone                 */
#define CmdGTP_ENG_MAX 0x87

/*--  Commands to the Trunk Control Program --*/

#define CmdRING        0x88   /* Generate audible ringing tone */
#define CmdBUSY        0x89   /* Generate busy tone            */
#define CmdRORD        0x8A   /* Generate reorder tone         */
#define CmdANSR        0x8B   /* Answer the call               */
#define CmdHGUP        0x8C   /* Hangup                        */
#define CmdDIAL        0x8D   /* Dial                          */

/*-- Special commands added for Alliance Generation: --*/
#define CmdEsc         0x86   /* Pass 1st parm as command to TCP         */
#define CmdLOADTCP     0x9F   /* Command from ME2 with TCP name in parms */


/*-- Events --*/

#define EvnGTP_MIN     0x80
#define EvnBEG         0x82   /* TCP started response              */
#define EvnEND         0x83   /* TCP stopped response              */
#define EvnFIN         0x84   /* TCP executed the FINISH macro     */
#define EvnNTC         0x85   /* No Trunk Control Program running. */
#define EvnOVF         0x86   /* Command Overflow                  */
#define EvnILG         0x87   /* Illegal Command                   */
#define EvnXOK         0x88   /* Dialling string transfered OK     */
#define EvnXNO         0x89   /* Dialling string not transfered    */
#define EvnSER         0x8A   /* Wrong Speech environment          */

#define EvnTMD1        0x90     /* timeout waiting for 1st digit              */
#define EvnTMDN        0x91     /* timeout waiting for nth digit              */
#define EvnRHUP        0x92     /* remote hang up detected                    */
#define EvnEODG        0x93     /* end of dial (in or out)                    */
#define EvnDT10        0x94     /* more than 10 pulses recd                   */
#define EvnHGUP        0x95     /* hangup request acknowledged                */
#define EvnTNBG        0x96     /* tone request ack                           */
#define EvnANSR        0x97     /* answer request ack                         */
#define EvnTMPC        0x98     /* time out waiting for tone/answer request   */
#define EvnDIAL        0x99     /* dial request ack                    901213 */
#define EvnPRSG        0x9a     /* permanent signal detected                  */
#define EvnOFHK        0x9b     /* off hook detected (incoming siezure)       */
#define EvnIGNR        0x9c     /* command ignored                            */
#define EvnNOWK        0x9d     /* timed out waiting for wink to start 901212 */
#define EvnWKSK        0x9e     /* timed out waiting for wink to end   901212 */

#define PLSDGT1        0xA1   /* Pulse digit */
#define PLSDGT2        0xA2
#define PLSDGT3        0xA3
#define PLSDGT4        0xA4
#define PLSDGT5        0xA5
#define PLSDGT6        0xA6
#define PLSDGT7        0xA7
#define PLSDGT8        0xA8
#define PLSDGT9        0xA9
#define PLSDGT0        0xAA
#define EvnGTP_MAX     0xAF

/*----------------------------------------| */
/* MF TABLE (in digit order)  910521-JP   | */
/* Event    Code   Frequencies   TT Equiv | */
/*--------|------|-------------|----------| */
/* MFST     0xD4   1500,1700     '*'      | */
/* MFKP     0xCC   1100,1700     '#'      | */
/* MFDGT0   0xCF   1300,1500     '0'      | */
/* MFDGT1   0xC0   700, 900      '1'      | */
/* MFDGT2   0xC1   700, 1100     '2'      | */
/* MFDGT3   0xC5   900, 1100     '3'      | */
/* MFDGT4   0xC2   700, 1300     '4'      | */
/* MFDGT5   0xC6   900, 1300     '5'      | */
/* MFDGT6   0xCA   1100,1300     '6'      | */
/* MFDGT7   0xC3   700, 1500     '7'      | */
/* MFDGT8   0xC7   900, 1500     '8'      | */
/* MFDGT9   0xCB   1100,1500     '9'      | */
/* n/a      n/a    (no equiv)    'A'      | */
/* MFST3P   0xC4   700, 1700     'B'      | */
/* MFSTP    0xC8   900, 1700     'C'      | */
/* MFST2P   0xD0   1300,1700     'D'      | */
/*----------------------------------------| */
#define MFMIN          0xC0
#define MFDGT1         0xC0
#define MFDGT2         0xC1
#define MFDGT4         0xC2
#define MFDGT7         0xC3
#define MFST3P         0xC4
#define MFDGT3         0xC5
#define MFDGT5         0xC6
#define MFDGT8         0xC7
#define MFSTP          0xC8
#define MFEND          0xC9        /* end of the MFtone */
#define MFDGT6         0xCA
#define MFDGT9         0xCB
#define MFKP           0xCC
/* unused              0xCD */
/* unused              0xCE */
#define MFDGT0         0xCF
#define MFST2P         0xD0
/* unused              0xD1 */
/* unused              0xD2 */
/* unused              0xD3 */
#define MFST           0xD4
#define MFMAX          0xD4
