/*****************************************************************************
FILE:        me2err.h
DESCRIPTION: This header file defines all possible negative error return codes
             and positive success codes.  Virtually all ME/2 ToolKit functions
             will return one of these return codes.

****!!!! This module for customer use; Begin stripping !!!!****
   $$History
*   1 ME2ERR.H 19-Jul-89,14:39:24,`GS' Placed under revision control
*   2 ME2ERR.H 26-Mar-90,18:34:44,`GS' Deleted ME2_TT_IN_PROGRESS.
*   3 ME2ERR.H 2-Aug-90,17:38:08,`GS' Added Trunk manager errors
*        Moved scheduler errors to VBXSCHED.H
*   4 ME2ERR.H 13-Dec-90,17:51:10,`GS' Added errors for OG Trunk support.
*   5 ME2ERR.H 29-Apr-91,16:58:06,`WEB' 'WEB' Synthesis error codes -103,-110
*   6 ME2ERR.H 7-Nov-91,9:51:04,`JP' Added internal_timeout. Paren codes (-1).
*   7 ME2ERR.H 10-Feb-92,18:05:12,`GS' Removed parens around 0 (SUCCESS)
*   8 ME2ERR.H 14-May-92,13:37:24,`GS' added -112
*   9 ME2ERR.H 10-Dec-93,17:38:22,`GS' New errors for vox mgr
*   10 ME2ERR.H 27-Jul-94,7:59:22,`JP' Note addition of errors to me2xlate
   $$History
****!!!! End stripping for customer use !!!!****

Copyright (c) 1987-94 Natural MicroSystems Corporation.  All rights reserved.
*****************************************************************************/

#define ME2_NO_CHANNEL           (-2)   /* No channel has been requested yet  */
#define ME2_NO_LINE              (-3)   /* No line has been requested yet     */
#define ME2_MEM_ALLOC_FAIL       (-4)   /* Necessary memory allocation failed */
#define ME2_SHARE_MEM_FAIL       (-5)   /* Necessary shared mem alloc. failed */
#define ME2_OUT_OF_SEQ           (-6)   /* Call made out of sequence.         */
#define ME2_FUNCTION_INUSE       (-7)   /* Board already in use by other func.*/
#define ME2_BAD_APPL_ID          (-8)   /* Invalid application ID             */
#define ME2_OUT_OF_CHANNELS      (-9)   /* Maximum number of channels exceeded*/
#define ME2_BAD_CHANNEL          (-10)  /* Bad channel number given as arg.   */
#define ME2_BAD_LINE             (-11)  /* Bad line number given as argument. */
#define ME2_BAD_QUEUE_TYPE       (-12)  /* Invalid queue type specified.      */
#define ME2_PARMS_NOT_FOUND      (-13)  /* Parameter Spec. files not found    */
#define ME2_BAD_PARM_FILE        (-14)  /* Corrupted Parameter Spec file.     */
#define ME2_BAD_PARM_CATEGORY    (-15)  /* Bad category number given.         */
#define ME2_BAD_PARM_NUMBER      (-16)  /* Bad parameter number given.        */
#define ME2_PARM_NOT_FOUND       (-17)  /* Parameter name not found.          */
#define ME2_FILE_OPEN_FAIL       (-18)  /* Failure during a file open.        */
#define ME2_CATEGORY_NOT_FOUND   (-19)  /* Category name not found.           */
#define ME2_BAD_PARM_NAME        (-20)  /* Bad parameter name specified       */
#define ME2_NO_GROUP_MEMBERS     (-21)  /* No members in group specified      */
#define ME2_TIMEOUT              (-22)  /* Timeout while waiting for event.   */
#define ME2_QUEUE_MISMATCH       (-23)  /* Mismatch when attaching to queue.  */
#define ME2_QUEUE_ATTACH_FAIL    (-24)  /* Internal attempt to attach failed. */
#define ME2_NO_FUNCTION          (-25)  /* No function in progress to stop    */
#define ME2_FILE_NOT_FOUND       (-26)  /* Specified file could not be found  */
#define ME2_FILE_WRITE_FAIL      (-27)  /* Failure during a file write        */
#define ME2_FILE_READ_FAIL       (-28)  /* Failure during a file read         */
#define ME2_BAD_DB_HANDLE        (-29)  /* Bad Database handle was specified  */
#define ME2_INVALID_RECORD       (-30)  /* Bad record number was specified    */
#define ME2_FIELD_NOT_FOUND      (-31)  /* Database field not found           */
#define ME2_BAD_OPEN_MODE        (-32)  /* Bad open mode argument specified   */
#define ME2_NO_INDEX_FILE        (-33)  /* No index file is open              */
#define ME2_NO_MATCH_FOUND       (-34)  /* No match to specified key was fnd. */
#define ME2_RECORD_DELETED       (-35)  /* Specified record is deleted        */
#define ME2_INVALID_FIELD        (-36)  /* Specified field number is invalid  */
#define ME2_LOCK_IN_USE          (-37)  /* Lock request failed, unlock first  */
#define ME2_NOT_LOCKED           (-38)  /* Unlock attempted when not locked   */
#define ME2_LOCK_FAIL            (-39)  /* Lock attempt(s) failed             */
#define ME2_BAD_GROUP            (-40)  /* Bad line group specified           */
#define ME2_DB_NOT_SUPPORTED     (-41)  /* Function not supported in DB Mgr.  */
#define ME2_BAD_NDX_HANDLE       (-42)  /* Bad index file handle specified    */
#define ME2_DB_NOT_FOUND         (-43)  /* Database Manager not found         */
#define ME2_QUEUE_TIMEOUT        (-44)  /* Timeout waiting for queue event    */
#define ME2_QUEUE_EMPTY          (-46)  /* Queue is currently empty           */
#define ME2_STATUSMON_INUSE      (-48)  /* Status monitor already requested   */
#define ME2_BAD_VOX_HANDLE       (-50)  /* Bad voice file handle specified    */
#define ME2_VOX_MSG_NOT_FOUND    (-51)  /* Specified message not found in file*/
#define ME2_VOX_NO_CURRENT_MSG   (-52)  /* No current message for frame edit  */
#define ME2_VOX_BAD_SEEKMODE     (-53)  /* Bad voice seek mode specified      */
#define ME2_VOX_OUT_OF_INDICES   (-54)  /* Out of indices in voice file       */
#define ME2_VOX_BAD_MSG_COUNT    (-55)  /* Message cound out of range         */
#define ME2_VOX_FRM_NOT_FOUND    (-56)  /* Specified frame not found in msg.  */
#define ME2_BAD_VOX_ARG          (-57)  /* Misc. bad argument specified       */
#define ME2_BAD_KEY_FIELD        (-58)  /* Invalid field specified for create */
#define ME2_KEY_FIELD_MISMATCH   (-59)  /* Cannot mix char and numeric fields */
#define ME2_DB_RWMODE_MISMATCH   (-60)  /* Open read/write mode inconsistent  */
#define ME2_NOT_IN_GROUP         (-61)  /* Specified line is not in group     */
#define ME2_DEVIOCTL_ERR         (-62)  /* Error while talking to driver      */
#define ME2_CREATE_THREAD_FAIL   (-63)  /* Handler thread creation error      */
#define ME2_HDLR_BEG_TIMEOUT     (-64)  /* Timeout waiting for handler thread */
#define ME2_VOX_NO_PLAY          (-66)  /* Not currently playing              */
#define ME2_BAD_QUEUE_SEL        (-67)  /* Bad queue selector specified       */
#define ME2_VOX_RATE_MISMATCH    (-68)  /* Rate mismatch in voice copy        */
#define ME2_DB_KEY_NOT_FOUND     (-70)  /* Database key to delete not found   */
#define ME2_DB_DUPLICATE_KEY     (-71)  /* Duplicate key (not allowed in DB)  */
#define ME2_NO_STATMON           (-72)  /* Function requires a status monitor */
#define ME2_BAD_OVLY_FILE        (-73)  /* Specified file not valid overlay   */
#define ME2_OVLY_BANK_FAIL       (-74)  /* Bank failure while loading overlay */
#define ME2_BAD_OVLY_ID          (-75)  /* Bad overlay ID, not ME/2 overlay   */
#define ME2_INDEX_FILE_OPEN      (-76)  /* Must close index files before main */
#define ME2_VMOS_ERROR           (-77)  /*                                    */
#define ME2_OPEN_FOR_PLAYONLY    (-78)  /*                                    */
#define ME2_BAD_LINE_TYPE        (-79)  /*                                    */
#define ME2_QUEUE_FULL           (-80)  /*                                    */
#define ME2_BAD_ARGUMENT         (-81)  /*                                    */
#define ME2_AUDIO_DISABLED       (-82)  /*                                    */
#define ME2_OUT_OF_MESSAGES      (-83)  /*                                    */
#define ME2_BAD_PROMPT_HANDLE    (-84)  /*                                    */
#define ME2_BAD_PROMPT_CMD       (-85)  /*                                    */
#define ME2_PROMPT_BUILD_FAIL    (-86)  /* error while building msg           */
#define ME2_PARM_EXCEED_MAX      (-87)  /*                                    */
#define ME2_ENVVAR_NOT_FOUND     (-88)  /*                                    */
/****!!!! This module for customer use; Begin stripping !!!!*******************
 * Note: -89 to -93 for Me2Scheduler which was not released.  These errors    *
 *      are located in VBXSCHED.H.                                            *
 ****!!!! End stripping for customer use !!!!**********************************/
#define ME2_MAX_FILES_OPEN       (-94)  /* max num of database or voice files */
#define ME2_WRONG_BOARD_TYPE     (-95)  /* function not supp. by this board   */
#define ME2_TRUNK_NOT_ACTIVE     (-96)  /* Trunk overlay not active           */
#define ME2_OVLY_NOT_LOADED      (-97)  /* Overlay was not loaded(Me2OvlyLock)*/
#define ME2_TRUNK_NOT_INITD      (-98)  /* Trunk initialization was not done  */
#define ME2_NO_TRUNK_MANAGER     (-99)  /* Me2Trunk.DLL not found (req line)  */
#define ME2_TRUNK_BAD_SIGNAL_TYPE (-100)/* Invalid TRUNK.SIGNAL_TYPE parameter*/
#define ME2_TRUNK_DIAL_TIMEOUT   (-101) /* Time out waiting for wink/wink-end */
#define ME2_TRUNK_NOT_IDLE       (-102) /* Out sieze failed, trunk is busy    */
#define ME2_SYN_TOO_MANY_VOICES  (-103) /* NVOICES parm must not exceed 10    */
#define ME2_SYN_DIP_TOO_BIG      (-104) /* Diphone table file too big (>64K)  */
#define ME2_SYN_BAD_VOICE_NUM    (-105) /* Current voice # >= number of voices*/
#define ME2_SYN_NULL_UTTERANCE   (-106) /* After processing, nothing to say   */
#define ME2_SYN_WD2PHN_OVERFLOW  (-107) /* Buff ovflow converting wds to phns */
#define ME2_SYN_HOMPROC_OVERFLOW (-108) /* Buff ovflow converting wds to phns */
#define ME2_SYN_PHN2FRM_OVERFLOW (-109) /* Buff ovflow converting wds to phns */
#define ME2_SYN_INTONE_OVERFLOW  (-110) /* Buff ovflow converting wds to phns */
#define ME2_INTERNAL_TIMEOUT     (-111) /* Poor CPU util.; semaphore timeout  */
#define ME2_NO_TRUNK_OVERLAY     (-112) /* Invalid TRUNK.OVERLAY parameter    */
#define ME2_INTERNAL_ERROR       (-113) /* An internal inconsistency occurred */
#define ME2_WRONG_FILE_TYPE      (-114) /* Wrong kind of file                 */
#define ME2_VOX_FILE_IN_USE      (-115) /* File is accessed by another thread */
#define ME2_TOO_MANY_BUFFERS     (-116) /* Buffer submitted too early         */
#define ME2_LINE_IN_USE          (-117) /* Attempt to grab a busy line        */

/****!!!! This module for customer use; Begin stripping !!!!*******************
 *                                                                            *
 * NOTE: WHEN ADDING ERROR CODES, ALSO MODIFY 'ME2XLATE' TO CONTAIN THE NEW   *
 *       ERROR NAME AND DESCRIPTION.                                          *
 *                                                                            *
 ****!!!! End stripping for customer use !!!!**********************************/


/*******************************************************************************
* Following are successful completion messages.  SUCCESS indicates that
* everything went as planned.  All other warning messages mean the intended
* function was accomplished but an unexpected situation arose which was taken
* care of automatically.
*/

#define SUCCESS                    0     /* function was successfull          */
#define ME2_NO_HIGHER_QUEUE        1     /* pushed entry was thrown away      */
