/****************************************************************************
FILE:	  vox.h
****!!!! THIS MODULE FOR CUSTOMER USE !!!!****
AUTHOR:   Michael Hoffman
CREATED:  871124
UPDATES:  880606    WEB     Vox defs split off into own file
****!!!! EXCISE ABOVE FOR CUSTOMER USE !!!!****

DESCRIPTION:
    The following mnemonic definitions are used to provide access to
the store & forward overlay functions when making IOCTL calls to the
device driver.	The parameters are accessed as offsets into the
dat[] array of the XFERPARM structure defined in ME2.H.

Copyright (c) 1987,88 Natural Microsystems Corporation.  All rights reserved.
****************************************************************************/



#define RECBG	    0x60	/* Command & event: recording begin
	  Parameters: five parameters in five words.  All parameters are
       single word.  The first is a number from 0 to 4 to choose a
       recording rate from 12 to 64kbps respectively.  The second
       chooses an amplitude level from 1 to 1023.  The default of 32 is
       used if this parameter is 0 (record and play with no gain).  If
       this paramter is negative, automatic gain control is used.  The
       third parameter chooses a noise level to be used in both
       automatic gain control and hangup detection (nominal 8).  The
       fourth and fifth parameters choose time periods in 100ths of a
       second for hangup to be reported before speech and after speech
       respectively.  Each of these may be disabled by setting a time
       period of negative or zero.

	  Data: direction up, any number of frames of fixed size
       depending on rate, framecount in use.  The frame sizes at each
       rate are 16, 21, 31, 41, and 81 words for 12, 16, 24, 32, and
       64kbps respectively.	    */

#define RECST	    0x61	/* Command & event: recording end   */

#define RECNOFRMS   0x62	/* Event: out of frames error	    */
#define RECBHGP     0x63	/* Event: hangup at start of record */
#define RECHGP	    0x64	/* Event: hangup during record	    */



#define PLYBG	    0x68	/* Command & event: play begin
	  Parameters: two parameters in two words.  The first parameter
       is a rate from 0 to 4 for 12 to 64kbps speech as for recording.
       This value MUST match the rate on recording.  The second
       parameter is amplitude level select from 1 to 1023 with 0
       indicating a default of 32.  Default values of 32 for both record
       and play will result in no overall gain and will cover the
       dynamic range in typical telephone environments.

	  Data: direction down, multiple frames of fixed size depending
       on rate, framecount in use.  The frame sizes at each rate are
       16, 21, 31, 41, and 81 words for 12, 16, 24, 32, and 64kbps
       respectively.	     */

#define PLYST	    0x69	/* Command & event: play stopped    */

#define PLYLOUD     0x6a	/* Command: play 50% louder	    */
#define PLYSOFT     0x6b	/* Command: play 25% softer	    */

#define PLYNOFRMS   0x6a	/* Event: out of frames error	    */
#define PLYAMPL     0x6b	/* Event: change in play amplitude  */



#define BEEPBG	    0x70	/* Command & event: beeping begin
	  Parameters: five parameters in five words.  The first
       parameter should be 0 to 7 to choose a 250Hz fundamental tone or
       one of its harmonics (500Hz, 750Hz, 1kHz, 1250Hz, 1500Hz, 1750Hz,
       or 2kHz respectively).  The second parameter chooses amplitude
       from 1 to 1023 with 0 defaulting to 32 as for record and play.
       The third parameter is a count of the number of beeps to give;
       all at the same frequency.  The fourth parameter gives the
       duration of each beep in 20mS periods.  The fifth parameter gives
       the duration of a silent pause between beeps in 20mS periods.

	  No data.		*/

#define BEEPST	    0x71	/* Command & event: beeping stopped */

#define BEEPDN	    0x72	/* Event: beeping done		    */



#define VTIMERR     0x7c	/* Event: out of execution time error  */
#define VFRMOV	    0x7d	/* Event: frame overrun error, PC late */
#define VCMDERR     0x7e	/* Event: command error 	       */
#define VLVLERR     0x7f	/* Event: speech env. level error      */


/* Clear-down-tone events, generated by PLAY.OVL and RECORD.OVL     910124 */

#define PLYTHGP     0x6C	/* Tone detected during play	    910124 */
#define RECTHGP     0x65	/* Tone detected during record	    910124 */
