/*******************************************************************************
FILE:     parminfo.c

****!!!! THIS MODULE FOR CUSTOMER USE !!!!****
AUTHOR:   Michael Hoffman
CREATED:  890428
UPDATES:  890701 MTH  Added ability to show individual categories.
   $$History
*   1 PARMINFO.C 2-Aug-89,11:07:04,`GS' Placed under revision control
*   2 PARMINFO.C 26-Mar-90,11:30:18,`DAR' Added function prototypes
*   3 PARMINFO.C 1-May-90,8:55:12,`GS' Added 'stripper' lines.
*   4 PARMINFO.C 4-Jan-91,11:27:50,`JP' Version 3.21 - Reformat; new options.
*        Added -x(hes) -p(pause) -n(catgnames) options.
*   5 PARMINFO.C 22-Jan-91,17:06:18,`JP' Added string search in parm values.
*   6 PARMINFO.C 14-Feb-91,8:46:42,`JP' Fixed bug where global buff was used
*        instead of a local buffer.
*   7 PARMINFO.C 12-Mar-91,12:12:34,`JP' chk_str of str > 128 not tested.
*   8 PARMINFO.C 7-Jul-94,18:15:32,`MEL' merged UNIX and OS/2 versions
   $$History
****!!!! EXCISE ABOVE FOR CUSTOMER USE !!!!****
DESCRIPTION:

This utility shows all parameters currently defined in the system.
The information can be presented in report format or in Master profile
format.

The report format is the default format.

The Master profile format is the same as appending all parameter source
files, *.prs, as one file.  This file can be split into source files if
the source file doesn't exist.

Copyright (c) 1989-91 Natural Microsystems Corporation.  All rights reserved.
*******************************************************************************/

#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#ifdef unix
#include "ctype.h"
#else
#include "conio.h"
#endif

#ifdef unix
#define far
#define pascal
#endif

#include "me2def.h"

void main( int, char ** );
void syntax( void );
int  chk_catg( char * );
int  chk_prm( char * );
int  chk_str( char * );
void getval( int, int, char *, char * );
char *spaces( int, char * );
void pause( int * );
#ifdef unix
int   comparech (int, int) ;
char *itoa (int, char *, int) ;
char *ltoa (long, char *, int) ;
int   strcmpi (const char *, const char *) ;
char *strupr (char *) ;
#endif

#define OUTSIZ 512
static char line[OUTSIZ];
static char buff[OUTSIZ];
static char fname[50];

#define MAXDEFS 20
static char CatList[MAXDEFS][40];   /*xx- specified list of catgs-xx*/
static char PrmList[MAXDEFS][40];   /*xx- specified list of prm names-xx*/
static char StrList[MAXDEFS][40];   /*xx- specified list of strings-xx*/
static int  CatCnt  = 0;            /*xx- number of catgs to view when specified-xx*/
static int  PrmCnt  = 0;            /*xx- number of names to view when specified-xx*/
static int  StrCnt  = 0;            /*xx- number of strings to search for-xx*/

static int  TotCat  = 0;            /*xx- running total of categories reported-xx*/
static int  TotPrm  = 0;            /*xx- running total of parameters reported-xx*/
static long TotByte = 0L;           /*xx- running total of bytes reported-xx*/

#define O_DISPLAY  0
#define O_CATEGORY 1
#define O_MASTER   2
#define O_GEN      3
static int  OutpMode    = O_DISPLAY;    /*xx- method/format of output-xx*/

static int  HexFlg      = 0;    /*xx- flag to include hex rep of int/long vals-xx*/
static int  PauseFlg    = 0;    /*xx- flag & page size for pausing display-xx*/
static int  CaseFlg     = 0;    /*xx- flag indicating case-insensitive strings-xx*/

static char *revision   = "1.06";

static char *MaxCatgs = "Warning!  Number of specified categories limited to 20.";
static char *MaxNames = "Warning!  Number of specified names limited to 20.";
static char *MaxStrgs = "Warning!  Number of specified strings limited to 20.";


/*******************************************************************************
 MODULE:      main (parminfo)
 DESCRIPTION:
*******************************************************************************/
void main( int argc, char *argv[] )
    {
    ME2INT cat;
    ME2INT prm;
    char catname[30];
    char prmname[30];
    ME2INT type;
    int size,i,len,r;
    char *pc, *pp;
    int  pcnt;
    FILE *fp;
    int catgshown;

    printf( "VBX Show Parameter Info Utility  Version %s\n", revision );
    puts  ( "Copyright (c) 1989-1991 Natural MicroSystems Corporation." );
    puts  ( "  All rights reserved.\n" );

    if( argc < 2 )
        {
        syntax();
        }

    for( i = 1; i < argc; i++ )           /*xx- parse command line for options-xx*/
        {
        pc = argv[i];
        if( *pc == '-' || *pc == '/' || *pc == '\\' )
            {
            pc++;
            switch( toupper( *pc ) )
                {
                case 'P':               /*xx- Pause flag-xx*/
                    pc++;
                    if( *pc )
                        PauseFlg = atoi( pc );
                    else
                        PauseFlg = 24;
                    pcnt = 0;
                    break;

                case 'X':               /*xx- additionally show heX data format-xx*/
                    HexFlg = 1;
                    break;

                case 'I':               /*xx- case-insensitive string search-xx*/
                    CaseFlg = 1;
                    break;

                case 'M':               /*xx- Master profile format-xx*/
                    OutpMode = O_MASTER;
                    break;

                case 'D':               /*xx- Display format-xx*/
                    OutpMode = O_DISPLAY;
                    break;

                case 'G':               /*xx- Generate profile (.PRS) sources-xx*/
                    if( !strcmpi( pc, "GENPRS" ) )
                        OutpMode = O_GEN;
                    else
                        syntax();
                    break;

                case 'C':               /*xx- Category switch-xx*/
                    strcpy( line, ++pc );
                    if( !*line )
                        {
                        OutpMode = O_CATEGORY;
                        }
                    else if( CatCnt == MAXDEFS )
                        {
                        puts( MaxCatgs );
                        }
                    else
                        {
                        pp = strtok( line, "#$%^&*-+" );
                        while( pp )
                            {
                            if( CatCnt == MAXDEFS )
                                {
                                puts( MaxCatgs );
                                break;
                                }
                            strcpy( CatList[CatCnt++], pp );
                            pp = strtok( NULL, "#$%^&*-+" );
                            }
                        }
                    break;

                case 'N':               /*xx- parameter Name specification-xx*/
                    strcpy( line, ++pc );
                    if( !*line )
                        {
                        syntax();
                        }
                    else if( PrmCnt == MAXDEFS )
                        {
                        puts( MaxNames );
                        }
                    else
                        {
                        pp = strtok( line, "#$%^&*-+" );
                        while( pp )
                            {
                            if( PrmCnt == MAXDEFS )
                                {
                                puts( MaxNames );
                                break;
                                }
                            strcpy( PrmList[PrmCnt++], pp );
                            pp = strtok( NULL, "#$%^&*-+" );
                            }
                        }
                    break;

                case 'S':               /*xx- parameter Name specification-xx*/
                    strcpy( line, ++pc );
                    if( !*line )
                        {
                        syntax();
                        }
                    else if( StrCnt == MAXDEFS )
                        {
                        puts( MaxStrgs );
                        }
                    else
                        {
                        pp = strtok( line, "#$%^&*-+" );
                        while( pp )
                            {
                            if( StrCnt == MAXDEFS )
                                {
                                puts( MaxStrgs );
                                break;
                                }
                            strcpy( StrList[StrCnt++], pp );
                            pp = strtok( NULL, "#$%^&*-+" );
                            }
                        }
                    break;

                default:
                    printf( "Illegal switch [-/]: '%c'\n\n", *pc );
                    syntax();
                    break;
                }
            }
        else
            {
            r = ME2PARMLOAD( pc );      /*xx- if not option, assume profile name-xx*/
            if ( r < SUCCESS )
                {
                printf( "Could not load profile %s:  %d\n", pc, r );
#ifndef unix
                printf( "Press any key to continue." );
                getch();
#endif
                printf( "\r                           \r" );
                }
            }
        }


    if( StrCnt && CaseFlg )         /*xx- Upper case for case-insensitivity-xx*/
        {
        for( i = 0; i < StrCnt; i++ )
            strupr( StrList[i] );
        }


    pcnt = 0;
    for( cat = 0; ; cat++ )         /*xx- process each category until we hit one-xx*/
        {                           /*xx- that does not exist-xx*/
        catgshown = 0;

        if( ME2PARMCATEGORYNAME( cat, catname ) < SUCCESS )
            break;

        if( !chk_catg( catname ) )
            continue;

        if( pcnt && ( OutpMode != O_CATEGORY ) )
            {                       /*xx- always pause after a whole category-xx*/
            pcnt = -1;
            pause( &pcnt );
            }

        if( ( OutpMode != O_MASTER ) && ( OutpMode != O_GEN ) )
            {                       /*xx- Show in display format-xx*/
            if( OutpMode == O_CATEGORY )
                {
                printf( "[%02d] %s\n", cat, catname );
                TotCat++;
                pcnt++;
                pause( &pcnt );
                }
            else                    /*xx- Show parameter in the category-xx*/
                {
                pcnt = 0;
                for( prm = 0; ; prm++ )
                    {
                    if( ME2PARMNAME( cat, prm, prmname ) < SUCCESS )
                        break;

                    if( !chk_prm( prmname ) )
                        continue;

                    ME2PARMTYPEBYNUM( cat, prm, &type );

                    sprintf( line, " [%02d] %-13s = ", prm, prmname );
                    pc = &line[strlen(line)];

                    getval( cat, prm, pc, buff );

                    if( !chk_str( pc ) )
                        continue;

                    if( !catgshown )    /*xx- When something found, make sure-xx*/
                        {               /*xx- the category name is displayed.-xx*/
                        printf( "\nCATEGORY:  [%02d] %s\n", cat, catname );
                        catgshown = 1;
                        TotCat++;
                        pcnt += 2;
                        }

                    pcnt++;
                    if( ( len = strlen( line ) ) > 79 )
                        pcnt++;

                    if( HexFlg )
                        {
                        if( ( type == -2 ) || ( type == 0 ) )
                            {
                            pc = &line[len=strlen(line)];
                            pc = spaces( 40-len, pc );
                            sprintf( pc, "(%s)", buff );
                            }
                        else if( type < -2 )
                            {
                            pc = &line[len=strlen(line)];
                            if( len > 40 )
                                {
                                *pc++ = '\n';
                                pcnt++;
                                pc = spaces( 40, pc );
                                }
                            else
                                pc = spaces( 40-len, pc );
                            sprintf( pc, "(%s)", buff );
                            }
                        /*xx- increase line count for each 80 chars beyond 40-xx*/
                        pcnt += (((int)(strlen(buff)+40))/80);
                        }

                    pause( &pcnt );
                    puts( line );
                    }
                }
            }

        else                        /*xx- Show in master profile format-xx*/
            {
            if( OutpMode == O_GEN )
                {
                strcpy( fname, catname );
                i = strlen( fname );
                if( i > 8 ) i = 8;
                strcpy( &fname[i], ".prs" );
                if( ( fp = fopen( fname, "wt" ) ) == NULL )
                    printf( "Can't create parameter source '%s'.\n", fname );
                else
                    {
                    printf( "Creating '%s'\n", fname );
                    fprintf( fp, "; Parameter source: %s\n", fname );
                    fprintf( fp, "; Created by Parminfo.\n" );
                    }
                }
            else
                fp = NULL;

            pcnt = 0;
            for( prm = 0; ; prm++ )
                {
                if( ME2PARMNAME( cat, prm, prmname ) < SUCCESS )
                    break;

                if( !chk_prm( prmname ) )
                    continue;

                if( !catgshown )
                    {
                    printf( "\n" );
                    catgshown = 1;
                    TotCat++;
                    pcnt++;
                    }

                ME2PARMTYPEBYNUM( cat, prm, &type );

                strcpy( line, catname );        /*xx- build full name-xx*/
                strcat( line, "." );
                strcat( line, prmname );
                strcat( line, " " );

                len = strlen( line );
                pc = &line[len];

                while( len++ < 22 )
                    *pc++ = ' ';
                *pc++ = '=';
                *pc++ = ' ';

                getval( cat, prm, pc, buff );   /*xx- get value in text form-xx*/

                if( !chk_str( pc ) )
                    continue;

                strcat( line, " " );
                len = strlen( line );
                pc  = line + len;

                if( !HexFlg )
                    pc = spaces( 70-len, pc );  /*xx- space fill to 70 chars-xx*/
                else
                    pc = spaces( 40-len, pc );  /*xx- fill to 40 chars if nec.-xx*/

                *pc++ = '/';
                if( !type )                 /*xx- if 0, then value is a long-xx*/
                    {
                    *pc++ = 'l';            /*xx- put '/l'-xx*/
                    *pc++ = '\0';
                    }
                else
                    {
                    if( type == -2 )
                        {
                        *pc++ = 'i';
                        size = 1;
                        }
                    else if( type < 0 )     /*xx- if < 0, then integer array-xx*/
                        {
                        *pc++ = 'a';
                        size = (-type)/2;   /*xx- put '/ann' where nn=type/2-xx*/
                        }
                    else
                        {
                        *pc++ = 'c';        /*xx- otherwise type > 0, so sub 1-xx*/
                        size = type - 1;    /*xx- len does not include the null-xx*/
                        }
                    itoa( size, pc, 10 );   /*xx- put text equiv into string-xx*/
                    }

                pcnt += ( (strlen(line)/80) + 1 );

                if( HexFlg )
                    {
                    len = strlen( line );
                    pc = &line[len];
                    if( ( type == -2 ) || ( type == 0 ) )
                        {
                        pc = spaces( 50-len, pc );
                        sprintf( pc, ";(%s)", buff );
                        }
                    else if( type < -2 )
                        {
                        if( len > 50 )
                            {
                            *pc++ = '\n';
                            pcnt++;
                            pc = spaces( 50, pc );
                            }
                        else
                            pc = spaces( 50-len, pc );
                        sprintf( pc, ";(%s)", buff );
                        }

                    /*xx- increase line count for each 80 chars beyond 30-xx*/
                    len = strlen( buff );
                    pcnt += ((len-50)/80);
                    }

                if( fp )
                    fprintf( fp, "%s\n", line );
                else
                    {
                    pause( &pcnt );
                    puts( line );                   /*xx- output text to screen-xx*/
                    }
                }
            printf( "\n" );                     /*xx- blank line between categories-xx*/
            }
        }

    printf( "\nNumber of categories:  %d",    TotCat  );
    printf( "\nNumber of parameters:  %d",    TotPrm  );
    printf( "\nNumber of data bytes:  %ld\n", TotByte );
    exit( 0 );
    }


/*******************************************************************************
 MODULE:      syntax
 DESCRIPTION:
*******************************************************************************/
void syntax( void )
    {
    puts( "Format:\tparminfo [options] [profiles]" );
    puts( "\twhere options are:" );
    puts( "\t\t-d              Output in display/report format (default)." );
    puts( "\t\t-m              Output in master parameter source format." );
#ifdef UNDOCUMENTED
    puts( "\t\t-genprs         Generate .PRS files from .PRMs" );
#endif
    puts( "\t\t-c              Show a list of all category names." );
    puts( "\t\t-cCategories    Show info about specified categories." );
    puts( "\t\t                categories= cat1[+cat2[+cat3]...[+catN]]]." );
    puts( "\t\t-nParmnames     Show info about specified names only."  );
    puts( "\t\t                parmnames= nam1[+nam2[+nam3]...[+namN]]]." );
    puts( "\t\t-sStrings       Search for strings in parameter values." );
    puts( "\t\t                strings= str1[+str2[+str3]...[+strN]]]." );
    puts( "\t\t-i              Case-insensitive during string compares." );
    puts( "\t\t-x              Also show hex equivalent values." );
    puts( "\t\t-p[##]          Pauses for each category and every ## lines," );
    puts( "\t\t                where ## defaults to 24." );
    puts( "\tand profiles is a list of files to load before showing values:" );
    puts( "\t\t[file1[.pro] [file2.[pro]...[fileN[.pro]]]]" );
    exit( 1 );
    }


/*******************************************************************************
 MODULE:      chk_catg
 DESCRIPTION: If the user specified certain categories, determine if the
              category is in the list.  For no categories specified, return
              true.
*******************************************************************************/
int chk_catg( char *catg )
    {
    int i;

    if( CatCnt )                /*xx- if list was specified-xx*/
        {                       /*xx- see if this category is in the list-xx*/
        for( i = 0; i < CatCnt; i++ )
            if( !strcmpi( catg, CatList[i] ) )
                return( 1 );
        return( 0 );
        }
    else
        return( 1 );
    }


/*******************************************************************************
 MODULE:      chk_prm
 DESCRIPTION: If the user specified certain parameter names, determine if the
              category is in the list.  For no names specified, return true.
*******************************************************************************/
int chk_prm( char *prm )
    {
    int i;

    if( PrmCnt )        /*xx- if list was specified-xx*/
        {               /*xx-  see if this name is in the list-xx*/
        for( i = 0; i < PrmCnt; i++ )
            if( !strcmpi( prm, PrmList[i] ) )
                return( 1 );
        return( 0 );
        }
    else
        return( 1 );
    }


/*******************************************************************************
 MODULE:      chk_str
 DESCRIPTION: If the user specified search strings, determine if the passed
              buffer contains any of the search strings.  For no strings
              specified, return true.
*******************************************************************************/
int chk_str( char *str )
    {
    int i;
    char lclbuff[128];

    if( !StrCnt )
        return( 1 );

    else if( strlen( str ) >= 128 )
        return( 0 );

    strcpy( lclbuff, str );
    if( CaseFlg )
        strupr( lclbuff );

    for( i = 0; i < StrCnt; i++ )   /*xx-  see if this name is in the list-xx*/
        if( strstr( lclbuff, StrList[i] ) )
            return( 1 );

    return( 0 );
    }


/*******************************************************************************
 MODULE:      getval
 DESCRIPTION:
*******************************************************************************/
void getval( int cat, int prm, char *cval, char *hval )
    {
    int i;
    char *pc;
    char *ph;
    long lval;
    ME2INT type;
    /* Note - 'short' type used to hold the values of 16 bit ME2 parameters
     * passed by reference to Me2ParmByName and Me2ParmByNum */
    short arry[256];

    TotPrm++;

    ME2PARMTYPEBYNUM( cat, prm, &type );

    if( !type )
        {
        TotByte += 4;                   /*xx- long value-xx*/

        ME2PARMBYNUM( cat, prm, (void far *)&lval, GETPARM );
        ltoa( lval, cval, 10 );
        sprintf( hval, "0x%08lx", lval );
        }

    else if( type > 0 )                 /*xx- char string-xx*/
        {
        TotByte += type;

        *cval++ = '"';
        *hval   = '\0';
        ME2PARMBYNUM( cat, prm, (void far *)cval, GETPARM );
        strcat( cval, "\"" );
        }

    else
        {
        type = -type;                   /*xx- integer array-xx*/
        TotByte += type;
        type = type/2;
        pc = cval;
        ph = hval;

        ME2PARMBYNUM( cat, prm, (void far *)arry, GETPARM );

        for( i = 0; i < type; i++ )     /*xx- create comma delimited list of #'s-xx*/
            {
            if( i )
                {
                *pc++ = ',';
                *ph++ = ',';
                }
            itoa((int)arry[i], pc, 10 );
            pc = &cval[strlen(cval)];
            sprintf( ph, "0x%04x", arry[i] );
            ph = &hval[strlen(hval)];
            }
        }
    return;
    }


/*******************************************************************************
 MODULE:      spaces
 DESCRIPTION:
*******************************************************************************/
char *spaces( int i, char *pc )
    {
    if( i > 0 )
        {
        while( i-- )
            *pc++ = ' ';
        }
    return( pc );
    }


/*******************************************************************************
 MODULE:      pause
 DESCRIPTION:
*******************************************************************************/
void pause( int *count )
    {
    int c;

    if( !PauseFlg ) return;
    if( *count >= PauseFlg || *count < 0 )
        {
        printf( "(C)ontinuous; (Q)uit; Any other key to continue." );
#ifdef unix
        c = getchar () ;
#else
        c = getch();
#endif
        printf( "\r                                                \r" );
        switch( toupper( c ) )
            {
            case 'C':   PauseFlg = 0;  break;
            case 'Q':   exit( 0 );     break;
            default:    *count = 1;    break;
            }
        }
    return;
    }

#ifdef unix
int strcmpi (const char *string1, const char *string2)
    {
    int i, rc = 0 ;

    for (i = 0 ; string1[i] != '\0' || string2[i] != '\0' ; i++)
        if (rc = comparech (string1[i], string2[i]))
            break ;

    return (rc) ;
    }

int comparech (int ch1, int ch2)
    {
    int lowerch1, lowerch2, rc ;

    lowerch1 = tolower (ch1) ;
    lowerch2 = tolower (ch2) ;

    if (lowerch1 < lowerch2)
        rc = -1 ;
    else if (lowerch1 > lowerch2)
        rc = 1 ;
    else
        rc = 0 ;

    return (rc) ;
    }

char *itoa (int value, char *string, int radix)
    {
    radix ;

    sprintf (string, "%d", value) ;
    return (string) ;
    }

char *ltoa (long value, char *string, int radix)
    {
    radix ;

    sprintf (string, "%ld", value) ;
    return (string) ;
    }

char *strupr (char *string)
    {
    char *tempstring ;

    tempstring = string ;

    while (*tempstring)
        *tempstring++ = (char) toupper (*tempstring) ;

    return (string) ;
    }
#endif









