/***********************************************************************
*    File - vbxload.c
*
*    Description - Initialize and download VMOS to VBX boards based
*      on ascii config file "./vbx.cfg"
*
*    Functions -
*
*  $$History
*   1 VBXLOAD.C 22-Feb-91,16:46:20,`GS' Placed under revision control
*   2 VBXLOAD.C 6-Mar-91,15:39:24,`GS' vbx.. was ln..
*   3 VBXLOAD.C 8-Apr-91,16:00:28,`GS' Don't quit on INIT error, decode errno
*   4 VBXLOAD.C 9-Apr-91,12:37:14,`GS' Added comments; CFG_UNIT was CFG_INIT.
*   5 VBXLOAD.C 30-Jan-92,12:13:00,`GS' Added error code.
*   6 VBXLOAD.C 29-Jun-92,10:23:52,`RP' '//' comments changed to ANSI C
*        comments.
*   7 VBXLOAD.C 16-Jun-94,16:09:02,`MEL' (UNIX) removed compiler warnings
*  $$History
*
*    Copyright (c) 1990  Natural MicroSystems Inc.
***********************************************************************/
/* $$Revision "&(#)%n r%v" */
static char Revision[] = "&(#)VBXLOAD.C r7";


#include    <fcntl.h>
#include    <stdio.h>
#include    "vbxdrv.h"
#include    <stdlib.h>
#include    <errno.h>
#include    <string.h>
#include    <ctype.h>

#ifdef unix
  #include <unistd.h>
#endif


int dh;

int read_config();

char *text[] = {

"No initialization attempted          ", /*  0   VBX_CFG_NO_INIT            */
"Successfully initialized VBX port    ", /*  1   VBX_CFG_SUCCESS            */
"Not found (no response)              ", /*  2   VBX_CFG_NO_RESPONSE        */
"Not found (invalid response to reset)", /*  3   VBX_CFG_BAD_RESPONSE       */
"Failure to download probe routine    ", /*  4   VBX_CFG_PROBE_DWNLD_FAILED */
"Failure to probe VBX port config     ", /*  5   VBX_CFG_PROBE_RSP_FAILED   */
"Failure to reboot VBX port           ", /*  6   VBX_CFG_REBOOT_FAILED      */
"Failure to download VMOS             ", /*  7   VBX_CFG_VMOS_DWNLD_FAILED  */
"Wrong response to VBX OS reset       ", /*  8   VBX_CFG_BAD_VMOS_RESP      */
"Timeout waiting for interrupt        ", /*  9   VBX_CFG_TMO_RESET          */
"Timeout on VBX/100 ID transfer       ", /* 10   VBX_CFG_TMO_IDXFER         */
"No Response to VBX OS reset          "  /* 11   VBX_CFG_NO_VMOS_RESP       */
               } ;

/*
       SAMPLE INPUT FILE FORMAT

# Base device  Number of ports Base address  Vmos         Comments
# ----------- ---------------- ------------  ----         -----------
   0                1             168                  #  vbx00  (no vmos)
   1                4             340      vmos4.bin   #  vbx01-05

*/

/* Databases - these are constructed by interpreting config file */

/* Device table - one entry for device */
#define UNIT_TABLE_SIZE 64
struct {
    short unit ;
    short baseio ;
    int vmos ;
    } Unit_table [UNIT_TABLE_SIZE] ;
int Max_unit = 0 ;

/* Vmos table - one entry for each different VMOS required */
#define VMOS_TABLE_SIZE 4
struct {
    char name[120];
    char *buf;
    short  size ;
    } Vmos_table[VMOS_TABLE_SIZE] = {0};
int Max_vmos = 0;


/*----------------------------------------------------------------*/
main()
    {
    char dn[40];
    PARMBLOCK rp;
    struct vbx_init_status status ;
    char *status_text ;
    char *ioctl_err_text ;
    char tmpbuf[120] ;
    int i ;
    short unit ;
    int vmos ;

    read_config () ;     /* Read config file */
#ifdef DEBUG_CONFIG
    for (i=0; i<Max_unit; i++)
        printf ("/dev/vbx%-3.2d  0x%4.4x   %s\n",
              Unit_table [i].unit, Unit_table [i].baseio,
              Unit_table [i].vmos == -1 ? "(none)" :
                Vmos_table [ Unit_table [i].vmos ]. name) ;
#else
    strcpy (dn, "/dev/vbxconfig") ;
    if ((dh=open(dn ,O_RDONLY)) == -1)   /* Open the pseudo-device */
        {
        fprintf(stderr, "Can't open %s.\n",dn);
        exit(0);
        }

    printf ("Device       address  boardid  status\n");
    printf ("------       -------  -------  ------------------------------------------\n");


    for (i = 0; i< Max_unit; i++)
        {

        unit = Unit_table[i].unit ;
        vmos = Unit_table[i].vmos ;

        rp.hdr.nparm = 2;              /* 2 wd parms for CONFIG */

        if (vmos == -1)
            {
            rp.hdr.dir = 0;                /* No data (no vmos) */
            rp.hdr.frmsz = 0;
            rp.hdr.nfrms = 0;
            rp.pbdbuf = NULL;
            }
        else
            {
            char *p;
            int fh ;
            if (Vmos_table[vmos].buf == NULL)
                {
                p = malloc(Vmos_table[vmos].size) ;
                if (p == NULL)
                    {
                    fprintf (stderr, "malloc error, size=%d\n",
                             Vmos_table[vmos].size) ;
                    exit (1) ;
                    }
                Vmos_table[vmos].buf = p ;
                if ((fh=open(Vmos_table[vmos].name ,O_RDONLY)) == -1)   /* Open the device */
                    {
                    fprintf(stderr, "Can't open %s.\n",Vmos_table[vmos].name);
                    Vmos_table[vmos].size = 0 ;
                    }
                else
                    {
                    Vmos_table[vmos].size = (short)
                      read (fh, Vmos_table[vmos].buf, Vmos_table[vmos].size);
                   /* printf ("read %d bytes\n", Vmos_table[vmos].size); */
                    }
                }
            if (Vmos_table[vmos].size == 0 )
                 continue ;

            rp.pbdbuf = Vmos_table[vmos].buf ;
            rp.hdr.dir = 1;                            /* Data down (vmos) */
            rp.hdr.frmsz = (unsigned short)(Vmos_table[vmos].size /2) ;  /* words            */
            rp.hdr.nfrms = 1;
            }

        rp.parm[0] = unit;
        rp.parm[1] = Unit_table[i].baseio ;

        ioctl_err_text = NULL ;
        if ( ioctl(dh, CFG_UNIT, &rp) == -1 )
            switch (errno)
            {
            case EBUSY:
                ioctl_err_text = "ERROR Device is busy" ;
                break ;
            case ENXIO:
                ioctl_err_text = "ERROR Device number is out of range." ;
                break ;
            case ENOMEM:
                ioctl_err_text = "ERROR: Insufficient memory." ;
                break ;
            default:
                fprintf(stderr, "Error issuing ioctl(CFG_UNIT); errno = %d.\n",
                        errno);
                close(dh);
                exit(1);
            }
        else
            {
            rp.hdr.nparm = 1 ;                  /* 1 parm = unit # */
            rp.hdr.dir = 2;                     /* Data UP */
            rp.hdr.frmsz = sizeof status / 2 ;
            rp.hdr.nfrms = 1;
            rp.pbdbuf = &status ;

            if ( ioctl(dh, CFG_GETSTAT, &rp) == -1 )
                {
                fprintf(stderr,"Error issuing ioctl(CFG_GETSTAT); errno = %d.\n",
                        errno);
                close(dh);
                exit(0);
                }
            }
        if ( ioctl_err_text != NULL  )
            status_text = ioctl_err_text ;
        else if (0 <= status.status
                   && status.status < sizeof text/sizeof text[0])
            status_text = text[status.status] ;
        else
            sprintf (tmpbuf, "Unknown status: %d", status.status) ;

        printf ("/dev/vbx%-3.2d  0x%4.4x   0x%4.4x   %s\n",
              unit, status.baseio, status.boardid, status_text) ;
        }
#endif
    return 0;
    }

/*----------------------------------------------------------------*/
int read_config()
    {
    FILE *fp ;
    char *fname ;
    char buf [120] ;
    int field ;
    char *p, *q, *p1 ;
    char token [40] ;
    int dev, dev_num, num_ports, baseio;
    char *vmos_name ;
    int i ;
    char *error_text ;
    int line ;

    fname = "vbx.cfg" ;
    if ((fp=fopen(fname ,"r")) == NULL)
        {
        fprintf(stderr, "Can't open %s\n",fname);
        exit(1);
        }
    line = 0 ;
    while (fgets(buf, sizeof buf, fp) != NULL)
        {

/* Parse input.  Each line consists of 3 mandatory fields and a fourth  */
/* optional field.  Fields are white-space separated. The fields are:   */
/*  device_number  number_of_ports  IO_address  vmos_file_name.         */
/* Any text following a '#' is ignored.  Lines containing no fields are */
/* ignored.                                                             */

        ++ line ;
        field = 0 ;
        p = buf;
        while (field <= 3 && *p != '\0' && *p != '#')
            {
            /* skip leading white space */
            while (isspace (*p)) ++p ;
            /* Copy chars up to next white or newline to token */
            q = token ;
            p1 = p ;
            while (! isspace(*p) && *p != '#' && *p != '\0')
                *q++ = *p++ ;
            if (q != token)
                {
                *q = '\0' ;
                if (field == 0)
                    {
                    dev_num = (int)strtol (token, &q, 10) ;
                    if (*q != '\0')
                        {
                        error_text = "Syntax_error" ;
                        p = p1 + (q-token) ;  /* to show error position */
                        goto error ;
                        }
                    }
                else if (field == 1)
                    {
                    num_ports = (int)strtol (token, &q, 10) ;
                    if (*q != '\0')
                        {
                        error_text = "Syntax_error" ;
                        p = p1 + (q-token) ;  /* to show error position */
                        goto error ;
                        }
                    }
                else if (field == 2)
                    {
                    baseio = (int)strtol (token, &q, 16) ;
                    if (*q != '\0')
                        {
                        error_text = "Syntax_error" ;
                        p = p1 + (q-token) ;  /* to show error position */
                        goto error ;
                        }
                    if ((baseio % 8) != 0)
                        {
                        p = p1 ;   /* to show error position */
                        error_text = "I/O address must be a multiple of 8.";
                        goto error ;
                        }
                    }
                else if (field == 3)
                    vmos_name = token ;

                field += 1 ;
                }
            } /* end while */

        if (field == 0)
            continue ;
        if (field < 3)
            {
            error_text = "Incomplete line";
            goto error ;
            }
        if (field == 3)
            vmos_name = "" ;

        /* Add an entry for each port of the device to the data table. */

        for (dev=dev_num; dev< dev_num+num_ports; dev++)
            {
            if (dev > MAXVBX_UNIT)
                {
                sprintf (error_text, "Unit %d must be less than %d",
                  dev, MAXVBX_UNIT+1);
                goto error ;
                }
            if (Max_unit == UNIT_TABLE_SIZE-1)
                {
                error_text = "Too many units!";
                goto error ;
                }
            Unit_table [Max_unit]. unit = (short)dev ;
            Unit_table [Max_unit]. baseio = (short)baseio ;
            baseio +=  8 ;

            if (strlen(vmos_name) != 0)
                {
                /* Find/add vmos name to vmos table */
                for (i=0;i<Max_vmos;i++)
                    {
                    if (strcmp(Vmos_table[i].name, vmos_name) == 0)
                        break ;
                    }
                if (i==Max_vmos && ++Max_vmos == VMOS_TABLE_SIZE)
                    {
                    error_text = "Too many versions of VMOS!";
                    goto error ;
                    }
                strcpy (Vmos_table[i].name, vmos_name) ;
                Vmos_table[i].size = 4096 ;   /* Default VMOS size */
                Vmos_table[i].buf  = NULL ;
                Unit_table [Max_unit]. vmos = i ;
                }
            else
                Unit_table [Max_unit]. vmos = -1 ;
            ++ Max_unit ;
            } /* end for */
        }/* end while fgets() != NULL */
    fclose (fp) ;
    return 0;

error:
    fprintf (stderr, "%s(%d) : %s\n", fname, line, error_text);
    fprintf (stderr, buf);
    fprintf (stderr, "%*s\n", (p-buf)+1, "^");
    fclose (fp) ;
    exit (1) ;
    }
