/****************************************************************************
FILE:     voxcopy.c
****!!!! THIS MODULE FOR CUSTOMER USE !!!!****
AUTHOR:   Michael Hoffman
CREATED:  880118

UPDATES:
    890601 MTH  Added better shave algorithm based on WEB's Watson shave
                program.
   $$History
*   1 VOXCOPY.C 22-Sep-89,9:51:54,`GS' Placed under revision control
*   2 VOXCOPY.C 26-Mar-90,11:49:22,`DAR' Added function prototypes
*   3 VOXCOPY.C 1-May-90,8:50:12,`GS' Added 'stripper' lines.
*   4 VOXCOPY.C 21-Sep-93,9:53:34,`GS' Added display of dos error
*   5 VOXCOPY.C 5-Jul-94,14:57:52,`MEL' merged UNIX and OS/2 versions
   $$History
****!!!! EXCISE ABOVE FOR CUSTOMER USE !!!!****

DESCRIPTION:

This program will copy one vox files contents to another with
the extra ability of shaving off silence from the front and back
of each message.

Copyright (c) 1987-89 Natural Microsystems Corporation.  All rights reserved.
****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#ifdef unix
#include <ctype.h>
#endif

#ifdef unix
#define far
#define pascal
#endif

#include "me2def.h"

/******************************************************************************/
/*   Function Prototypes                                                      */
/*   -------------------                                                      */
int  main( int, char ** );
int  getegy( register int, long );
void clregycache( void );
void doexit( int );

/******************************************************************************/

#define  MAXIDX     "VOICE.MAX_INDEX"
#define  COMPRS     "VOICE.COMPRESSION"

#define FRMAVERANGE   20L       /* number for frames to to average at a time*/
#define MINOVERFRMS   10L       /* minimum # of frms in range which must be */
                                /* over to qualify as non silence threshold */
#define BACKUPFRMS    6L        /* once boundary is found, back away a few  */

#define  EGYDFLT     25         /* if changed also change help text below   */
#define  MAXCACHE    30

typedef struct EGYCACHE {
    long frm;
    int  egy;
    } EGYCACHE;

static ME2INT vsrc = 0;
static ME2INT vdst = 0;

static EGYCACHE EgyCache[MAXCACHE];
static int      ECLast = 0;

main( ac, av )
    int ac;
    char *av[];
    {
    int EgyMax = 0;
    int OvrWrt = 0;
    int IdxCnt = 0;
    int MsgNum = 0;
    int DstMsg = 0;
    int MaxIdx = 0;
    ME2INT i,beg,max,msg,egy,rate,frmsover;
    char buff[80];
    char *pc;
    long l,f,begfrm,endfrm,lastfrm,maxfrm;
    /* Note - 'short' type used to hold the values of 16 bit ME2 parameters
     * passed by reference to Me2ParmByName and Me2ParmByNum */
    short m;     /* 910801 */

    puts(
"VBX Voice File Copy Utility   Version 1.03" );
    puts(
"Copyright (c) 1988-93 Natural MicroSystems Corp.  All Rights Reserved.\n" );

    if ( ac < 3 )
        {
        puts(
"Format:  voxcopy srcfile[.vox] destfile[.vox] [opts]\n" );
        puts(
"   /S[nn]    - Shave silence off with maximum average energy nn (0 to 72).");
        puts(
"               Default is energy level of 25." );
        puts(
"   /X        - Overwrite existing destination file." );
        puts(
"               Default is to merge into existing file, if any." );
        puts(
"   /Inn      - Allocate nn indices in destination file, (48 to 6400)." );
        puts(
"               Otherwise, VOICE.MAX_INDEX value or exact number needed," );
        puts(
"               whichever is greater, will be used." );
        puts(
"   /Mss[,dd] - Copy only message number ss to destination file as message dd.");
        puts(
"               If dd is not specified, ss will be used as destination message.");

        return( 1 );
        }

    for ( i = 3; i < ac; i++ )      /* Parse options */
        {
/*      strncpy( temp, av[i], 80 );
        pc = temp;      */
        pc = av[i];

        if ( *pc == '/' || *pc == '-' || *pc == '\\' )
            {
            pc++;
            switch( toupper(*pc) )
                {
            case 'X':                       /* Overwrite any existing file */
                OvrWrt = 1;
                break;

            case 'S':                       /* Shave silence */
                EgyMax = atoi( pc+1 );
                if ( EgyMax <= 1 )  EgyMax = EGYDFLT;
                break;

            case 'I':                       /* Number of indices to allocate */
                IdxCnt = atoi( pc+1 );
                break;

            case 'M':                       /* Explicit message number       */
                MsgNum = atoi( pc+1 );
                if ( (pc=strrchr(pc, ',')) )
                    DstMsg = atoi( pc+1 );

                break;

            default:
                printf( "Option %s not recognized.", pc );
                doexit(1);
                }
            }
        else
            {
            printf( "Argument %s not recognized.", pc );
            doexit(1);
            }
        }

    if ( !DstMsg )  DstMsg = MsgNum;

    i = ME2REQUESTCHANNEL( 11, (char far *)"VBX Voice Copy" );
    if ( i < SUCCESS )
        {
        printf( "Could not request a VBX Channel.  Error: %d\n", i );
        doexit(1);
        }

    i = ME2VOXOPEN( av[1], &vsrc, PLAYONLY );    /* Open source */
    if ( i < SUCCESS )
        {
        printf( "Could not find or open source voice file.  Error: %d (%d)\n",
           i, Me2GetError() );
        doexit(1);
        }

    /* find out how many real messages there are and make sure that the
       destination file gets created with enough indices.
    */

    ME2VOXHIGHMSG( vsrc, &max );               /* go until the end */

    if ( DstMsg )          /* If only 1 message to copy, we only need 1 index */
        MaxIdx = 1;
    else                                   /* Count number of messages active */
        for( MaxIdx=0,m=1; m <= (short) max; m++ )
            if ( ME2VOXSETCURMSG(vsrc, m ) == SUCCESS )  MaxIdx++;

    if ( IdxCnt )
        m = (short) IdxCnt;            /* If # of indices given, use it */
    else
        ME2PARMBYNAME( (char far *)MAXIDX, (void far *)&m, GETPARM );

                                       /* If not enough, use our calculated # */
    if ( m < (short) MaxIdx )   m = (short) MaxIdx;
    ME2PARMBYNAME( (char far *)MAXIDX, (void far *)&m, SETPARM);

                                      /* set compression for subsequent opens */
    ME2VOXGETRATE( vsrc, &rate );
    ME2PARMBYNAME( (char far *)COMPRS, (void far *)&rate, SETPARM);

    if ( OvrWrt )                     /* If overwrite mode, unlink file       */
        {
        strcpy( buff, av[2] );
        if ( !strrchr( buff, '.' ) )
            strcat( buff, ".VOX" );

        unlink( buff );
        }

    i = ME2VOXOPEN( av[2], &vdst, PLAYREC );
    if ( i < SUCCESS )
        {
        printf( "Could not open destination file.  Error: %d (%d)\n", i, Me2GetError() );
        doexit(1);
        }

    ME2VOXGETRATE( vdst, &i );
    if ( i != rate )
        {
        printf( "Destination voice file has different compression rate." );
        doexit(1);
        }


    if ( MsgNum )
        beg = max = MsgNum;                /* copy just the specified message */
    else
        beg = 1;                                      /* start at beginning   */

    for( msg = beg; msg <= max; msg++ )
        {
        printf( "Message %03d ", msg );

        if ( DstMsg )
            m = (short) DstMsg; /* if single destination, use it      */
        else
            m = (short) msg;    /* else, use same message # in dest.  */

        if ( ME2VOXSETCURMSG( vsrc, msg ) == SUCCESS )
            {
            if ( EgyMax )           /* if trimming, do all the extra work */
                {
                clregycache();      /* mark cache as empty                */

                /* shave beginning of message */

                ME2VOXSETCURMSG( vsrc, msg );
                ME2VOXSEEKFRM( vsrc, 0L, VOX_SEEK_END, &maxfrm );

                lastfrm = maxfrm - FRMAVERANGE;
                begfrm  = 0L;

                /*
                  Find first frame where 'FRMAVERANGE' frames in a row have
                  less than 'MINOVERFRMS' frames over the threshold energy.
                  Once this position is found, back up 'BACKUPFRMS' frames
                  to be sure at least a few of the lower energy frames are
                  included to ensure a clean startup of the message when
                  played back.
                */

                do {
                    egy = getegy( vsrc, begfrm );
                    if ( egy >= EgyMax )
                        {
                        frmsover = 1;
                        for ( f = 1; f < FRMAVERANGE; f++ )
                            if ( getegy(vsrc,begfrm+f) >= EgyMax )  frmsover++;
                        }
                    begfrm++;                       /* go to next frame */

                    } while( (egy      < EgyMax       ||
                              frmsover < MINOVERFRMS) &&
                              begfrm   < lastfrm );

                begfrm -= BACKUPFRMS;               /* back up a few frames  */
                if ( begfrm < 0L )  begfrm = 0L;


                /* shave end of message (Same algorithm as above in reverse) */

                lastfrm = MINOVERFRMS;
                endfrm  = --maxfrm;

                do {
                    egy = getegy( vsrc, endfrm );
                    if ( egy >= EgyMax )
                        {
                        frmsover = 1;
                        for ( f = 1; f < FRMAVERANGE; f++ )
                            if ( getegy(vsrc,endfrm-f) >= EgyMax )  frmsover++;
                        }
                    endfrm--;

                    } while( (egy      < EgyMax       ||
                              frmsover < MINOVERFRMS) &&
                              endfrm   > lastfrm );

                endfrm += BACKUPFRMS;
                if ( endfrm > maxfrm )  endfrm = maxfrm;

                if ( endfrm <= begfrm )
                    {
                    begfrm = 0L;
                    endfrm = 1L;
                    }

                ME2VOXERASEMSG( vdst, m );  /* Delete old dest msg (if any) */
                ME2VOXSETCURMSG( vdst, m ); /* Then copy new shaved message */
                ME2VOXSEEKFRM( vsrc, begfrm, VOX_SEEK_BEG, &l );

                i = ME2VOXCOPYFRM( vdst, vsrc, endfrm-begfrm+1L, &l );
                if ( i < SUCCESS )
                    printf( "Copy error: %d  ", i );
                else
                    printf(
"Beg:%03ld/End:%03ld frames trimmed and %4ld frames copied.  ",
                              begfrm, maxfrm - endfrm, endfrm-begfrm+1L );
                }
            else
                {
                i = ME2VOXCOPYMSG( vdst, m, vsrc, msg );     /* Straight copy */
                if ( i < SUCCESS )
                    printf( "Copy error: %d", i );
                else
                    printf( "Copied." );
                }
            }
        else
            {
            printf( "Not found." );
            }

        putchar ( '\n' );
        }

    doexit(0);
    }



int getegy( register int vh, long frm )
    {
    long l;
    ME2INT egy;
    register int i;

    /*
       A cache of frame energies is maintained so shave algorithm only really
       goes to disk once for each frame's energy, even though it uses it
       multiple times.
    */

    for( i=0; i< MAXCACHE; i++ )        /* 1st see if in cache */
        {
        if ( frm == EgyCache[i].frm )  return( EgyCache[i].egy );
        }

    if ( ME2VOXSEEKFRM( vh, frm, VOX_SEEK_BEG, &l) < SUCCESS )  return(0);
    if ( ME2VOXFRMENERGY( vh, &egy )               < SUCCESS )  return(0);

    EgyCache[ECLast].egy = egy;
    EgyCache[ECLast].frm = frm;
    if ( ++ECLast >= MAXCACHE )  ECLast = 0;

    return( egy );
    }




void clregycache()
    {
    register int i;

    for( i=0; i < MAXCACHE; i++ )  EgyCache[i].frm = -1L;
    ECLast = 0;
    }





void doexit(i)
    int i;
    {
    if ( vsrc > 0 )  ME2VOXCLOSE( vsrc );
    if ( vdst > 0 )  ME2VOXCLOSE( vdst );
    ME2RELEASECHANNEL();
    exit(i);
    }
