/*****************************************************************************
 FILE:        voxplay.c
 DESCRIPTION: This utility will play all messages in one or more VBX
              Voice files.

****!!!! This module for customer use; to be stripped.
   $$History
*   1 VOXPLAY.C 4-Aug-89,15:02:30,`GS' Placed under revision control
*   2 VOXPLAY.C 26-Mar-90,8:32:48,`DAR' Continuing support for VBX/4
*   3 VOXPLAY.C 3-Apr-90,13:04:24,`DAR' Updated version label
*   4 VOXPLAY.C 12-Apr-90,17:05:28,`GS' Flush TT Q before play.
*        Attempt to load VUTILS.PRO
*        Made help more verbose.  Put connection stuff in a subroutine.
*   5 VOXPLAY.C 25-May-93,9:50:04,`JP' v.2.00 to work with VBX100-I
*   6 VOXPLAY.C 18-Jun-93,16:31:26,`JP' v.2.03 debug with VBX100-I,local phone
*   7 VOXPLAY.C 9-Nov-93,10:08:22,`GS' CPRNGQT->CPRINGEND. Rmv Me2VoxSetCurMsg
*   8 VOXPLAY.C 9-Nov-93,10:46:26,`GS' Update version; remove extra SETPARMs
*   9 VOXPLAY.C 10-Nov-93,15:13:34,`GS' Detect/display play failed
*   10 VOXPLAY.C 15-Jun-94,10:53:00,`JP' V.2.05 test/integ. with 1.7 and on AG.
*   11 VOXPLAY.C 30-Jun-94,15:23:34,`MEL' merged UNIX and OS/2 versions
*   12 VOXPLAY.C 30-Jun-94,16:04:20,`JP' Removed sleep before waitevent.
*        Note: DOS version of DosSleep in api.lib under OS/2 2.0 unpredictable.
*   13 VOXPLAY.C 11-Jul-94,20:22:38,`MEL' removed compiler warnings (UNIX)
*   14 VOXPLAY.C 27-Jul-94,7:45:34,`JP' Fixed keyboard '*'
*   15 VOXPLAY.C 12-Aug-94,9:35:48,`MEL' DOSMODE is defined in termio.h so
*        only check DOSMODE if 'unix' is not defined
   $$History
****!!!! End of stripping for customer use.

 Copyright (c) 1989-94 Natural Microsystems Corporation.  All rights reserved.
*****************************************************************************/

#ifdef unix
#define far
#define pascal
#elif defined( OS2MODE )
#define INCL_BASE
#define INCL_DOS
#define INCL_NOPM
#include <os2.h>
#elif defined( DOSMODE )
#include <dos.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef unix
#include <unistd.h>
#include <termio.h>
#else
#include <conio.h>
#endif

#include <me2.h>
#include <me2def.h>
#include <me2err.h>
#include <voxutl.h>

#ifdef unix
 /* because DOSMODE is defined in termio.h */
#elif defined( OS2MODE )
 static FILEFINDBUF ffb;
#elif defined( DOSMODE )
 static struct find_t ffb;
#endif

static char fvox[80];
static char fdir[80];
static ME2INT  msg;
static ME2INT  highmsg;
static ME2INT  vsel = -1;

void main( int ac, char *av[] );
int  waitevent( ME2INT ev );
void doexit( int xcode );
void getline( void );
int  callspv( void );
void lnreqerr( int error );

#ifdef unix
extern int rdchk(int);
int getch (void) ;
int kbhit (void) ;
void resetkbd (void) ;
void setkbd (void) ;

static struct termio oldtty ;
#endif


/*****************************************************************************
 Module:      main
 Description:
*****************************************************************************/
void main( int ac, char *av[] )
    {
    int delta, rtn;
    int filecount = 0;
    ME2INT i;
    ME2INT cat;
    char buff[255];
    int e;
#ifdef OS2MODE
    int dirh;
#endif
#ifndef unix
    ME2INT state;
    unsigned short cnt;
    char *pc,c;
#endif

#ifdef unix
    setkbd () ;
#endif

    puts( "NMS Voice File Player   Version 2.09" );
    puts( "Copyright (c) 1989-94, Natural MicroSystems Corp.  All Rights Reserved." );

    if ( ac < 2 )
        {
        puts( "\nFormat:  voxplay file[.vox]  (file name may contain * and ? wildcards)" );
        puts( "" );
        puts( " While playing, the following Touch Tones are recognized:" );
        puts( "   0  First message" );
        puts( "   1  Previous message" );
        puts( "   2  Repeat current message" );
        puts( "   3  Skip to next message" );
        puts( "   *  Skip to next voice file" );
        puts( "   #  Exit VOXPLAY" );

        if( ( cat = Me2ParmCategoryNum( "VUTILS" ) ) >= 0 )
            {
            Me2ParmLoad( "VUTILS" );
            puts( "\nCurrent parameter settings for audio connection are:\n" );
            if( Me2ParmByNum( cat, 0, buff, GETPARM ) >= SUCCESS )
                {
                printf( "    Connection Type: %d ", buff[0] );
                switch( buff[0] )
                    {
                    case CALLOUT:
                        printf( "(Dial out)" );
                        if( Me2ParmByNum( cat, 1, buff, GETPARM ) >= SUCCESS )
                             printf( "   Number to dial: %s\n", buff );
                        break;
                    case LPHONE:
                        printf( "(Use Local phone)\n" );
                        break ;
                    case CALLIN:
                        printf( "(Call in)\n" );
                        break ;
                    default:
                        printf( "INVALID- Must be 1, 2, or 3\n" );
                    }
                }

            if( Me2ParmByNum( cat, 2, buff, GETPARM ) >= SUCCESS )
                {
                printf( "    Line number:     %d", buff[0] );
                if( buff[0] == ANYLINE ) printf( " (Any line)" );
                printf( "\n" );
                }
            }
        doexit( 0 );
        }

    if ( ( i = Me2RequestChannel( 12, "VBX Voice Play" ) ) < SUCCESS )
        {
        printf( "Could not allocate an ME/2 Channel, ME/2 error: %d\n", i );
        doexit( 0 );
        }

    strcpy( fvox, av[1] );            /*xx- parse file info from command line-xx*/

#ifndef unix
    if( ( c = fvox[strlen(fvox)-1] ) == '\\' || c == ':' )
        strcat( fvox, "*.vox" );
    else if( ( pc = strchr( fvox, '.' ) ) == NULL )
        strcat( fvox, ".vox" );

    strcpy( fdir, av[1] );            /*xx- create directory specification-xx*/
    pc = fdir+strlen(fdir);
    while( pc > fdir && *pc != ':' && *pc != '\\' )
        pc--;
    if( pc > fdir )
        ++pc;
    *pc = '\0';

    cnt = 1;
#if defined( OS2MODE )
    dirh = 0xffff;
    i = DosFindFirst( fvox, (PHFILE)&dirh, 0,
                      &ffb, (USHORT)sizeof(ffb), &cnt, 0L );
#elif defined( DOSMODE )
    i = _dos_findfirst( fvox, 0, &ffb );
#endif
    if ( i )
        {
        printf( "File or directory not found\n");
        doexit( 1 );
        }

#endif

    getline();

    while( kbhit() )    /*xx- clear keyboard buffer-xx*/
        getchar();

    puts( "");
    puts( "  Touch-Tone/Keyboard Commands:                                              ");
    puts( "       0: 1st   1: Prev   2: Repeat   3: Next   *: Next file   #/ESC: Exit   ");
    puts( "");
    puts( "\t\t\tPress any key to start play" );

    if( getch() == ESC )
        doexit( 0 );

    do  {                           /*xx- process each speech file in this loop-xx*/

#ifdef unix
#elif defined( OS2MODE )
        strcpy( fvox, fdir );
        strcat( fvox, ffb.achName );
#elif defined( DOSMODE )
        strcpy( fvox, fdir );
        strcat( fvox, ffb.name );
#endif

        printf( "\nProcessing %s... ", fvox );

        if( (rtn=Me2VoxOpen( fvox, &vsel, PLAYONLY )) >= SUCCESS )
            {
            delta = 1;
            msg   = 1;
            Me2VoxHighMsg( vsel, &highmsg );

            while( msg <= highmsg )
                {
                while( Me2GetTTC( (char*)&e, QREMOVE ) == SUCCESS )
                    ;          /*xx- flush the touch tone queue-xx*/

                if((rtn= Me2VoxPlayMsg( vsel, msg )) >= SUCCESS )
                    {
                    printf( "\nMessage #%d", msg );
                    delta = waitevent( EXITCODE );
                    }
                else
                    {
                    if (rtn == ME2_VOX_MSG_NOT_FOUND)
                        delta = 1;
                    else
                        {
                        printf ("Me2VoxPlayMsg(%d): %d\n", msg, rtn);
                        doexit ( 1 );
                        }
                    }
                msg += delta;
                if ( msg <= 0 )
                    {
                    msg = 1;
                    delta = 1;
                    }
                }
            if ((rtn=Me2VoxClose( vsel )) < 0)
                printf ("Me2VoxClose: %d\n", rtn);
            }
        else
            printf ("Me2VoxOpen(%s) failed.  ME/2 error: %d (OS error %d)\n",
                     fvox, rtn, Me2GetError() );

        filecount++;
#ifdef unix
#elif defined( OS2MODE )
        i = DosFindNext( dirh, &ffb, sizeof(ffb), &cnt );
#elif defined( DOSMODE )
        i = _dos_findnext( &ffb );
#endif

#ifdef unix
        i = 1;
#else
        Me2TelState( &state, 0 );
        if ( !state & TELLOCLOFF )
            i = 1;
#endif
        } while( i == 0 );

    printf("\nDone.  %d file%s processed\n", filecount, filecount==1?"":"s" );
#if defined( OS2MODE )
    DosFindClose( dirh );
#endif /* not unix or DOS */

    doexit( 0 );
    }


/*****************************************************************************
 Module:      waitevent()
 Description: Main event handler
 Returns:     Number of messages to advance by
*****************************************************************************/
int waitevent( ME2INT expected )
    {
    ME2INT event;
    int ret = 0;

    for( ;; )
        {
        if( kbhit() )                /*xx- first check for any keystrokes-xx*/
            {
            event = (ME2INT)getch();
            if( ( event == CR ) || ( event == ESC ) || ( event == '#' ) )
                event = TTPOUND;     /*xx- translate exit request to DTMF terms-xx*/
            else if( event == '*' )
                event = TTSTAR;
            Me2SendEvent( Me2GetChannel(), event );
            Me2Stop();               /*xx- stop recording-xx*/
            }

        if( Me2WaitEvent( 250L ) >= SUCCESS )
            Me2GetEvent( &event, QREMOVE );
        else
            event = 0;                  /*xx- for polling in DOS-xx*/

        switch( event )
            {
            case TT1:       ret = -1;     break;
            case PLAYDONE:  ret =  1;     break;
            case TT3:       ret =  1;     break;
            case TT0:       ret = -9999;  break;
            case PLAYFAIL:
                printf( "  Play failed.\n" );
                /* FALL THROUGH */
            case TTSTAR:    ret =  9999;  break;

            case LINRQFAIL:
                lnreqerr( ret );
                /* FALL THROUGH */
            case LCLONHK:
            case TTPOUND:   doexit( 0 );
            }

        if( ( event == expected ) ||
            ( expected == EXITCODE &&
              (unsigned)event >= EXITCODE && (unsigned)event <= EXITCMAX ) )
            return( ret );
        }
    }


/*****************************************************************************
 Module:      doexit()
 Description: This function closes everything and exits the program
*****************************************************************************/
void doexit( int xcode )
    {
    Me2Stop();
    Me2ReleaseLine();
    Me2ReleaseChannel();
    Me2VoxClose( vsel );

#ifdef unix
    resetkbd ();
#endif

    exit( xcode );
    }


/*****************************************************************************
 Function:    getline()
 Description: Establish talk path to telephone
*****************************************************************************/
void getline( void )
    {
    /* Note - 'short' type used to hold the values of 16 bit ME2 parameters
     * passed by reference to Me2ParmByName and Me2ParmByNum */
    short maxrings;
    short conntype;
#ifndef unix
    short groupnum;
#endif
    short linenum;
    int   ret;
    int   boardid;
    char  dialstr[32];

    Me2ParmLoad( "vutils" );
                                     /*xx- Get connection type-xx*/
    if( Me2ParmByName( "vutils.vconn_line", &linenum, GETPARM )
        < SUCCESS )
        {                           /*xx- If there is an error reading vutils-xx*/
#ifndef unix
        groupnum = GRP0;            /*xx-  info, default = group 0-xx*/
#endif
        linenum  = ANYLINE;         /*xx- Any line from that group-xx*/
        conntype = CALLOUT;         /*xx- call out-xx*/
        }
    else
        {                           /*xx- Get chosen connection type.-xx*/
        if( Me2ParmByName( "vutils.vconn_type", (char *)&conntype, GETPARM )
           < SUCCESS )
            conntype = CALLOUT;     /*xx- Default = call out-xx*/

#ifndef unix
        if( linenum == ANYLINE )
            groupnum = GRP0;        /*xx- If line = -1, get any avail line-xx*/
#endif
        }

    if( linenum != ANYLINE )
        {
        if( ( ret = Me2GrabLine( linenum ) ) < SUCCESS )
            {
            lnreqerr( ret );
            doexit( 1 );
            }
        boardid = Me2GetBoardId();
        }
#ifdef unix
    else
        {
        lnreqerr( ME2_BAD_LINE );
        doexit( 1 );
        }
#else
    else if( conntype == CALLIN )
        {
        if( ( ret = Me2RequestLine( groupnum, LINERING, 15000L ) ) < SUCCESS )
            {
            lnreqerr( ret );
            doexit( 1 );
            }
        }
    else
        {
        if( ( ret = Me2RequestLine( groupnum, LINEFREE, 5000L ) ) != SUCCESS )
            {
            lnreqerr( ret );
            doexit( 1 );
            }
        else
            {
            waitevent( EXITCODE ) ;
            boardid = Me2GetBoardId();
            }
        }
#endif

    switch( conntype )
        {
        case CALLOUT:               /*xx- Get max rings before no answer-xx*/
            if( ( Me2ParmByName( "vutils.vmax_rings", &maxrings,
                                 GETPARM ) ) == SUCCESS )
                 {
                 Me2ParmByName( "call_progress.max_ring",
                                &maxrings, SETPARM );
                 }

            if( ( ( ret = Me2ParmByName( "vutils.vconn_num", dialstr,
                                GETPARM ) ) < SUCCESS ) || ( !dialstr[0] ) )
                {
                if( IS_VBX100( boardid ) )       /*xx- If it's VBX/100, default-xx*/
                    {                            /*xx-   to local phone instead.-xx*/
                    conntype  = LPHONE;
                    Me2SetAttachMode( DETACH );  /*xx- Tell ME2 using local phone-xx*/
                    Me2SetMuteMode( MUTED );     /*xx- Make sure no dialtone-xx*/
                    Me2SetLineMode( HANGUP );
                    break;
                    }
                else                          /*xx- Prompt for number to dial if-xx*/
                    {                         /*xx- none specified in .pro/.prm-xx*/
                    printf( "\nEnter phone number to dial:  " );
                    if( !gets( dialstr ) )  /*xx- Quit if nothing entered.-xx*/
                        {
                        puts( "Error interpreting dialstring" );
                        doexit( 1 );
                        }
                    else if (ret == SUCCESS)  /*xx- Save the number for next time.-xx*/
                        {
                        Me2ParmByName( "vutils.vconn_num",
                                       (char *)dialstr, SETPARM );
                        Me2ParmSave( "VUTILS.PRM" );
                        }
                   printf("\n");
                   }
                }
                                              /*xx- Make the connection-xx*/
            if ( ( ret = Me2Call( dialstr ) ) < SUCCESS )
                {
                printf( "Error dialing number.  ME/2 error: %d\n", ret );
                doexit( 1 );
                }
            else
                puts( "Please wait while dialing... " );

            if( callspv() < SUCCESS )    /*xx- Make sure remote end answered.-xx*/
                {
                puts( "Please try again." );
                doexit( 1 );
                }
            break;

        case LPHONE:
            if( IS_VBX100( boardid ) )   /*xx- Mute if local request and VBX/100-xx*/
                 {
                 Me2SetAttachMode( DETACH );
                 Me2SetMuteMode( MUTED );
                 Me2SetLineMode( HANGUP );
                 }
            break;

        case CALLIN:
            puts( "Waiting for call. Press ESC to abort" );
            if( linenum != ANYLINE )
                waitevent( RINGBEG );
            else
                waitevent( LINRQDONE );
            boardid = Me2GetBoardId();
            Me2SetLineMode( PICKUP );
            Me2SetMuteMode( NOTMUTED );
            break;

         default:
             puts( "Unsupported configuration specified in VUTILS.prm/.pro file");
             doexit( 1 );
        }
    return;
    }


/*****************************************************************************
 Module:      callspv()
 Description: This function handles call progress if remote connection
              request is specified.
*****************************************************************************/
int callspv( void )
    {
    ME2INT event;
    int done = 0;

    for(;;)
        {
        Me2WaitEvent( 250L );
        if( Me2GetEvent( &event, QREMOVE ) != SUCCESS )
            continue;

        switch( event )
            {
            case CALLSTOP:  puts( "Error: Call Stopped" );
                            done = 1;  break;

            case CALLFAIL:  puts( "Error: Call Failed" );
                            done = 1;  break;

            case CPRINGEND: puts( "Error: Call Progress Ring Quit" );
                            done = 1;  break;

            case CPTIMEOUT: puts( "Error: Call Progress TimeOut" );
                            done = 1;  break;

            case CPFASTBSY: puts( "Fast Busy at dialed number" );
                            done = 1;  break;

            case CPBUSY:    puts( "Busy Signal at dialed number" );
                            done = 1;  break;

            case CPNOANSWR: puts( "No Answer at dialed number" );
                            done = 1;  break;

            case CPSITTONE: puts( "Special Info Tone at dialed number" );
                            done = 1;  break;

            default:
                if( (unsigned)event >= EXITCODE &&
                    (unsigned)event <= EXITCMAX )
                    {
                    puts( "Call Completed" );
                    return( SUCCESS );
                    }
                else
                    {
                    continue;
                    }
            }

        if( done )
            {
            if( Me2Stop() >= SUCCESS )
                waitevent( EXITCODE );
            return( FAILURE );
            }
        }
    }


/*****************************************************************************
 Module:      lnreqerr
 Description: Display specific error message indicating the reason
              the line request failed.
*****************************************************************************/
void lnreqerr( int error )
    {
    printf( "Error requesting a line, error %d, additional error %d\n",
                error, Me2GetLineError() );
    return;
    }

#ifdef unix
int getch (void)
    {
    return (getchar ()) ;
    }

int kbhit (void)
    {
    int rc ;

    if (rdchk (1) > 0)
        rc = 1 ;
    else
        rc = 0 ;

    return (rc) ;
    }

void resetkbd (void)
    {
    ioctl (0, TCSETAF, &oldtty) ; /* Reset keyboard to default parameters */
    }

void setkbd (void)
    {
    struct termio tty ;

    ioctl (0, TCGETA, &tty) ;     /* Get parameters associated with terminal */
    oldtty = tty ;                /* Duplicate to reset later                */

    tty.c_lflag     &= ~ICANON ;  /* Set ICANON off                          */
    tty.c_lflag     &= ~ECHO ;    /* Set ECHO off                            */
    tty.c_cc[VMIN]  =  (char) 1 ; /* Set VMIN to 1                           */
    tty.c_cc[VTIME] =  (char) 0 ; /* Set VTIME to 0                          */

    ioctl (0, TCSETAF, &tty) ;    /* Set new parameters associated with tty  */
    }
#endif

