/*****************************************************************************
 FILE:        voxrec.c
 DESCRIPTION: This program allows voice messages to be recorded one after
              another in order to facilitate the building of a voice prompts
              library.

****!!!! This module for customer use; to be stripped.
   $$History
*   1 VOXREC.C 18-Sep-89,11:15:20,`GS' Placed under revision control
*   2 VOXREC.C 29-Mar-90,9:30:52,`DAR' Removed DosSleep for touchtone bug
*        (Bug now fixed in ME2)
*   3 VOXREC.C 3-Apr-90,13:02:58,`DAR' Updated version label
*   4 VOXREC.C 9-Apr-90,20:40:30,`GS' Flush TT Q before record.
*        Made help more descriptive of current parms.
*        Attempts to load a VUTILS.PRO
*   5 VOXREC.C 12-Apr-90,16:50:58,`GS' Don't delete file if error
*        require filename as arg 1 - no default to PROMPT_TEXT.LANGUAGE
*        Use ME2FINDILE so can delete file if not in current dir
*        Wait for end-tone before going on to next.
*        Put connection stuff in a subroutine.  Put help stuff in one also.
*        Report error if RECFAIL.
*   6 VOXREC.C 25-May-93,9:42:06,`JP' v.2.0 support VBX100-I
*   7 VOXREC.C 18-Jun-93,16:33:36,`JP' v.2.03 debug with VBX100-I,local phone
*   8 VOXREC.C 9-Nov-93,10:59:16,`GS' CPRNGQT->CPRINGEND. Filter **PARM ERR**
*   9 VOXREC.C 2-Dec-93,8:44:08,`GS' removed #include me2drv.h
*   10 VOXREC.C 15-Jun-94,10:55:02,`JP' V.2.05 test/integ. with 1.7 and on AG.
*   11 VOXREC.C 30-Jun-94,15:24:10,`MEL' merged UNIX and OS/2 versions
*   12 VOXREC.C 11-Jul-94,20:23:08,`MEL' removed compiler warnings (UNIX)
   $$History
****!!!! End of stripping for customer use.

 Copyright (c) 1989-93 Natural MicroSystems Corporation.  All rights reserved.
*****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#ifdef unix
#include <unistd.h>
#include <termio.h>
#else
#include <conio.h>
#endif

#ifdef unix
#define far
#define pascal
#endif

#include "me2.h"
#include "me2err.h"
#include "me2def.h"
#include "voxutl.h"

void help( void );
void main( int ac, char *av[] );
ME2INT  recordmsg( ME2INT vsel, ME2INT msg, ME2INT time );
ME2INT  waitevent( ME2INT expected );
void getline( void );
int  callspv( void );
void lnreqerr( int error );
void showerr( char *msg, int err_val );

static ME2INT  vsel = -1;

#ifdef unix
extern int rdchk(int);
int getch (void) ;
int kbhit (void) ;
void resetkbd (void) ;
void setkbd (void) ;

static struct termio oldtty ;
#endif

/*****************************************************************************
 Module:      help()
 Description:
*****************************************************************************/
void help( void )
    {
    ME2INT cat;
    char buff[255];

    puts( "\nFormat:   VOXREC name [options]" );
    puts( " where:" );
    puts( "    name      - records into name.VOX voice message file, and loads" );
    puts( "                the optional name.PRO message text file.\n" );
    puts( "    options   - may include the following, separated by spaces:\n");
    puts( "    /X        - Overwrite existing destination file." );
    puts( "                Default is to merge into existing file, if any." );
    puts( "    /Inn      - Allocate nn indices in destination file, (48 to 6400)." );
    puts( "                Otherwise, VOICE.MAX_INDEX value or exact number needed," );
    puts( "                whichever is greater, will be used." );
    puts( "    /Mbb[,ee] - Record messages beginning at number bb and ending with ee." );
    puts( "                If ee is not specified, use ESC key or Touch-Tone '#' to quit.");
    puts( "    /Rnnn     - Maximum nnn seconds per message. (Default: unlimited)." );

    if( ( cat = Me2ParmCategoryNum( "VUTILS" ) ) >= SUCCESS )
        {
        Me2ParmLoad( "VUTILS" );
        puts( "\nCurrent parameter settings for audio connection are:\n" );
        if( Me2ParmByNum( cat, 0, buff, GETPARM ) >= SUCCESS )
            {
            printf( "    Connection Type: %d ", buff[0] );
            switch( buff[0] )
                {
                case CALLOUT:
                    printf( "(Dial out)" );
                    if( Me2ParmByNum( cat, 1, buff, GETPARM ) >= SUCCESS )
                         printf( "   Number to dial: %s\n", buff );
                    break;
                case LPHONE:
                    printf( "(Use Local phone)\n" );
                    break ;
                case CALLIN:
                    printf( "(Call in)\n" );
                    break ;
                default:
                    printf( "INVALID- Must be 1, 2 or 3\n" );
                    break;
                }
            }
        if( Me2ParmByNum( cat, 2, buff, GETPARM ) >= SUCCESS )
            {
            printf( "    Line number:     %d", buff[0] );
            if( buff[0] == ANYLINE ) printf( " (Any line)" );
            printf( "\n" );
            }
        }

#ifdef unix
         resetkbd () ;
#endif

    exit( 0 );
    }


/*****************************************************************************
 Module:      main()
 Description:
*****************************************************************************/
void main( int ac, char *av[] )
    {
    char *fname ;
    ME2INT Cat    = -1;
    /* Note - 'short' type used to hold the values of 16 bit ME2 parameters
     * passed by reference to Me2ParmByName and Me2ParmByNum */
    short IdxCnt = 0;
    int OvrWrt = 0;
    int MsgBeg = 1;
    int PrmFnd = 0;
    int MsgEnd = 9999;
    int Time   = 9999;
    ME2INT i,m;
    char *pc,c;
    char fullnm [80];
    char rootnm [20];
    int  e;

#ifdef unix
    setkbd () ;
#endif

    puts( "VBX Voice Library Record Utility.  Version 2.05" );
    puts( "Copyright (c) 1989-94 Natural MicroSystems Corporation." );

    if( ac == 1 )
        {
        help();
        }

    if( ( c=*(av[1]) ) != '/' && c != '-' )
        {
        fname = av[1]  ;
        }
    else
        {
/*xx-      Me2ParmByname( "PROMPT_TEXT.LANGUAGE", (char *)buff, GETPARM );-xx*/
        help();
        }

    for ( i = 2; i < ac; i++ )
        {
        pc = av[i];
        if ( *pc == '/' || *pc == '-' || *pc == '\\' )
            {
            pc++;
            switch( toupper(*pc) )
                {
                case 'M':
                    MsgBeg = atoi( pc+1 );
                    if( (pc=strrchr(pc, ',')) )
                        MsgEnd = atoi( pc+1 );
                    break;

                case 'X':
                    OvrWrt = 1;
                    break;

                case 'I':
                    IdxCnt = (short)atoi( pc+1 );
                    break;

                case 'R':
                    Time   = atoi( pc+1 );
                    break;

                default:
                    printf( "Option %s not recognized.", pc );
                    doexit( 1 );
                }
            }
        else
            {
            help();
            }
        }

    /*xx- Get full path name of file, and also get just the basic 'name.ext'-xx*/
    if ( Me2FindFile( fname, "VOX", ME2LOCAL, rootnm,
                      fullnm, sizeof fullnm ) == SUCCESS )
        {
        if( OvrWrt )
            {
            remove( fullnm );  /*xx- Delete file-xx*/
            }
        }

    Cat = Me2ParmCategoryNum( "PROMPT_TEXT" );

    if( Cat >= SUCCESS )
        {
        /*xx- Get filename only (no extension) for PARMLOAD.-xx*/
        if( ( pc = strrchr(rootnm, '.' ) ) != NULL )
           *pc = '\0';

        Me2ParmLoad( rootnm );
        }

    if( IdxCnt )
        Me2ParmByName( "VOICE.MAX_INDEX", (void *)&IdxCnt, SETPARM );

    printf ("Opening file: %s\n", fullnm);
    i = Me2VoxOpen( fullnm, &vsel, PLAYREC );
    if( i < SUCCESS )
         {
         showerr("Couldn't find or open specified voice file.", i );
         doexit( 1 );
         }

    if( ( i = Me2RequestChannel( 6, "VoxRec" ) ) < SUCCESS )
         {
         showerr( "Error requesting a channel.", i );
         doexit( 1 );
         }

    getline();

    while( kbhit() )
        getchar();  /*xx- Clear keyboard buffer-xx*/

    puts( "");
    puts( "  Touch-Tone/Keyboard Commands:                                              ");
    puts( "       1: Re-record   2: Next   #/ESC: Quit.                                 " );
    puts( "");
    puts( "\t\t\tPress any key to start record" );

    if( getch() == ESC )
        doexit( 0 );

    /* Flush the touch tone queue */
    while( Me2GetTTC( (char*)&e, QREMOVE ) == SUCCESS )
        ;

    Me2Beep( 1 );
    waitevent( EXITCODE );

    for( m = MsgBeg; m <= MsgEnd; m++ ) /*xx- Loop to record all messages-xx*/
        {
        if( Cat >= 0 )
            {
            char *msg = Me2ParmMsg( Cat, m ) ;
            if (strcmp (msg, "**PARM ERR**") == 0)
                msg = "" ;
            printf( "Message #%03d: %s\n", m, msg );
            }
        else
            printf( "Message #%03d now\n", m );

        i = recordmsg( vsel, m, Time );
        switch( i )
            {
            case TTPOUND:              /*xx- Process normal exit request-xx*/
                doexit( 0 );
                break;

            case TT1:                  /*xx- Process re-record request-xx*/
                m--;
                break;

            case RECFAIL:
                printf("MESSAGE ENDED: Ran out of indices, or file write failed\n");
                break;
            }
        }
    doexit( 0 );
    }


/*****************************************************************************
 Module:      recordmsg()
 Description: This function is responsible for recording the individual
              messages and processing any events that occur while the
              recording is in process.
*****************************************************************************/
ME2INT recordmsg( ME2INT vsel, ME2INT msg, ME2INT time )
    {
    ME2INT event;
    ME2INT ret = 0;

    if( ( ret = Me2VoxRecordMsg( vsel, msg, time ) ) < SUCCESS )
        {
        printf( "Error while recording: %d\n", ret );
        doexit( 1 );
        }

    for( ;; )
        {
        if( kbhit() )         /*xx- First check for any keystrokes-xx*/
            {
            event = (ME2INT)getch();
            if( ( event == CR ) || ( event == ESC ) )
                 ret = TTPOUND;    /*xx- Translate exit request to DTMF terms-xx*/
            Me2SendEvent( Me2GetChannel(), event );
            Me2Stop();        /*xx- Stop recording-xx*/
            }

        Me2WaitEvent( 250L );
        if( Me2GetEvent( &event, QREMOVE ) == SUCCESS )
            {
            if( TTSTAR <= event && event <= TT9 )
                 {
                 if( ret == 0 )
                     ret = event;
                 /*xx- Wait for end of tone-xx*/
                 Me2GetTTC( (char*)&event, QREMOVE );
                 }
            else if (EXITCODE <= (unsigned)event && (unsigned)event <= EXITCMAX)
                 {
                 if( event == RECFAIL )
                     if( ret == 0 )
                         ret = event ;
                 return( ret );
                 }
            }
        }
    }


/*****************************************************************************
 Module:      waitevent()
 Description: Main event handler
*****************************************************************************/
ME2INT waitevent( ME2INT expected )
    {
    ME2INT event;

    for( ;; )
         {
         Me2WaitEvent( 250L );
         if ( Me2GetEvent( &event, QREMOVE ) >= SUCCESS )
              {
              if ( event == expected )
                   return( SUCCESS );

              if( ( expected == EXITCODE ) &&
                    ( (unsigned)event >= EXITCODE &&
                      (unsigned)event <= EXITCMAX ) )
                   {
                   if( event >= FAILCODE && event <= FAILCMAX )
                        return( FAILURE );
                   else
                        return( SUCCESS );
                   }
              }
         }
    }


/*****************************************************************************
 Module:      doexit()
 Description: This function closes everything and exits the program
*****************************************************************************/
void doexit( int xcode )
    {
    Me2Stop();
    Me2ReleaseLine();
    Me2ReleaseChannel();
    Me2VoxClose( vsel );
#ifdef unix
    resetkbd () ;
#endif
    exit( xcode );
    }


/*****************************************************************************
 Function:    getline()
 Description: Establish talk path to telephone
*****************************************************************************/
void getline( void )
    {
    /* Note - 'short' type used to hold the values of 16 bit ME2 parameters
     * passed by reference to Me2ParmByName and Me2ParmByNum */
    short maxrings;
    short conntype;
#ifndef unix
    short groupnum;
#endif
    short linenum;
    ME2INT   ret;
    ME2INT   boardid;
    char  dialstr[32];

    Me2ParmLoad( "vutils" );
                                     /*xx- Get connection type-xx*/
    if( Me2ParmByName( "vutils.vconn_line", &linenum, GETPARM )
        < SUCCESS )
        {                           /*xx- If there is an error reading vutils-xx*/
#ifndef unix
        groupnum = GRP0;            /*xx-  info, default = group 0-xx*/
#endif
        linenum  = ANYLINE;         /*xx- Any line from that group-xx*/
        conntype = CALLOUT;         /*xx- call out-xx*/
        }
    else
        {                           /*xx- Get chosen connection type.-xx*/
        if( Me2ParmByName( "vutils.vconn_type", &conntype, GETPARM )
           < SUCCESS )
            conntype = CALLOUT;     /*xx- Default = call out-xx*/

#ifndef unix
        if( linenum == ANYLINE )
            groupnum = GRP0;        /*xx- If line = -1, get any avail line-xx*/
#endif
        }

    if( linenum != ANYLINE )
        {
        if( ( ret = Me2GrabLine( linenum ) ) < SUCCESS )
            {
            lnreqerr( ret );
            doexit( 1 );
            }
        boardid = Me2GetBoardId();
        }
#ifdef unix
    else
        {
        lnreqerr( ME2_BAD_LINE );
        doexit( 1 );
        }
#else
    else if( conntype == CALLIN )
        {
        if( ( ret = Me2RequestLine( groupnum, LINERING, 15000L ) ) < SUCCESS )
            {
            lnreqerr( ret );
            doexit( 1 );
            }
        }
    else
        {
        if( ( ret = Me2RequestLine( groupnum, LINEFREE, 5000L ) ) != SUCCESS )
            {
            lnreqerr( ret );
            doexit( 1 );
            }
        else
            {
            waitevent( EXITCODE ) ;
            boardid = Me2GetBoardId();
            }
        }
#endif

    switch( conntype )
        {
        case CALLOUT:               /*xx- Get max rings before no answer-xx*/
            if( ( Me2ParmByName( "vutils.vmax_rings", &maxrings,
                                 GETPARM ) ) == SUCCESS )
                 {
                 Me2ParmByName( "call_progress.max_ring",
                                &maxrings, SETPARM );
                 }

            if( ( ( ret = Me2ParmByName( "vutils.vconn_num", dialstr,
                                GETPARM ) ) < SUCCESS ) || ( !dialstr[0] ) )
                {
                if( IS_VBX100( boardid ) )       /*xx- If it's VBX/100, default-xx*/
                    {                            /*xx-   to local phone instead.-xx*/
                    conntype  = LPHONE;
                    Me2SetAttachMode( DETACH );  /*xx- Tell ME2 using local phone-xx*/
                    Me2SetMuteMode( MUTED );     /*xx- Make sure no dialtone-xx*/
                    Me2SetLineMode( HANGUP );
                    break;
                    }
                else                          /*xx- Prompt for number to dial if-xx*/
                    {                         /*xx- none specified in .pro/.prm-xx*/
                    printf( "\nEnter phone number to dial:  " );
                    if( !gets( dialstr ) )  /*xx- Quit if nothing entered.-xx*/
                        {
                        puts( "Error interpreting dialstring" );
                        doexit( 1 );
                        }
                    else if (ret == SUCCESS)  /*xx- Save the number for next time.-xx*/
                        {
                        Me2ParmByName( "vutils.vconn_num",
                                       (char *)dialstr, SETPARM );
                        Me2ParmSave( "VUTILS.PRM" );
                        }
                   printf("\n");
                   }
                }
                                              /*xx- Make the connection-xx*/
            if ( ( ret = Me2Call( dialstr ) ) < SUCCESS )
                {
                printf( "Error dialing number.  ME/2 error: %d\n", ret );
                doexit( 1 );
                }
            else
                puts( "Please wait while dialing... " );

            if( callspv() < SUCCESS )    /*xx- Make sure remote end answered.-xx*/
                {
                puts( "Please try again." );
                doexit( 1 );
                }
            break;

        case LPHONE:
            if( IS_VBX100( boardid ) )   /*xx- Mute if local request and VBX/100-xx*/
                 {
                 Me2SetAttachMode( DETACH );
                 Me2SetMuteMode( MUTED );
                 Me2SetLineMode( HANGUP );
                 }
            break;

        case CALLIN:
            puts( "Waiting for call. Press ESC to abort" );
            if( linenum != ANYLINE )
                waitevent( RINGBEG );
            else
                waitevent( LINRQDONE );
            boardid = Me2GetBoardId();
            Me2SetLineMode( PICKUP );
            Me2SetMuteMode( NOTMUTED );
            break;

         default:
             puts( "Unsupported configuration specified in VUTILS.prm/.pro file");
             doexit( 1 );
        }
    return;
    }

/*****************************************************************************
 Module:      callspv()
 Description: This function handles call progress if remote connection
              request is specified.
*****************************************************************************/
int callspv( void )
    {
    ME2INT event;
    int done = 0;

    for(;;)
        {
        Me2WaitEvent( 250L );
        if( Me2GetEvent( &event, QREMOVE ) != SUCCESS )
            continue;

        switch( event )
            {
            case CALLSTOP:  puts( "Error: Call Stopped" );
                            done = 1;  break;

            case CALLFAIL:  puts( "Error: Call Failed" );
                            done = 1;  break;

            case CPRINGEND: puts( "Error: Call Progress Ring Quit" );
                            done = 1;  break;

            case CPTIMEOUT: puts( "Error: Call Progress TimeOut" );
                            done = 1;  break;

            case CPFASTBSY: puts( "Fast Busy at dialed number" );
                            done = 1;  break;

            case CPBUSY:    puts( "Busy Signal at dialed number" );
                            done = 1;  break;

            case CPNOANSWR: puts( "No Answer at dialed number" );
                            done = 1;  break;

            case CPSITTONE: puts( "Special Info Tone at dialed number" );
                            done = 1;  break;

            default:
                if( (unsigned)event >= EXITCODE &&
                    (unsigned)event <= EXITCMAX )
                    {
                    puts( "Call Completed" );
                    return( SUCCESS );
                    }
                else
                    {
                    continue;
                    }
            }

        if( done )
            {
            if( Me2Stop() >= SUCCESS )
                waitevent( EXITCODE );
            return( FAILURE );
            }
        }
    }


/*****************************************************************************
 Module:      lnreqerr
 Description: Display specific error message indicating the reason
              the line request failed.
*****************************************************************************/
void lnreqerr( ME2INT error )
    {
    printf( "Error requesting a line, error %d, additional error %d\n",
                error, Me2GetLineError() );
    return;
    }


/*****************************************************************************
 Module:      showerr
 Description: Display error message and accompanying error return code.
*****************************************************************************/
void showerr( char *msg, int err_val )
    {
    printf( "%s  ME/2 error: %d\n", msg, err_val );
    return;
    }

#ifdef unix
int getch (void)
    {
    return (getchar ()) ;
    }

int kbhit (void)
    {
    int rc ;

    if (rdchk (1) > 0)
        rc = 1 ;
    else
        rc = 0 ;

    return (rc) ;
    }

void resetkbd (void)
    {
    ioctl (0, TCSETAF, &oldtty) ; /* Reset keyboard to default parameters */
    }

void setkbd (void)
    {
    struct termio tty ;

    ioctl (0, TCGETA, &tty) ;     /* Get parameters associated with terminal */
    oldtty = tty ;                /* Duplicate to reset later                */

    tty.c_lflag     &= ~ICANON ;  /* Set ICANON off                          */
    tty.c_lflag     &= ~ECHO ;    /* Set ECHO off                            */
    tty.c_cc[VMIN]  =  (char) 1 ; /* Set VMIN to 1                           */
    tty.c_cc[VTIME] =  (char) 0 ; /* Set VTIME to 0                          */

    ioctl (0, TCSETAF, &tty) ;    /* Set new parameters associated with tty  */
    }
#endif


