/***********************************************************************
*    File - ogt.c
*
*    Description - Example OutGoing Trunk application
*
*     This example obtains a trunk, dials a number, waits for answer,
*     and uses call progress detection to determine the disposition of
*     the call.  It then delays for a few seconds, hangs up and repeats.
*     (You must Control-C to exit).
*
*     This code is applicable to both T1 and E&M ports, but you must set
*     the parameter TRUNK.SIGNAL_TYPE appropriately.  See the comments
*     in the code regarding possible settings.
*
*
***********************************************************************/
#ifndef unix
#define  INCL_DOSPROCESS
#define  INCL_NOPM
#include <os2.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef unix
#include <unistd.h>
#endif

#include "mixed.h"
#include "me2def.h"
#include "me2err.h"

#define REMOTE_ANSWER LPCURON
#define REMOTE_HANGUP LPCUROFF

/*xx- LOCAL FUNCTIONS-xx*/

void ogt_setparms();
void ogt_getport();
ME2INT ogt_dial();
void ogt_callprogress();
void ogt_hangup();
void setparm();
ME2INT waitdone() ;
ME2INT waittrunkevent() ;
void showtrunkevent();
void  ogt_exit() ;

/*xx- GLOBALS-xx*/

ME2INT Answered = 0;        /*xx- set in waitdone-xx*/
ME2INT Hungup   = 0;        /*xx- set in waitdone-xx*/
ME2INT LastCallprogevent = 0;   /*xx- set in waitdone-xx*/
ME2INT Voice = 0;           /*xx- set in waitdone-xx*/
#ifdef unix
ME2INT Trunk;
#endif

short trunk_type = 255;

/*xx----------------------------------------------------------------------------xx*/
/*xx- MAIN-xx*/

void main(argc, argv)
  int argc;
  char **argv;
    {
    ME2INT i ;
    char *digits ;

#ifdef unix
    if (argc < 3 || argc > 4)
        {
        puts ("usage: ogt {VBX port #} {dial string} [{trunk_type}]");
        exit (0) ;
        }
    else
        {
        Trunk = atoi (argv[1]) ;
        digits = argv[2] ;
        if (argc == 4)
          trunk_type = (short) atoi (argv[3]);
        else trunk_type = 255;
        }
#else
    /*xx- Expect one argument, the number to dial-xx*/
    if (argc < 2)
        {
        puts ("usage: OGT <number to dial> [/S<type>]");
        ogt_exit (0);
        }
    else
        digits = argv[1] ;

    if (argc > 3 || argc == 3 && (  (argv[2][0] != '/' && argv[2][0] != '-')
                                  ||(argv[2][1] != 's' && argv[2][1] != 'S')))
        {
        printf ("Invalid argument.  Usage: OGT <number to dial> [/S<type>]\n" );
        exit (1);
        }
    if (argc == 3)
        trunk_type = atoi (&argv[2][2]) ;
    else
        trunk_type = 255 ;
#endif

    /*xx- Get ME/2 channel-xx*/
    ME2REQUESTCHANNEL(99,"OGT");

    /*xx- Set ME2 parameters-xx*/
    ogt_setparms();

    /*xx- Get a port-xx*/
    ogt_getport();

    /*xx-  Start the Trunk Control Program at its initial entry point.-xx*/
    /*xx-  This is an idle state, where it is ready for the DIAL command.-xx*/

    if ((i=Me2TrunkStart(0)) != SUCCESS)
        {
        printf ("Me2TrunkStart returned error %d\n", i) ;
        ogt_exit (1) ;
        }
    printf("Trunk Control Program started\n");

    /*xx- Enter a loop to make calls-xx*/
    while (1)
        {
        /*xx- Make an outgoing call-xx*/
        if (ogt_dial(digits) == 0)
            {
            /*xx- Start Call progress to look for busy tone, SI Tone, etc.-xx*/
            ogt_callprogress();

    /*xx-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;-xx*/
    /*xx-   Called party interaction would go here      ;-xx*/
    /*xx-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;-xx*/


            /*xx- Wait 3 seconds, then hangup-xx*/
            ME2DELAY (300) ;
            waitdone() ;    /*xx- wait for delay to complete-xx*/
            }
        /*xx- Hangup, restarting the Trunk Manager-xx*/
        ogt_hangup ();

        }
    /*xx- while(1)-xx*/
    }
/*xx- main-xx*/


/*******************************************************************
* ogt_setparm - set ME2 parameters
*
* For this example, parameters are set by the program.  The same result
* could be obtained by adding the settings to ME2.PRO, or to a user profile
* that is loaded with Me2ParmLoad.  Note that setting of trunk-related
* parameters must be done before getting a port with Me2RequestLine.
********************************************************************/
void ogt_setparms()
    {
    /* Note - 'short' type used to hold the values of 16 bit ME2 parameters
     * passed by reference to Me2ParmByName and Me2ParmByNum */
    short val ;
    short cpflags[6] ;
    char  *trunk_type_text ;
    ME2INT i ;

    /*xx- Set the name of the Trunk Control overlay and associated parameter-xx*/
    /*xx- category.-xx*/
    setparm ("TRUNK.overlay", "TCPOGT" ) ;

    /*xx- Set the parameter that defines hardware type, unless already set-xx*/
    /*xx- in ME2.PRO.-xx*/

    /*xx- If hardware type selected via command line argument, set the ME2-xx*/
    /*xx- parameter, otherwise verify that parameter is already set (presumably-xx*/
    /*xx- by an entry in ME2.PRO.)-xx*/

    if (trunk_type != 255)
        {
        i=ME2PARMBYNAME("TRUNK.SIGNAL_TYPE", &trunk_type, SETPARM) ;
        if (i != SUCCESS)
            {
            printf("Error %d setting parameter TRUNK.Signal_Type\n", i) ;
            exit(1);
            }
        }
    else
        {
        i=ME2PARMBYNAME("TRUNK.SIGNAL_TYPE", &trunk_type, GETPARM) ;
        if (i != SUCCESS)
            {
            printf("Error %d getting parameter TRUNK.Signal_Type\n", i) ;
            exit(1);
            }
        }

    if (trunk_type == 255)
        {
        puts ("\nYou must specify trunk type either via an ME/2 parameter:");
        puts ("\n         TRUNK.SIGNAL_TYPE = <type>");
        puts ("\nor with a command line argument:");
#ifdef unix
        puts ("           ogt {VBX port #} {dial string} [{trunk_type}]");
        puts ("\n where {trunk_type} is one of");
#else
        puts ("\n         OGT <number to dial> /S<type>");
        puts ("\n where <type> is one of");
#endif
        puts ("     1 - VBX 400  or AG-8 DID");
        puts ("     2 - VBX 400  or AG-8 E&M");
        puts ("     3 - VBX 1200 or AG Digital (connected to T1 via MVIP)");
        exit (1) ;
        }

    switch (trunk_type)
        {
        case 1: trunk_type_text = "DID" ; break ;
        case 2: trunk_type_text = "E&M" ; break ;
        case 3: trunk_type_text = "T1" ; break ;
        default: trunk_type_text = "???" ; break ;
        }
    printf ("Trunk type is %d (%s)\n", trunk_type, trunk_type_text);

    /*xx- Here we modify default OGT paramters settings.-xx*/

    /*******
           FYI: OGT PARAMETER DEFAULTS:

       TCPOGT.WAIT_FOR_WINK = 1000    (10 sec) time to wait for wink (0=no wait)
       TCPOGT.MINIMUM_WINK  = 10      (100 ms) Minimum duration of the wink
       TCPOGT.MAXIMUM_WINK  = 490     (4.9 s) Maximum duration of the wink
       TCPOGT.PRE_DIAL_DLY  = 7       (70 ms) Delay before dialing, after wink
       TCPOGT.TONE_ON_TIME  = 8       (80 ms) Duration of DTMF tone on
       TCPOGT.TONE_OFF_TIME = 8       (80 ms) Duration of DTMF tone off
       TCPOGT.PULSE_MAKE    = 4       (40 ms) Dial pulse make
       TCPOGT.PULSE_BREAK   = 6       (60 ms) Dial pulse break
       TCPOGT.PULSE_IDT     = 70      (700 ms) Dial pulse inter-digit-time
       TCPOGT.TONE_LO       = 512     Amplitude of low DTMF frequencies
       TCPOGT.TONE_HI       = 640     Amplitude of high DTMF frequencies
       TCPOGT.PERMANENT_SIG = 2000    (20 sec) Timeout waiting for remote hangup
    *******/


    /*xx- Note: If any OGT parameters are to be changed from their defaults, this-xx*/
    /*xx- must be done before calling Me2RequestLine.-xx*/


    /*xx- EXAMPLE: Change default minimum wink time to 120 ms-xx*/

    val = 12 ;
    setparm ("TCPOGT.MINIMUM_WINK", &val) ;

    /*xx- Modify Call Progress parameters to allow detection of varying-xx*/
    /*xx-  length of voice-xx*/

    cpflags[0] = 0 ;   /*xx- no debug-xx*/
    cpflags[1] = 1 ;   /*xx- quit after CPVOCLNG-xx*/
    cpflags[2] = 0 ;   /*xx- don't quit after CPVOCMED-xx*/
    cpflags[3] = 0 ;   /*xx- don't quit after CPVOCBEG-xx*/
    cpflags[4] = 1 ;   /*xx- quit after CPRINGEND-xx*/
    cpflags[5] = 0 ;   /*xx- don't quit after CPRINGBEG-xx*/
    setparm  ("CALL_PROGRESS.FLAGS", cpflags) ;

    return ;
    }
/*xx- ogt_setparms-xx*/

/**********************************************************************
* ogt_getport - obtain a VBX port to dial out on
************************************************************************/
void ogt_getport()
    {
    ME2INT i ;
#ifndef unix
    ME2INT event ;
#endif

#ifdef unix
    if ( (i=ME2GRABTRUNKLINE( Trunk )) < SUCCESS )
        {
        printf ("Me2GrabTrunkLine returned error %d\n", i) ;
        ogt_exit (1) ;
        }
#else
    /*xx- The LINETRUNK argument causes an uninitialized port ("line")-xx*/
    /*xx- to be returned.-xx*/

    if ( (i=ME2REQUESTLINE( 0 /* group */, LINETRUNK, 0L )) < SUCCESS )
        {
        printf ("Me2RequestLine returned error %d\n", i) ;
        ogt_exit (1) ;
        }
    if ((event=waitdone()) == LINRQFAIL)
        {
        printf ("Line request failed. Me2GetLineError returned %d\n",
              ME2GETLINEERROR());
        ogt_exit(1);
        }
    else if (event != LINRQDONE)
        {
        printf ("Unexpected completion event from line request: %d (0x%x)\n",
                   event, event);
        ogt_exit(0);
        }
    else
        printf ("Line request succeeded.  Got VBX port # %d\n", ME2GETLINE());
#endif
    return ;
    }
/*xx- ogt_getport-xx*/

/************************************************************************
*  ogt_dial - Dial a call using the Trunk Manager
*    If the correct wink is not detected, a failure is returned so
*    the call can be re-attempted.
***********************************************************************/

ME2INT ogt_dial(digits)
  char *digits;
    {
    ME2INT i ;
    ME2INT event ;
    int dial_failed = FALSE ;

    if ((i=Me2TrunkDial (digits)) != SUCCESS)
        {
        printf ("Me2TrunkDial returned error %d\n", i) ;
        ogt_exit (1) ;
        }

    /*xx- Wait for dial completion - the trunk control overlay will exit-xx*/
    /*xx-  automatically-xx*/

    /*xx- The TRUNKDONE indicates that the trunk overlay (Trunk Control Program-xx*/
    /*xx- running on the DSP) has stopped and answer detection is now being done-xx*/
    /*xx- by the VMOS supervision detector.-xx*/

    while ((event = waittrunkevent()) != TRUNKDONE)
        {
        if (event == TRUNKFAIL)
            ogt_exit (1) ;         /*xx- internal protocol error-xx*/
        else if (event == TRUNKWINKTIMEOUT)
            {
            printf ("FAILED: Timed out waiting for wink\n");
            dial_failed = TRUE ;
            }
        else if (event == TRUNKSTUCKSENDER)
            {
            printf ("FAILED: Timed out waiting for end of wink\n");
            dial_failed = TRUE ;
            }
        }
    if (dial_failed)
        return -1 ;
    else
        return 0;
    }
/*xx- ogt_dial-xx*/

/************************************************************************
* ogt_callprogress - Run call progress detection to look for busy tone,
*                    SI Tone, etc.
* NOTES:
*   If the called party answers, there will be a change in the incoming
*   signaling on the trunk, which will cause a REMOTE_ANSWER event.  This
*   is independent of Call Progress Analysis.  A global "answered" flag
*   will be set
*   by the waitdone subroutine.
*
*   If voice is detected without such an answer, one can assume some sort
*   of recorded message has been encountered.
*
*   The last call progress event received will be stored in a global by
*   the waitdone subroutine.
*************************************************************************/
void ogt_callprogress()
    {
    ME2INT i ;
    ME2INT event ;

    if ((i=ME2CALLPROGRESS()) != SUCCESS)
        {
        printf ("Me2CallProgress returned error %d\n", i) ;
        ogt_exit (1) ;
        }
    printf ("Starting call progress analysis.\n");

    LastCallprogevent = Answered = Voice = Hungup = 0;
    if ((event=waitdone()) == CPROGSTOP )
        {
        printf ("Call progress detection stopped.\n");
        if (Hungup)
            if ( Answered )
                printf ("Called party hung up before voice detected. \n");
            else
                printf ("Unexpected hangup event during call progress\n");
        }

    else if (event != CPROGDONE)
        {
        if (event == CPROGFAIL)
            printf ("Call progress FAILED\n");
        else
            printf("Unexpected completion event from call progress:%d (0x%x)\n",
                   event, event);
        ogt_exit(1);
        }
    else
        {
        printf ("Call progress detection completed.\n");

        switch (LastCallprogevent)
            {
            case CPDIALTON : puts ("Dial tone detected!"); break ;
            case CPBUSY    : puts ("Busy tone detected");  break ;
            case CPFASTBSY : puts ("Fast busy tone detected");  break ;
            case CPRINGEND : puts ("Ring ended without voice");  break ;
            case CPNOANSWR : puts ("No answer - max rings detected");  break ;
            case CPSITTONE : puts ("Special Information tone detected"); break ;
            case CPVOCLNG  : puts ("Long voice detected") ;   break ;
            case CPVOCEND  : puts ("End of voice detected") ; break ;
            case CPTIMEOUT : puts ("Timed out with no call progress indication") ;
                             break ;
            default:
                printf ("Call progress terminated for unexpected reason: 0x%x\n",
                          LastCallprogevent );
                break ;
            }

        if (Voice > 0)
            {
            printf ("%s voice detected",
             Voice == 1 ? "Short" : Voice == 2 ? "Medium" : "Long") ;
            if ( !Answered)
                printf ("; no answer indication\n");
            else
                printf ("\n");
            }
        }
    return ;
    }
/*xx- ogt_callprogress-xx*/


/***********************************************************************
* ogt_hangup  - Hangup the call.
*
* Hangup by restarting the Trunk Control Program at entry point 1.
* There will be a hangup event from the overlay when it sees that
* the remote is on-hook.
************************************************************************/
void ogt_hangup ()
    {
    ME2INT i ;
    ME2INT event ;

    if ((i=Me2TrunkStart(1)) != SUCCESS)
        {
        printf ("Me2TrunkStart returned error %d\n", i) ;
        ogt_exit (1) ;
        }

    if (Answered && !Hungup)
        puts ("Waiting for remote to hang up");

    while ((event = waittrunkevent()) != TRUNKHANGUP)
        {
        if (event == TRUNKFAIL)
            ogt_exit (1) ;          /*xx- internal protocol error-xx*/
        }
    /*xx- Before returning to the top of the loop to make another call,-xx*/
    /*xx-  a delay is necessary so that a sufficiently long disconnect-xx*/
    /*xx-  interval is sent to the central office.  Note that Me2Delay-xx*/
    /*xx-  canNOT be used here because the Trunk Manager is active.-xx*/

#ifdef unix
    sleep (1) ;
#else
    DosSleep (1000L);
#endif
    return ;
    }
/*xx- ogt_hangup-xx*/


/*********************************************************************
* ogt_exit - cleanup and exit
*********************************************************************/
void  ogt_exit (stat)
  int stat;
    {
    ME2RELEASELINE();
    ME2RELEASECHANNEL();
    exit (stat) ;
    }


/*******************************************************************
* setparm - subroutine to call Me2ParmByName and test the return code.
******************************************************************/
void setparm(parm, val)
  char *parm;
  void *val;
    {
    ME2INT i=ME2PARMBYNAME(parm, val, SETPARM) ;
    if (i != SUCCESS)
        {
        if (i==ME2_CATEGORY_NOT_FOUND)
            printf("Category not found for %s\n", parm);
        else
            printf("Error %d setting parameter %s\n", i, parm) ;
        ogt_exit(1);
        }
    return ;
    }




/*******************************************************************
* waitdone - local subroutine to wait for completion of a background
*   function.  This routine looks for a hangup indication
*   and will stop the background function and return immediately if one
*   is detected.  It also records, but does not stop on, answer
*   indication and call progess events.
******************************************************************/
ME2INT waitdone()
    {
    ME2INT i;
    ME2INT ev;

    while (1)
        {
        i = ME2WAITEVENT( -1L );       /* Wait for an event to come through */
                                       /* the event queue.                  */
        if ( i == ME2_QUEUE_EMPTY)     /*xx- DOS case-xx*/
            continue ;

        if ( i != SUCCESS )
            {
            printf ("Me2WaitEvent returned error %d\n", i) ;
            ogt_exit (1) ;
            }

        if ((i=ME2GETEVENT(&ev, QREMOVE)) != SUCCESS)
            {
            printf ("Me2GetEvent returned error %d\n", i) ;
            ogt_exit (1) ;
            }
        printf ("(waitdone) Got event 0x%x\n", ev);
        if (ev == REMOTE_ANSWER)
            {
            printf("REMOTE ANSWER DETECTED\n") ;
            Answered = 1;
            continue ;
            }
        else if (ev == REMOTE_HANGUP)
            {
            printf("REMOTE HANGUP DETECTED\n") ;
            Hungup = 1;                       /*xx- If called party hangs up,-xx*/
            ME2STOP();                        /*xx- abort active function (if any)-xx*/
            }
        else if (CPDIALTON <= ev && ev <= CPTIMEOUT)
            {
            LastCallprogevent = ev ;
            switch (LastCallprogevent)
                {
                case CPVOCBEG  : Voice = 1; break ;
                case CPVOCMED  : Voice = 2; break ;
                case CPVOCLNG  : Voice = 3; break ;
                default:   break ;
                }
            }

        else if (EXITCODE <= ev && ev <= EXITCMAX ) /*xx- Check for done code.-xx*/
            return ev ;
        }
    }
/*xx- waitdone-xx*/


/*******************************************************************
* waittrunkevent - local subroutine to wait for an event from the
*   trunk handler thread.  This routine returns on events in the
*   TRUNKMIN - TRUNK MAX range and also on any exit code.
******************************************************************/
ME2INT waittrunkevent()
    {
    ME2INT i;
    ME2INT ev;

    while (1)
        {
        i = ME2WAITEVENT( 10000L );    /* Wait for an event to come through */
                                       /* the event queue.                  */
        if ( i == ME2_QUEUE_EMPTY)     /*xx- DOS case-xx*/
            continue ;

        if ( i == ME2_QUEUE_TIMEOUT)
            {
            puts ("still waiting for trunk event...");
            continue ;
            }

        if ( i != SUCCESS )
            {
            printf ("Me2WaitEvent returned error %d\n", i) ;
            ogt_exit (1) ;
            }

        if ((i=ME2GETEVENT(&ev, QREMOVE)) != SUCCESS)
            {
            printf ("Me2GetEvent returned error %d\n", i) ;
            ogt_exit (1) ;
            }

        else if (EXITCODE <= ev && ev <= EXITCMAX ) /*xx- Check for done code.-xx*/
            {
            if (ev == TRUNKFAIL)
                printf ("TRUNKFAIL received\n");
            else if (ev == TRUNKDONE)
                printf ("TRUNKDONE received\n");
            else if (ev == TRUNKSTOP)
                printf ("TRUNKSTOP received\n");

            return ev ;
            }
        else if (TRUNKCODEMIN <= ev && ev <= TRUNKCODEMAX ) /*xx- Check for trunk-xx*/
            {
            showtrunkevent(ev);
            return ev ;
            }
        else
            {
            printf ("(waittrunkevent) Got event 0x%x\n", ev);
            continue ;
            }
        }
    }
/*xx- waittrunkevent-xx*/

/********************************************************************/
void showtrunkevent(event)
   ME2INT event;
{
static char     *trunkevent [ (TRUNKCODEMAX-TRUNKCODEMIN) +1 ] = {
       "TRUNKBEG        ",
       "TRUNKCALLRECD   ",
       "TRUNKHANGUP     ",
       "TRUNKTIMEOUT    ",
       "TRUNKSEIZED     ",
       "TRUNKNODIGITS   ",
       "TRUNKPARTIALDIAL",
       "TRUNKPERMANENT  ",
       "TRUNKWINKTIMEOUT",
       "TRUNKSTUCKSENDER",
       "TRUNKENDOFDIAL" };

    printf ("Received event: %s\n", trunkevent[event-TRUNKCODEMIN]);
    return ;
    }


