/***********************************************************************
*
*              FILE: HBcallprocs.c
*
*    Description - DSP Telephony application
*
*  This application waits for incoming calls, obtains a trunk, 
*  makes voice responce, get the destinatdials 
*  the destination number, waits for answer, and uses call progress 
*  detection to determine the disposition of the call.  It then delays 
*  for a few seconds, hangs up and repeats.
*  (You must Control-C to exit).
*
*    Build instructions:
*    Microsoft 6.0:
*        CL /AL /c /Zi /Od /W3 /I\VBX\INCLUDE didapp.c
*        link /NOD /COD /PM:VIO didapp,,,os2+LLIBCEP+..\VBX\LIB\me2;
*
*    Borland C 1.0:
*        bcc -c -I\VBX\INCLUDE didapp.c
*        bcc -L\VBX\LIB didapp.obj me2.lib
*
*    IBM C Set/2:
*        icc /c /Gt+ /Ti /Kacoprx /Ss /Sp1 /I\vbx\include didapp.c
*        link386 /De /PM:VIO didapp.obj,didapp.exe,didapp.map,\vbx\lib\me2.lib;
*
*    UNIX:
*        cc -I /opt/nms/mex/include didapp.c /opt/nms/mex/lib/me2lib.a -o didapp -lx
*
*
***********************************************************************/

#define NUM_DID_DIGITS 10  /*- Central office sends 10 digits in HBdsp -xx*/

#ifdef unix
#include <unistd.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "mixed.h"
#include "me2def.h"
#include "me2err.h"

#define FALSE 0
#define TRUE  1
#define REMOTE_ANSWER LPCURON
#define REMOTE_HANGUP LPCUROFF
#define MAXWAIT   600000L
#define FOREVER   (-1L)

/* Message Numbers */
#define MWELCOME        1
#define MGOODBYE        2
#define MMENU           3
#define MSEL1           4
#define MSEL2           5
#define MSEL3           6
#define MINVALID        7
#define MPMPT1          8
#define MPMPT2          9
#define MRECORD         1

/*xx- LOCAL FUNCTIONS-xx*/
ME2INT ogt_waitdone() ;
ME2INT vox_waitdone();
ME2INT did_waitdone();
void showtrunkevent();
ME2INT waittrunkevent() ;
void docall();
int  play();
int  recplay();
void doexit();
void flushttqueue();
void ogt_setparms();
void ogt_getport();
ME2INT ogt_dial();
void ogt_callprogress();
void ogt_hangup();
void setparm();
void  ogt_exit() ;

/*xx- GLOBALS-xx*/
ME2INT Vmsg = -1;       /* Voice file handles.*/
ME2INT Vtst = -1;
ME2INT Answered = 0;        /*xx- set in waitdone-xx*/
ME2INT Hungup   = 0;        /*xx- set in waitdone-xx*/
ME2INT LastCallprogevent = 0;   /*xx- set in waitdone-xx*/
ME2INT Voice = 0;           /*xx- set in waitdone-xx*/
ME2INT Trunk; /* for tone detection (port #;0 for demo) */
short  trunk_type = 255; /* for tone and ogt (3 for T1) */
char   *digits; /* dest. dial # */
short  Maxrec = 10;	 /*  maximum voice recording sec. */
char Vmsgfil[20];    /* Name of voice file containing prompts used by demo.*/
char Vtstfil[80];    /* Name of voice file to record test message to. */


/***************************************************************************
 * main()
 *
 *
 ***************************************************************************/
void main()
{
  ME2INT i ;
  char *digits ;
  char  buff[20];           /* Array used to make ascii channel number. */
  ME2INT ev ;
#ifdef unix
    short linenum;
#else
    short line_group;
#endif

    /*xx- Get ME/2 channel-xx*/
  ME2REQUESTCHANNEL(99,"OGT");

    /*xx- Set ME2 parameters-xx*/
  ogt_setparms(buff);


    /*xx- Get a port-xx*/
  ogt_getport();

    /*xx-  Start the Trunk Control Program at its initial entry point.-xx*/
    /*xx-  This is an idle state, where it is ready for the DIAL command.-xx*/

    if ((i=Me2TrunkStart(0)) != SUCCESS)
        {
        printf ("Me2TrunkStart returned error %d\n", i) ;
        ogt_exit (1) ;
        }
    printf("Trunk Control Program started\n");

    /*xx- Enter a loop to make calls-xx*/
    while (1)
        {
#ifdef unix    
    while(1){                /* Wait for a call to come into the system. */
        i = Me2WaitEvent(-1);    /* Wait for an event to come through */
                                 /* the event queue.                  */
  
        Me2GetEvent(&ev, QREMOVE );  /* Get the event off the queue */

        if (ev == RINGBEG)           /* Wait for an incoming call */
          break ;
     }
#else
    vox_waitdone( FOREVER );    /* Wait for a call to come into the system. */

    if( i != SUCCESS )      /* Check to see if call to ME/2 function returns */
        {                   /* successfully.                                 */
        puts( "\tCall not received." );
        doexit( i );        /* If not, process routine to provide error msg. */
        }
#endif

    /* Answer the call */
    i = Me2SetLineMode( PICKUP ) ;
    if( i != SUCCESS )      /* Check to see if call to ME/2 function returns */
        {                   /* successfully.                                 */
        doexit( i );        /* If not, process routine to provide error msg. */
        }


    /*
      Remove any existing events from the event queue, in order
      to start with an empty queue.
    */
    while( ( Me2GetEvent( &ev, QREMOVE ) ) != ME2_QUEUE_EMPTY )
        ;

    docall();               /* Function that actually starts demo. */

    puts( "\tPrompt msg COMPLETED.\n" );
    

        /*xx- Make an outgoing call-xx*/
        if (ogt_dial(digits) == 0)
            {
            /*xx- Start Call progress to look for busy tone, SI Tone, etc.-xx*/
            ogt_callprogress();

    /*xx-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;-xx*/
    /*xx-   Called party interaction would go here      ;-xx*/
    /*xx-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;-xx*/


            /*xx- Wait 3 seconds, then hangup-xx*/
            ME2DELAY (300) ;
            ogt_waitdone() ;    /*xx- wait for delay to complete-xx*/
            }
        /*xx- Hangup, restarting the Trunk Manager-xx*/
        ogt_hangup ();

        }
    /*xx- while(1)-xx*/
    }
/*xx- main-xx*/


/*******************************************************************
* ogt_setparm - set ME2 parameters
*
* For this example, parameters are set by the program.  The same result
* could be obtained by adding the settings to ME2.PRO, or to a user profile
* that is loaded with Me2ParmLoad.  Note that setting of trunk-related
* parameters must be done before getting a port with Me2RequestLine.
********************************************************************/
void ogt_setparms(buff)
     char  *buff;
{
    /* Note - 'short' type used to hold the values of 16 bit ME2 parameters
     * passed by reference to Me2ParmByName and Me2ParmByNum */
  short val ;
  short id=0 ;
  short cpflags[6] ;
  short linenum;
  char  *trunk_type_text ;
  ME2INT i ;

    /*xx- Set the name of the Trunk Control overlay and associated parameter-xx*/
    /*xx- category.-xx*/
  setparm ("TRUNK.overlay", "TCPOGT" ) ;

    /*xx- Set the parameter that defines hardware type, unless already set-xx*/
    /*xx- in ME2.PRO.-xx*/

  i = ME2PARMLOAD("DSP");		/* loading DSP.PRM */

  if( i != SUCCESS ) {	/* see if load DSP.PRO parameter file is success */
    puts("\tError processing .PRO file information.");
    doexit(i);		/* if not, process routine to provide error msg. */
  }
 i = ME2PARMBYNAME( "DSP.APPL_ID", &id, GETPARM );
 if( i != SUCCESS )     /* See if call to ME/2 function is successful */
    {
      puts( "\tError reading DSP.APPL_ID parameter." );
      printf("\t%d",id);
      doexit( i );       /* If not, process routine to provide error msg. */
    } 
  i = Me2ParmByName( "DSP.MAX_RECORD", &Maxrec, GETPARM);
  if( i != SUCCESS )     /* See if call to ME/2 function is successful */
    {
      puts( "\tError reading DSP.MAX_RECORD parameter." );
      doexit( i );       /* If not, process routine to provide error msg. */
    }
#ifdef unix
  i = Me2ParmByName( "DSP.LINENUM", &linenum, GETPARM);
  if( i != SUCCESS )     /* See if call to ME/2 function is successful */
    {
      puts( "\tError reading DSP.LINENUM parameter." );
      doexit( i );       /* If not, process routine to provide error msg. */
    }

  i = Me2ParmByName( "DSP.TRUNK", &Trunk, GETPARM);
  if( i != SUCCESS )     /* See if call to ME/2 function is successful */
    {
      puts( "\tError reading DSP.TRUNK parameter." );
      doexit( i );       /* If not, process routine to provide error msg. */
    }
printf("\tTrunk => %d\n", Trunk);

  i = Me2ParmByName( "DSP.TRUNK_TYPE", &trunk_type, GETPARM);
  if( i != SUCCESS )     /* See if call to ME/2 function is successful */
    {
      puts( "\tError reading DSP.TRUNK_TYPE parameter." );
      doexit( i );       /* If not, process routine to provide error msg. */
    }
#else
  i = Me2ParmByName( "DSP.GROUPNUM", &line_group, GETPARM);
  if( i != SUCCESS )     /* See if call to ME/2 function is successful */
    {
      puts( "\tError reading DSP parameter." );
      doexit( i );       /* If not, process routine to provide error msg. */
    }
#endif
  i = Me2ParmByName( "DSP.VOX_MSG_FILE", Vmsgfil, GETPARM);
  if( i != SUCCESS )     /* See if call to ME/2 function is successful */
    {
      puts( "\tError reading DSP.VOX_MSG_FILE parameter." );
      doexit( i );       /* If not, process routine to provide error msg. */
    }

  /*  Here we are appending our channel number to the end of the test
      voice file name in order to make sure it is unique from the test voice
      file used by any other channels.  It is because of this appending
      process that we only allow six characters in in the test voice file
      name, instead of eight (see parameter file).
    */
    i = Me2ParmByName( "DSP.VOX_REC_FILE", Vtstfil, GETPARM);

    if( i != SUCCESS )     /* See if call to ME/2 function is successful */
        {
        puts( "\tError reading DSP.VOX_REC_FILE parameter." );
        doexit( i );       /* If not, process routine to provide error msg. */
        }

#ifdef unix
    sprintf (buff, "%d", linenum );          /* Turn line # into ascii. */
#else
    sprintf (buff, "%d", Me2GetChannel() );  /* Turn channel # into ascii. */
#endif
    strcat( Vtstfil, buff );                 /* Append to end of name.       */

    /*
      Open the voice file to use for all messages played during the
      demo.  Then open the file that will be used for voice recordings
      during the demonstration.  The function Me2VoxOpen searches the path
      specified by the ME2LOCAL variable for a voice file that is opened for
      PLAYONLY.  (If a voice file is opened for PLAYREC and cannot be found
      through this path, it is created in the current, local subdirectory.)

      It is also recommended practice not to specify extensions on voice
      file names when creating the source code.  This allows the flexibility
      of using other voice modules to support another file format or
      compression algorithim.  The replacement module will append the
      appropriate extension for it, as ME/2 does with the '.VOX' extension.
      NOTE:  This also applies to .PRO files and DATABASES.
    */
    i = Me2VoxOpen(Vmsgfil, &Vmsg, PLAYONLY);
    if( i != SUCCESS )     /* See if call to ME/2 function is successful */
        {
        puts( "\tDEMO.vox file could not be found in the correct subdirectory!" );
        doexit( i );       /* If not, process routine to provide error msg. */
        }

    i = Me2VoxOpen( Vtstfil, &Vtst, PLAYREC);
    if( i != SUCCESS )     /* See if call to ME/2 function is successful */
        {
        puts( "\tError creating/opening the test file." );
        doexit( i );       /* If not, process routine to provide error msg. */
        }

/* upto here set parameter for voicresp */

/* from here, set parm for ogt */

    /*xx- If hardware type selected via command line argument, set the ME2-xx*/
    /*xx- parameter, otherwise verify that parameter is already set (presumably-xx*/
    /*xx- by an entry in ME2.PRO.)-xx*/

  if (trunk_type != 255)
    {
      i=ME2PARMBYNAME("TRUNK.SIGNAL_TYPE", &trunk_type, SETPARM) ;
      if (i != SUCCESS)
	{
	  printf("Error %d setting parameter TRUNK.Signal_Type\n", i) ;
	  exit(1);
	}
    }
  else
    {
      i=ME2PARMBYNAME("TRUNK.SIGNAL_TYPE", &trunk_type, GETPARM) ;
      if (i != SUCCESS)
	{
	  printf("Error %d getting parameter TRUNK.Signal_Type\n", i) ;
	  exit(1);
	}
    }
  
  if (trunk_type == 255)
    {
      puts ("\nYou must specify trunk type either via an ME/2 parameter:");
      puts ("\n         TRUNK.SIGNAL_TYPE = <type>");
      puts ("\nor in the  DSP.PRO file argument, DSP.TRUNK_TYPE:");
#ifdef unix
      puts ("\n where {trunk_type} is one of");
#else
      puts ("\n         OGT <number to dial> /S<type>");
      puts ("\n where <type> is one of");
#endif
      puts ("     1 - VBX 400  or AG-8 DID");
      puts ("     2 - VBX 400  or AG-8 E&M");
      puts ("     3 - VBX 1200 or AG Digital (connected to T1 via MVIP)");
      exit (1) ;
    }

  switch (trunk_type) {
     case 1: trunk_type_text = "DID" ; break ;
     case 2: trunk_type_text = "E&M" ; break ;
     case 3: trunk_type_text = "T1" ; break ;
     default: trunk_type_text = "???" ; break ;
  }
  printf ("Trunk type is %d (%s)\n", trunk_type, trunk_type_text);

    /*xx- Here we modify default OGT paramters settings.-xx*/

    /*******
           FYI: OGT PARAMETER DEFAULTS:

       TCPOGT.WAIT_FOR_WINK = 1000    (10 sec) time to wait for wink (0=no wait)
       TCPOGT.MINIMUM_WINK  = 10      (100 ms) Minimum duration of the wink
       TCPOGT.MAXIMUM_WINK  = 490     (4.9 s) Maximum duration of the wink
       TCPOGT.PRE_DIAL_DLY  = 7       (70 ms) Delay before dialing, after wink
       TCPOGT.TONE_ON_TIME  = 8       (80 ms) Duration of DTMF tone on
       TCPOGT.TONE_OFF_TIME = 8       (80 ms) Duration of DTMF tone off
       TCPOGT.PULSE_MAKE    = 4       (40 ms) Dial pulse make
       TCPOGT.PULSE_BREAK   = 6       (60 ms) Dial pulse break
       TCPOGT.PULSE_IDT     = 70      (700 ms) Dial pulse inter-digit-time
       TCPOGT.TONE_LO       = 512     Amplitude of low DTMF frequencies
       TCPOGT.TONE_HI       = 640     Amplitude of high DTMF frequencies
       TCPOGT.PERMANENT_SIG = 2000    (20 sec) Timeout waiting for remote hangup
    *******/


    /*xx- Note: If any OGT parameters are to be changed from their defaults, this-xx*/
    /*xx- must be done before calling Me2RequestLine.-xx*/


    /*xx- EXAMPLE: Change default minimum wink time to 120 ms-xx*/

    val = 12 ;
    setparm ("TCPOGT.MINIMUM_WINK", &val) ;

    /*xx- Modify Call Progress parameters to allow detection of varying-xx*/
    /*xx-  length of voice-xx*/

    cpflags[0] = 0 ;   /*xx- no debug-xx*/
    cpflags[1] = 1 ;   /*xx- quit after CPVOCLNG-xx*/
    cpflags[2] = 0 ;   /*xx- don't quit after CPVOCMED-xx*/
    cpflags[3] = 0 ;   /*xx- don't quit after CPVOCBEG-xx*/
    cpflags[4] = 1 ;   /*xx- quit after CPRINGEND-xx*/
    cpflags[5] = 0 ;   /*xx- don't quit after CPRINGBEG-xx*/
    setparm  ("CALL_PROGRESS.FLAGS", cpflags) ;

    return ;
    }


/* from here, set parm for did  */




/*xx- ogt_setparms-xx*/

/**********************************************************************
* ogt_getport - obtain a VBX port to dial out on
************************************************************************/
void ogt_getport()
    {
    ME2INT i ;
#ifndef unix
    ME2INT event ;
#endif

#ifdef unix
    if ( (i=ME2GRABTRUNKLINE( Trunk )) < SUCCESS )
        {
        printf ("Me2GrabTrunkLine returned error %d\n", i) ;
        ogt_exit (1) ;
        }
#else
    /*xx- The LINETRUNK argument causes an uninitialized port ("line")-xx*/
    /*xx- to be returned.-xx*/

    if ( (i=ME2REQUESTLINE( 0 /* group */, LINETRUNK, 0L )) < SUCCESS )
        {
        printf ("Me2RequestLine returned error %d\n", i) ;
        ogt_exit (1) ;
        }
    if ((event=ogt_waitdone()) == LINRQFAIL)
        {
        printf ("Line request failed. Me2GetLineError returned %d\n",
              ME2GETLINEERROR());
        ogt_exit(1);
        }
    else if (event != LINRQDONE)
        {
        printf ("Unexpected completion event from line request: %d (0x%x)\n",
                   event, event);
        ogt_exit(0);
        }
    else
        printf ("Line request succeeded.  Got VBX port # %d\n", ME2GETLINE());
#endif
    return ;
    }
/*xx- ogt_getport-xx*/

/************************************************************************
*  ogt_dial - Dial a call using the Trunk Manager
*    If the correct wink is not detected, a failure is returned so
*    the call can be re-attempted.
***********************************************************************/

ME2INT ogt_dial(digits)
  char *digits;
    {
    ME2INT i ;
    ME2INT event ;
    int dial_failed = FALSE ;

    if ((i=Me2TrunkDial (digits)) != SUCCESS)
        {
        printf ("Me2TrunkDial returned error %d\n", i) ;
        ogt_exit (1) ;
        }

    /*xx- Wait for dial completion - the trunk control overlay will exit-xx*/
    /*xx-  automatically-xx*/

    /*xx- The TRUNKDONE indicates that the trunk overlay (Trunk Control Program-xx*/
    /*xx- running on the DSP) has stopped and answer detection is now being done-xx*/
    /*xx- by the VMOS supervision detector.-xx*/

    while ((event = waittrunkevent()) != TRUNKDONE)
        {
        if (event == TRUNKFAIL)
            ogt_exit (1) ;         /*xx- internal protocol error-xx*/
        else if (event == TRUNKWINKTIMEOUT)
            {
            printf ("FAILED: Timed out waiting for wink\n");
            dial_failed = TRUE ;
            }
        else if (event == TRUNKSTUCKSENDER)
            {
            printf ("FAILED: Timed out waiting for end of wink\n");
            dial_failed = TRUE ;
            }
        }
    if (dial_failed)
        return -1 ;
    else
        return 0;
    }
/*xx- ogt_dial-xx*/

/************************************************************************
* ogt_callprogress - Run call progress detection to look for busy tone,
*                    SI Tone, etc.
* NOTES:
*   If the called party answers, there will be a change in the incoming
*   signaling on the trunk, which will cause a REMOTE_ANSWER event.  This
*   is independent of Call Progress Analysis.  A global "answered" flag
*   will be set
*   by the waitdone subroutine.
*
*   If voice is detected without such an answer, one can assume some sort
*   of recorded message has been encountered.
*
*   The last call progress event received will be stored in a global by
*   the waitdone subroutine.
*************************************************************************/
void ogt_callprogress()
    {
    ME2INT i ;
    ME2INT event ;

    if ((i=ME2CALLPROGRESS()) != SUCCESS)
        {
        printf ("Me2CallProgress returned error %d\n", i) ;
        ogt_exit (1) ;
        }
    printf ("Starting call progress analysis.\n");

    LastCallprogevent = Answered = Voice = Hungup = 0;
    if ((event=ogt_waitdone()) == CPROGSTOP )
        {
        printf ("Call progress detection stopped.\n");
        if (Hungup)
            if ( Answered )
                printf ("Called party hung up before voice detected. \n");
            else
                printf ("Unexpected hangup event during call progress\n");
        }

    else if (event != CPROGDONE)
        {
        if (event == CPROGFAIL)
            printf ("Call progress FAILED\n");
        else
            printf("Unexpected completion event from call progress:%d (0x%x)\n",
                   event, event);
        ogt_exit(1);
        }
    else
        {
        printf ("Call progress detection completed.\n");

        switch (LastCallprogevent)
            {
            case CPDIALTON : puts ("Dial tone detected!"); break ;
            case CPBUSY    : puts ("Busy tone detected");  break ;
            case CPFASTBSY : puts ("Fast busy tone detected");  break ;
            case CPRINGEND : puts ("Ring ended without voice");  break ;
            case CPNOANSWR : puts ("No answer - max rings detected");  break ;
            case CPSITTONE : puts ("Special Information tone detected"); break ;
            case CPVOCLNG  : puts ("Long voice detected") ;   break ;
            case CPVOCEND  : puts ("End of voice detected") ; break ;
            case CPTIMEOUT : puts ("Timed out with no call progress indication") ;
                             break ;
            default:
                printf ("Call progress terminated for unexpected reason: 0x%x\n",
                          LastCallprogevent );
                break ;
            }

        if (Voice > 0)
            {
            printf ("%s voice detected",
             Voice == 1 ? "Short" : Voice == 2 ? "Medium" : "Long") ;
            if ( !Answered)
                printf ("; no answer indication\n");
            else
                printf ("\n");
            }
        }
    return ;
    }
/*xx- ogt_callprogress-xx*/


/***********************************************************************
* ogt_hangup  - Hangup the call.
*
* Hangup by restarting the Trunk Control Program at entry point 1.
* There will be a hangup event from the overlay when it sees that
* the remote is on-hook.
************************************************************************/
void ogt_hangup ()
    {
    ME2INT i ;
    ME2INT event ;

    if ((i=Me2TrunkStart(1)) != SUCCESS)
        {
        printf ("Me2TrunkStart returned error %d\n", i) ;
        ogt_exit (1) ;
        }

    if (Answered && !Hungup)
        puts ("Waiting for remote to hang up");

    while ((event = waittrunkevent()) != TRUNKHANGUP)
        {
        if (event == TRUNKFAIL)
            ogt_exit (1) ;          /*xx- internal protocol error-xx*/
        }
    /*xx- Before returning to the top of the loop to make another call,-xx*/
    /*xx-  a delay is necessary so that a sufficiently long disconnect-xx*/
    /*xx-  interval is sent to the central office.  Note that Me2Delay-xx*/
    /*xx-  canNOT be used here because the Trunk Manager is active.-xx*/

#ifdef unix
    sleep (1) ;
#else
    DosSleep (1000L);
#endif
    return ;
    }
/*xx- ogt_hangup-xx*/


/*********************************************************************
* ogt_exit - cleanup and exit
*********************************************************************/
void  ogt_exit (stat)
  int stat;
    {
    ME2RELEASELINE();
    ME2RELEASECHANNEL();
    exit (stat) ;
    }


/*******************************************************************
* setparm - subroutine to call Me2ParmByName and test the return code.
******************************************************************/
void setparm(parm, val)
  char *parm;
  void *val;
    {
    ME2INT i=ME2PARMBYNAME(parm, val, SETPARM) ;
    if (i != SUCCESS)
        {
        if (i==ME2_CATEGORY_NOT_FOUND)
            printf("Category not found for %s\n", parm);
        else
            printf("Error %d setting parameter %s\n", i, parm) ;
        ogt_exit(1);
        }
    return ;
    }




/*******************************************************************
* waitdone - local subroutine to wait for completion of a background
*   function.  This routine looks for a hangup indication
*   and will stop the background function and return immediately if one
*   is detected.  It also records, but does not stop on, answer
*   indication and call progess events.
******************************************************************/
ME2INT ogt_waitdone()
    {
    ME2INT i;
    ME2INT ev;

    while (1)
        {
        i = ME2WAITEVENT( -1L );       /* Wait for an event to come through */
                                       /* the event queue.                  */
        if ( i == ME2_QUEUE_EMPTY)     /*xx- DOS case-xx*/
            continue ;

        if ( i != SUCCESS )
            {
            printf ("Me2WaitEvent returned error %d\n", i) ;
            ogt_exit (1) ;
            }

        if ((i=ME2GETEVENT(&ev, QREMOVE)) != SUCCESS)
            {
            printf ("Me2GetEvent returned error %d\n", i) ;
            ogt_exit (1) ;
            }
        printf ("(waitdone) Got event 0x%x\n", ev);
        if (ev == REMOTE_ANSWER)
            {
            printf("REMOTE ANSWER DETECTED\n") ;
            Answered = 1;
            continue ;
            }
        else if (ev == REMOTE_HANGUP)
            {
            printf("REMOTE HANGUP DETECTED\n") ;
            Hungup = 1;                       /*xx- If called party hangs up,-xx*/
            ME2STOP();                        /*xx- abort active function (if any)-xx*/
            }
        else if (CPDIALTON <= ev && ev <= CPTIMEOUT)
            {
            LastCallprogevent = ev ;
            switch (LastCallprogevent)
                {
                case CPVOCBEG  : Voice = 1; break ;
                case CPVOCMED  : Voice = 2; break ;
                case CPVOCLNG  : Voice = 3; break ;
                default:   break ;
                }
            }

        else if (EXITCODE <= ev && ev <= EXITCMAX ) /*xx- Check for done code.-xx*/
            return ev ;
        }
    }
/*xx- waitdone-xx*/


/*******************************************************************
* waittrunkevent - local subroutine to wait for an event from the
*   trunk handler thread.  This routine returns on events in the
*   TRUNKMIN - TRUNK MAX range and also on any exit code.
******************************************************************/
ME2INT waittrunkevent()
    {
    ME2INT i;
    ME2INT ev;

    while (1)
        {
        i = ME2WAITEVENT( 10000L );    /* Wait for an event to come through */
                                       /* the event queue.                  */
        if ( i == ME2_QUEUE_EMPTY)     /*xx- DOS case-xx*/
            continue ;

        if ( i == ME2_QUEUE_TIMEOUT)
            {
            puts ("still waiting for trunk event...");
            continue ;
            }

        if ( i != SUCCESS )
            {
            printf ("Me2WaitEvent returned error %d\n", i) ;
            ogt_exit (1) ;
            }

        if ((i=ME2GETEVENT(&ev, QREMOVE)) != SUCCESS)
            {
            printf ("Me2GetEvent returned error %d\n", i) ;
            ogt_exit (1) ;
            }

        else if (EXITCODE <= ev && ev <= EXITCMAX ) /*xx- Check for done code.-xx*/
            {
            if (ev == TRUNKFAIL)
                printf ("TRUNKFAIL received\n");
            else if (ev == TRUNKDONE)
                printf ("TRUNKDONE received\n");
            else if (ev == TRUNKSTOP)
                printf ("TRUNKSTOP received\n");

            return ev ;
            }
        else if (TRUNKCODEMIN <= ev && ev <= TRUNKCODEMAX ) /*xx- Check for trunk-xx*/
            {
            showtrunkevent(ev);
            return ev ;
            }
        else
            {
            printf ("(waittrunkevent) Got event 0x%x\n", ev);
            continue ;
            }
        }
    }
/*xx- waittrunkevent-xx*/

/********************************************************************/
void showtrunkevent(event)
   ME2INT event;
{
static char     *trunkevent [ (TRUNKCODEMAX-TRUNKCODEMIN) +1 ] = {
       "TRUNKBEG        ",
       "TRUNKCALLRECD   ",
       "TRUNKHANGUP     ",
       "TRUNKTIMEOUT    ",
       "TRUNKSEIZED     ",
       "TRUNKNODIGITS   ",
       "TRUNKPARTIALDIAL",
       "TRUNKPERMANENT  ",
       "TRUNKWINKTIMEOUT",
       "TRUNKSTUCKSENDER",
       "TRUNKENDOFDIAL" };

    printf ("Received event: %s\n", trunkevent[event-TRUNKCODEMIN]);
    return ;
    }



/*******************************************************************
* waitdone - local subroutine to wait for completion of a background
*   function.  If a hangup event is detected, this routine
*   will stop the background function, wait for the completion code,
*   and return a hangup indication.
******************************************************************/

ME2INT did_waitdone()
    {
    ME2INT ev, hangup_ev;
    ME2INT i ;

    hangup_ev = 0 ;
    while (1)
        {
        i = Me2WaitEvent( -1L );       /* Wait for an event to come through */
                                       /* the event queue.                  */
        if ( i != SUCCESS )
            {
            printf ("Me2WaitEvent returned error %d\n", i) ;
            exit (1) ;
            }

        if ((i=Me2GetEvent(&ev, QREMOVE)) != SUCCESS)
            {
            printf ("Me2GetEvent returned error %d\n", i) ;
            exit (1) ;
            }
        if (ev == LPCUROFF)                       /*xx- If caller hangs up, abort-xx*/
            {                                     /*xx- active function (if any)-xx*/
            printf("REMOTE HANGUP DETECTED\n") ;
            Me2Stop();
#ifdef unix
            return ev;
#else
            hangup_ev  = ev;
#endif
            }

          /*xx- Check for done code.-xx*/
        else if (EXITCODE <= (unsigned)ev && (unsigned)ev <= EXITCMAX )
            if (hangup_ev != 0)
                return hangup_ev ;
            else
                return ev ;

        }
    }


/*******************************************************************************
 DOCALL:
      This routine starts the interactive segments of the demonstration.

       Return Value:    None
******************************************************************************/
void docall()
    {
    char sel[5];     /* Character array for storing touchtones. */
    ME2INT result;    /* Return code from Me2GetTTS. */

    puts("\tSTART OF DEMO");
    Me2Delay( 40L );
    vox_waitdone( MAXWAIT );
    flushttqueue();                            /* Empty the touch-tone queue.*/

    if( play( Vmsg, MWELCOME ) ) return;     /* Play the WELCOME message   */

    for( ;; )
        {
        if( play(Vmsg, MMENU ) ) return;     /* Play the main menu message.*/

        /* This function collects one touchtone digit */
        Me2GetTTS( "X", "", sel, &result, 1 );

        if( vox_waitdone( MAXWAIT ) ) return;    /* Abort if no touch-tones */
                                             /* entered.                */

        /* See what touchtone was entered */

        if ( result == 'T' || sel[0] == '0' )   /* If timeout occurred or */
            {                                   /* '0' entered, play      */
            play( Vmsg, MGOODBYE );             /* GOODBYE message        */
            return;
            }
        else if( sel[0]>='1' && sel[0] <= '3' ) /* If touchtones '1'-'3'  */
            {                                   /* entered, play MENU     */
            if( play( Vmsg, (sel[0]-'0')+3) )   /* selection message.     */
                return;
            }
        else if( sel[0] == '4' )                /* If touchtone '4' play */
            {                                   /* prompt for recording  */
            if( recplay() )                     /* and playing voice.    */
                return;
            }
        else if( play( Vmsg, MINVALID ) )       /* If anything other than */
            return;
                                                /* '1-4' or '0' play      */
                                                /* invalid selection msg. */

        flushttqueue();                        /* Empty the touch-tone queue.*/
        }
    }


/*****************************************************************************
PLAY:
   This routine plays the specified message from a particular voice file.
   It waits for the completion of play before returning.  While playing special
   event processing occurs to detect local phone hangup, <ESC> key press, etc.

   Argument List:    ME2INT vsel    => Voice file handle.

                     ME2INT msg     => Message number to be played.

   Return Value:          value from waitdone()
*****************************************************************************/
ME2INT play( vsel, msg )
    ME2INT vsel, msg;  
    {
    ME2INT i;

    i = Me2VoxPlayMsg( vsel, msg );
    if( i != SUCCESS )                 /* Display error msg if play fails.   */
        {
        puts( "\tProblem with demo voice file.  Reinstall demo to correct." );
        doexit( i );
        }
    return( vox_waitdone( MAXWAIT ) );
    }


/****************************************************************************
RECPLAY:
   This function records a test message and plays it
   back when the record is done executing.


   Return Value:        value from waitdone()
*****************************************************************************/
ME2INT recplay()
    {
    ME2INT i ;

    if ((i=play( Vmsg, MPMPT1 )) != 0)  /* Play prompt for recording test msg */
        return i;

    Me2Beep( 1 );                          /* Beep tone for start of record. */
    if ((i=vox_waitdone( MAXWAIT )) != 0)
        return i;

    i = Me2VoxRecordMsg( Vtst, MRECORD, Maxrec ); /* Record up to Maxrec secs */
    if( i != SUCCESS )                            /* Display error message if */
        {                                         /*  record failed.          */
        puts( "\tError recording speech! Bad file handle or no available disk space." );
        doexit( i );
        }
    if ((i=vox_waitdone( MAXWAIT )) != 0)
        return i;

    flushttqueue();                       /*Empty the touch-tone queue.         */

    if ((i=play( Vmsg, MPMPT2 )) != 0)  /* Play prompt that prepares caller         */
        return i;                       /* to hear what was recorded.        */

    i = Me2VoxPlayMsg( Vtst, MRECORD ); /* Play msg that contains their voice.*/
    if( i != SUCCESS )                  /* Abort if problem with that voice file. */
        {
        puts( "\tError playing message back." );
        doexit( i );
        }
    return( vox_waitdone( MAXWAIT ) );
    }


/*****************************************************************************
WAITDONE:
   This function waits for a 'DONE' code to appear on the event queue
   while checking any other events coming in to see if they need special
   handling. All other events are thrown away.  Note that even though Touch-
   Tone events are thrown away here, ME/2 maintains a copy of them in the TT
   queue for Me2GetTTS() and Me2GetTTC() to process.

   All ME/2 functions which take more than a fraction of a second to
   complete are designed to return immediately upon startup and then
   report a 'DONE' event code when the action is totally finished.  A
   function of this type generally has three valid 'DONE' codes to
   indicate one of the following:  finished normally, stopped by
   Me2Stop(), or the function failed and was aborted.

   All events which indicate the termination of a ME/2 function have been
   placed in a special range of values which can be easily checked for.
   Waitdone() returns when an event code in the 'DONE' code value range
   appears on the event queue.


   Return Value:        1   A hangup has been detected. Stop currently
                            active function and return.
                        0   No special event has been found.
                       <0   Timeout
*******************************************************************************/
ME2INT vox_waitdone( timeout )
     long timeout;
{
  ME2INT ev ;
  ME2INT i;

  while (1)
    {
      i = Me2WaitEvent( timeout );   /* Wait for an event to come through */
      /* the event queue.                  */
      if ( i == ME2_QUEUE_TIMEOUT )  /* Check and see if a timeout.       */
	{
	  Me2Stop();                 /* If timeout, stop everything and   */
	  return( i );               /* abort.                            */
	}

      Me2GetEvent( &ev, QREMOVE );   /* Get the event off the queue*/
      if ( ev == LPCUROFF )          /* Check and see if a hangup occurred*/
	{
	  Me2Stop();                 /* If so, stop everything and   */
	  return( 1 );               /* return.                       */
	}

          /* Check for done code.  */
      if( EXITCODE <= (unsigned)ev && (unsigned)ev <= EXITCMAX )
	return( 0 );
    }
}



/*******************************************************************************
DOEXIT:
   This function is used to abort cleanly from the application if a failure
   occurs.  Error descriptions can found in ME2ERR.H.
   Argument List:    ME2INT n       =>    Error code.
   Return Value:     No return
*******************************************************************************/
void doexit( n )
    ME2INT n;
    {
    if( Vmsg >= 0 )                 /* Close both voice files if opened. */
        Me2VoxClose( Vmsg );
    if( Vtst >= 0 )
        Me2VoxClose( Vtst );

    Me2ReleaseLine();               /* Release the line                  */
    Me2ReleaseChannel();            /* and channel before exiting.       */

    strcat( Vtstfil, ".vox");
#ifdef __STDC__
    remove( Vtstfil );
#else
    unlink( Vtstfil );
#endif

    if( n != SUCCESS )
        {
        printf( "ME/2 error code: %d, process aborted.\n", n );
        printf( "Refer to 'me2err.h' and your manual for a list of errors.\n" );
        }

    exit( n );
    }


/******************************************************************************
FLUSHTTQUEUE:
   This function removes any existing touchtones from the touch-tone queue.
   Return Value:     none
******************************************************************************/
void flushttqueue()
    {
    char c;

    while( Me2GetTTC( &c, QREMOVE ) == 0 )
        ;
    return ;
    }
