/***********************************************************************
 *
 * Name:        Plugin_procs.c
 *
 * Description: NCS video plugin procedures               
 *
 * Author:	Janusz Bulawa & Tomasz Stachowiak, NPAC
 *
 * Date:	October 1996
 *
 ***********************************************************************/
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/param.h>

#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <Xm/MwmUtil.h>
#include <Xm/MenuShell.h>
#include <Xm/RowColumn.h>
#include <Xm/Scale.h>
#include <Xm/MainW.h>

#include "VideoProtocol.h"
#include "../lib/common.h"        /* NCS application API header */

#define GLOBAL                    /* H.263 includes */
#include "config.h"
#include "tmndec.h"
#include "global.h"
#include "rtp.h"

/* H.263 */
static unsigned char *localPict[3];
static unsigned char *exlocalPict[3];
extern unsigned char *frame;
extern char* videobuffer;
extern int numbytes;
extern int writeoffset;

/* Widgets */
Widget MainWindow;
extern app;
extern toplevel;
extern start_but;
extern stop_but;
extern shell;

/* application state */
#define INTRA_GAP 10

ncsApplication *ncsapp;
XtWorkProc recworkproc;  

extern localName;
extern int expand;
extern state;
int localHandle;
int    currentUser;
int totalBytesSent=0;
int totalFramesSent=0;
int tmp_state;
int numUsers = 0;         /* number of users */
char *wlabel;
int code_type = H263;     /* encoding type */
int intraFrame = 1;       /* if 0 - next frame sent INTRA */
int interval;/* variables for calculation of inter coding gap (rate control) */
float delayCoeff = 1.1;
int first[MAX_USERS+1];
XtIntervalId timerId=0;
int lastDecodeTime = 2000;
extern XtWorkProc workProc;
char *data;
int messageNum=0;          /* number of current message */
int lastMsgNum[MAX_USERS+1]; /* last received message number for each member */
int allintra = 0; 
int conn_type=M288;


/* display */
extern Display *display;
Window window;
Window win[MAX_USERS+1];
int dpy_depth;
int depth[MAX_USERS+1];


void partyLeft(int no);
Boolean recWorkProc();
void start_playing();
void start_conference(void);
void started();
void QuitPlugin();
void initdecoder (void);
void initdpy(int userno, unsigned char sourceType);
void SendVideoData(char *data, int len);
extern SetMessage(const char *msg);
Boolean scheduleFrame();
Boolean nextFrame();
void YVU9toYUV411(unsigned char *input, 
                         unsigned char *y, unsigned char *u, unsigned char *v, 
                         int width, int heigth);

void setModem();
void setISDN();
void setISDN2();
void setLAN();

/*************************************************
 *
 * Data sending procedures
 *
 *************************************************/

/* Video data sending */
void SendVideoData(char *data, int len)
{
  VideoPluginFrame *frame;

   frame = (VideoPluginFrame *)malloc(sizeof(VideoPluginFrame)+len-1);
   if (!frame)
   {
     fprintf(stderr, "Message allocation error\n");
     return;
   }

  totalBytesSent+=len;
  totalFramesSent++;
  if (len>0)
  {
    messageNum = (messageNum + 1) % 256;
    frame->num = (unsigned char)messageNum;
    gettimeofday(&frame->time);
    memcpy(frame->data, data, len);

    ncsAppSendData(ncsapp, -1, code_type,frame,sizeof(VideoPluginFrame)+len-1);

  }
  else 
    fprintf(stderr, "SendVideoData(): Illegal data length value!\n");

  free(frame);

  return;
}

/* Special data sending */
void SendSpecificMessage(VideoTag messageType)
{
   static VideoPluginArg dane;
   char *title;

   switch (messageType) {
   case videoTagPlay:
     dane.sourceType = SGI;
     dane.frate=10;
     dane.format=SF_QCIF;
     break;
   case videoTagConfirm:
     break;
   case videoTagStop:
     break;
   case videoTagExit:
     break;
   case videoTagFrameDecoded:
     break;
   case videoTagNewMember:
     break;
   default:
     fprintf(stderr, "Unknown request in SendSpecific...!\n");
     break;
   }

   ncsAppSendData(ncsapp, -1, messageType, &dane, sizeof(dane)); 
}

/* Special data sending */
void SendSpecificMessageToMember(int userNo, VideoTag messageType)
{
   VideoPluginArg dane;

   switch (messageType) {

   case videoTagPlay:
     dane.sourceType = SGI;
     dane.frate=10;
     dane.format=SF_QCIF;
     break;
     
   }

   ncsAppSendData(ncsapp, userNo, messageType, &dane, sizeof(dane));
}



/******************************************
 *
 * Received video data processing
 *
 ******************************************/
static void ProcessVideoData(void *data, int size, int type, int userNo)
{
  int i, f;
  VideoPluginFrame *frame;
  struct timeval start, stop;
  char *yuv;

  
  frame = (VideoPluginFrame *)data;

  if((frame->num < lastMsgNum[userNo]) 
     && ((lastMsgNum[userNo] - frame->num) < 100))
    {
      lastMsgNum[userNo] = frame->num;
      return; /* misplaced packet */
    }
  
  lastMsgNum[userNo] = frame->num;
  gettimeofday(&start);
  memset(invideobuffer, 0, MAX_FRAME_SIZE);
  avail=size - sizeof(VideoPluginFrame) + 1;
  memcpy(invideobuffer, frame->data, avail);
  invideooffset=0;
  initbits();
  if(type == 0)
    {
       if (getheader())
	 {
	    getpicture(&framenum, 0);
	    framenum++;
	 }
    }
  else
    if(type == 1)
      {
	getpicture(&framenum, 1);
      }
    else
      {
	  yuv = malloc(176*144*3/2);
	  YVU9toYUV411(frame->data, yuv, &yuv[25344], &yuv[25344+6336], 176, 144);
	  localPict[0]=yuv; 
	  localPict[1]=yuv+25344;
	  localPict[2]=yuv+25344+6336;
	  currentUser=(int)userNo;
	  window=win[userNo];
	  dpy_depth=depth[userNo];
	  if (expand && outtype == T_X11) 
	    {
	      interpolate_image(localPict[0], exlocalPict[0],
				coded_picture_width, coded_picture_height);
	      interpolate_image(localPict[1], exlocalPict[1], 
				chrom_width, chrom_height);
	      interpolate_image(localPict[2], exlocalPict[2], 
				chrom_width, chrom_height);
	      dither(exlocalPict);
	    }
	  else
	    {
	      dither(localPict);
	    }
	  free(yuv);
      }

  gettimeofday(&stop);

  lastDecodeTime = (stop.tv_sec - start.tv_sec)*1000 
    + (stop.tv_usec - start.tv_usec)/1000;

  doReceiveStatistics(userNo, &frame->time, lastDecodeTime);
}


/********************************************************
 *
 * Main coding and sending procedure, invoked with timer 
 *
 ********************************************************/
Boolean nextFrame()
{
  int wait_time;
  struct timespec wait;

      if((state == PLAY))
	{
	  
	  /* frame capturing and compression */
	  switch(code_type)
	    {
	    case H263:
	      if(intraFrame == 0)
		{
		  frame = getFrame(176, 144);

		  codeIntraFrame(frame); 
		  writeoffset=0;
		}
	      else
		{		
		  numbytes=codeFrame(&frame);
		}
	      break;
	      
	    case H261:
	      frame = getFrame(176, 144);
	      
	      if(intraFrame == 0)
		numbytes = p64EncodeIntraFrame(frame, videobuffer);
	      else
		numbytes = p64EncodeFrame(frame, videobuffer);
		
	      break;
	    }
	  
	  intraFrame = (intraFrame + 1) % INTRA_GAP;

	  doSendStatistics(numbytes);

	  /* mirror image handling */
	  localPict[0]=frame; 
	  localPict[1]=frame+25344;
	  localPict[2]=frame+25344+6336;
	  currentUser=(int)localHandle;
	  window=win[currentUser];
	  dpy_depth=depth[currentUser];
	  if (expand && outtype == T_X11) 
	    {
	      interpolate_image(localPict[0], exlocalPict[0],
				coded_picture_width, coded_picture_height);
	      interpolate_image(localPict[1], exlocalPict[1], 
				chrom_width, chrom_height);
	      interpolate_image(localPict[2], exlocalPict[2], 
				chrom_width, chrom_height);
	      dither(exlocalPict);
	    }
	  else
	    {
	      dither(localPict);
	    }

	  free(frame);
	  
	  /* compressed frame sending */
	  
	  SendVideoData(videobuffer, numbytes);
	  
	  /*interval = (int)(delayCoeff *(float)((numUsers + 1)*lastDecodeTime));*/

	  interval = statCalcInterval();

	  timerId = XtAppAddTimeOut(app, interval, nextFrame, NULL);

	  if(lastDecodeTime == 2000)
	    lastDecodeTime = 10;
	}

      return False;
}


/**********************************
 *
 * Main panel callbacks
 *
 **********************************/

/*
 * Buttons pushing callbacks
 */
void start_pushed(Widget w, XtPointer data, XmPushButtonCallbackStruct *cbs)
{
  if(state == STOP)
    {
      XtSetSensitive(start_but, False);	 
      XtSetSensitive(stop_but, True);  
      
      state=PLAY;
      
      source_format=SF_QCIF;
      
      sendStart();
      if (first[localHandle]) 
	{
	  initdpy(localHandle, SGI);
	  first[localHandle] = 0; 
	}
      
      
      start_playing();
      SetMessage("Video is running...");
    }
  return;
}

void stop_pushed(Widget w, XtPointer data, XmPushButtonCallbackStruct *cbs)
{
  if(state == PLAY)
    {
      XtSetSensitive(start_but, True);
      XtSetSensitive(stop_but, False);
      
      state=STOP;
      sendStop();
      SendSpecificMessage(videoTagStop);

      partyLeft(localHandle);
      
      SetMessage("Video stopped...");
    }
}

/*
 * File menu callback
 */
void file_menu(Widget menu_item, int item_no,
		  XmAnyCallbackStruct *cbs)
{
   switch(item_no)
     {
	case 0:
	QuitPlugin();
	break;
     }
}


/*
 * Tool menu callback 
 */
void tool_menu(Widget menu_item, int item_no,
		  XmAnyCallbackStruct *cbs)
{
   switch(item_no)
     {
	case 0:
	  if (timerId!=0) 
	    {
	      XtRemoveTimeOut(timerId);
	      timerId=0;
	    }
	  tmp_state = state;
	  if(state == PLAY)
	    {
	      SendSpecificMessage(videoTagStop);
	      sendStop();
	    }
	  state = PANEL;
	  if(shell==NULL) PanelDialog();
	  /*
	  else 
	    XtPopup(shell, XtGrabNone);*/
	  break;

        case 1:
	  if(state == PLAY)
	    {
	      sendStop();
	      statShowDlg(toplevel);
	      sendStart();
	    }
	  else
	      statShowDlg(toplevel);
	  /*
	  sendTimeReport();
	  */
	  break;
     }
}


/****************************************************
 *
 * Video control panel callbacks
 *
 ***************************************************/

/* Compression type menu callback */
void
comp_option_cb(w, item_no, cbs)
Widget w;
int item_no;
XmAnyCallbackStruct *cbs;
{
   switch(item_no)
     {
	case 0:
	code_type = H263;
	break;

	case 1:
	code_type = H261;
	break;
     }
}

void
conn_option_cb(w, item_no, cbs)
Widget w;
int item_no;
XmAnyCallbackStruct *cbs;
{
   switch(item_no)
     {
	case 0:
	setModem();
	break;

	case 1:
	setISDN();
	break;

	case 2:
	setISDN2();
	break;

	case 3:
	setLAN();
	break;
     }
}

/* Video size menu callback */
void
size_option_cb(w, item_no, cbs)
Widget w;
int item_no;
XmAnyCallbackStruct *cbs;
{
   switch(item_no)
     {
	case 0:
	  /* printf("QCIF size\n"); */
	break;

	case 1:
	  /* printf("CIF size\n"); */
	break;
     }
}

/* expand toggle button callback */
void expand_cb(Widget w, char *data, XmToggleButtonCallbackStruct *cbs)
{
  int err, i;


  if(cbs->set)
    {
      expand = 1;
      for(i=0; i<MAX_USERS+1; i++)
	{
	  if(!first[i])                /*win[i] != NULL*/
	    {
	      err = XDestroyWindow(display, win[i]);
	      initdpy(i, SGI);
	    }
	}
    }
	       
   else
     {
       expand = 0;
       for(i=0; i<MAX_USERS+1; i++)
	 {
	   if(!first[i])
	     {
	       XDestroyWindow(display, win[i]);
	       initdpy(i, SGI);
	     }
	 }
     }
}

/* intra mode toggle button callback */
void intra_cb(Widget w, char *data, XmToggleButtonCallbackStruct *cbs)
{
   if(cbs->set)
     {
       setIntraMode();
       allintra = 1;
     }
   else
     {
       resetIntraMode();
       intraFrame = 0;
       allintra = 0;
     }
}

/*
 * Video panel dialog callbacks
 */
void v_file_menu(Widget menu_item, int item_no,
		  XmAnyCallbackStruct *cbs)
{
   switch(item_no)
     {
	case 0:
	  state = tmp_state;
	  XtDestroyWidget(shell);
	  shell = NULL;
	  /*XtPopdown(shell);*/
	  if(state == PLAY)
	    {
	      start_playing();
	      sendStart();
	    }
	  break;
     }
}



void start_playing()
{
  state=PLAY;

  SendSpecificMessage(videoTagPlay);
  messageNum = 0;

  if(code_type == H263)
    {
      frame = getFrame(176, 144);
      codeIntraFrame(frame);
      writeoffset=0;
      SendVideoData(videobuffer, numbytes);
      nextFrame();
    }
  else
    {
      nextFrame();
    }
}

/***********************************************************************
 *
 *  Application messages handler
 *
 ***********************************************************************/
void VideoControlMsg(RTPheader *header, void *data, int len)
{
  static VideoPluginArg dane;
  VideoPluginArg *cfg;
  int sender;
  short type;

  if(header->type != 2)
    {
      /*printf("Incompatible application version\n");*/
      return;
    }

  sender = header->ssrc;
  type = header->pt;

  memcpy(&dane, data, sizeof(VideoPluginArg));

  switch (type)
   { 
      case H263:
	if(!first[sender])
	  {
	    currentUser = (int)sender;
	    window = win[currentUser];
	    dpy_depth = depth[currentUser];
	    ProcessVideoData(data, len, 0, sender);
	  }
	break;

      case H261:
	if(!first[sender])
	  {
	    currentUser = (int)sender;
	    window = win[currentUser];
	    dpy_depth = depth[currentUser];
	    ProcessVideoData(data, len, 1, sender);
	  }
	 break;

      case YUV9:
	if(!first[sender])
	  {
	    currentUser = (int)sender;
	    window = win[currentUser];
	    dpy_depth = depth[currentUser];
	    ProcessVideoData(data, len, 2, sender);
	  }
	 break;

      case videoTagExit:
	/*
         if (timerId!=0) XtRemoveTimeOut(timerId);
	 timerId=0;
	 QuitPlugin();
	 */
         break;

      case videoTagConfirm:
	break;

      case videoTagPlay:
	 cfg = (VideoPluginArg *)data;
	 source_format=dane.format;
	 receiveStart(sender);
	 numUsers++;
	 lastMsgNum[sender] = 0;
         if (first[sender]) 
	   {
	     initdpy(sender, cfg->sourceType);
	     usrTimeInit(sender);
	     first[sender] = 0;
	   }
         SendSpecificMessage(videoTagConfirm);
	 SetMessage("Receiving video...");
         break;

       case videoTagStop:
	 receiveStop(sender);
	 numUsers--;
	 partyLeft(sender);
         break;

       case videoTagFrameDecoded:
	 break;

       case videoTagNewMember:
	 if(state == PLAY)
	   {
	     intraFrame = 0;
	     SendSpecificMessageToMember(sender, videoTagPlay);
	   }
         break;

       default:
	 printf("Unknown request in VideoControlMsg %d from %d!\n", type, sender);
	 break;
   }
    
}


/***********************************************************************
 *
 *  Plugin management section 
 *
 ***********************************************************************/
void starting()
{
  /*localHandle=ncsAppGetLocalHandle(ncsapp);*/
  /*numUsers = ncsAppGetNumMembers(ncsapp);*/

  SendSpecificMessage(videoTagNewMember);

}


void partyLeft(int no)
{
  if((!first[no]) && (win[no] != NULL))
    {
      XDestroyWindow(display, win[no]);    
      win[no] = 0;
      first[no] = 1;
    }
}

void retrieveConnection()
{
  char *ncs_path;
  char path[100];
  char conn_name[50];
  FILE *file;

  ncs_path = getenv("NCS_PATH");
  strcpy(path, ncs_path);
  strcat(path, "/Video/video.cfg");
  file = fopen(path, "r");
  if(file == NULL)
    {
      setModem();
      return;
    }

  fgets(conn_name, 50, file);

  setModem();

  if(strcmp(conn_name, "Modem 28.8") == 0)
    setModem();
  if(strcmp(conn_name, "ISDN") == 0)
    setISDN();
  if(strcmp(conn_name, "ISDN 2") == 0)
    setISDN2();
  if(strcmp(conn_name, "LAN") == 0)
    setLAN();
}

void saveConnection()
{
  char *ncs_path;
  char path[100];
  FILE *file;

  ncs_path = getenv("NCS_PATH");
  strcpy(path, ncs_path);
  strcat(path, "/Video/video.cfg");
  file = fopen(path, "w");
  if(file == NULL)
    {
      return;
    }

  switch(conn_type)
    {
    case M288:
      fprintf(file, "Modem 28.8");
      break;

    case ISDN:
      fprintf(file, "ISDN");
      break;

    case ISDN2:
      fprintf(file, "ISDN 2");
      break;

    case LAN:
      fprintf(file, "LAN");
      break;
    }
}

void QuitPlugin()
{
  /*  printf("Video Plugin: EXIT requested. Exiting...\n");
  printf("Average video data rate: %f bytes per frame\n",
	 (float)totalBytesSent/(float)totalFramesSent);*/
  if(state == PLAY)
    {
      state=STOP;
      sendStop();
      SendSpecificMessage(videoTagStop);
    }
  SendSpecificMessage(videoTagExit);
  shutdownEncoder();
  closecapture();
  ncsAppDettach(ncsapp);
  saveConnection();
  exit(0);
}
 
void introduceYourself()
{
  int i;

  /*
   * Start up NCS.
   */
  if((ncsapp = ncsAppAttach(VIDEO_PLUGIN_PORT)) == 0)
    {
      /*fprintf(stderr, "Application already started. Exiting...\n");*/
      exit(0);
    }

  ncsAppAddCallback(ncsapp, tagAppStarting, starting);
  ncsAppAddCallback(ncsapp, tagAppQuit, QuitPlugin);
  ncsAppAddCallback(ncsapp, tagAppDelMbr, partyLeft);

  for(i=0; i<MAX_USERS+1; i++)
    first[i]=1;

  quiet=1; 
  ld = &base; 

  framenum = 0;
  temp_ref = 0;
  prev_temp_ref= -1;
  state=STOP; 
  outputname = "";
  videobuffer=(char*)malloc(MAX_FRAME_SIZE);
  invideobuffer=(char*)malloc(MAX_FRAME_SIZE);
  vdata = malloc(MAX_FRAME_SIZE);
  avail=0;

  localHandle = MAX_USERS;

  retrieveConnection();

  recworkproc = XtAppAddWorkProc(app, recWorkProc, NULL);

   return;
}



/***********************************************************************
 *
 *  H263 display section 
 *
 ***********************************************************************/
void initdpy(int userno, unsigned char sourceType)
{
  MainWindow = XtWindow(toplevel);

  wlabel=(char*)malloc(128);
  init_display("", "Display", (int)MainWindow, userno, sourceType);
  free(wlabel);
}


initexpand()
{
  int cc, k, size;

  for (cc=0; cc<3; cc++) 
    {
      if (cc==0)
	size = coded_picture_width*coded_picture_height*4;
      else
	size = chrom_width*chrom_height*4;
    
      if (!(exlocalPict[cc] = (unsigned char *)malloc(size)))
	error("malloc failed\n");

      for (k=0;k<MAX_USERS+1;k++) 
	{
	  if (!(exnewframe[k][cc] = (unsigned char *)malloc(size)))
	    error("malloc failed\n");
	}
    }
}

void initdecoder(void)
{
  int i, k, cc, size;
  FILE *cleared;

  source_format = SF_QCIF;

  expand = 0;
  outtype = T_X11;

  /* clip table */
  if (!(clp=(unsigned char *)malloc(1024)))
    error("malloc failed\n");

  clp += 384;

  for (i=-384; i<640; i++)
    clp[i] = (i<0) ? 0 : ((i>255) ? 255 : i);

  /* MPEG-1 = TMN parameters */
  matrix_coefficients = 5;

  switch (source_format) {
    case (SF_SQCIF):
      horizontal_size = 128;
      vertical_size = 96;
      break;
    case (SF_QCIF):
      horizontal_size = 176;
      vertical_size = 144;
      break;
    case (SF_CIF):
      horizontal_size = 352;
      vertical_size = 288;
      break;
    case (SF_4CIF):
      horizontal_size = 704;
      vertical_size = 576;
      break;
    case (SF_16CIF):
      horizontal_size = 1408;
      vertical_size = 1152;
      break;
    default:
      printf("ERROR: Illegal input format\n");
      exit(-1);
      break;
  }


  mb_width = horizontal_size/16;
  mb_height = vertical_size/16;
  coded_picture_width = horizontal_size;
  coded_picture_height = vertical_size;
  chrom_width =  coded_picture_width>>1;
  chrom_height = coded_picture_height>>1;
  blk_cnt = 6;

  for (cc=0; cc<3; cc++) {
    if (cc==0)
      size = coded_picture_width*coded_picture_height;
    else
      size = chrom_width*chrom_height;

    for (k=0;k<MAX_USERS+1;k++) 
      {
	if (!(refframe[k][cc] = (unsigned char *)malloc(size)))
	  error("malloc failed\n");
	if (!(oldrefframe[k][cc] = (unsigned char *)malloc(size)))
	  error("malloc failed\n");
	if (!(bframe[k][cc] = (unsigned char *)malloc(size)))
	  error("malloc failed\n");
      }
  }

  for (cc=0; cc<3; cc++) {
    if (cc==0) {
      size = (coded_picture_width+64)*(coded_picture_height+64);
      for (k=0;k<MAX_USERS+1;k++)
	{
	  if (!(edgeframeorig[k][cc] = (unsigned char *)malloc(size)))
	    error("malloc failed\n");
	  edgeframe[k][cc] = edgeframeorig[k][cc] + (coded_picture_width+64) * 32 + 32;
	}
    }
    else {
      size = (chrom_width+32)*(chrom_height+32);
      for (k=0;k<MAX_USERS+1;k++) 
	{
	  if (!(edgeframeorig[k][cc] = (unsigned char *)malloc(size)))
	    error("malloc failed\n");
	  edgeframe[k][cc] = edgeframeorig[k][cc] + (chrom_width+32) * 16 + 16;
	}
    }
  }

  /*  if (expand) 
    {*/
      initexpand();
      /*    }*/

  /* IDCT */
  if (refidct)
    init_idctref();
  else
    init_idct();

  window = 0;
  for(i=0; i<MAX_USERS+1; i++)
    {
      win[i] = NULL;
    }

  init_global_dpy();
}

void error(text)
char *text;
{
  fprintf(stderr,text);
  exit(1);
}

/* trace output */
void printbits(code,bits,len)
int code,bits,len; 
{
  int i;
  for (i=0; i<len; i++)
    printf("%d",(code>>(bits-1-i))&1);
}

/* end of H263 display section */


Boolean recWorkProc()
{
  struct timeval timeout;
  fd_set set;
  int size;
  int sockfd;
  char *data;
  char *msg;
  RTPheader *header;

  timeout.tv_sec = 0;
  timeout.tv_usec = 100000;

  sockfd = ncsAppGetFD(ncsapp, &set);
  if(select(sockfd+1, &set, 0, 0, &timeout) > 0)
    {
      ncsAppProcessIncoming(ncsapp, &set);
      size = ncsAppGetData(ncsapp, &set, &data);
      if(size > 0)
	{
	  size = size - RTP_HEADER_SIZE;
	  msg = data + RTP_HEADER_SIZE;
	  header = (RTPheader *)data;
	  VideoControlMsg(header, msg, size);
	  free(data);
	}

    }

  return False;
}

void YVU9toYUV411(unsigned char *input, 
                         unsigned char *y, unsigned char *u, unsigned char *v, 
                         int width, int heigth)
{
        int i, j;
        int QCIFwidth, QCIFheigth;
        unsigned char *cr1, *cr2;
	unsigned char pom;
        
        cr1 = &input[width * heigth];
        cr2 = &input[width * heigth*17/16];

        QCIFwidth = 176;
        QCIFheigth = 144;

        for(i=0; i<heigth; i++)
        {
                for(j=0; j<width; j++)
                {
                        y[i*QCIFwidth + j] = input[i*width + j];
                }
        }
        
        for(i=0; i<QCIFheigth/4; i++)
                for(j=0; j<QCIFwidth/4; j++)
                {
                        pom = cr2[i*width/4 + j];
                        u[i*QCIFwidth + j*2] = pom;
                        u[i*QCIFwidth + j*2 + 1] = pom;
                        u[(2*i+1)*(QCIFwidth/2) + j*2] = pom;
                        u[(2*i+1)*(QCIFwidth/2) + j*2 + 1] = pom;
        
                        pom = cr1[i*width/4 + j];
                        v[i*QCIFwidth + j*2] = pom;
                        v[i*QCIFwidth + j*2 + 1] = pom;
                        v[(2*i+1)*(QCIFwidth/2) + j*2] = pom;
                        v[(2*i+1)*(QCIFwidth/2) + j*2 + 1] = pom;
                }

  }


void setModem()
{
  statSetFpsLimit(4);
  code_type = H263;
  resetIntraMode();
  allintra = 0;
  conn_type = M288;
}

void setISDN()
{
  statSetFpsLimit(5);
  code_type = H261;
  resetIntraMode();
  allintra = 0;
  conn_type = ISDN;
}

void setISDN2()
{
  statSetFpsLimit(10);
  code_type = H261;
  resetIntraMode();
  allintra = 0;
  conn_type = ISDN2;
}

void setLAN()
{
  statSetFpsLimit(10);
  code_type = H261;
  setIntraMode();
  allintra = 1;
  conn_type = LAN;
}


