/*
 * File         : decode.c
 * Author       : Tomasz Stachowiak
 * Created      : Tue Aug 11 16:53:15 1998
 * Revision     : $Revision$
 * Updated      : $Date$
 * Copyright: Northeast Parallel Architectures Center
 *            at Syracuse University 1998
 */

#define GLOBAL                    /* H.263 includes */
#include <stdlib.h>
#include <string.h>

#include "config.h"
#include "tmndec.h"
#include "global.h"

/*#include "decode.h"*/

int currentUser;
static unsigned char *exlocalPict[3];

void initexpand(void);
char *getpicture(int *framenum, int type);

int H263initDecoder()
{
  int i, k, cc, size;
  FILE *cleared;

  source_format = SF_QCIF;

  expand = 0;
  outtype = T_X11;

  /* clip table */
  if (!(clp=(unsigned char *)malloc(1024)))
    error("malloc failed\n");

  clp += 384;

  for (i=-384; i<640; i++)
    clp[i] = (i<0) ? 0 : ((i>255) ? 255 : i);

  /* MPEG-1 = TMN parameters */
  matrix_coefficients = 5;

  switch (source_format) {
    case (SF_SQCIF):
      horizontal_size = 128;
      vertical_size = 96;
      break;
    case (SF_QCIF):
      horizontal_size = 176;
      vertical_size = 144;
      break;
    case (SF_CIF):
      horizontal_size = 352;
      vertical_size = 288;
      break;
    case (SF_4CIF):
      horizontal_size = 704;
      vertical_size = 576;
      break;
    case (SF_16CIF):
      horizontal_size = 1408;
      vertical_size = 1152;
      break;
    default:
      printf("ERROR: Illegal input format\n");
      exit(-1);
      break;
  }


  mb_width = horizontal_size/16;
  mb_height = vertical_size/16;
  coded_picture_width = horizontal_size;
  coded_picture_height = vertical_size;
  chrom_width =  coded_picture_width>>1;
  chrom_height = coded_picture_height>>1;
  blk_cnt = 6;

  for (cc=0; cc<3; cc++) {
    if (cc==0)
      size = coded_picture_width*coded_picture_height;
    else
      size = chrom_width*chrom_height;

    for (k=0;k<MAX_USERS+1;k++) 
      {
	if (!(refframe[k][cc] = (unsigned char *)malloc(size)))
	  error("malloc failed\n");
	if (!(oldrefframe[k][cc] = (unsigned char *)malloc(size)))
	  error("malloc failed\n");
	if (!(bframe[k][cc] = (unsigned char *)malloc(size)))
	  error("malloc failed\n");
      }
  }

  for (cc=0; cc<3; cc++) {
    if (cc==0) {
      size = (coded_picture_width+64)*(coded_picture_height+64);
      for (k=0;k<MAX_USERS+1;k++)
	{
	  if (!(edgeframeorig[k][cc] = (unsigned char *)malloc(size)))
	    error("malloc failed\n");
	  edgeframe[k][cc] = edgeframeorig[k][cc] + (coded_picture_width+64) * 32 + 32;
	}
    }
    else {
      size = (chrom_width+32)*(chrom_height+32);
      for (k=0;k<MAX_USERS+1;k++) 
	{
	  if (!(edgeframeorig[k][cc] = (unsigned char *)malloc(size)))
	    error("malloc failed\n");
	  edgeframe[k][cc] = edgeframeorig[k][cc] + (chrom_width+32) * 16 + 16;
	}
    }
  }

  /*  if (expand) 
    {*/
      initexpand();
      /*    }*/

  /* IDCT */
  if (refidct)
    init_idctref();
  else
    init_idct();

  invideobuffer = malloc(10000);
  framenum = 0;
  ld = &base; 
  quiet=1; 
  temp_ref = 0;
  prev_temp_ref= -1;
  avail=0;
/*
  window = 0;
  for(i=0; i<MAX_USERS+1; i++)
    {
      win[i] = NULL;
    }

  init_global_dpy();
*/
 
  return 0;
}

void initexpand(void)
{
  int cc, k, size;

  for (cc=0; cc<3; cc++) 
    {
      if (cc==0)
	size = coded_picture_width*coded_picture_height*4;
      else
	size = chrom_width*chrom_height*4;
    
      if (!(exlocalPict[cc] = (unsigned char *)malloc(size)))
	error("malloc failed\n");

      for (k=0;k<MAX_USERS+1;k++) 
	{
	  if (!(exnewframe[k][cc] = (unsigned char *)malloc(size)))
	    error("malloc failed\n");
	}
    }
}

char *H263decodeFrame(int user_id, char *frame, int len)
{
  char *buf;
  int tmp_source_format = source_format;

  currentUser = user_id;
  memcpy(invideobuffer, frame, len);
  invideooffset=0;
  avail = len;
  initbits();

  if (getheader())
    {
      if(source_format != tmp_source_format)
	/* add restarting decoder procedure here */
	printf("Error: Invalid source format!\n");
      buf = (char *)getpicture(&framenum, 0);
      framenum++;
    }
  else
    return NULL;
  
  return buf;
}
