/************************************************************************
 *
 *  main.c, main module of tmn (TMN encoder).
 *  tmn is an H.263 encoder somewhat based on the Test Model Near-term
 *  (TMN5) in the ITU-T LBC Experts Group.
 *
 *  Copyright (C) 1996  Telenor R&D, Norway
 *        Karl Olav Lillevold <Karl.Lillevold@nta.no>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Karl Olav Lillevold               <Karl.Lillevold@nta.no>
 *  Telenor Research and Development
 *  P.O.Box 83                        tel.:   +47 63 84 84 00
 *  N-2007 Kjeller, Norway            fax.:   +47 63 81 00 76
 *
 *  Robert Danielsen                  e-mail: Robert.Danielsen@nta.no
 *  Telenor Research and Development  www:    http://www.nta.no/brukere/DVC/
 *  P.O.Box 83                        tel.:   +47 63 84 84 00
 *  N-2007 Kjeller, Norway            fax.:   +47 63 81 00 76
 *  
 ************************************************************************/

#include"sim.h"
#include <sys/time.h>

FILE *streamfile;
int start, end, frame_no, writediff;  

char* videobuffer;
int writeoffset=0;
int numbytes;

PictImage *prev_image = NULL;
PictImage *curr_image = NULL;
PictImage *curr_recon = NULL;
PictImage *prev_recon = NULL;

/* PB-frame specific */
PictImage *B_recon = NULL;
PictImage *B_image = NULL;

Pict *pic;
unsigned char *frame;
FILE *cleared;
  
#ifdef LESSIO
FILE *inpf;
#endif  

int i;
float mean_frame_rate, ref_frame_rate, frame_rate, seconds;
int first_loop_finished=0;
int total_frames_passed, PPFlag=0, targetrate;
int frames,bframes,pframes,wcopies,icopies, write_repeated,pdist=0,bdist=0;
/*  int start, end, frame_no, writediff;  */
int first_frameskip, chosen_frameskip, orig_frameskip, frameskip;
int QP,QPI;
Bits *bits;
Bits *total_bits;
Bits *intra_bits;
Results *res;
Results *total_res;
Results *b_res;
char *seqfilename;
char *outputfile;

#ifndef OFFLINE_RATE_CONTROL
float DelayBetweenFramesInSeconds;
int CommBacklog;
#else
PictImage *stored_image = NULL;
int start_rate_control;
#endif

extern int arith_used;

int image_no = 0;
int intraflag = 0;
struct timeval tstart,tstop;
int runtime;

/* Function Prototypes */
extern void decodeFrame(int n, int start);
void initbitsenc();

/*
 * Encoder initialization
 */
int H263initEncoder(int x, int y)
{
  int framel;
  
  pic = (Pict *)malloc(sizeof(Pict));

  bits = (Bits *)malloc(sizeof(Bits));
  total_bits = (Bits *)malloc(sizeof(Bits));
  intra_bits = (Bits *)malloc(sizeof(Bits));
  res = (Results *)malloc(sizeof(Results));
  total_res = (Results *)malloc(sizeof(Results));
  b_res = (Results *)malloc(sizeof(Results));
  seqfilename = (char *)malloc(sizeof(char)*100);
  outputfile =  (char *)malloc(sizeof(char)*100);

  headerlength = DEF_HEADERLENGTH;
 
#ifndef FASTIDCT
  init_idctref();
#endif

  /* Default variable values */
  advanced = DEF_ADV_MODE;
  syntax_arith_coding = DEF_SAC_MODE;
  pic->unrestricted_mv_mode = DEF_UMV_MODE;
  mv_outside_frame = DEF_UMV_MODE || DEF_ADV_MODE;
  long_vectors = DEF_UMV_MODE;
  pb_frames = DEF_PBF_MODE;

  QP = DEF_INTER_QUANT;
  QPI = DEF_INTRA_QUANT; 
  pic->BQUANT = DEF_BQUANT; 

  /* Format  */
  if((x == 176) && (y = 144))     
     pic->source_format = SF_QCIF;
  else
    if((x == 352) && (y = 288))     
      pic->source_format = SF_CIF;
    else
      pic->source_format = DEF_CODING_FORMAT;

  ref_frame_rate = (float)DEF_REF_FRAME_RATE;
  chosen_frameskip = DEF_FRAMESKIP + 1;
  orig_frameskip = DEF_ORIG_SKIP + 1;
#ifdef OFFLINE_RATE_CONTROL
  start_rate_control = DEF_START_RATE_CONTROL;
#else
  pic->target_frame_rate = (float)DEF_TARGET_FRAME_RATE;
#endif

  write_repeated = DEF_WRITE_REPEATED;
  pic->seek_dist = DEF_SEEK_DIST;
  pic->use_gobsync = DEF_INSERT_SYNC;
  start = DEF_START_FRAME;
  end = DEF_STOP_FRAME;

  targetrate = 0; 
  /* default is variable bit rate (fixed quantizer) will be used */

/* DECODE */
/*  videobuffer=(char*)malloc(10000); */
/* DECODE */
  intra_gap = 1000000;
  frames = 0;
  pframes = 0;
  bframes = 0;
  total_frames_passed = 0;
  pic->PB = 0;
  wcopies = icopies = 1;

  pic->TR = 0;
  pic->QP_mean = (float)0.0;

  pels=x;
  lines=y;
  
  cpels = pels/2;

#ifndef OFFLINE_RATE_CONTROL
  /* rate control variables */
  pic->bit_rate = targetrate;
  pic->src_frame_rate = (int)(ref_frame_rate / orig_frameskip);
  DelayBetweenFramesInSeconds = (float) 1.0/(float)pic->src_frame_rate;
  InitializeRateControl();
#endif

  if (QP == 0 || QPI == 0) {
    fprintf(stderr,"Warning:");
    fprintf(stderr,"QP is zero. Bitstream will not be correctly decodable\n");
  }

  if (ref_frame_rate != 25.0 && ref_frame_rate != 30.0) {
    fprintf(stderr,"Warning: Reference frame rate should be 25 or 30 fps\n");
  }

  frame_rate =  ref_frame_rate / (float)(orig_frameskip * chosen_frameskip);

  return 0;
}

void H263shutdownEncoder()
{
  /* Free memory */
  /*FreeImage(curr_recon);
  FreeImage(curr_image);*/
  free(bits);
  free(total_bits);
  free(intra_bits);
  free(res);
  free(total_res);
  free(b_res);
  free(pic);
  /*  exit(0);*/
}

/*
 * Coding INTRA frame
 */
int H263codeIntraFrame(unsigned char *image, char *buffer)
{
  videobuffer = buffer;

  /* Initialize bitcounters */
  initbitsenc ();
  
  /* correction T.S. 10/97 -- elimination of INTRA frame coding memory leak */
  if(curr_image != NULL)
    FreeImage(curr_image);
  if(curr_recon != NULL)
    FreeImage(curr_recon);


/* Intra image */
  curr_image = FillImage(image);
  pic->picture_coding_type = PCT_INTRA;
  pic->QUANT = QPI;

  curr_recon = CodeOneIntra(curr_image, QPI, bits, pic);

  if (arith_used) {
    bits->header += encoder_flush();
    arith_used = 0;
  }
  bits->header += alignbits (); /* pictures shall be byte aligned */

  AddBitsPicture(bits);
/*  printf("Coded INTRA bits: %d\n", bits->total);*/
  numbytes=bits->total/8;
  memcpy(intra_bits,bits,sizeof(Bits));
  ZeroBits(total_bits);
  ZeroRes(total_res);
  ZeroRes(b_res);
  
  /* number of seconds to encode */
  seconds = (end - start + chosen_frameskip) * orig_frameskip/ ref_frame_rate;

  
  /* compute first frameskip */
#ifdef OFFLINE_RATE_CONTROL
  first_frameskip = chosen_frameskip;
  frameskip = chosen_frameskip;
#else
  CommBacklog = intra_bits->total - 
    (int)(DelayBetweenFramesInSeconds * pic->bit_rate);

  if (pic->bit_rate == 0) {
    frameskip = chosen_frameskip;
  }
  else {  /* rate control is used */
    frameskip = 1;
    while ( (int)(DelayBetweenFramesInSeconds*pic->bit_rate) <= CommBacklog) {
      CommBacklog -= (int) ( DelayBetweenFramesInSeconds * pic->bit_rate );
      frameskip += 1;
    }
  }
  first_frameskip = frameskip;
#endif

  if (first_frameskip > 256)
    fprintf(stderr,"Warning: frameskip > 256\n");

  pic->picture_coding_type = PCT_INTER;
  pic->QUANT = QP;
  bdist = chosen_frameskip;

  /* always encode the first frame after intra as P frame.
     This is not necessary, but something we chose to make
     the adaptive PB frames calculations a bit simpler */
  if (pb_frames) {
    pic->PB = 0;
    pdist = 2*chosen_frameskip - bdist;
  }

  if (write_repeated) 
    icopies = chosen_frameskip;

  /*FreeImage(curr_recon);*/

  return numbytes;
}




int H263codeFrame(unsigned char **image, char *buffer)
{
  int framel;
  char *tmp_image = *image;

  videobuffer = buffer;
  writeoffset=0;

  prev_image = curr_image;
  prev_recon = curr_recon;
  
  image_no++;
  if (image_no % intra_gap == 0) 
    {
      intraflag = 1;
      pic->PB = 0;
    }
  else 
    {
      intraflag = 0;
    }


  /* Set QP to pic->QUANT from previous encoded picture */
  QP = pic->QUANT;
  
  if (!PPFlag) 
    {
      if (pic->PB) 
	{ 
	  bdist = frameskip;
	  pdist = 2*frameskip - bdist;
	  pic->TRB = bdist * orig_frameskip;
	  if (pic->TRB > 8)
	    fprintf(stdout,"distance too large for B-frame\n");
	  /* Read the frame to be coded as B */
	  *image=(unsigned char *)getFrame(pels, lines);
	  B_image = FillImage(*image);
	  first_loop_finished = 1;
	  if (frame_no + pdist <= end) 
	    {
	      *image=(unsigned char *)getFrame(pels, lines);
	    }
	  else 
	    {
	      pic->PB = 0; /* end of sequence, encode as P */
	      *image=(unsigned char *)getFrame(pels, lines);
	    }
	} /* pic->PB */
      else 
	{
	  *image=(unsigned char *)getFrame(pels, lines);
	}
      free(tmp_image);
      curr_image = FillImage(*image);

      if (pic->PB) 
	{
	  if (pic->TRB > 8 || !NextTwoPB(curr_image, B_image, prev_image, 
					 bdist, pdist, pic->seek_dist)) 
	    {
	      /* curr_image and B_image were not suitable to be coded
		 as a PB-frame - encoding as two P-frames instead */
	      pic->PB = 0;
#ifdef OFFLINE_RATE_CONTROL
	      stored_image = curr_image;
#else
	      FreeImage(curr_image);
#endif
	      frameskip = bdist;
	      
	      curr_image = B_image;
	      PPFlag = 1;
	    }
	  else 
	    {
	      frame_no += pdist;
	    }
	}
    } 
  else 
    {
      /* PPFlag is set when the second of the two P-frames 
         is due to be coded */
#ifdef OFFLINE_RATE_CONTROL
      curr_image = stored_image;
#else
      *image = (unsigned char *)getFrame(pels, lines);
      curr_image = FillImage(*image);
#endif
      pic->PB = 0;
      PPFlag = 0;
    }
  
  /* Temporal Reference is the distance between encoded frames compared
     the reference picture rate which is 25.0 or 30 fps */
  pic->TR += (( (frameskip+(pic->PB?pdist:0)) *orig_frameskip) % 256); 
  if (frameskip+(pic->PB?pdist:0) > 256)
    fprintf(stdout,"Warning: frameskip > 256\n");
  
  
  frames += (pic->PB ? 2: 1);
  bframes += (pic->PB ? 1 : 0);
  pframes += 1;
  
  if (pic->PB) 
    { /* Code two frames as a PB-frame */
      B_recon = InitImage(pels*lines);
    }
  
  if (!intraflag) 
    {
      curr_recon = InitImage(pels*lines);
      CodeOneOrTwo(curr_image, B_image, prev_image, prev_recon, 
		   QP, (bdist+pdist)*orig_frameskip, bits, pic, 
		   B_recon, curr_recon);
    }
  else 
    {
      curr_recon = CodeOneIntra(curr_image, QP, bits, pic);
    }

  if (targetrate != 0)
    fprintf(stdout,"Inter QP: %d\n", QP);
  fflush(stdout);
  
  if (arith_used) 
    {
      bits->header += encoder_flush();
      arith_used = 0;
    }

  bits->header += alignbits ();  /* pictures shall be byte aligned */
  AddBitsPicture(bits);
  AddBits(total_bits, bits);
  
#ifndef OFFLINE_RATE_CONTROL
  if (pic->bit_rate != 0 && pic->PB)
    CommBacklog -= (int) 
      ( DelayBetweenFramesInSeconds*pic->bit_rate ) * pdist;
  
  if (pic->bit_rate != 0) {
    UpdateRateControl(bits->total);
    
    CommBacklog += bits->total;
    frameskip = 1;
    CommBacklog -= (int) 
      (frameskip * DelayBetweenFramesInSeconds *pic->bit_rate);
    
    while ( (int)(DelayBetweenFramesInSeconds*pic->bit_rate) <= CommBacklog)
      {
        CommBacklog -= (int) ( DelayBetweenFramesInSeconds * pic->bit_rate );
        frameskip += 1;
      }
  }
#else
  /* Aim for the targetrate with a once per frame rate control scheme */
  if (targetrate != 0) 
    if (frame_no - start > (end - start) * start_rate_control/100.0) 
      /* when generating the MPEG-4 anchors, rate control was started
	 after 70% of the sequence was finished.
	 Set start_rate_control with option "-R <n>" */
      
      pic->QUANT = FrameUpdateQP(total_bits->total + intra_bits->total,
				 bits->total / (pic->PB?2:1),
				 (end-frame_no) / chosen_frameskip + PPFlag,
				 QP, targetrate, seconds);
  frameskip = chosen_frameskip;
#endif
  
  if (pic->PB) 
    {
      if (write_repeated) 
        wcopies = pdist;
      FreeImage(B_image);
      FreeImage(B_recon);
    }
  
  if (write_repeated) 
    wcopies = (pb_frames ? bdist : frameskip);
  
  if (pb_frames)
    pic->PB = 1;
  
  FreeImage(prev_image);
  FreeImage(prev_recon);
  fflush(stdout);
  framel=bits->total/8;
  /*    printf("Coded Frame has %d bytes\n", numbytes);*/
  writeoffset=0;
  
  return framel;
}

/**********************************************************************
 *
 *	Name:        NextTwoPB
 *	Description:    Decides whether or not to code the next
 *                      two images as PB
 *      Speed:          This is not a very smart solution considering
 *                      the encoding speed, since motion vectors
 *                      have to be calculation several times. It 
 *                      can be done together with the normal
 *                      motion vector search, or a tree search 
 *                      instead of a full search can be used.
 *	
 *	Input:	        pointers to previous image, potential B- 
 *                      and P-image, frame distances
 *	Returns:        1 for yes, 0 otherwise
 *	Side effects:
 *
 *	Date: 950824	Author: Karl.Lillevold@nta.no
 *
 ***********************************************************************/

int NextTwoPB(PictImage *next2, PictImage *next1, PictImage *prev, 
              int bskip, int pskip, int seek_dist)
{
  int adv_is_on = 0, mof_is_on = 0, lv_is_on = 0;
  int psad1, psad2, bsad, psad;
  MotionVector *MV[6][MBR+1][MBC+2];
  MotionVector *mvp, *mvbf, *mvbb;
  int x,y;
  int i,j,k,tmp;

  /* Temporarily disable some options to simplify motion estimation */
  if (advanced) {
    advanced = OFF;
    adv_is_on = ON;
  }
  if (mv_outside_frame) {
    mv_outside_frame = OFF;
    mof_is_on = ON;
  }
  if (long_vectors) {
    long_vectors = OFF;
    lv_is_on = ON;
  }

  for (j = 1; j <= (lines>>4); j++) 
    for (i = 1; i <= (pels>>4); i++) 
      for (k = 0; k < 3; k++) {
        MV[k][j][i] = (MotionVector *)calloc(1,sizeof(MotionVector));
        /* calloc to avoid Checker warnings about reading of
           unitizalized memory in the memcpy's below */
      }

  mvbf = (MotionVector *)malloc(sizeof(MotionVector));
  mvbb = (MotionVector *)malloc(sizeof(MotionVector));

  psad = 0;
  psad1 = 0;
  psad2 = 0;
  bsad = 0;

  /* Integer motion estimation */
  for ( j = 1; j < lines/MB_SIZE - 1; j++) {
    for ( i = 1; i < pels/MB_SIZE - 1 ; i++) {
      x = i*MB_SIZE;
      y = j*MB_SIZE;

      /* picture order: prev -> next1 -> next2 */
      /* next1 and next2 can be coded as PB or PP */
      /* prev is the previous encoded picture */

      /* computes vectors (prev <- next2) */
      MotionEstimation(next2->lum,prev->lum,x,y,0,0,seek_dist,MV,&tmp);
      if (MV[0][j+1][i+1]->x == 0 && MV[0][j+1][i+1]->y == 0)
        MV[0][j+1][i+1]->min_error += PREF_NULL_VEC;
      /* not necessary to prefer zero vector here */
      memcpy(MV[2][j+1][i+1],MV[0][j+1][i+1],sizeof(MotionVector));

      /* computes sad(prev <- next1) */
      MotionEstimation(next1->lum,prev->lum,x,y,0,0,seek_dist,MV,&tmp);
      if (MV[0][j+1][i+1]->x == 0 && MV[0][j+1][i+1]->y == 0)
        MV[0][j+1][i+1]->min_error += PREF_NULL_VEC;
      memcpy(MV[1][j+1][i+1],MV[0][j+1][i+1],sizeof(MotionVector));

      /* computes vectors for (next1 <- next2) */
      MotionEstimation(next2->lum,next1->lum,x,y,0,0,seek_dist,MV,&tmp);
      if (MV[0][j+1][i+1]->x == 0 && MV[0][j+1][i+1]->y == 0)
        MV[0][j+1][i+1]->min_error += PREF_NULL_VEC;

      /* scales vectors for (prev <- next2 ) */
      mvp = MV[2][j+1][i+1];
      mvbf->x =   bskip * mvp->x / (bskip + pskip);
      mvbb->x = - pskip * mvp->x / (bskip + pskip);
      mvbf->y =   bskip * mvp->y / (bskip + pskip);
      mvbb->y = - pskip * mvp->y / (bskip + pskip);

      psad1 += MV[0][j+1][i+1]->min_error;
      psad2 += MV[1][j+1][i+1]->min_error;
      psad +=  mvp->min_error;

      /* computes sad(prev <- next1 -> next2) */
      bsad += SAD_MB_Bidir(next1->lum + x + y*pels,
           next2->lum + x + mvbb->x + (y + mvbb->y)*pels,
           prev->lum + x + mvbf->x + (y + mvbf->y)*pels,
           pels, INT_MAX);
    }
  }

  for (j = 1; j <= (lines>>4); j++) 
    for (i = 1; i <= (pels>>4); i++) 
      for (k = 0; k < 3; k++) 
        free(MV[k][j][i]);
  free(mvbf);
  free(mvbb);

  /* restore advanced parameters */
  advanced = adv_is_on;
  mv_outside_frame = mof_is_on;
  long_vectors = lv_is_on;

  /* do the decision */
  if (bsad < (psad1+psad2)/2)
    fprintf(stdout,"Chose PB - bsad %d, psad %d\n", 
            bsad, (psad1+psad2)/2);
  else
    fprintf(stdout,"Chose PP  - bsad %d, psad %d\n", 
            bsad, (psad1+psad2)/2);

  if (bsad < (psad1 + psad2)/2)
    return 1;
  else 
    return 0;
}
