/***************************************************************************
 * VideoConferencing Tool 
 *      on Linux X Window (with v4l2 driver)
 *      all rights reserved 
 *      by Bo-kyun Na (bkna@npac.syr.edu), IntellActive Research Inc.
 ***************************************************************************
 *	gcc -o vdo -L/usr/X11R6/lib/ -lXt -lXaw -Wall vdo.c
 ***************************************************************************/

#define X_DEPTH  32
/*  Set these according to your capture device set-up */
#define MY_DEVICE "/dev/video0"
#define MY_STD	  "NTSC"	/*-> "PAL" or "NTSC" */
#define MY_WIDTH  360
#define MY_HEIGHT 240
#define MY_INPUT  0		/* 0 for MXC, 1 for VID, and 2 for SVHS */

/*  MY_PIXELFORMAT should match your X Window screen mode  */
/*  If MY_PIXELFORMAT is not defined, capturing will be set to the
    screen format automatically (I hope)  */
// #define MY_PIXELFORMAT V4L2_PIX_FMT_RGB565 
/* V4L2_PIX_FMT_YUYV */
/* V4L2_PIX_FMT_GREY */
/* V4L2_PIX_FMT_YUV420 */
/* V4L2_PIX_FMT_RGB555 */
/* V4L2_PIX_FMT_RGB565 */
/* V4L2_PIX_FMT_BGR24 */
/* V4L2_PIX_FMT_BGR32 */

/*  Uncomment the following to test stuff  */
#define TEST_INPUTS      /* enumerate the inputs */
#define TEST_FORMATS     /* enumerate the image formats */
#define TEST_CONTROLS    /* enumerate the controls */
#define TEST_CAPTURE     /* test video capture */

#define STREAMBUFS	4

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/mman.h>
#include <errno.h>

/* These are needed to use the Videum driver */
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/videodev.h>  /* Video for Linux Two */

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Porthole.h>

typedef struct tag_vimage {
  struct v4l2_buffer vidbuf;
  char		     *data;
  XImage	     *ximage;
} VIMAGE;

VIMAGE	vimage[STREAMBUFS];
int	vid;
int	x_depth;


__u32 frame_buffer_format (Display *d, Window w)
{
  __u32	fbf = V4L2_PIX_FMT_RGB565;

  XWindowAttributes wa;
  XGetWindowAttributes(d, w, &wa);

  if (wa.depth == 15)
    fbf = V4L2_PIX_FMT_RGB555;
  else if (wa.depth == 16) 
    fbf = V4L2_PIX_FMT_RGB565;
  else if (wa.depth == 24) 
    fbf = V4L2_PIX_FMT_BGR32;
  else if (wa.depth == 32) 
    fbf = V4L2_PIX_FMT_BGR32;
  else 
    printf("Unrecognized display depth. You may get an X Windows error.\n");

  return fbf;
}


void quit (Widget w, XtPointer client, XtPointer call)
{
  struct v4l2_performance	perf;
  
  if (vid >= 0) {
    if (ioctl(vid, VIDIOC_G_PERF, &perf) != 0)
      printf("G_PERF returned error code %d\n",errno);
    else
      printf("Captured %d frames\n", perf.frames);
  }
  exit(0);
}

void printctrl (struct v4l2_queryctrl *qc)
{	
  printf("  %-12s %-6s %8d %8d %7d %8d  %s\n", qc->name,
	 (qc->type == V4L2_CTRL_TYPE_MENU) ? "(menu)" : 
	 (qc->type == V4L2_CTRL_TYPE_BOOLEAN) ? "(bool)" : "(int)",
	 qc->minimum, qc->maximum, qc->step, qc->default_value, qc->catname);
}

/* *************************************************************************
 * Function : int main (int argc, char *argv[])
 *
 * ************************************************************************* */
int main (int argc, char **argv)
{
  XtAppContext		app;
  Widget		w_toplevel;
  Widget		w_topbox;
  Widget		w_menubox;
  Widget		w_quit;
  Widget		w_viewport;

  char			my_device[64];
  int			err;
  int			i;
  struct v4l2_capability	cap;
  struct v4l2_requestbuffers    req;
  struct v4l2_format	        fmt, fmt_cap;
  struct v4l2_captureparm	parm;
  struct v4l2_standard	        std;
  int			        input;

#ifdef TEST_CONTROLS
  struct v4l2_queryctrl	qc;
#endif

  struct v4l2_buffer	tempbuf;
  GC			imagegc;
  int			argi;

  /*->	Put in the device node name */
  strcpy(my_device, MY_DEVICE);
  for (argi = 1; argi < argc; ++argi) {
    if (strcmp(argv[argi], "-b") == 0) {
      x_depth = 32;
      continue;
    }
    strcpy(my_device, argv[argi]);
  }

  vid = open(my_device, O_RDONLY);
  if (vid < 0) {
    printf("No video device \"%s\"\n", my_device);
    return 1;
  }

  x_depth = 32;
  req.count = 2;		/* # of requested window displays */

  if (ioctl(vid, VIDIOC_QUERYCAP, &cap)) {
    printf("QUERYCAP returned error %d\n", errno);
    return 1;
  }
  printf("Device node:  %s\n", my_device);
  printf("Device:       %s\n", cap.name);
  printf("Capabilities:");
  if (cap.flags & V4L2_FLAG_READ)       printf(" capture via read();");
  if (cap.flags & V4L2_FLAG_STREAMING)  printf(" streaming capture;");
  if (cap.flags & V4L2_FLAG_PREVIEW)    printf(" automatic preview;");
  if (cap.flags & V4L2_FLAG_SELECT)     printf(" select() call;");
  if (cap.flags & V4L2_FLAG_TUNER)      printf(" tuner;");
  if (cap.flags & V4L2_FLAG_MONOCHROME) printf(" monochrome only;");
  printf("\n");
  if (cap.type != V4L2_TYPE_CAPTURE) {
    printf("Not a capture device.\n");
    return 1;
  }

#ifdef TEST_INPUTS
  printf("Video inputs on this device:\n");
  for (i = 0, err = 0; err == 0; ++i) {	
    struct v4l2_input inp;
    inp.index = i;
    err = ioctl(vid, VIDIOC_ENUMINPUT, &inp);
    if (!err)
      printf("  %d: \"%s\" is a%s, %s audio\n", i, inp.name,
	     (inp.type == V4L2_INPUT_TYPE_TUNER) ?
	     " TV tuner" : "n analog input",
	     (inp.capability & V4L2_INPUT_CAP_AUDIO) ? "has" : "no");
  }

#ifdef MY_INPUT
  input = MY_INPUT;
  if (ioctl(vid, VIDIOC_S_INPUT, input))
    printf("S_INPUT returned error %d\n",errno);
#endif
#endif

  printf("Video standards supported:");
  for (i = 0, err = 0; err == 0; ++i) {
    struct v4l2_enumstd	estd;
    estd.index = i;
    err = ioctl(vid, VIDIOC_ENUMSTD, &estd);
    if (!err) {
      printf("%c %s", i?',':' ', estd.std.name);
      if (strcmp(MY_STD, estd.std.name) == 0)
	std = estd.std;
    }
  }
  printf("\n");

#ifdef TEST_FORMATS
  printf("Video capture image formats supported:\n");
  for (i = 0, err = 0; err == 0; ++i) {
    struct v4l2_fmtdesc	fmtd;
    fmtd.index = i;
    err = ioctl(vid, VIDIOC_ENUM_CAPFMT, &fmtd);
    if (!err)
      printf("  %d: %s  (%s)\n", i, fmtd.description,
	     (fmtd.flags & V4L2_FMT_FLAG_COMPRESSED) ?
	     "compressed" : "uncompressed");
  }
#endif

#ifdef TEST_CONTROLS
  printf("Controls supported:\n");
  printf("  Label        type    Minimum  Maximum    Step  Default"
	 " Category\n");
  qc.id = V4L2_CID_BRIGHTNESS;
  if (ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0)
    printctrl(&qc);
  qc.id = V4L2_CID_CONTRAST;
  if (ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0)
    printctrl(&qc);
  qc.id = V4L2_CID_SATURATION;
  if (ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0)
    printctrl(&qc);
  qc.id = V4L2_CID_HUE;
  if (ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0)
    printctrl(&qc);
#endif

#ifndef TEST_CAPTURE
  return 0;
#endif

  if (!(cap.flags & V4L2_FLAG_READ)) {
    printf("Device does not support the read() call.\n");
    return 1;
  }

  ioctl(vid, VIDIOC_G_PARM, &parm);
  if (ioctl(vid, VIDIOC_S_PARM, &parm))
    printf("S_PARM returned error %d\n",errno);

  if (ioctl(vid, VIDIOC_S_STD, &std))
    printf("S_STD returned error %d\n",errno);

  ioctl(vid, VIDIOC_G_FMT, &fmt);
  fmt.width = MY_WIDTH;
  fmt.height = MY_HEIGHT;

#ifdef MY_PIXELFORMAT
  fmt.pixelformat = MY_PIXELFORMAT;
#endif
  if (ioctl(vid, VIDIOC_S_FMT, &fmt)) {
    printf("S_FMT returned error %d\n",errno);
    return 1;
  }

  w_toplevel = XtAppInitialize(&app, "xvideum", NULL, 0,
			       &argc, argv, NULL, NULL, 0);
  w_topbox =
    XtVaCreateManagedWidget("topbox", boxWidgetClass, w_toplevel,
			    XtNorientation, XtorientVertical, XtNborderWidth,
			    0, XtNhSpace, 0, XtNvSpace, 0, NULL);
  w_menubox =
    XtVaCreateManagedWidget("menubox", boxWidgetClass, w_topbox,
			    XtNorientation, XtorientHorizontal, XtNborderWidth,
			    0, XtNhSpace, 4, XtNvSpace, 1, NULL);
  w_quit =
    XtVaCreateManagedWidget("Quit!", commandWidgetClass, w_menubox,
			    XtNborderWidth, 0, NULL);
  XtAddCallback(w_quit, XtNcallback, quit, NULL);
  w_viewport =
    XtVaCreateManagedWidget("port", portholeWidgetClass, w_topbox, XtNwidth,
			    fmt.width, XtNheight, fmt.height, NULL);
  XtRealizeWidget(w_toplevel);

#ifndef MY_PIXELFORMAT
  ioctl(vid, VIDIOC_G_FMT, &fmt);
  //  fmt.depth = 32;
  printf ("fmt.depth = %d, pixformat = %4.4s \n", fmt.depth, (char *)&fmt.pixelformat);
  fmt.pixelformat = 
    frame_buffer_format(XtDisplay(w_viewport),
			DefaultRootWindow(XtDisplay(w_viewport)));

  err = ioctl(vid, VIDIOC_S_FMT, &fmt);
  if (err) {
    printf("S_FMT returned error %d\n",errno);
    return 1;
  }

  fmt_cap.width = fmt.width;
  fmt_cap.height = fmt.height;
printf ("width %d, height %d \n", fmt.width, fmt.height);
  fmt_cap.depth = 32;
  fmt_cap.pixelformat = V4L2_PIX_FMT_RGB32; // 565;
  fmt_cap.flags = fmt.flags;
  fmt_cap.bytesperline = fmt.bytesperline;
  fmt_cap.sizeimage = fmt.sizeimage;
  fmt_cap.priv = fmt.priv;

    if (ioctl(vid, VIDIOC_S_FMT, &fmt_cap)) { // set vidcapture image format 
      printf("S_FMT returned error %d\n", errno);
      exit (1);
    }
  
#endif

  vimage[0].data = malloc(fmt.sizeimage);
    if (vimage[0].data == NULL) {
      printf("malloc(%d) failed\n", fmt.sizeimage);
      return 1;
    }

  printf("Capturing %dx%dx%d \"%4.4s\" images\n", fmt.width, fmt.height,
	 fmt.depth, (char *)&fmt.pixelformat);
  printf("Images are %d bytes each\n", fmt.sizeimage);

  imagegc = XCreateGC(XtDisplay(w_viewport), XtWindow(w_viewport), 0, NULL);
  if (imagegc == NULL) {
    printf("XCreateGC failed.\n");
    return 1;
  }
  for (i = 0; i < req.count; ++i) {
    vimage[i].ximage = XCreateImage(XtDisplay(w_viewport),
				    DefaultVisual(XtDisplay(w_viewport), 0),
				    (x_depth == 32) ? 24 : x_depth, ZPixmap,
				    0, vimage[i].data, fmt.width, fmt.height, 
				    16, 0);
    if (vimage[i].ximage == NULL) {
      printf("No XImage\n");
      return 1;
    }
  }

  for (;;) {
    XEvent event;

    if (XtIsRealized(w_viewport)) {
      tempbuf.index = 0;
      read(vid, vimage[0].data, fmt.sizeimage);

      XPutImage(XtDisplay(w_viewport), XtWindow(w_viewport), imagegc,
              vimage[tempbuf.index].ximage, 0, 0, 0, 0, fmt.width, fmt.height);
    }
    else {
      XtAppPeekEvent(app, &event);
    }

    while (XtAppPending(app)) {
      XtAppNextEvent(app, &event);
      XtDispatchEvent(&event);
    }
  }

  close (vid);
  XCloseDisplay(XtDisplay(w_viewport));
  return 0;
}
