
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <semaphore.h>

#include "params.h"
#include "network.h"
#include "transceive.h"
#include "vdo.h"
#include "h261.h"

int verbose = 0;

void YUVtoRGB32 ();
void init_dither_tab ();


/* ********************************************************************
 * Thread : void *Thread_RxVideo(void *arg)
 *
 *
 * ******************************************************************** */
void main (int argc, char **argv)
{
  FILE *df;
  char datafile[80];

  int userno = 0;
  uchar *y, *u, *v;
  char inbuf[VDO_WIDTH * VDO_HEIGHT * 3/2]; 

  V4L2 v4l2;
  XWA xwa;
  VIMAGE vimage;

  int packetcnt = 0;
  int currentframe;
  int lastw, lasth;
  int i;
  unsigned int currentsession;
  unsigned short myflags;

  Parameters *param;
  TransceiveState *receiver;
  PacketType packets[MAXPACKETS];	/* circular buffer of rec'vd packets */
  PacketType *packetlist[MAXPACKETS];	/* packets for this frame */
  PacketType *curpacket;
  MsgType *message;

  xwa.xdepth = 0;

  /* .... initialize everything ....  */
  param = (Parameters *) init_RxVidParam(argc, argv);
  receiver = initialize_receive(param, VIDEO);

  y = inbuf;
  u = &inbuf[VDO_WIDTH * VDO_HEIGHT];
  v = &inbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];

  param->data_via_file = 0;	/* Save data to a file */
  receiver->session = 1;	/* choose remote host to receive */
  //  param->codec = RAW;

  /* 
  (char *) receiver->name[receiver->session].sin_addr = 
                                  (char *) param->hostname[receiver->session];
  printf ("remotehost = %s\n", receiver->name[receiver->session]);
  */

  /* Initialization of image display  */
  BV_init_display (&xwa, &v4l2, argc, argv, &vimage);

  /* Init. of Color Conversion */
  init_dither_tab ();

  init_codec ();		/* Initialization of codec of H261 */
  init_decoder ();		/* Initilization of H261 decoder */

  if (param->data_via_file) {
    strcpy (datafile, "vimage.raw");
    printf ("image file name is %s\n", datafile);
    df = fopen(datafile, "w");
  }

  /* .... receive video ....      */
  currentframe = 0;
  currentsession = 0;
  receiver->session = 0;
  lastw = param->width;
  lasth = param->height;
   
  myflags = 0xffff;
  for (i = 0; i < MAXPACKETS; i++) packetlist[i] = NULL;

  for (;;) {
    message = (MsgType *) BV_VDO_receive (receiver, message);
    
    if (param->codec == H261) {
      /* Decode of recvd message and conversion to RGB from YUV  */
      decode_frame (message->data, message->header.size, inbuf, 
		    ((curpacket->header.frame % 10) == 0) ? 1 : 0, userno);
      YUVtoRGB32(y, u, v, vimage.data, VDO_WIDTH, VDO_HEIGHT);
      //	memcpy(vimage->data, message->data, message->header.size); 
    } 
    else if (param->codec == H263) {

    }
    else if (param->codec == JPEG) {

    }
    else if (param->codec == MPEG) {

    }
    else if (param->codec == RAW) {
      memcpy(vimage.data, message->data, message->header.size); 
    }
    else {
      printf ("Error on creating message. Specify the codec type\n");
      exit (0);
    }

    //      if (verbose)
    printf ("Rx: The %dth image frame w/ size = %d bytes\n", 
	    message->header.frame, message->header.size);

    /* Display images */
    BV_display_vdoframe (&xwa, &vimage, &v4l2);

    /* Store received video data as a file */
    if (param->data_via_file)	
      fwrite(vimage.data, 1, message->header.size, df);

  }
}


