/* =========================================================================
 * FILE       : TxODO.c
 * -----------:-------------------------------------------------------------
 * PROGRAMMER : Bo-kyun Na (bkna@npac.syr.edu), IntellActive Research Inc.
 *            : 1999.10.12.
 * -----------:-------------------------------------------------------------
 * ********** :     All rights are reserved by Bo-kyun Na.        **********
 * -----------:-------------------------------------------------------------
 * Revision   : 
 * -----------:-------------------------------------------------------------
 * This example records audio from /dev/dsp device, transmits compressed
 * audio over the local network.
 * ========================================================================= */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>		/* only for linux */
#include <malloc.h>
#include <sys/types.h>
#include <sys/param.h>

#include "network.h"
#include "params.h"
#include "transceive.h"
#include "odo.h"

extern int transmit_packet (TransceiveState *tstate, PacketType *packet);
void BV_odo_packetize (unsigned char *data, int msg_sz, 
		       TransceiveState *tstate, Parameters *param);


/* *************************************************************************
 * Function    : main ()
 * ------------:------------------------------------------------------------
 * Inputs      : none
 * Outputs     : int 
 * Description : void *Thread_TxAudio (void *arg)
 * ************************************************************************* */
int main(int argc, char **argv)
{
  int fd;			/* sound device file descriptor */
  int status;			/* return status of system calls */
  int rd, wr;			/* size in byte unit */
  int odo_frame;

  char dev_odo[64];		/* device driver of audio device */
  unsigned char *buf;		/* this buffer holds the digitized audio */

  /* Network parameters */
  Parameters *param;
  TransceiveState *tstate;


  /* audio device driver */
  strcpy (dev_odo, ODO_DEVICE);		

  odo_frame = ODO_LENGTH * ODO_RATE * ODO_SIZE * ODO_CHANNELS / 8;
  buf = (unsigned char *) malloc(odo_frame);
  if (buf == NULL) {
    printf ("malloc fail: buf sizeof (buf) = %d\n", odo_frame);
    exit (1);
  }

  /* Initialization of network parameters */
  param = (Parameters *) init_TxVidParam (argc, argv);

  /* Initialization of transmission parameters */
  if (param->transmit)
    tstate = (TransceiveState *) initialize_transmit(param, AUDIO);

  /* Initialization of audio */
  fd = BV_ODO_initialize (dev_odo);

  for ( ; ; ) {			/* loop until Control-C */
    rd = BV_ODO_record (fd, buf, odo_frame);
    printf ("Record size is %d\n", rd);

    BV_odo_packetize (buf, odo_frame, tstate, param);

    //    wr = BV_ODO_play (fd, buf, odo_frame);

    /* wait for playback to complete before recording again */
    status = ioctl(fd, SOUND_PCM_SYNC, 0); 
    if (status == -1)
      perror("SOUND_PCM_SYNC ioctl failed");
  }
}



/* *************************************************************************
 * Function    : Bv_odo_packetize ()
 * ------------:------------------------------------------------------------
 * Inputs      : 
 * Outputs     : 
 * Description : 
 * ************************************************************************* */
void BV_odo_packetize (unsigned char *data, int msg_sz, 
		       TransceiveState *tstate, Parameters *param)
{
  PacketType packet;
  static int frame = 0;
  int count = 0;
  
  packet.header.frame = frame++;
  packet.header.session = tstate->session | (AUDIO_PACKET << 28);

  if (msg_sz > PACKETDATASIZE * MAXPACKETS) {
    if (verbose)
      fprintf(stderr, "Not sending frame %d due to size (%d) > maximum\n",
              frame, msg_sz);
  }
  else {
    while (msg_sz > 0) {
      int curbytes = 0;
      
      if (msg_sz <= PACKETDATASIZE) {
        curbytes = msg_sz;
	packet.header.flags = ((1 << count) - 1);
      }
      else {
        curbytes = PACKETDATASIZE;
        packet.header.flags = ~(1 << count);
      }
      
      packet.header.size = curbytes;
      count++;
      msg_sz -= curbytes;
      memcpy (packet.payload, data, curbytes);
      data += curbytes;

      if (verbose)
	printf ("Tx: AUDIO w/ size %d\n", msg_sz);

      transmit_packet(tstate, &packet);

      if (param->polling)
        poll(0, 0, param->polling);
    }
  }
}
