
#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <semaphore.h>

#include "params.h"
#include "network.h"
#include "transceive.h"
#include "vdo.h"
#include "h261.h"

struct pollfd pollfds[2];	/* polling file descriptors for socket */
unsigned char *clp;

extern int verbose;


/* ********************************************************************
 * Thread : void *Thread_RxVideo(void *arg)
 *
 *
 * ******************************************************************** */
void main (int argc, char **argv)
{
  FILE *df;
  char datafile[80];

  int userno = 0;
  uchar *y, *u, *v;
  char inbuf[VDO_WIDTH * VDO_HEIGHT * 3/2]; 

  V4L2 v4l2;
  XWA xwa;
  VIMAGE vimage;

  int packetcnt = 0;
  int currentframe;
  int lastw, lasth;
  int i;
  unsigned int currentsession;
  unsigned short myflags;

  Parameters *param;
  TransceiveState *receiver;
  PacketType packets[MAXPACKETS];	/* circular buffer of rec'vd packets */
  PacketType *packetlist[MAXPACKETS];	/* packets for this frame */
  PacketType *curpacket;
  MsgType message;

  xwa.xdepth = 0;

  /* .... initialize everything ....  */
  param = (Parameters *) init_RxVidParam(argc, argv);
  receiver = initialize_receive(param, VIDEO);

  y = inbuf;
  u = &inbuf[VDO_WIDTH * VDO_HEIGHT];
  v = &inbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];

  param->data_via_file = 0;	/* Save data to a file */
  receiver->session = 1;	/* choose remote host to receive */
  //  param->codec = RAW;

  /* 
  (char *) receiver->name[receiver->session].sin_addr = 
                                  (char *) param->hostname[receiver->session];
  printf ("remotehost = %s\n", receiver->name[receiver->session]);
  */

  /* Initialization of image display  */
  BV_init_display (&xwa, &v4l2, argc, argv, &vimage);

  init_codec ();		/* Initialization of codec of H261 */
  init_decoder ();		/* Initilization of H261 decoder */

  /* clip table */
  if (!(clp = (unsigned char *) malloc(1024))) error("malloc failed\n");
  clp += 384;
  for (i=-384; i<640; i++) clp[i] = (i<0) ? 0 : ((i>255) ? 255 : i);

  if (param->data_via_file) {
    strcpy (datafile, "vimage.raw");
    printf ("image file name is %s\n", datafile);
    df = fopen(datafile, "w");
  }

  /* .... receive video ....      */
  currentframe = 0;
  currentsession = 0;
  receiver->session = 0;
  lastw = param->width;
  lasth = param->height;
   
  myflags = 0xffff;
  for (i = 0; i < MAXPACKETS; i++) packetlist[i] = NULL;

  for (;;) {
    //    sem_wait (&VR);
    //    sem_post (&VS); 

    //   fprintf (stderr, "RecvVidThread ");

    curpacket = &packets[packetcnt++];
    packetcnt %= MAXPACKETS; 

    /* Get video packet from remote if available. */
#ifdef XEVENT
    receive_packet(receiver, curpacket, &viewport);
#else
     receive_packet(receiver, curpacket);
#endif

     if (verbose)
       printf ("cursession = %d\n", curpacket->header.session);

    if (curpacket->header.frame < currentframe) {
      /* if this packet is from an earlier frame, disregard it. */
      if (verbose)
	fprintf(stderr, "received stale packet %d, current frame %d\n",
		curpacket->header.frame, currentframe);
      //    thrd_stop (&AR); 
      continue;
    }

    if (curpacket->header.frame > currentframe) {
      /* If the frame number increased then we probably missed the last
       * packet of the previous frame. We display what we can drop of the
       * previous frame if '-b' option for verbose was selected, otherwise 
       * we just drop the partial frame.                                  
       */
      if (myflags != 0) {
	if (verbose)
	  fprintf (stderr, "dropping %d due to getting packet %d (flags %d)\n",
		   currentframe, curpacket->header.frame, myflags);
      }

      for (i = 0; i < MAXPACKETS; i++) 
	packetlist[i] = NULL;
      myflags = 0xffff;
      currentframe = curpacket->header.frame;
    }

    /* Save the packet for decoding */
    packetlist[packetnum(curpacket->header.flags)] = curpacket;
    myflags &= curpacket->header.flags;

    if (myflags == 0) {
      if (param->codec == H261) {
	create_message (&message, packetlist);

      /* Decode of recvd message and conversion to RGB from YUV  */
	decode_frame (message.data, message.header.size, inbuf, 
		      ((curpacket->header.frame % 10) == 0) ? 1 : 0, userno);
	ConvertYUVtoRGB(y, u, v, vimage.data, VDO_WIDTH, VDO_HEIGHT);
	//	memcpy(vimage.data, message.data, message.header.size); 
      } 
      else if (param->codec == H263) {
	create_message(&message, packetlist);
      }
      else if (param->codec == JPEG) {
	create_message(&message, packetlist);
      }
      else if (param->codec == MPEG) {
	create_message(&message, packetlist);
      }
      else if (param->codec == RAW) {
	create_message(&message, packetlist);
	memcpy(vimage.data, message.data, message.header.size); 
      }
      else {
	printf ("Error on creating message. Specify the codec type\n");
	exit (0);
      }

      //      if (verbose)
	printf ("Rx: The %dth image frame w/ size = %d bytes\n", 
		message.header.frame, message.header.size);

      /* Display images */
      BV_display_vdoframe (&xwa, &vimage, &v4l2);

      /* Store received video data as a file */
      if (param->data_via_file)	
	fwrite(vimage.data, 1, message.header.size, df);

    }
    //    thrd_stop (&AR); 
  }
}


