
/* *************************************************************************
 *  File : MTvodo.c
 * -------------------------------------------------------------------------
 *  All rights are reserved by Bo-kyun Na, IntellActive Research Inc.
 * -------------------------------------------------------------------------
 * This example reads an image from /dev/video, transmits compressed video 
 * over the local network, and optionally displays the image in Xwindow.
 * ************************************************************************* */

#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>
#include <semaphore.h>

#include "params.h"
#include "network.h"
#include "transceive.h"
#include "vdo.h"
#include "h261.h"

struct pollfd pollfds[2];	/* polling file descriptors for socket */
unsigned char *clp;

extern int verbose;


/* *************************************************************************
 * void *Thread_TxVideo (void *arg)
 *
 *
 * ************************************************************************* */
void main (int argc, char **argv)
{
  FILE *fp;
  char fname[80];
  int readable = 0, writable = 0;

  int i;

  Parameters *param;
  TransceiveState *tstate;

  int vid;
  char	devname[64];
  XWA xwa;
  V4L2 v4l2;
  VIMAGE vimage;

  int first = 0;
  int CISsize;
  char inbuf[VDO_WIDTH * VDO_HEIGHT * 3/2]; 
  char outbuf[VDO_WIDTH * VDO_HEIGHT * 3/2];
  char tmpbuf[VDO_WIDTH * VDO_HEIGHT * 3/2];
  unsigned char *y, *u, *v;
  unsigned char *y1, *u1, *v1;
  int userno = 0;

  /* Put in the device node name */
  strcpy(devname, VDO_DEVICE);

  /* Initialization of video devices */
  param = (Parameters *) init_TxVidParam (argc, argv);

  /* Process command line arguments */
  for (i = 1; i < argc; i++) {
    if (!strcmp (argv[i], "-C")) {
      //      switch (argv[++i]) {
      //      case "H261":
      //	docis = H261;
      //	cis_type = "H261";
      //	break;
      //      case "Jpeg" :
      //	docis = JPEG;
      //	cis_type = "Jpeg";
      //	break;
      //      case "Mpeg1":
      //	docis = MPEG;
      //	cis_type = "Mpeg1";
      //	break;
      //      default:
      //      }
    }
    if(!strcmp(argv[i], "-D")) {
      strcpy (devname, argv[++i]);
    }
    if (!strcmp(argv[i], "-i")) {
      strcpy (fname, argv[++i]);
      readable = 1;
      fp = fopen(fname, "r");
      if (fp == NULL) printf ("Can't open image file: %s\n", fname);
      else  printf ("Image file name is %s\n", fname);
    }
    if (!strcmp(argv[i], "-o")) {
      strcpy (fname, argv[++i]);
      writable = 1;
      fp = fopen(fname, "w");
      if (fp == NULL) printf ("Can't open image file: %s\n", fname);
      else  printf ("Image file name is %s\n", fname);
    }
    if(!strcmp(argv[i], "-r")) {
      //      rate = atoi(argv[++i]);
    }
    if(!strcmp(argv[i], "-s")) {
      //      width = atoi(argv[++i]);
      //      height = atoi(argv[++i]);
    }
    if(!strcmp(argv[i], "-x")) {
      //    exit_on_done = 1;
    }
  }

  xwa.xdepth = 0;		/* init. of depth on xwindow */

  /* Pointers for the conversion of RGBtoYUV */
  y = inbuf;
  u = &inbuf[VDO_WIDTH * VDO_HEIGHT];
  v = &inbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];

  y1 = tmpbuf;
  u1 = &tmpbuf[VDO_WIDTH * VDO_HEIGHT];
  v1 = &tmpbuf[VDO_WIDTH * VDO_HEIGHT * 5/4];

  /* Initialization of transmission parameters */
  if (param->transmit)
    tstate = (TransceiveState *) initialize_transmit(param, VIDEO);

  /* Function: Init. of video capture to get the video from my camera */
  vid = BV_init_vidcap (devname, &v4l2);

  /* Init. and format of xwindow display */
  BV_init_display (&xwa, &v4l2, argc, argv, &vimage);

  init_codec ();		/* Initialization of codec of H261 */
  init_encoder ();		/* Initilization of H261 encoder */
  init_decoder ();

  /* clip table */
  if (!(clp = (unsigned char *) malloc(1024))) error("malloc failed\n");
  clp += 384;
  for (i=-384; i<640; i++) clp[i] = (i<0) ? 0 : ((i>255) ? 255 : i);

  /* Loop processing related to a video frame */
  for (;;) {
    //    sem_wait (&VS);
    //    fprintf (stderr, "SendVidThread ");

    if (readable)
      fread (vimage.data, v4l2.fmt.sizeimage, 1, fp);
    else
      BV_get_vdoframe (vid, vimage.data, &v4l2);

    /* display local video image */
    //    BV_display_vdoframe (&xwa, &vimage, &v4l2);

    /* Packetization and sending of video data */
    if (param->transmit) {
      if (param->codec == H261) {
	/* Conversion image pixel format */
	RGB16toYUV(vimage.data, y, u, v, VDO_WIDTH, VDO_HEIGHT);

	CISsize = ((first %= 10) == 0) ? 
	          p64EncodeIntraFrame (inbuf, outbuf):
	          p64EncodeFrame (inbuf, outbuf);
 
	/* transmit compressed image sequence data */
	BV_packetize_vdoframe(outbuf,CISsize,VDO_WIDTH,VDO_HEIGHT,tstate,param);

	/* Decode of recvd message and conversion to RGB from YUV  */
	//	decode_frame (outbuf, CISsize, tmpbuf, (first == 0) ? 1 : 0, userno);
	// ConvertYUVtoRGB(y1, u1, v1, vimage.data, VDO_WIDTH, VDO_HEIGHT);
	//	memcpy (vimage.data, inbuf, VDO_WIDTH * VDO_HEIGHT * 3/2 );

	ConvertYUVtoRGB(y, u, v, vimage.data, VDO_WIDTH, VDO_HEIGHT);

	/* Display compressed local images */
	BV_display_vdoframe (&xwa, &vimage, &v4l2);

	first ++;  
      }
      else if (param->codec == H263) {

      }
      else if (param->codec == JPEG) {

      }
      else if (param->codec == MPEG) {

      }
      else if (param->codec == RAW) {	/* transmit raw video data */
	BV_packetize_vdoframe (vimage.data, v4l2.fmt.sizeimage, 
			       VDO_WIDTH, VDO_HEIGHT, tstate, param);
      }
      else {
	printf ("Error on sending. Specifiy a correct codec type.\n");
	exit (0);
      }
    }

    /* Store compressed video data as a file */
    if (writable)	
      fwrite(vimage.data, 1, CISsize, fp);

    //    thrd_stop (&AR);   /* Thread stop */
  }
}
