
#include <stdio.h>
#include <stdlib.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

/* private prototypes */



/* X11 related variables */
static Display *display;
static Window window;
static GC GC_Instance;
static XImage *Ximg;
static unsigned char Pixel[256];


/* connect to server, create and map window,
 * allocate colors and (shared) memory
 */
void Initialize_Display_Process(char *name)
{
  int crv, cbu, cgu, cgv;
  int Y, Cb, Cr, R, G, B;
  int i;
  char dummy;
  int screen;
  Colormap cmap;
  int private;
  XColor xcolor;
  unsigned int fg, bg;
  char *wtitle = "Local";
  XSizeHints hint;
  XVisualInfo vinfo;
  XEvent xev;
  unsigned long tmp_pixel;
  XWindowAttributes xwa;

  display = XOpenDisplay(name);
  if (display == NULL) Error("Can not open display\n");
  screen = DefaultScreen(display);

  hint.x = 200;
  hint.y = 200;
  hint.width = horizontal_size;
  hint.height = vertical_size;
  hint.flags = PPosition | PSize;

  /* Get some colors */
  bg = WhitePixel (display, screen);
  fg = BlackPixel (display, screen);

  /* Make the window */
  if (!XMatchVisualInfo(display, screen, 8, PseudoColor, &vinfo)) {
    if (!XMatchVisualInfo(display, screen, 8, GrayScale, &vinfo))
      Error("requires 8 bit display\n");
  }

  window = XCreateSimpleWindow (display, DefaultRootWindow (display), hint.x,
				hint.y, hint.width, hint.height, 4, fg, bg);

  XSelectInput(display, window, StructureNotifyMask);

  /* Tell other applications about this window */
  XSetStandardProperties (display, window, wtitle, wtitle, None, NULL, 0, &hint);

  /* Map window. */
  XMapWindow(display, window);

  /* Wait for map. */
  do {
    XNextEvent(display, &xev);
  } while (xev.type != MapNotify || xev.xmap.event != window);

  XSelectInput(display, window, NoEventMask);

  /* allocate colors */
  GC_Instance = DefaultGC(display, screen);
  cmap = DefaultColormap(display, screen);
  private = 0;

  YUVtoRGB16 ();

  Ximg = XCreateImage (display, None, 8, ZPixmap, 0, &dummy,
		       Coded_Picture_Width, Coded_Picture_Height, 8, 0);
}


YUVtoRGB16 ()
{

  /* color allocation:
   * i is the (internal) 8 bit color number, it consists of separate
   * bit fields for Y, U and V: i = (yyyyuuvv), we don't use yyyy=0000
   * and yyyy=1111, this leaves 32 colors for other applications
   *
   * the allocated colors correspond to the following Y, U and V values:
   * Y:   24, 40, 56, 72, 88, 104, 120, 136, 152, 168, 184, 200, 216, 232
   * U,V: -48, -16, 16, 48
   *
   * U and V values span only about half the color space; this gives
   * usually much better quality, although highly saturated colors can
   * not be displayed properly
   *
   * translation to R,G,B is implicitly done by the color look-up table
   */
  for (i=16; i<240; i++) {
    /* color space conversion */
    Y  = 16*((i>>4)&15) + 8;
    Cb = 32*((i>>2)&3)  - 48;
    Cr = 32*(i&3)       - 48;

    Y = 76309 * (Y - 16); /* (255/219)*65536 */

    R = Clip[(Y + crv*Cr + 32768)>>16];
    G = Clip[(Y - cgu*Cb - cgv*Cr + 32768)>>16];
    B = Clip[(Y + cbu*Cb + 32786)>>16];

    /* X11 colors are 16 bit */
    xcolor.red   = R << 8;
    xcolor.green = G << 8;
    xcolor.blue  = B << 8;

    if (XAllocColor(display, cmap, &xcolor) != 0)
      Pixel[i] = xcolor.pixel;
    else {
      /* allocation failed, have to use a private colormap */
      if (private)
        Error("Couldn't allocate private colormap");

      private = 1;

      if (!Quiet_Flag)
        fprintf(stderr, "Using private colormap (%d colors were available).\n", i-16);

      /* Free colors. */
      while (--i >= 16) {
        tmp_pixel = Pixel[i];	/* because XFreeColors expects unsigned long */
        XFreeColors(display, cmap, &tmp_pixel, 1, 0);
      }

      /* i is now 15, this restarts the outer loop */

      /* create private colormap */
      XGetWindowAttributes(display, window, &xwa);
      cmap = XCreateColormap(display, window, xwa.visual, AllocNone);
      XSetWindowColormap(display, window, cmap);
    }
  }

}
