// tchord.h

// definitions for guitar chord database and sound functions

#ifndef TCHORD_H
#define TCHORD_H

// maximum number of chords, increase if needed
const int maxChords = 100;

// it's a 6 string guitar
const int numStrings = 6;

// show this many frets
const int numFrets = 8;

typedef char strings_t[numStrings];

// data structure for one guitar chord
typedef struct tutChord
{
  char longName[40];   // long name (e.g. "C Major"), must be unique
  char shortName[8];   // short name (e.g. "C")
                       // each string is either 0 (open), X (dead),
                       // or a fret number (first fret is 1)
  strings_t strings;   // finger for each string (ordered EADGBE)
  char sampleFile[80]; // sound sample filename
};

// type to hold array of chord names
typedef char *chordList_t[maxChords];

// class for a database of chords
class tutChords
{
public:
  tutChords() { fileName[0] = 0; numChords = 0; }
  tutChords(char *filename);
 ~tutChords() { fileName[0] = 0; numChords = 0; }
  void chordNames(chordList_t list);
  // these return -1 on error
  int load(char *fileName); // load chord file
  int save(char *fileName); // save chord file
  int play(char *longName); // play a chord
  int fingering(char *longName, strings_t &strings);

private:
  char fileName[80];         // database file name
  tutChord chord[maxChords]; // array of chords
  chordList_t chordList;     // list of long chord names
  int numChords;             // current number of chords
};

#endif
