static struct v4l2_queryctrl capture_control[] =
{
 {V4L2_CID_BRIGHTNESS, "Brightness", 0, 63, 1, 32, V4L2_CTRL_TYPE_INTEGER},
 {V4L2_CID_CONTRAST,   "Contrast",   0, 63, 1, 32, V4L2_CTRL_TYPE_INTEGER},
 {V4L2_CID_SATURATION, "Saturation", 0, 63, 1, 32, V4L2_CTRL_TYPE_INTEGER},
 /*{V4L2_CID_HUE,        "Hue",        0, 63, 1, 32, V4L2_CTRL_TYPE_INTEGER},*/
};
#define MAXCONTROLS	(sizeof(capture_control)/sizeof(capture_control[0]))
#define VCTRL_BRIGHTNESS	0
#define VCTRL_CONTRAST		1
#define VCTRL_SATURATION	2
#define VCTRL_HUE		3
static int
find_vctrl(int id)
{
        int i;
        for (i = MAXCONTROLS - 1; i >= 0; i--)
                if (capture_control[i].id == id)
                        break;
        return i;
}



struct video_source
{
	struct v4l2_input	input;
	int			control[MAXCONTROLS];
	struct v4l2_tuner	tuner;
	int			vcrmode;
};
/*  Indices into the array of video_source's */
#define VSOURCE_COMP		0
#define VSOURCE_SVIDEO		1
//#define VSOURCE_TUNER		3
#define VSOURCE_COUNT		2


/*  Per-open data for handling multiple opens on one device */
struct device_open
{
	int			isopen;
	int			noncapturing;
	struct capture_device	*dev;
};
#define MAX_OPENS	3

/*  Streaming data buffer  */
struct stream_buffer
{
	struct v4l2_q_node	qnode;
	struct v4l2_buffer	vidbuf;
	int			requested;
	__u8			*vaddress;  /* vmalloc() */
	struct scatter_node	*dma_list;  /* get_free_page() */
};
#define MAX_CAPTURE_BUFFERS	30
#define MAX_LOCKED_MEMORY	2000000

/*
 *	Capture device structure
 *
 *	One for each handled device in the system.
 *	This structure holds all the global information the driver
 *	needs about each device.
 */
struct capture_device
{
	struct v4l2_device	v;	/*  Must be first */
	char			shortname[16];
	int			is_registered;
	int			open_count;
	struct device_open	open_data[MAX_OPENS];
	int			capturing_opens;

/*	Per-bus index number for each device	*/
	int			index;
/*	General type of device  */
	int			type;
/*	Pointer to the pci_dev structure for this board	*/
	struct pci_dev		*pci;
/*	I/O Base address for non-PCI devices	*/
	unsigned int		iobase;
/*	Interrupts	*/
	int			irq;
	int			ints_enabled;
	struct tq_struct	tqnode_dpc;/* for Bottom Half routine */
	struct timer_list	tlnode;/* for polling interrupts */
	struct wait_queue	*new_video_frame;

/*	Video decoder stuff		*/
	struct video_decoder	videc;
	struct video_source	source[VSOURCE_COUNT];
	int			source_width;
	int			source_height;

/*	Client capture format and capture modes	*/
	struct v4l2_format	clientfmt;
	struct v4l2_captureparm	capture;
	int			input;/* which video source is selected */

/*	Hardware capture format	(see ucode_parms for width & height) */
	int			capture_bypp;
	int			capture_size;
	__u8			*capture_buffer;/* vmalloc() */
	int			capture_buffer_size;
	struct scatter_node	*capture_dma_list;/* get_free_page() */

/*	Capture state	*/
	int			ready_to_capture;
	int			grabber_enabled;
	int			capture_completed;
	unsigned long		time_acquired;/* millisecond time stamp */
	int			streaming;
	struct stream_buffer	stream_buf[MAX_CAPTURE_BUFFERS];
	int			stream_buffers_mapped;
	struct v4l2_queue	stream_q_capture;
	struct v4l2_queue	stream_q_done;
	struct timeval		stream_begin;
	unsigned long		stream_last_frame;
	__u8			*stream_capture_buffer;

/*	Image format conversions	*/
	struct translation	translation;
	__u8			*xlat_temp;/* vmalloc() */

/*	Performance statistics	*/
	struct v4l2_performance perf;

	/* frame counter for test images */
	int	h,m,s,f;
};
/*	Values for type field	*/
#define DEVICE_TYPE_0		0
/*	Extreme video dimensions	*/
#define MIN_WIDTH		32
#define MIN_HEIGHT		24
#define MAX_WIDTH		640
#define MAX_HEIGHT		288
#define MAX_FRAME_AGE		200 /* ms */


/*  These macros can be used to make device I/O operations atomic  */
/* static spinlock_t device_lock = SPIN_LOCK_UNLOCKED; */
/* #define BEGIN_CRITICAL_SECTION	\ */
/*         do{unsigned long flags;spin_lock_irqsave(&wavi_lock,flags) */
/* #define END_CRITICAL_SECTION	\ */
/* 	spin_unlock_irqrestore(&wavi_lock,flags);}while(0) */



/*
 *	D E V I C E   F U N C T I O N S
 */

static void
device_initialize(struct capture_device *dev)
{
	/*  TODO: Put hardware into a sensible state and	*/
	/*        do the one-time startup things		*/
}

static void
device_brightness(struct capture_device *dev, int x)
{
}

static void
device_contrast(struct capture_device *dev, int x)
{
}

static void
device_saturation(struct capture_device *dev, int x)
{
}

static void
device_tone_controls(struct capture_device *dev)
{
	int	*ctrl;
	ctrl = dev->source[dev->input].control;
	device_brightness(dev, ctrl[VCTRL_BRIGHTNESS]);
	device_contrast(dev, ctrl[VCTRL_CONTRAST]);
	device_saturation(dev, ctrl[VCTRL_SATURATION]);
/* 	device_hue(dev, ctrl[VCTRL_HUE]); */
}

static void
grabbing_enable(struct capture_device *dev, int x)
{
	/*  TODO: Start the grabbing hardware  */
}

static int
isqrt(unsigned int q)
{/*	A little integer square root routine */
	int		i;
	int		r;
	unsigned int	b2	= 0x40000000;
	unsigned int	t;

	for (i = 16, r = 0; i > 0 && q; --i)
	{
		t = ((unsigned int)r << i) + b2;
		if (t <= q)
		{
			q -= t;
			r |= (1 << (i - 1));
		}
		b2 >>= 2;
	}
	return r;
}

static unsigned long
current_time_ms(void)
{
	struct timeval	now;

	do_gettimeofday(&now);
	return now.tv_sec * 1000 + now.tv_usec / 1000;
}

/*
 *
 *	V I D E O   D E C O D E R S
 *
 */

static int
decoder_initialize(struct capture_device *dev)
{
	/*  Video decoder information fields	*/
	dev->videc.standards		= (1 << V4L2_STD_NTSC) |
					  (1 << V4L2_STD_PAL);
	dev->videc.ntsc_hskip		= 30;
	dev->videc.ntsc_vskip		= 12;
	dev->videc.ntsc_width		= 640;
	dev->videc.ntsc_height		= 240;
	dev->videc.ntsc_field_order	= 0;
	dev->videc.pal_hskip		= 62;
	dev->videc.pal_vskip		= 14;
	dev->videc.pal_width		= 768;
	dev->videc.pal_height		= 288;
	dev->videc.pal_field_order	= 0;
	dev->videc.preferred_field	= 0;

	dev->videc.num_inputs		= 2;

	dev->videc.decoder_is_stable	= 1;

	return 1;
}

static int
decoder_set_input(struct capture_device *dev, int i)
{
	dev->input = i;
	//dev->videc.decoder_is_stable = 0;
	/*  TODO: Switch the hardware to the new input  */
	return 1;
}

static int
decoder_set_standard(struct capture_device *dev, int x)
{
	dev->videc.standard = x;
	/*  TODO: Switch the hardware to the new standard  */
	switch (x)
	{
	case V4L2_STD_NTSC:
		dev->videc.frame_period = 333667;
		break;
	case V4L2_STD_PAL:
		dev->videc.frame_period = 400000;
		break;
	case V4L2_STD_SECAM:
		dev->videc.frame_period = 400000;
		break;
	}
	return 1;
}

static int
decoder_set_vcrmode(struct capture_device *dev, int x)
{
	dev->source[dev->input].vcrmode = x;
	/*  TODO: Switch decoder to VCR sync timing mode  */
	return 1;
}

static int
decoder_is_stable(struct capture_device *dev)
{
	/*  TODO: Check if decoder is synced to input  */
	return 1;
}

static int
decoder_probe(struct capture_device *dev)
{
	/*  TODO: Probe I2C bus or whatever for the video decoder */

	/*  Fill in the method fields  */
	dev->videc.initialize = decoder_initialize;
	dev->videc.set_input = decoder_set_input;
	dev->videc.set_standard = decoder_set_standard;
	dev->videc.set_vcrmode = decoder_set_vcrmode;
	dev->videc.is_stable = decoder_is_stable;

	//info_msg("Found decoder chip\n");
	return 1;/*  Found  */
}

/*
 *
 *	V I D E O   C A P T U R E   F U N C T I O N S
 *
 */

/*
 *  Supported capture formats
 */
static struct v4l2_fmtdesc capfmt[] = 
{
	{	0, {"RGB-16 (5-5-5)"},
		V4L2_PIX_FMT_RGB555,  0, 16, {0, 0},
	},
	{	1, {"RGB-16 (5-6-5)"},
		V4L2_PIX_FMT_RGB565,  0, 16, {0, 0},
	},
	{	2, {"RGB-24 (B-G-R)"},
		V4L2_PIX_FMT_BGR24,   0, 24, {0, 0},
	},
	{	3, {"RGB-32 (B-G-R-?)"},
		V4L2_PIX_FMT_BGR32,   0, 32, {0, 0},
	},
	{	4, {"Greyscale-8"},
		V4L2_PIX_FMT_GREY,    V4L2_FMT_CS_601YUV, 8, {0, 0},
	},
	{	5, {"YUV 4:2:2 (Y-U-Y-V)"},
		V4L2_PIX_FMT_YUYV,    V4L2_FMT_CS_601YUV, 16, {0, 0},
	},
	{	6, {"YUV 4:2:0 (planar)"},
		V4L2_PIX_FMT_YUV420,  V4L2_FMT_CS_601YUV, 12, {0, 0},
	},
};
#define NUM_CAPFMT (sizeof(capfmt)/sizeof(capfmt[0]))


static void interrupt_enable(struct capture_device *dev);

/*  The image format has changed, width, height, pixel format.
 *  Decide if the format is ok or take the closest valid format.
 */
static void
capture_new_format(struct capture_device *dev)
{
	int	ntsc;
	int	max_height;
	int	max_width;
	int	max_pixels;
	int	t;

	ntsc = (dev->videc.standard == V4L2_STD_NTSC);
	dev->ready_to_capture = 0;

	dev->source_width = 
		(ntsc) ? dev->videc.ntsc_width : dev->videc.pal_width;
	dev->source_height = 
		(ntsc) ? dev->videc.ntsc_height : dev->videc.pal_height;

	dev->clientfmt.flags = V4L2_FMT_CS_601YUV;
	switch (dev->clientfmt.pixelformat)
	{
	case V4L2_PIX_FMT_GREY:
		dev->clientfmt.depth = 8;
		break;
	case V4L2_PIX_FMT_YUV420:
		dev->clientfmt.depth = 12;
		break;
	case V4L2_PIX_FMT_RGB555:
	case V4L2_PIX_FMT_RGB565:
		dev->clientfmt.flags = 0;
		/* fall thru */
	case V4L2_PIX_FMT_YUYV:
	case V4L2_PIX_FMT_UYVY:
		dev->clientfmt.depth = 16;
		break;
	case V4L2_PIX_FMT_BGR24:
		dev->clientfmt.depth = 24;
		dev->clientfmt.flags = 0;
		break;
	case V4L2_PIX_FMT_BGR32:
		dev->clientfmt.depth = 32;
		dev->clientfmt.flags = 0;
		break;
	default:
debug_msg("unknown format %08X\n", dev->clientfmt.pixelformat);
		dev->clientfmt.depth = 16;
		dev->clientfmt.pixelformat = V4L2_PIX_FMT_RGB565;
		dev->clientfmt.flags = 0;
		break;
	}

	dev->capture_bypp = 2;

	if (dev->clientfmt.width < MIN_WIDTH)
		dev->clientfmt.width = MIN_WIDTH;
	if (dev->clientfmt.height < MIN_HEIGHT)
		dev->clientfmt.height = MIN_HEIGHT;

#define MAX_IMAGE_SIZE 850000
	max_width = dev->source_width;
	max_height = dev->source_height;
	max_pixels = MAX_IMAGE_SIZE / dev->capture_bypp;
	t = isqrt((max_pixels * dev->clientfmt.width) / dev->clientfmt.height);
	if (t < max_width)
		max_width = t;
	t = isqrt((max_pixels * dev->clientfmt.height) / dev->clientfmt.width);
	if (t < max_height)
		max_height = t;

	if (dev->clientfmt.width > max_width)
		dev->clientfmt.width = max_width;
	if (dev->clientfmt.height > max_height)
		dev->clientfmt.height = max_height;

	dev->clientfmt.width &= ~3;
	dev->clientfmt.height &= ~3;
	dev->clientfmt.sizeimage = (dev->clientfmt.width
		* dev->clientfmt.height
		* dev->clientfmt.depth)
		/ 8;

	dev->capture_size = dev->clientfmt.width
		* dev->clientfmt.height
		* dev->capture_bypp;

	/*  TODO: Any other driver state related to the image format  */
}

/*  Stop the music!
 */
static void
capture_abort(struct capture_device *dev)
{
	dev->grabber_enabled = 0;
	/*  Turn off the capture hardware  */
	grabbing_enable(dev, 0);
}

/*  Allocate buffers, and get everything ready to capture
 *  an image, but don't start capturing yet.
 */
static int
capture_begin(struct capture_device *dev)
{
	capture_abort(dev);
	if (dev->ready_to_capture)
		return dev->ready_to_capture;

	if (dev->capture_buffer_size < dev->capture_size)
	{
		if (dev->capture_buffer != NULL)
			vfree(dev->capture_buffer);
		dev->capture_buffer_size = 
			(dev->capture_size + PAGE_SIZE - 1) & ~(PAGE_SIZE - 1);
		dev->capture_buffer = (__u8 *)
			vmalloc(dev->capture_buffer_size);
		if (dev->capture_buffer == NULL)
		{
			dev->capture_buffer_size = 0;
			err_msg("Can't allocate capture buffer"
				" %d bytes\n", dev->capture_size);
			return dev->ready_to_capture;
		}
	}

#if 0
	if (dev->irq && (can use DMA))
		bm_build_scatter_list(dev, dev->capture_buffer, 
				      &dev->capture_dma_list);
#endif

	if (!translate_setup(dev))
		return dev->ready_to_capture;

	interrupt_enable(dev);
debug_msg("Ready to capture!\n");
	return (dev->ready_to_capture = 1);
}

/*  Start an image capture
 */
static void
capture_grab_frame(struct capture_device *dev)
{
	if (dev->ready_to_capture && dev->grabber_enabled)
		return;

	capture_begin(dev);
	if (!dev->ready_to_capture)
		return;

	/*  TODO: Prepare the hardware for the next capture  */

	//list = dev->capture_dma_list;/* DMA list for capture_buffer */

	/*  Set up stream_capture_buffer to point to the buffer to  */
	/*  capture the next frame into  */
	if (dev->streaming)
	{
		struct stream_buffer	*buf;

		dev->stream_capture_buffer = dev->capture_buffer;

		/* Go straight into streaming buffer? */
		if (dev->translation.type == XLAT_NULL)
		{
			buf = v4l2_q_peek_head(&dev->stream_q_capture);
			if (buf != NULL)
			{
				dev->stream_capture_buffer = buf->vaddress;
				//list = buf->dma_list;
			}
		}
	}

	/*  Start the hardware  */
	grabbing_enable(dev, 1);

	dev->grabber_enabled = 1;
	dev->capture_completed = 0;
}

/*
 *	STREAMING CAPTURE
 */

static int/* 1 = success; 0 = failed */
capture_queuebuffer(struct capture_device *dev,
		    __u32		 type)
{
	int			i;
	struct stream_buffer	*buf	= NULL;

	if (!dev->stream_buffers_mapped || type != V4L2_BUF_TYPE_CAPTURE)
	{
		debug_msg("QBUF no buffers mapped or wrong type\n");
		return 0;
	}

	for (i = 0; i < MAX_CAPTURE_BUFFERS; ++i)
	{
		if ((dev->stream_buf[i].vidbuf.flags &
		     (V4L2_BUF_FLAG_MAPPED | V4L2_BUF_FLAG_QUEUED)) ==
		    V4L2_BUF_FLAG_MAPPED)
		{/*	found one mapped, but not queued */
			buf = &dev->stream_buf[i];
			break;
		}
	}
	if (buf == NULL)
	{
		debug_msg("QBUF no unqueued buffers\n");
		return 0;/* no availabe buffers */
	}

	buf->vidbuf.flags &= ~V4L2_BUF_FLAG_DONE;
	v4l2_q_add_tail(&dev->stream_q_capture, &buf->qnode);
	buf->vidbuf.flags |= V4L2_BUF_FLAG_QUEUED;
	return 1;
}

static int/* 1 = got a buffer; 0 = timed out */
capture_waitforbuffer(struct capture_device *dev,
		      struct timeval	   *timeout)
{
	long	my_timeout = 0;

	if (!dev->streaming)
	{
		debug_msg("NEXTBUF called when not streaming\n");
		return 0;
	}

	if (v4l2_q_peek_head(&dev->stream_q_done) != NULL)
		return 1;  /* frame(s) ready now */

	if (v4l2_q_peek_head(&dev->stream_q_capture) == NULL)
	{
		debug_msg("NEXTBUF called when no buffers are queued\n");
		return 0;  /* no frames queued */
	}

	/*  Sleep with timeout? */
	if (timeout != NULL && (timeout->tv_sec || timeout->tv_usec))
	{
		my_timeout = HZ * timeout->tv_sec
			   + timeout->tv_usec / (1000000 / HZ);
		/* watch out for race condition going to sleep! */
		cli();
		if (!dev->capture_completed)
		{
			my_timeout = interruptible_sleep_on_timeout(
				&dev->new_video_frame, my_timeout);
		}
		sti();
		if (my_timeout == 0)
			return 0; /* timed out */
		return 1;
	}

	/* watch out for race condition going to sleep! */
	cli();
	if (!dev->capture_completed)
		interruptible_sleep_on(&dev->new_video_frame);
	sti();
	return 1;
}

static int/* 1 = got a buffer; 0 = no buffers */
capture_dequeuebuffer(struct capture_device *dev,
		      struct v4l2_buffer *buf)
{
	struct stream_buffer *newbuf;

	if (!dev->streaming || buf->type != V4L2_BUF_TYPE_CAPTURE)
	{
		debug_msg("DQBUF not streaming or wrong buffer type\n");
		return 0;
	}
	newbuf = v4l2_q_del_head(&dev->stream_q_done);
	if (newbuf == NULL)
	{
		debug_msg("DQBUF nothing on done queue\n");
		return 0;
	}
	newbuf->vidbuf.flags &= ~V4L2_BUF_FLAG_QUEUED;

	*buf = newbuf->vidbuf;
	return 1;
}

static int
capture_streamon(struct capture_device	*dev,
		 __u32			type)
{
	struct stream_buffer *buf;

	if (dev->streaming || type != V4L2_BUF_TYPE_CAPTURE)
	{
		debug_msg("STREAMON wrong buffer type or already streaming\n");
		return 0;
	}

	capture_abort(dev);/* cancel any capture that might be in progress */

	/*  -2 is a magic number that triggers start-of-stream logic in */
	/*    capture_interrupt()  */
	dev->stream_last_frame = -2;

	dev->perf.frames = 0;
	dev->perf.framesdropped = 0;
	dev->perf.bytesout = 0;

	/*  Can't capture frames faster than the video input  */
	if (dev->capture.timeperframe < dev->videc.frame_period)
		dev->capture.timeperframe = dev->videc.frame_period;

	/*  Move any leftover DONE buffers to the free pool */
	while ((buf = v4l2_q_del_head(&dev->stream_q_done)))
		buf->vidbuf.flags &= ~V4L2_BUF_FLAG_QUEUED;

	/*  Kick off the machine */
	dev->streaming = 1;
	capture_grab_frame(dev);
	return 1;
}

static void
capture_streamoff(struct capture_device	*dev,
		 __u32			type)
{
	if (!dev->streaming || type != V4L2_BUF_TYPE_CAPTURE)
	{
		debug_msg("STREAMOFF wrong buffer type or not streaming\n");
		return;
	}
	capture_abort(dev);
	dev->streaming = 0;

	/* Note: should really delay this till next capture */
	dev->perf.frames = 0;
	dev->perf.framesdropped = 0;
	dev->perf.bytesout = 0;
}


/*	Read out and convert the next frame
 */
static int /* returns length of data or negative for error */
capture_imagereadout(struct capture_device	*dev,
		     __u8			*capture_buffer,
		     __u8			*output_buffer,
		     int			output_size,
		     int			output_is_user)
{
	int	len;

	len = dev->capture_size;
	/*  If not using DMA then get data into capture buffer  */
	{
		/*  In this example driver we have no hardware to get  */
		/*  data from, so we will make a synthetic image  */
		synthetic_image(dev, capture_buffer);
	}
	/*  else if using DMA the image is already in the buffer  */
	/*  Set 'len' to 0 or negative if frame is no good  */

	dev->grabber_enabled = 0;
	grabbing_enable(dev, 0);

	if (len <= 0)
	{
		return len;
	}

	len = translate_image(dev, capture_buffer, output_buffer,
			      output_size, output_is_user);
	if (len < 0)
		return len;
	++dev->perf.frames;
	dev->perf.bytesout += len;
	return len;
}

/*  The hardware has issued the interrupt signal, do any post-capture
 *  processing that may be necessary.
 *  [This function is called indirectly through the immediate task queue;
 *  it executes at elevated IRQL, but it is interruptible. (It's a b.h.)]
 */
static void
capture_interrupt(void *v)
{
	struct capture_device	*dev = (struct capture_device *)v;
	struct stream_buffer	*buf;
	int			len;
	struct timeval		timestamp_rough;
	unsigned long		raw_frame_num;
	unsigned long		next_raw_frame_to_keep;
	unsigned long		stream_frame_num;
	u64			temp64;

	/*  TODO: Check for an interrupt pending on the device, and  */
	/*        return if there is no interrupt pending  */
	/*  (In this hardware-less demo I'll just check the completed flag) */
	if (!dev->grabber_enabled ||
	    dev->capture_completed)
		return;

	if (dev->capture_buffer == NULL || !dev->ints_enabled)
	{
		err_msg("Can't process the interrupt\n");
		return;
	}
	dev->capture_completed = 1;

	if (!dev->streaming)
	{
		dev->time_acquired = current_time_ms();
		/* DMA might not have finished, but we'll check in read() */
		//debug_msg("New frame ready\n");
		wake_up_interruptible(&dev->new_video_frame);
		return;
	}

	/*  Only get here in streaming mode  */

	if (dev->stream_last_frame == -2)
	{/*	First frame of the stream  */
		v4l2_masterclock_gettime(&dev->stream_begin);
		dev->stream_last_frame = -1;
	}

	buf = v4l2_q_peek_head(&dev->stream_q_capture);
	if (buf == NULL)
	{/*	No available buffers. Skip this frame. This is not an  */
	 /*	error, it's a normal way to throttle the capture rate  */
		dev->grabber_enabled = 0;
		grabbing_enable(dev, 0);
		capture_grab_frame(dev);
		return;
	}

	/*  Compute current stream time  */
	v4l2_masterclock_gettime(&timestamp_rough);
	v4l2_timeval_delta(&timestamp_rough,
			   &dev->stream_begin, &timestamp_rough);

	/*  Capture rate control  */
	raw_frame_num = v4l2_timeval_divide(
		&timestamp_rough, dev->videc.frame_period);
	temp64 = (u64)dev->capture.timeperframe
		* (dev->stream_last_frame + 1)
		+ (dev->videc.frame_period >> 1);
	next_raw_frame_to_keep = 
		v4l2_math_div6432(temp64, dev->videc.frame_period, NULL);
	if (raw_frame_num < next_raw_frame_to_keep)
	{/*	Not time yet, don't keep this frame */
		dev->grabber_enabled = 0;
		grabbing_enable(dev, 0);
		capture_grab_frame(dev);
		return;
	}

	/*  Want this frame  */
	len = capture_imagereadout(dev, dev->stream_capture_buffer,
				   buf->vaddress, buf->vidbuf.length, 0);
	/*  Begin capturing the next frame now  */
	capture_grab_frame(dev);

	if (len <= 0)
	{/*	Frame no good, DMA did not finish, etc. */
		return;
	}

	/*  Fill in the buffer information fields  */
	buf->vidbuf.bytesused = len;
	buf->vidbuf.flags |= V4L2_BUF_FLAG_DONE | V4L2_BUF_FLAG_KEYFRAME;
	buf->vidbuf.timestamp = timestamp_rough;
	stream_frame_num = v4l2_timeval_correct(&buf->vidbuf.timestamp,
						dev->capture.timeperframe);
	//debug_msg("Stream frame %4lu T= %lu.%06lu\n", stream_frame_num,
	//	  buf->vidbuf.timestamp.tv_sec,buf->vidbuf.timestamp.tv_usec);
	if (stream_frame_num > dev->stream_last_frame + 1)
	{/*	We have missed one or more frames  */
		dev->perf.framesdropped += stream_frame_num
			- dev->stream_last_frame + 1;
	}
	dev->stream_last_frame = stream_frame_num;

	/*  Move buffer to done queue  */
	buf = v4l2_q_del_head(&dev->stream_q_capture);
	v4l2_q_add_tail(&dev->stream_q_done, &buf->qnode);

	/*  A new frame is ready!  */
	wake_up_interruptible(&dev->new_video_frame);
}


/*  Read captured data into a user buffer.
 *  Return: negative = error
 *	    0        = keep waiting
 *	    positive = count of bytes read successfully
 */
static long
capture_read(struct capture_device *dev,
	     __u8	*user_buffer,
	     int	user_buffer_size)
{
	int		len = user_buffer_size;
	unsigned long	now;

	if (!dev->ints_enabled)
		return -EIO;

	if (!dev->capture_completed)
	{/* No interrupt has occurred yet, or DMA didn't finish.  */
		//debug_msg("No data ready.\n");
		if (!dev->grabber_enabled)
			capture_grab_frame(dev);
		return 0;/* caller should keep waiting */
	}
	
	now = current_time_ms();
	if (now - dev->time_acquired > MAX_FRAME_AGE)
	{/* Frame in buffer is stale, get a new one */
		debug_msg("Stale frame, re-acquiring.\n");
		dev->grabber_enabled = 0;
		grabbing_enable(dev, 0);
		capture_grab_frame(dev);
		return 0;/* caller should keep waiting */
	}

	len = capture_imagereadout(dev, dev->capture_buffer,
				   user_buffer, user_buffer_size, 1);
	capture_grab_frame(dev);
	return len;
}

/*  Stop capturing and free all resources used for capture.
 */
static void
capture_close(struct capture_device *dev)
{
	int	i;

	if (dev->streaming)
		capture_streamoff(dev, V4L2_BUF_TYPE_CAPTURE);
	capture_abort(dev);
	dev->ready_to_capture = 0;
	translate_close(dev);
	if (dev->capture_dma_list)
		free_page((unsigned long)dev->capture_dma_list);
	dev->capture_dma_list = 0;
	if (dev->capture_buffer != NULL)
		vfree(dev->capture_buffer);
	dev->capture_buffer = NULL;
	dev->capture_buffer_size = 0;
	for (i = 0; i < MAX_CAPTURE_BUFFERS; ++i)
	{
		dev->stream_buf[i].requested = 0;
		if (dev->stream_buf[i].vaddress)
			vfree(dev->stream_buf[i].vaddress);
		dev->stream_buf[i].vaddress = NULL;
		if (dev->stream_buf[i].dma_list)
			free_page((unsigned long)dev->stream_buf[i].dma_list);
		dev->stream_buf[i].dma_list = NULL;
	}
}

/*
 *
 *	I N T E R R U P T   R O U T I N E S
 *
 */

/*  This function runs at interrupt time, either in response to a hardware
 *  interrupt, or on each timer tick if there is no hardware interrupt.
 */
static void
interrupt_handler(void *v)
{
	struct capture_device *dev = (struct capture_device *)v;

	if (!dev->ints_enabled)
		return;

	/*  Call "bottom half" of handler  */
	dev->tqnode_dpc.routine = capture_interrupt;
	dev->tqnode_dpc.data = dev;
	queue_task(&dev->tqnode_dpc, &tq_immediate);
	mark_bh(IMMEDIATE_BH);

	if (dev->tlnode.function != NULL &&
	    dev->ints_enabled)
	{/*	Poll again on next timer tick  */
		dev->tlnode.expires = jiffies + HZ/100;
		add_timer(&dev->tlnode);
	}
}

/*  Called by the system when our hardware interrupt occurs
 */
static void
interrupt_hw(int irq, void *v,
	     struct pt_regs *regs)
{
/*	struct capture_device *dev = (struct capture_device *)v; */

	if (v == NULL)
		return;

/* 	if (!(our device has interrupt pending)) */
/* 	{ */
/* 		Not us. Likely another device is sharing the IRQ */
/* 		debug_msg("HW interrupt (not this device)\n"); */
/* 		return; */
/* 	} */

	//debug_msg("HW interrupt (real)\n");
	interrupt_handler(v);
}

static void
interrupt_disable(struct capture_device *dev)
{
	if (!dev->ints_enabled)
		return;
	dev->ints_enabled = 0;

	/*  TODO: Disable interrupts on the device  */

	if (dev->irq == 0)
	{
		del_timer(&dev->tlnode);
	}
	/*  Wake up any processes that might be waiting for a frame  */
	/*  and let them return an error  */
	wake_up_interruptible(&dev->new_video_frame);
}

static void
interrupt_enable(struct capture_device *dev)
{
	if (dev->ints_enabled)
		interrupt_disable(dev);
	dev->ints_enabled = 1;

	/*  TODO: Enable interrupts on the device  */

	dev->tlnode.function = NULL; /* NULL indicates h/w interrupts */
	if (dev->irq == 0)
	{
		init_timer(&dev->tlnode);
		dev->tlnode.function = 
			(void(*)(unsigned long))interrupt_handler;
		dev->tlnode.data = (unsigned long)dev;
		dev->tlnode.expires = jiffies + HZ/100;
		add_timer(&dev->tlnode);
	}
	debug_msg("%s interrupts enabled\n",
		  dev->tlnode.function?"Polled":"Hardware");
}

/*
 *
 *	M E M O R Y   M A P P I N G
 *
 */

static struct stream_buffer *
mmap_stream_buffer_from_offset(struct capture_device *dev,
			       unsigned long offset)
{
	int	i;
	for (i = 0; i < MAX_CAPTURE_BUFFERS; ++i)
		if (offset == dev->stream_buf[i].vidbuf.offset)
			return &dev->stream_buf[i];
	return NULL;
}

static int
mmap_request_buffers(struct capture_device *dev,
		     struct v4l2_requestbuffers *req)
{
	int	i;
	u32	buflen;

	if (dev->stream_buffers_mapped)
		return 0;/* can't make requests if buffers are mapped */
	if (req->count > MAX_CAPTURE_BUFFERS)
		req->count = MAX_CAPTURE_BUFFERS;
	req->type = V4L2_BUF_TYPE_CAPTURE; /* only kind I know */

	/*  The buffer length needs to be a multiple of the page size  */
	buflen = (dev->clientfmt.sizeimage + PAGE_SIZE - 1)
		& ~(PAGE_SIZE - 1);

	debug_msg("Granting %d buffers\n",req->count);

	/*  Now initialize the buffer structures. Don't allocate the */
	/*  buffers until they're mapped. */
	for (i = 0; i < req->count; ++i)
	{
		dev->stream_buf[i].requested = 1;
		dev->stream_buf[i].vidbuf.index = i;
		dev->stream_buf[i].vidbuf.type = req->type;
		dev->stream_buf[i].vidbuf.offset = 4*i;/* anything unique */
		dev->stream_buf[i].vidbuf.length = buflen;
		dev->stream_buf[i].vidbuf.bytesused = 0;
		dev->stream_buf[i].vidbuf.timestamp.tv_sec = 0;
		dev->stream_buf[i].vidbuf.timestamp.tv_usec = 0;
		dev->stream_buf[i].vidbuf.flags = 0;
	}
	for (i = req->count; i < MAX_CAPTURE_BUFFERS; ++i)
		dev->stream_buf[i].requested = 0;

	return 1;
}

static void
mmap_vma_open(struct vm_area_struct *vma)
{
	struct capture_device *dev =
		capture_device_from_file(vma->vm_file);
	if (dev == NULL)
		return;
	//debug_msg("vma_open called\n");
	++dev->stream_buffers_mapped;
	//MOD_INC_USE_COUNT;
}

static void
mmap_vma_close(struct vm_area_struct *vma)
{
	struct capture_device *dev =
		capture_device_from_file(vma->vm_file);
	struct stream_buffer *buf =
		mmap_stream_buffer_from_offset(dev, vma->vm_offset);

	if (dev->streaming)
	{
		info_msg("Warning- munmap() called while streaming\n");
		capture_streamoff(dev, buf->vidbuf.type);
	}
	v4l2_q_yank_node(&dev->stream_q_capture, &buf->qnode);
	v4l2_q_yank_node(&dev->stream_q_done, &buf->qnode);

	if (buf->vaddress != NULL)
		vfree(buf->vaddress);
	buf->vaddress = NULL;
	if (buf->dma_list)
		free_page((unsigned long)buf->dma_list);
	buf->dma_list = NULL;
	buf->vidbuf.flags = 0;
	debug_msg("Buffer %d deallocated\n",(int)vma->vm_offset/4);

	if (dev->stream_buffers_mapped > 0)
		--dev->stream_buffers_mapped;
	//MOD_DEC_USE_COUNT;
}

static unsigned long
mmap_vma_nopage(struct vm_area_struct *vma,
		unsigned long address, int write)
{
	struct capture_device	*dev;
	struct stream_buffer	*buf;
	unsigned long		offset_into_buffer;
	unsigned long		page;

	dev = capture_device_from_file(vma->vm_file);
	if (dev == NULL)
		return 0;
	buf = mmap_stream_buffer_from_offset(dev, vma->vm_offset);
	if (buf == NULL)
		return 0;
	offset_into_buffer = address - vma->vm_start;
	if (offset_into_buffer >= buf->vidbuf.length)
	{
		err_msg("Attempt to read past end of mmap() buffer\n");
		return 0;
	}
	page = v4l2_vmalloc_to_page(buf->vaddress + offset_into_buffer);
	if (page == 0)
		return 0;

	atomic_inc(&mem_map[MAP_NR(page)].count);
	return page;
}

static struct vm_operations_struct capture_vma_operations =
{
	mmap_vma_open, mmap_vma_close, NULL, NULL, NULL, NULL,
	mmap_vma_nopage,
};


/*
 *
 *	V I D E O   F O R   L I N U X   I N T E R F A C I N G
 *
 */

static int
v4l2_open(struct v4l2_device *v, int flags, void **idptr)
{
	struct capture_device *dev = (struct capture_device *)v;
	int	i, n;
	int	cap;

	for (i = 0, n = -1, cap = 0; i < MAX_OPENS; ++i)
	{
		if (!dev->open_data[i].isopen)
			n = i;/* available open_data structure */
		else if (!dev->open_data[i].noncapturing)
			cap = 1;/* another open is already capturing */
	}
	if (n == -1)/* No available open_data structures */
	{
		debug_msg("No more opens on this device\n");
		return -EBUSY;
	}

	if (flags & O_NONCAP)/*  Non-capturing open */
		dev->open_data[n].noncapturing = 1;
	else if (cap)
	{
		debug_msg("No more capturing opens on this device\n");
		return -EBUSY;
	}
	else
	{
		dev->open_data[n].noncapturing = 0;
		/*  Keep track of whether there is a capturing open  */
		++dev->capturing_opens;
	}

	//MOD_INC_USE_COUNT;
	++dev->open_count;
	dev->open_data[n].isopen = 1;
	dev->open_data[n].dev = dev;
	*idptr = &dev->open_data[n];

	if (dev->open_count == 1)
	{
		if (dev->pci && dev->irq == 0)
		{
			dev->irq = dev->pci->irq;
			if (request_irq(dev->irq, interrupt_hw, SA_SHIRQ,
					dev->shortname, dev) < 0)
			{
				err_msg("Denied IRQ %d\n", dev->irq);
				dev->irq = 0;
			}
		}
		dev->ready_to_capture = 0;/* benchmark changes parameters! */
		dev->capture_completed = 0;
		dev->grabber_enabled = 0;
		v4l2_q_init(&dev->stream_q_capture);
		v4l2_q_init(&dev->stream_q_done);
	}
	debug_msg("Open succeeded\n");

	/* frame counter for test images only */
	if (!dev->open_data[n].noncapturing)
		dev->h=dev->m=dev->s=dev->f=0;
	return 0;
}

static void
v4l2_close(void *id)
{
	struct device_open *o = (struct device_open *)id;
	struct capture_device *dev = o->dev;

	if (!o->noncapturing)
	{
		--dev->capturing_opens;
		debug_msg("Close\n");
	}
	o->isopen = 0;
        --dev->open_count;
	if (dev->open_count == 0)
	{
		interrupt_disable(dev);
		capture_close(dev);
		if (dev->irq)
			free_irq(dev->irq, dev);
		dev->irq = 0;
	}
	//MOD_DEC_USE_COUNT;
}

static int
v4l2_ioctl(void		*id,
	     unsigned int	cmd,
	     void		*arg)
{
	struct device_open *o = (struct device_open *)id;
	struct capture_device *dev = o->dev;

//debug_msg("ioctl %d\n", _IOC_NR(cmd));
	switch(cmd)
	{
	case VIDIOC_QUERYCAP:
	{
		struct v4l2_capability b;
		strcpy(b.name, dev->v.name);
		b.type = V4L2_TYPE_CAPTURE;
		b.flags = V4L2_FLAG_READ |
			  V4L2_FLAG_STREAMING |
			  V4L2_FLAG_SELECT;
		b.inputs = dev->videc.num_inputs;
		b.outputs = 0;
		b.audios = 0;
		b.maxwidth = MAX_WIDTH;
		b.maxheight = MAX_HEIGHT;
		b.minwidth = MIN_WIDTH;
		b.minheight = MIN_HEIGHT;
		b.maxframerate = 30;
		if (copy_to_user(arg, &b, sizeof(b)))
			return -EFAULT;
		return 0;
	}

	case VIDIOC_ENUM_CAPFMT:
	{
		struct v4l2_fmtdesc f;
		if (copy_from_user(&f, arg, sizeof(f)))
			return -EFAULT;
		if (f.index < 0 || f.index >= NUM_CAPFMT)
			return -EINVAL;
		f = capfmt[f.index];
		if (copy_to_user(arg, &f, sizeof(f)))
			return -EFAULT;
		return 0;
	}

	case VIDIOC_G_FMT:
	{
		if (copy_to_user(arg, &dev->clientfmt, sizeof(dev->clientfmt)))
			return -EFAULT;
		return 0;
	}

	case VIDIOC_S_FMT:
	{
		struct v4l2_format fmt;
		if (o->noncapturing || dev->streaming)
		{
			debug_msg("Can't set format now\n");
			return -EPERM;
		}
		if (copy_from_user(&fmt, arg, sizeof(fmt)))
			return -EFAULT;
		dev->clientfmt = fmt;
		capture_new_format(dev);
		if (copy_to_user(arg, &dev->clientfmt, sizeof(fmt)))
			return -EFAULT;
		return 0;
	}

	case VIDIOC_G_COMP:	return -EINVAL;
	case VIDIOC_S_COMP:	return -EINVAL;

	case VIDIOC_REQBUFS:
	{
		struct v4l2_requestbuffers req;
		if (o->noncapturing)
		{
			debug_msg("REQBUFS illegal in non-capturing open\n");
			return -EPERM;
		}
		if (copy_from_user(&req, arg, sizeof(req)))
			return -EFAULT;
		capture_begin(dev);
		if (!mmap_request_buffers(dev, &req))
			return -EINVAL;
		if (copy_to_user(arg, &req, sizeof(req)))
			return -EFAULT;
		return 0;
	}

	case VIDIOC_QUERYBUF:
	{
		struct v4l2_buffer buf;
		int	i;
		if (o->noncapturing)
		{
			debug_msg("QUERYBUF illegal in non-capturing open\n");
			return -EPERM;
		}
		if (copy_from_user(&buf, arg, sizeof(buf)))
			return -EFAULT;
		i = buf.index;
		if (i >= MAX_CAPTURE_BUFFERS ||
		    !dev->stream_buf[i].requested ||
		    (buf.type & V4L2_BUF_TYPE_field) != 
		     (dev->stream_buf[i].vidbuf.type & V4L2_BUF_TYPE_field))
		{
			debug_msg("QUERYBUF bad parameter\n");
			return -EINVAL;
		}
		buf = dev->stream_buf[i].vidbuf;
		if (copy_to_user(arg, &buf, sizeof(buf)))
			return -EFAULT;
		return 0;
	}

	case VIDIOC_QBUF:
	{
		__u32	type = (__u32)arg;
		if (o->noncapturing)
		{
			debug_msg("QBUF illegal in non-capturing open\n");
			return -EPERM;
		}
		if (!dev->stream_buffers_mapped)
		{
			debug_msg("QBUF no buffers of that type are mapped\n");
			return -EINVAL;
		}
		if (!capture_queuebuffer(dev, type))
			return -EINVAL;
		return 0;
	}

	case VIDIOC_NEXTBUF:
	{
		struct timeval tv;
		if (arg)
		{
			if (copy_from_user(&tv, arg, sizeof(tv)))
				return -EFAULT;
			if (!capture_waitforbuffer(dev, &tv))
				return -EINVAL;
		}
		else if (!capture_waitforbuffer(dev, NULL))
			return -EINVAL;
		return 0;
	}

	case VIDIOC_DQBUF:
	{
		struct v4l2_buffer buf;
		if (o->noncapturing)
		{
			debug_msg("DQBUF illegal in non-capturing open\n");
			return -EPERM;
		}
		if (copy_from_user(&buf, arg, sizeof(buf)))
			return -EFAULT;
		if (!capture_dequeuebuffer(dev, &buf))
			return -EINVAL;
		if (copy_to_user(arg, &buf, sizeof(buf)))
			return -EFAULT;
		return 0;
	}

	case VIDIOC_STREAMON:
	{
		__u32	type = (__u32)arg;
		if (o->noncapturing)
		{
			debug_msg("STREAMON illegal in non-capturing open\n");
			return -EPERM;
		}
		if (!capture_streamon(dev, type))
			return -EINVAL;
		return 0;
	}

	case VIDIOC_STREAMOFF:
	{
		__u32	type = (__u32)arg;
		if (o->noncapturing)
		{
			debug_msg("STREAMOFF illegal in non-capturing open\n");
			return -EPERM;
		}
		capture_streamoff(dev, type);
		return 0;
	}

	case VIDIOC_ENUM_FBUFFMT:	return -EINVAL;
	case VIDIOC_G_FBUF:		return -EINVAL;
	case VIDIOC_S_FBUF:		return -EINVAL;
	case VIDIOC_G_WIN:		return -EINVAL;
	case VIDIOC_S_WIN:		return -EINVAL;
	case VIDIOC_PREVIEW:		return -EINVAL;

	case VIDIOC_G_PERF:
	{
		if (copy_to_user(arg, &dev->perf, sizeof(dev->perf)))
			return -EFAULT;
		return 0;
	}

	case VIDIOC_G_INPUT:
	{
		if (copy_to_user(arg, &dev->input, sizeof(dev->input)))
			return -EFAULT;
		return 0;
	}

	case VIDIOC_S_INPUT:
	{
		int	input = (int)arg;
		if (input < 0 || input >= dev->videc.num_inputs)
		{
			debug_msg("Input out of range %d\n", input);
			return -EINVAL;
		}
		if (input != dev->input)
		{
			dev->input = input;
			set_video_input(dev, input);
		}
		return 0;
	}

	case VIDIOC_G_PARM:
	{
		if (copy_to_user(arg, &dev->capture, sizeof(dev->capture)))
			return -EFAULT;
		return 0;
	}

	case VIDIOC_S_PARM:
	{
		struct v4l2_captureparm vp;
		if (copy_from_user(&vp, arg, sizeof(vp)))
			return -EFAULT;
		if (vp.capturemode & ~dev->capture.capability)
		{
			debug_msg("PARM unsupported capture capability\n");
			return -EINVAL;
		}
		if ((dev->capture.capability & V4L2_CAP_TIMEPERFRAME) &&
		    vp.timeperframe < 10000)
		{
			debug_msg("PARM time per frame out of range %ld\n",
				  vp.timeperframe);
			return -EINVAL;
		}
		if (vp.capturemode != dev->capture.capturemode &&
		    !o->noncapturing && dev->streaming)
			return -EINVAL;

		if (o->noncapturing)
			return 0;
		if (vp.capturemode != dev->capture.capturemode)
		{
			dev->capture.capturemode = vp.capturemode;
			capture_new_format(dev);
		}
		if ((vp.capturemode & V4L2_CAP_TIMEPERFRAME) &&
		    vp.timeperframe >= dev->videc.frame_period)
			dev->capture.timeperframe = vp.timeperframe;
		else
			dev->capture.timeperframe = dev->videc.frame_period;
		return 0;
	}

	case VIDIOC_G_STD:
	{
		struct v4l2_standard std;
		v4l2_video_std_construct(&std, dev->videc.standard, 0);
		if (copy_to_user(arg, &std, sizeof(std)))
			return -EFAULT;
		return 0;
	}

	case VIDIOC_S_STD:
	{
		struct v4l2_standard	std;
		int			id;
		if ((o->noncapturing && dev->capturing_opens) ||
		    dev->streaming)
			return -EPERM;
		if (copy_from_user(&std, arg, sizeof(std)))
			return -EFAULT;
		id = v4l2_video_std_confirm(&std);
		if (!((1 << id) & dev->videc.standards))
		{
			debug_msg("Bad standard: %u\n",
				  (unsigned)id);
			return -EINVAL;
		}
		dev->videc.set_standard(dev, id);
		return 0;
	}

	case VIDIOC_ENUMSTD:
	{
		struct v4l2_enumstd estd;
		__u32	b, i;
		if (copy_from_user(&estd, arg, sizeof(estd)))
			return -EFAULT;
		if (estd.index < 0 || estd.index > 30)
			return -EINVAL;
		for (b = 1, i = 0; b < 32; ++b)
		{
			if (((1 << b) & dev->videc.standards) == 0)
				continue;
			if (i == estd.index)
			{
				v4l2_video_std_construct(&estd.std, b, 0);
				estd.ports = (__u32)-1; /* all inputs */
				if (copy_to_user(arg, &estd, sizeof(estd)))
					return -EFAULT;
				return 0;
			}
			++i;
		}
		return -EINVAL;
	}

	case VIDIOC_ENUMINPUT:
	{
		struct v4l2_input vi;
		if (copy_from_user(&vi, arg, sizeof(vi)))
			return -EFAULT;
		if (vi.index >= dev->videc.num_inputs)
			return -EINVAL;
		if (copy_to_user(arg, &dev->source[vi.index].input,
				 sizeof(vi)))
			return -EFAULT;
		return 0;
	}

	case VIDIOC_QUERYCTRL:
	{
		struct v4l2_queryctrl	qc;
		int			i;
		if (copy_from_user(&qc, arg, sizeof(qc)))
			return -EFAULT;
		i = find_vctrl(qc.id);
		if (i < 0)
			return -EINVAL;
		qc = capture_control[i];
		if (copy_to_user(arg, &qc, sizeof(qc)))
			return -EFAULT;
		return 0;
	}

	case VIDIOC_G_CTRL:
	{
		struct v4l2_control	vc;
		int			i;
		if (copy_from_user(&vc, arg, sizeof(vc)))
			return -EFAULT;
		i = find_vctrl(vc.id);
		if (i < 0)
			return -EINVAL;
		vc.value = dev->source[dev->input].control[i];
		if (copy_to_user(arg, &vc, sizeof(vc)))
			return -EFAULT;
		return 0;
	}

	case VIDIOC_S_CTRL:
	{
		struct v4l2_control	vc;
		int			i;
		if (copy_from_user(&vc, arg, sizeof(vc)))
			return -EFAULT;
		i = find_vctrl(vc.id);
		if (i < 0)
			return -EINVAL;
		dev->source[dev->input].control[i] = vc.value;
		device_tone_controls(dev);
		return 0;
	}

	case VIDIOC_G_TUNER:	return -EINVAL;
	case VIDIOC_S_TUNER:	return -EINVAL;
	case VIDIOC_G_FREQ:	return -EINVAL;
	case VIDIOC_S_FREQ:	return -EINVAL;

	case VIDIOC_G_AUDIO:	return -EINVAL;
	case VIDIOC_S_AUDIO:	return -EINVAL;

	default:
		return -ENOIOCTLCMD;
	}
	return 0;
}

static int
v4l2_mmap(void			*id,
	  struct vm_area_struct *vma)
{
	struct device_open	*o   = (struct device_open *)id;
	struct capture_device	*dev = o->dev;
	struct stream_buffer	*buf;

	if (o->noncapturing)
		return -ENODEV;
	buf = mmap_stream_buffer_from_offset(dev, vma->vm_offset);
	if (buf == NULL)
	{
		debug_msg("mmap() Invalid offset parameter\n");
		return -EINVAL;/* no such buffer */
	}
	if (!buf->requested)
	{
		debug_msg("mmap() No buffers requested- no REQBUFS\n");
		return -EINVAL;/* not requested */
	}
	if (buf->vidbuf.flags & V4L2_BUF_FLAG_MAPPED)
	{
		debug_msg("mmap() Buffer is already mapped\n");
		return -EINVAL;/* already mapped */
	}
	if (buf->vidbuf.length != vma->vm_end - vma->vm_start)
	{
		debug_msg("mmap() Wrong length parameter\n");
		return -EINVAL;/* wrong length */
	}

	if (buf->vaddress != NULL)
		vfree(buf->vaddress);
	buf->vaddress = vmalloc(buf->vidbuf.length);
	if (buf->vaddress == NULL)
	{
		err_msg("Could not allocate mmap() buffer\n");
		return -ENODEV;
	}
#if 0
	if ((using DMA) &&
	    !bm_build_scatter_list(dev, buf->vaddress, &buf->dma_list))
		return -ENODEV;
#endif
	buf->vidbuf.flags |= V4L2_BUF_FLAG_MAPPED;

	vma->vm_ops = &capture_vma_operations;
	if (vma->vm_ops->open)
		vma->vm_ops->open(vma);
	/*  Note: vma->vm_file will be set up by V4L2  */

	return 0;
}

static int
v4l2_poll(void	*id,
	    struct file	*file,
	    poll_table	*table)
{
	struct device_open *o = (struct device_open *)id;
	struct capture_device *dev = o->dev;

	if (o->noncapturing)
	{
		debug_msg("poll() illegal in non-capturing open\n");
		return POLLERR;
	}

	if (dev->streaming)
	{
		void	*node;
		node = v4l2_q_peek_head(&dev->stream_q_done);
		if (node != NULL)
			return (POLLIN | POLLRDNORM);/* data is ready now */
		node = v4l2_q_peek_head(&dev->stream_q_capture);
		if (node == NULL)
			return POLLERR;  /* no frames queued */
		poll_wait(file, &dev->new_video_frame, table);
		return 0;
	}

	/*  Capture is through read() call */

	if (dev->capture_completed)/* data is ready now */
		return (POLLIN | POLLRDNORM);
	capture_grab_frame(dev);/* does nothing if capture is in progress */
	if (!dev->ready_to_capture)/* Can't grab frames! */
		return POLLERR;
	poll_wait(file, &dev->new_video_frame, table);
	return 0;
}

static long
v4l2_read(void		*id,
	    char		*buf,
	    unsigned long	count,
	    int			noblock)
{
	struct device_open *o = (struct device_open *)id;
	struct capture_device *dev = o->dev;
	long	len = 0;
	long	my_timeout;

	if (o->noncapturing)
	{
		debug_msg("read() illegal in non-capturing open\n");
		return -EPERM;
	}
	if (dev->streaming)
	{
		debug_msg("Can't read() when streaming is on\n");
		return -EPERM;
	}
	capture_grab_frame(dev);/* does nothing if capture is in progress */
	if (!dev->ready_to_capture)
	{
		debug_msg("Can't grab frames!\n");
		return 0;
	}

	my_timeout = HZ / 5;
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,127)
	current->timeout = jiffies + my_timeout;
#endif
	while (len == 0)
	{
		if (noblock)
		{
			if (!dev->capture_completed)
				return -EAGAIN;
		}
		else
		{
			/* watch out for race condition going to sleep! */
			cli();
			if (!dev->capture_completed)
			{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,127)
				interruptible_sleep_on(&dev->new_video_frame);
#else
				my_timeout = interruptible_sleep_on_timeout(
					&dev->new_video_frame, my_timeout);
#endif
			}
			sti();
		}
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,127)
		if (current->timeout <= jiffies)
#else
		if (my_timeout == 0)
#endif
		{
			debug_msg("Timeout on read\n");
			break;
		}
		len = capture_read(dev, buf, count);
	}
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,127)
	current->timeout = 0;
#endif
	//debug_msg("read %d\n", (int)len);
	return len;
}

/*
 *	Remaining initialization of video decoder etc. This is only
 *	done when the device is successfully identified and registered.
 */
static int
v4l2_init_done(struct v4l2_device *v)
{
	struct capture_device *dev = (struct capture_device *)v;
	int	i;

	/*  Initialize video input array	*/
	for (i = 0; i < VSOURCE_COUNT; ++i)
	{
		dev->source[i].input.index = i;
		dev->source[i].input.type = V4L2_INPUT_TYPE_CAMERA;
		dev->source[i].input.capability = 0;
		/*  Initialize video control properties	*/
		dev->source[i].control[VCTRL_BRIGHTNESS] =
			capture_control[VCTRL_BRIGHTNESS].default_value;
		dev->source[i].control[VCTRL_CONTRAST] =
			capture_control[VCTRL_CONTRAST].default_value;
		dev->source[i].control[VCTRL_SATURATION] =
			capture_control[VCTRL_SATURATION].default_value;
/* 		dev->source[i].control[VCTRL_HUE] = */
/* 			capture_control[VCTRL_HUE].default_value; */
	}
	strcpy(dev->source[VSOURCE_COMP].input.name, "Composite");
	strcpy(dev->source[VSOURCE_SVIDEO].input.name, "S-Video");
	/*strcpy(dev->source[VSOURCE_TUNER].input.name, "Tuner");*/
	/*dev->source[VSOURCE_TUNER].input.type = V4L2_INPUT_TYPE_TUNER;*/

	/*  Initialize the video decoder hardware	*/
	dev->videc.initialize(dev);

	/*  BUG: get defaults from user somehow...  */
	dev->videc.set_standard(dev, V4L2_STD_NTSC);
	dev->videc.set_vcrmode(dev, 0);
	set_video_input(dev, VSOURCE_COMP);

	/*  Capture parameters  */
	dev->capture.capability = V4L2_CAP_TIMEPERFRAME;
	dev->capture.capturemode = 0;
	dev->capture.extendedmode = 0;
	dev->capture.timeperframe = dev->videc.frame_period;

	/*  Default capture dimensions	*/
	dev->clientfmt.width = 160;
	dev->clientfmt.height = 120;
	dev->clientfmt.depth = 16;
	dev->clientfmt.pixelformat = V4L2_PIX_FMT_RGB565;
	dev->clientfmt.flags = 0;
	dev->clientfmt.bytesperline = 0;
	dev->clientfmt.sizeimage = 0;
	capture_new_format(dev);

	return 0;
}

