/*
 * console TV application.  Uses a framebuffer device.
 *
 *   (c) 1998 Gerd Knorr <kraxel@goldbach.in-berlin.de>
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <fcntl.h>
#include <termios.h>
#include <signal.h>
#include <ctype.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <X11/Intrinsic.h>
#include <curses.h>
#include <math.h>
#include <endian.h>

#include <linux/kd.h>
#include <linux/fb.h>

#include "config.h"

#include "fbtools.h"
#include "writefile.h"
#include "sound.h"
#include "channel.h"
#include "channels.h"
#include "grab.h"

#define MAX(x,y)        ((x)>(y)?(x):(y))
#define MIN(x,y)        ((x)<(y)?(x):(y))

/* ---------------------------------------------------------------------- */
/* framebuffer                                                            */

char  *fbdev    = NULL;
char  *fontfile = NULL;
char  *mode     = NULL;

unsigned short red[256],  green[256],  blue[256];
struct fb_cmap cmap  = { 0, 256, red,  green,  blue };

int switch_last,fb;
int keep_dma_on = 0;

int have_config;
int x11_native_format,have_dga=1,x11_is_remote=0;
int debug,sig;
int fs_width,fs_height,fs_xoff,fs_yoff,pix_width,pix_height;
int ww,hh;
float fbgamma = 1.0;

char v4l_conf[128] = "v4l-conf -q";

/*--- channels ------------------------------------------------------------*/

struct STRTAB *cmenu = NULL;
char   title[256];

int cur_color;
int cur_bright;
int cur_hue;
int cur_contrast;
int cur_capture;

int cur_mute   = 0;
int cur_volume = 65535;

char              *snapbase;
char              message[128] = "";

/* ---------------------------------------------------------------------- */
/* framebuffer stuff                                                      */

void
linear_palette(int bit)
{
    int             i, size = 256 >> (8 - bit);

    for (i = 0; i < size; i++)
	red[i] = green[i] = blue[i] = (unsigned short)(65535.0
		* pow(i/(size - 1.0), fbgamma));
}

void
dither_palette(int r, int g, int b)
{
    int             rs, gs, bs, i;

    rs = 256 / (r - 1);
    gs = 256 / (g - 1);
    bs = 256 / (b - 1);
    for (i = 0; i < r*g*b; i++) {
	green[i+16] = (gs * ((i / (r * b)) % g)) * 255;
	red[i+16]   = (rs * ((i / b) % r)) * 255;
	blue[i+16]  = (bs * ((i) % b)) * 255;
    }
}

void
fb_initcolors(int fd, int gray)
{
    /* get colormap */
    if (fb_var.bits_per_pixel == 8 ||
	fb_fix.visual == FB_VISUAL_DIRECTCOLOR) {
	if (-1 == ioctl(fd,FBIOGETCMAP,&cmap))
	    perror("ioctl FBIOGETCMAP");
    }
    
    switch (fb_var.bits_per_pixel) {
    case 8:
	if (gray) {
	    linear_palette(8);
	    x11_native_format = VIDEO_GRAY;
	} else {
	    dither_palette(5,9,5);
	    x11_native_format = VIDEO_RGB08;
	}
	break;
    case 15:
    case 16:
	if (fb_fix.visual == FB_VISUAL_DIRECTCOLOR)
	    linear_palette(5);
#if __BYTE_ORDER == __BIG_ENDIAN
	x11_native_format = (fb_var.green.length == 6) ?
	    VIDEO_RGB16_BE : VIDEO_RGB15_BE;
#else
	x11_native_format = (fb_var.green.length == 6) ?
	    VIDEO_RGB16_LE : VIDEO_RGB15_LE;
#endif
	break;
    case 24:
	if (fb_fix.visual == FB_VISUAL_DIRECTCOLOR)
	    linear_palette(8);
#if __BYTE_ORDER == __BIG_ENDIAN
	x11_native_format = VIDEO_RGB24;
#else
	x11_native_format = VIDEO_BGR24;
#endif
	break;
    case 32:
	if (fb_fix.visual == FB_VISUAL_DIRECTCOLOR)
	    linear_palette(8);
#if __BYTE_ORDER == __BIG_ENDIAN
	x11_native_format = VIDEO_RGB32;
#else
	x11_native_format = VIDEO_BGR32;
#endif
	break;
    default:
	fprintf(stderr, "Oops: %i bit/pixel ???\n",
		fb_var.bits_per_pixel);
	exit(1);
    }

    /* set colormap */
    if (fb_var.bits_per_pixel == 8 ||
	fb_fix.visual == FB_VISUAL_DIRECTCOLOR) {
	if (-1 == ioctl(fd,FBIOPUTCMAP,&cmap))
	    perror("ioctl FBIOPUTCMAP");
    }
}

void
tty_init()
{
    /* we use curses just for kbd input */
    initscr();
    cbreak();
    noecho();
    keypad(stdscr,1);
}

void
tty_cleanup()
{
    clear();
    refresh();
    endwin();
}


/* ---------------------------------------------------------------------- */

void
ctrlc(int signal)
{
    sig=1;
}

void
change_audio(int mode)
{
    if (grabber->grab_audio)
	grabber->grab_audio(-1,-1,&mode);
}

void set_capture(int j)
{
    int x,y,w,h;
    cur_capture = j;

    if (j) {
	if (ww && hh) {
	    w = ww;
	    h = hh;
	    x = fb_var.xres-w;
	    y = 0;
	} else {
	    w = fb_var.xres-24;
	    h = fb_var.yres-16;
	    x = 24;
	    y = 16;
#if 0
	    if (h < fb_var.yres-16)
		y += 8;
#endif
	}
	grabber->grab_overlay(x,y,w,h,x11_native_format,NULL,0);
    } else
	grabber->grab_overlay(0,0,0,0,0,NULL,0);
}

void set_norm(int j)
{
    cur_norm = j;

    grabber->grab_input(-1,cur_norm);
}

void set_source(int j)
{
    cur_input = j;
    
    grabber->grab_input(cur_input,-1);
}

void set_freqtab(int j)
{
    chan_tab = j;
}

void
set_picparams(int color, int bright, int hue, int contrast)
{
    if (color != -1)
	cur_color = color;
    if (bright != -1)
	cur_bright = bright;
    if (hue != -1)
	cur_hue = hue;
    if (contrast != -1)
	cur_contrast = contrast;
    grabber->grab_picture(cur_color,cur_bright,cur_hue,cur_contrast);
}

void
set_channel(struct CHANNEL *channel)
{
    /* image parameters */
    set_picparams(channel->color, channel->bright,
		  channel->hue, channel->contrast);

    /* input source */
    if (cur_input   != channel->source)
	set_source(channel->source);
    if (cur_norm    != channel->norm)
	set_norm(channel->norm);

    /* station */
    cur_channel  = channel->channel;
    cur_fine     = channel->fine;
    grabber->grab_tune(channel->freq);

    set_capture(channel->capture);
}

void
set_volume()
{
    int vol;
    
    if (have_mixer) {
	/* sound card */
	vol = cur_volume * 100 / 65536;
	mixer_set_volume(vol);
	cur_mute ? mixer_mute() : mixer_unmute();
    } else {
	/* v4l */
	grabber->grab_audio(cur_mute,cur_volume,NULL);
    }
}

void
channel_menu()
{
    int  i,f,max,len;
    char key[32],ctrl[16];

    cmenu = malloc((count+1)*sizeof(struct STRTAB));
    memset(cmenu,0,(count+1)*sizeof(struct STRTAB));
    for (i = 0, max = 0; i < count; i++) {
	len = strlen(channels[i]->name);
	if (max < len)
	    max = len;
    }
    for (i = 0; i < count; i++) {
	cmenu[i].nr      = i+1;
	cmenu[i].str     = channels[i]->name;
	if (channels[i]->key) {
	    if (2 != sscanf(channels[i]->key,"%15[A-Za-z0-9_]+%31[A-Za-z0-9_]",
			    ctrl,key))
		strcpy(key,channels[i]->key);
	    if (1 == sscanf(key,"F%d",&f)) {
		channels[i]->ckey = KEY_F(f);              /* Function keys */
	    } else if (strlen(key) == 1) {
		if (isalpha(key[0]))
		    key[0] = tolower(key[0]);
		channels[i]->ckey = (int)key[0];           /* single letter/digit */
	    }
	}
    }
}

/*--- grab (single frames) to file ---------------------------------------*/

void
snap(int jpeg)
{
    void *buffer;
    char *filename = NULL;
    char *name;
    int width  = 2048;
    int height = 1572;
    char line[80];
    int  i;

    if (!grabber->grab_setparams ||
	!grabber->grab_capture)
	return;

    if ( -1 != cur_sender ) {
	name = channels[cur_sender]->name;
    } else {
	name = tvtuner[cur_channel].name;
    }

    if (NULL == snapbase)
	snapbase = "snap";
    filename = snap_filename(snapbase, name, jpeg ? "jpeg" : "ppm");

    sprintf(line,"grabbing image to %s ... ", filename);
    i = fb_puts(0,0,line);
    
    if (0 != grabber_setparams(VIDEO_RGB24,&width,&height,0) ||
	NULL == (buffer = grabber_capture(NULL,1))) {
	fb_puts(i,0,"FAILED");
    } else {
	if (jpeg) {
#ifdef HAVE_LIBJPEG
	    write_jpeg(filename,buffer,width,height,jpeg_quality,0);
#endif
	} else {
	    write_ppm(filename,buffer,width,height);
	}
	fb_puts(i,0,"OK");
    }
    sleep(1);
}

/*--- main ---------------------------------------------------------------*/

static void
grabber_init()
{
    grabber_open(fb_var.xres_virtual,
		 fb_var.yres_virtual,
		 fb_fix.smem_start,
		 x11_native_format,
		 fb_fix.line_length);
}

void
console_switch()
{
    switch (fb_switch_state) {
    case FB_REL_REQ:
	if (!keep_dma_on)
	    set_capture(0);
	fb_switch_release();
	break;
    case FB_ACQ_REQ:
        fb_switch_acquire();
	fb_memset(fb_mem+fb_mem_offset,0,fb_fix.smem_len);
	ioctl(fb,FBIOPAN_DISPLAY,&fb_var);
	set_capture(1);
	break;
    case FB_ACTIVE:
    case FB_INACTIVE:
    default:
	break;
    }
    switch_last = fb_switch_state;
}

int
main(int argc, char *argv[])
{
    int             freq=0,key,i,c,gray=0,rc,vt=0;
    struct timeval  tv;
    time_t          t;
    char            text[80];
    fd_set          set;

    for (;;) {
	double val;
	c = getopt(argc, argv, "gvxkd:o:s:c:f:m:z:t::");
	if (c == -1)
	    break;
	switch (c) {
	case 'z':
	    if(sscanf(optarg, "%lf", &val) == 1) {
		if(val < 0.1 || val > 10)
		    fprintf(stderr, "gamma value is out of range.  must be "
			"0.1 < value < 10.0\n");
		else
		    fbgamma = 1.0 / val;
	    }
	    break;
	case 'f':
	    fontfile = optarg;
	    break;
	case 'm':
	    mode = optarg;
	    break;
	case 'g':
	    gray = 1;
	    break;
	case 'k':
	    keep_dma_on = 1;
	    break;
	case 'v':
	    debug = 1;
	    break;
	case 'd':
	    fbdev = optarg;
	    break;
	case 'o':
	    snapbase = strdup(optarg);
	    break;
	case 's':
	    sscanf(optarg,"%dx%d",&ww,&hh);
	    break;
	case 'c':
	    device = optarg;
	    /* v4l-conf needs this too */
	    strcat(v4l_conf," -c ");
	    strcat(v4l_conf,device);
	    break;
	case 't':
	    if (optarg)
		vt = strtoul(optarg, 0, 0);
	    else
		vt = -1;
	    break;
	default:
	    exit(1);
	}
    }

    fb = fb_init(fbdev,fontfile,mode,vt);
    fb_cleanup_fork();
    fb_initcolors(fb,gray);
    fb_switch_init();
    switch_last = fb_switch_state;
    
    switch (system(v4l_conf)) {
    case -1: /* can't run */
	fprintf(stderr,"could'nt start v4l-conf\n");
	break;
    case 0: /* ok */
	break;
    default: /* non-zero return */
	fprintf(stderr,"v4l-conf had some trouble, "
		"trying to continue anyway\n");
	break;
    }

    grabber_init();
    read_config();

    tty_init();
    atexit(tty_cleanup);
    signal(SIGINT,ctrlc);
    signal(SIGTSTP,SIG_IGN);

    set_freqtab(chan_tab);
    channel_menu();
    if (have_mixer)
	cur_volume = mixer_get_volume() * 65535/100;
    set_volume();
    cur_capture = 0;

    if (optind < argc)
	for (i = 0; i < count; i++)
	    if (0 == strcasecmp(channels[i]->name,argv[optind]))
		cur_sender = i;

    if (count) {
	if ((cur_sender < 0) || (cur_sender >= count))
	    cur_sender = 0;
	set_channel(channels[cur_sender]);
    } else {
	set_channel(&defaults);
    }

    fb_memset(fb_mem+fb_mem_offset,0,fb_fix.smem_len);
    for (;!sig;) {
	if (fb_switch_state == FB_ACTIVE || (keep_dma_on && !ww && !hh)) {
	    /* clear first lines */
	    fb_memset(fb_mem+fb_mem_offset,0,16*fb_fix.line_length);
	    if (message[0] != '\0') {
		strcpy(text,message);
	    } else if (-1 != cur_sender) {
		sprintf(text, "Framebuffer TV - %s", channels[cur_sender]->name);
	    } else {
		sprintf(text,"Framebuffer TV - channel %s",tvtuner[cur_channel].name);
		if (cur_fine != 0)
		    sprintf(text+strlen(text)," (%+d)",cur_fine);
		sprintf(text+strlen(text)," (%s/%s)",
			grabber->norms[cur_norm].str,
			chan_names[chan_tab].str);
	    }
	    fb_puts(0,0,text);

	    if (!ww && !hh) {
		/* display time */
		time(&t);
		strftime(text,16,"%H:%M",localtime(&t));
		fb_puts((fb_var.xres/8)-5,0,text);
	    }
	}
	if (switch_last != fb_switch_state) {
	    console_switch();
	    continue;
	}
	FD_ZERO(&set);
	FD_SET(0,&set);
	tv.tv_sec  = 5;
	tv.tv_usec = 0;
	message[0] = '\0';
	rc = select(1,&set,NULL,NULL,&tv);
	if (switch_last != fb_switch_state) {
	    console_switch();
	    continue;
	}
	if (rc <= 0)
	    continue;
	switch (key = getch()) {
	case 27: /* ESC */
	case 'q':
	case 'Q':
	    sig=1;
	    break;

	case '+':
	    cur_volume += 512;
	    if (cur_volume > 65535)
		cur_volume = 65535;
	    set_volume();
	    sprintf(message,"volume: %d%%", cur_volume * 100 / 65536);
	    break;
	case '-':
	    cur_volume -= 512;
	    if (cur_volume < 0)
		cur_volume = 0;
	    set_volume();
	    sprintf(message,"volume: %d%%", cur_volume * 100 / 65536);
	    break;
	case 10: /* ENTER/RETURN */
	case KEY_ENTER:
	    cur_mute = !cur_mute;
	    set_volume();
	    sprintf(message,"volume: %smute", cur_mute ? "" : "un");
	    break;

	case 'G':
	case 'g':
	    snap(0);
	    break;
	case 'J':
	case 'j':
	    snap(1);
	    break;

	case 339 /* PgUp  --  is'nt in curses.h ??? */:
	    cur_sender = (cur_sender+1)%count;
	    set_channel(channels[cur_sender]);
	    break;
	case 338 /* PgDn */:
	    cur_sender = (cur_sender+count-1)%count;
	    set_channel(channels[cur_sender]);
	    break;

	case KEY_UP:
	    do {
		cur_channel = (cur_channel+1) % CHAN_ENTRIES;
	    } while (!tvtuner[cur_channel].freq[chan_tab]);
	    cur_fine = 0;
	    cur_sender = -1;
	    freq = get_freq(cur_channel)+cur_fine;
	    grabber->grab_tune(freq);
	    break;
	case KEY_DOWN:
	    do {
		cur_channel = (cur_channel+CHAN_ENTRIES-1) % CHAN_ENTRIES;
	    } while (!tvtuner[cur_channel].freq[chan_tab]);
	    cur_fine = 0;
	    cur_sender = -1;
	    freq = get_freq(cur_channel)+cur_fine;
	    grabber->grab_tune(freq);
	    break;
	case KEY_LEFT:
	    cur_fine--;
	    cur_sender = -1;
	    freq = get_freq(cur_channel)+cur_fine;
	    grabber->grab_tune(freq);
	    break;
	case KEY_RIGHT:
	    cur_fine++;
	    cur_sender = -1;
	    freq = get_freq(cur_channel)+cur_fine;
	    grabber->grab_tune(freq);
	    break;

	case -1:
	    break;

	default:
	    if (isalpha(key))
		key = tolower(key);
	    for (i = 0; i < count; i++)
		if (channels[i]->ckey == key) {
		    cur_sender = i;
		    break;
		}
	    if (i < count) {
		set_channel(channels[cur_sender]);
	    } else {
		sprintf(message,"key: %d 0x%x ",key,key);
		if (key > 0x20 && key < 128)
		    sprintf(message+strlen(message),"'%c' ",key);
	    }
	}
    }
    grabber->grab_close();
    fb_memset(fb_mem+fb_mem_offset,0,fb_fix.smem_len);
    /* parent will clean up */
    exit(0);
}
