/*
 * this is a simple test app for playing around with the xvideo extention
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>

#include <X11/Xlib.h>
#include <X11/Xatom.h>

#include "config.h"
#include "grab.h"
#include "xv.h"

#ifndef HAVE_LIBXV
/* dummy stubs */
int have_xv;
void xv_init()
{
    if (debug)
	fprintf(stderr,"Xv: compiled without Xvideo extention support\n");
}
void xv_video(Window win, int width, int height, int on) {}
#else

#include <X11/extensions/Xv.h>
#include <X11/extensions/Xvlib.h>

extern Display    *dpy;
int               have_xv;

static int              ver, rel, req, ev, err;
static int              adaptors;
static int              adaptor = -1, port = -1;
static XvAdaptorInfo    *ai;

static int  xv_error;
static Atom xv_encoding;
static Atom xv_hue;
static Atom xv_saturation;
static Atom xv_brightness;
static Atom xv_contrast;

static struct STRTAB norms[] = {
    { -1, "PAL" },
    { -1, "NTSC" },
    { -1, "SECAM" },
    { -1, NULL }
};

static struct STRTAB input[] = {
    { -1, "default" },
    { -1, NULL }
};

static struct GRABBER xv = {
    "Xv",
    0,
    norms,input,
};

static int
xv_overlay(int x, int y, int width, int height, int format,
	   struct OVERLAY_CLIP *oc, int count)
{
    if (debug)
	fprintf(stderr,"Ouch: xv_overlay called\n");
    return 0;
}

int
xv_error_dev_null(Display * dpy, XErrorEvent * event)
{
    xv_error++;
    return 0;
}

int
xv_probe(XvPortID port, Atom attr)
{
    void            *old_handler;
    int              value;

    xv_error = 0;
    old_handler = XSetErrorHandler(xv_error_dev_null);
    XvGetPortAttribute(dpy, port, attr, &value);
    XSync(dpy, False);
    XSetErrorHandler(old_handler);
    if (debug)
	fprintf(stderr,"xv: probe: %s: %s\n",
		XGetAtomName(dpy,attr), xv_error ? "no" : "yes");
    return xv_error > 0 ? 0 : 1;
}

/* map: 0 - 65536  =>  -1000 - 1000 */
int me_to_xv(int val) {
    val = val * 2000 / 65536 - 1000;
    if (val < -1000) val = -1000;
    if (val >  1000) val =  1000;
    return val;
}

int
xv_picture(int color, int bright, int hue, int contrast)
{
    if (-1 != bright && xv_brightness != None)
	XvSetPortAttribute(dpy,port,xv_brightness,me_to_xv(bright));
    if (-1 != hue && xv_hue != None)
	XvSetPortAttribute(dpy,port,xv_hue,me_to_xv(hue));
    if (-1 != contrast && xv_contrast != None)
	XvSetPortAttribute(dpy,port,xv_contrast,me_to_xv(contrast));
    return 0;
}

void
xv_video(Window win, int width, int height, int on)
{
    static GC gc;
    
    if (debug)
	fprintf(stderr,"Xv: video: win=0x%lx, size=%dx%d, %s\n",
		win, width, height, on ? "on" : "off");
    if (on) {
	if (!gc)
	    gc = XCreateGC(dpy, win, 0, NULL);
	XvPutVideo(dpy,port,win,gc,0,0,width,height,0,0,width,height);
    } else {
	XvStopVideo(dpy,port,win);
    }
}

void xv_init()
{
    int i;
    
    if (Success != XvQueryExtension(dpy,&ver,&rel,&req,&ev,&err)) {
	if (debug)
	    fprintf(stderr,"Xv: Server has no Xvideo extention support\n");
	return;
    }
    if (Success != XvQueryAdaptors(dpy,DefaultRootWindow(dpy),&adaptors,&ai)) {
	fprintf(stderr,"Xv: XvQueryAdaptors failed");
	exit(1);
    }
    if (debug)
	fprintf(stderr,"Xv: %d adaptors available.\n",adaptors);
    for (i = 0; i < adaptors; i++) {
	if (debug)
	    fprintf(stderr,"Xv: %s:%s%s, ports %ld-%ld\n",
		    ai[i].name,
		    (ai[i].type & XvInputMask)  ? " input"  : "",
		    (ai[i].type & XvOutputMask) ? " output" : "",
		    ai[i].base_id,
		    ai[i].base_id+ai[i].num_ports-1);
	if ((ai[i].type & XvInputMask) && (port == -1)) {
	    port = ai[i].base_id;
	    adaptor = i;	    
	}
    }
    if (port == -1) {
	if (debug)
	    fprintf(stderr,"Xv: no usable port found\n");
	return;
    }

    /* probe was successfull */
    fprintf(stderr,"Xv: using port %d\n",port);
    have_xv = 1;
    xv.grab_overlay = xv_overlay;
    grabber = &xv;

    /* build atoms */
    xv_encoding   = XInternAtom(dpy, "XV_ENCODING", False);
    xv_hue        = XInternAtom(dpy, "XV_HUE", False);
    xv_saturation = XInternAtom(dpy, "XV_SATURATION", False);
    xv_brightness = XInternAtom(dpy, "XV_BRIGHTNESS", False);
    xv_contrast   = XInternAtom(dpy, "XV_CONTRAST", False);

    /* check attributes */
    xv_probe(port,xv_encoding)   || (xv_encoding   = None);
    xv_probe(port,xv_hue)        || (xv_hue        = None);
    xv_probe(port,xv_saturation) || (xv_saturation = None);
    xv_probe(port,xv_brightness) || (xv_brightness = None);
    xv_probe(port,xv_contrast)   || (xv_contrast   = None);

    /* set hooks */
    if (xv_hue        != None ||
	xv_brightness != None ||
	xv_contrast   != None) {
	xv.grab_picture = xv_picture;
    }
}

#endif
