/* This example reads an image from MMACo1k and 
 * displays the image in a Xwindow.             */

#include <stdio.h>
#include <stdlib.h>
#include <xil/xil.h>

#include "ddither.h"
#include "CamPlayer.h"

/* - - - - - - - - - - - - - - - - - - - - - - */
main(int argc, char **argv)
{
  struct subimage subimg = {NULL, 0, 0, 0, NULL, NULL, NULL};
  struct vdo_attr vdotype= {512, 512, 512, 512, 512, 100, -1, 0};
  struct xwin xwin = {0, 0, 0, 1,"/dev/o1k0", FALSE, 0, 0, };
  struct img_proc imgproc;

  XilSystemState state;
  XilImage img[4];     /* img[0] = cam_img, img[1] = display_img     */
                       /* img[2] = rescaled_img, img[3] = scaled_img */
  Xil_boolean error_out = FALSE;
  int format;
  int c;

  imgproc.scale.doscale = 2;
  imgproc.scale.grayscale=0;

  /* Getting the option parameters */
  while ((c = getopt(argc, argv, GETOPTARGS)) != EOF)
    switch (c) {
    case 's':
      imgproc.scale.doscale = atoi(optarg);
      break;
    case 'I':
      xwin.port = atoi(optarg);
      break;
    case 'D':
      xwin.devname = optarg;
      break;
    case 'i':
      vdotype.image_skip = atoi(optarg);
      break;
    case 'm':
      vdotype.max_buffers = atoi(optarg);
      break;
    case 'x':
      xwin.winx = atoi(optarg);
      xwin.xyflag = 1;
      break;
    case 'y':
      xwin.winy = atoi(optarg);
      xwin.xyflag = 1;
      break;
    case 'f':
      vdotype.total_frames = atoi(optarg);
      break;
    case 'w':
      subimg.child_width = atoi(optarg);
      break;
    case 'h':
      subimg.child_height = atoi(optarg);
      break;
    case 'c':
      dither_version = atoi(optarg);
      break;
    case 'g':
      imgproc.scale.grayscale = 1;
      break;
    case 'd':
      xwin.nodisplay = TRUE;	/* useful for timing raw capture speed */
      break;
    case 'E':
      error_out = TRUE;
      break;
    case 'B':
      vdotype.brightness = atoi(optarg);
      if (vdotype.brightness < -128)
	vdotype.brightness = -128;
      if (vdotype.brightness > 127)
	vdotype.brightness = 127;
      break;
    case 'C':
      vdotype.contrast = atoi(optarg);
      if (vdotype.contrast < 0)
	vdotype.contrast = 0;
      if (vdotype.contrast > 511)
	vdotype.contrast = 511;
      break;
    case 'H':
      vdotype.hue = atoi(optarg);
      if (vdotype.hue < -128)
	vdotype.hue = -128;
      if (vdotype.hue > 127)
	vdotype.hue = 127;
      break;
    case 'U':
      vdotype.gain_u = atoi(optarg);
      if (vdotype.gain_u < 0)
	vdotype.gain_u = 0;
      if (vdotype.gain_u > 511)
	vdotype.gain_u = 511;
      break;
    case 'V':
      vdotype.gain_v = atoi(optarg);
      if (vdotype.gain_v < 0)
	vdotype.gain_v = 0;
      if (vdotype.gain_v > 511)
	vdotype.gain_v = 511;
      break;
    case '?':
    default:
      usage(argv[0], NULL);
      break;
    }
  /* expect no args left over */
  if (argv[optind]) {
    usage(argv[0], argv[optind]);
  }

  /* Initialization of xil libraries */
  state = xil_open();
  if (state == NULL) {
    fprintf(stderr, "unable to open xil library\n");
    exit(1);
  }
    
  /* setup error-handler */
  if (error_out)
    xil_install_error_handler(state, error_handler);

  /* Initialize camera */
  if (init_cam(state, &img[0], &vdotype, &xwin) == 0) {
    fprintf(stderr, "Unknown video format, exiting.\n");
    xil_close(state);
    exit(1);
  }
    
  /* Initialization of Display */
  if (init_display(argc, argv, state, img, &subimg, &imgproc, &xwin) == 0) {
    fprintf(stderr, "Can't initialize display, exiting.\n");
    xil_close(state);
    exit(1);
  }

  /* Display by frame or by handle */
  if (display_by_handle(img, &subimg, &imgproc, &xwin) == 0) {
    if (!xwin.nodisplay) {	   /* It should exist in main(), also */
      xil_destroy(img[1]);	   /* must destroy display image      */
      XCloseDisplay(xwin.xdisplay);     /* before calling XCloseDisplay.   */
    }
    else {
      fprintf(stderr, "Can't display, exiting.\n");
      xil_close(state);
      exit(1);
    }
  }

  xil_close(state);
    
  return 0;
}
/* - - - - - - - - - - - - - - - - - - - - - - */
