
#include <stdio.h>
#include <xil/xil.h>
 
#include "CamPlayer.h"

/* ************************************ */
/* Initialization of xil display image  */
/* ************************************ */
int init_display (int argc, char **argv, XilSystemState state, 
		  struct image *img, struct img_proc *imgproc, 
		  struct xwin *xwin)
{
  XilLookup yuv_to_rgb;
  XilColorspace cspace_ycc601;
  XilColorspace cspace_rgb709;

  xil_get_info(img[0].handle, &(img[0].width), &(img[0].height), 
	       &(img[0].nbands), &(img[0].datatype));
  
  /* check for width or height too big */
  if (img[0].subimg.child_width > img[0].width ||
      img[0].subimg.child_height > img[0].height) {
    fprintf(stderr, "Maximum width, height = %d,%d\n", 
	    img[0].width, img[0].height);
    xil_close(state);
    exit(1);
  }
    
  if ((img[0].subimg.child_width) || (img[0].subimg.child_height)) {
    if (!img[0].subimg.child_width)
      img[0].subimg.child_width = img[0].width;
    if (!img[0].subimg.child_height)
      img[0].subimg.child_height = img[0].height;
    if (!(img[0].handle = 
	  xil_create_child(img[0].handle,
			   (img[0].width - img[0].subimg.child_width) / 2,
			   (img[0].height - img[0].subimg.child_height) / 2,
			   img[0].subimg.child_width, 
			   img[0].subimg.child_height, 0, img[0].nbands))) { 
      fprintf(stderr, "failed to open child image on MMACo1k\n");
      xil_close(state);
      exit(1);
    }
      
    xil_get_info(img[0].handle, &(img[0].width), &(img[0].height), 
		 &(img[0].nbands), &(img[0].datatype));
  }
    
  if (imgproc->scale.grayscale) {
    img[4].handle = xil_create_child(img[0].handle, 0, 0, img[0].width,
				       img[0].height, 0, 1);      
  }
  else {
    img[2].handle = xil_create(state, img[0].width, img[0].height, 
			       img[0].nbands, XIL_BYTE); 
  }
    
  if (imgproc->scale.doscale) {
    imgproc->scale.scale_factor = 1.0 / imgproc->scale.doscale;
    img[0].width = (int) (((float) img[0].width * \
			    imgproc->scale.scale_factor) + 0.5);
    img[0].height = (int) (((float) img[0].height * \
			     imgproc->scale.scale_factor) + 0.5);
    if (!imgproc->scale.grayscale) {
      img[3].handle = xil_create(state, img[0].width, img[0].height, 
				  img[0].nbands, XIL_BYTE);
    }
  }
  else {
    imgproc->scale.scale_factor = 1.0;
  }
  if (imgproc->scale.grayscale) {
    /* create a 1-band images with the scaled dimensions 
     *  for capture threshold */
    img[3].subimg.a = xil_create(state, img[0].width, img[0].height,
				  1, XIL_BYTE);
    img[3].subimg.b = xil_create(state, img[0].width, img[0].height,
				  1, XIL_BYTE);
    img[3].subimg.c = xil_create(state, img[0].width, img[0].height, 
				  1, XIL_BYTE);
  }
    
  if (xwin->nodisplay) {
    /* for simplicity's sake, the 'xwin->nodisplay' option assumes a 1 band */
    /* XIL_BYTE destination image */
    img[1].handle = xil_create(state, img[0].width, img[0].height, 
				1, XIL_BYTE);
    img[1].subimg.display_depth = 8;
  }
  else {
    XSizeHints hints;
    int port;
    char *format;
      
    /* xlib window creation */
    xwin->xdisplay = XOpenDisplay(NULL);
    if (!xwin->xdisplay) {
      fprintf(stderr, "Unable to connect to server\n");
      xil_close(state);
      exit(1);
    }
    img[1].subimg.display_depth = DefaultDepth(xwin->xdisplay, 
					 DefaultScreen(xwin->xdisplay));
    xwin->xwindow = XCreateSimpleWindow(xwin->xdisplay, 
					DefaultRootWindow(xwin->xdisplay),
					xwin->winx, xwin->winy, img[0].width, 
					img[0].height, 0, 0, 0);
    if (!xwin->xwindow) {
      fprintf(stderr, "Unable to create window\n");
      xil_close(state);
      exit(1);
    }
    if (xwin->xyflag) {
      hints.flags = USPosition;
      hints.x = xwin->winx;
      hints.y = xwin->winy;
    }
    else {
      hints.flags = 0;
    }
    xil_get_device_attribute(img[0].handle, "FORMAT", (void **) &format);
    xil_get_device_attribute(img[0].handle, "PORT_V", 
			     (void **) &(xwin->port));

    /* window title when the display window opened  */
    if (xwin->titlebar[0] == 0 )
      sprintf(xwin->titlebar, "%s: Port %d [%s]", 
	      (xwin->devname ? xwin->devname : "/dev/o1k0"), 
	      xwin->port, format);
    
    /* This Function open a window on Xwindow */
    XSetStandardProperties(xwin->xdisplay, xwin->xwindow, xwin->titlebar, 
			   "o1k_display", 0, argv, argc, &hints);
    
    /* if the user tries to shut down the window using the menu */
    /* it may be used as close_display(). */
    if (closedownAtom = XInternAtom(xwin->xdisplay, "WM_DELETE_WINDOW", False))
      XSetWMProtocols(xwin->xdisplay, xwin->xwindow, &closedownAtom, 1);
      
    /* list window events used */
    XSelectInput(xwin->xdisplay, xwin->xwindow,
		 ExposureMask | ButtonPressMask);
    
    /* make the window visible */
    XMapWindow(xwin->xdisplay, xwin->xwindow);
      
    /* wait for the window to be mapped (an Expose event) */
    do
      XNextEvent(xwin->xdisplay, &(xwin->event));
    while (xwin->event.xany.type != Expose);
      
    img[1].handle = 
      xil_create_from_window(state, xwin->xdisplay, xwin->xwindow);
  }
    
  if (img[1].subimg.display_depth == 8) {
    if (imgproc->scale.grayscale) {
      if (!xwin->nodisplay) {
	XilLookup grayramp;
	int num_entries = 256;
	int i;
          
	Xil_unsigned8 *graydata = (Xil_unsigned8 *) malloc (3 * num_entries); 

	for (i = 0; i < num_entries; i++) {
	  graydata[i * 3 + 2] = graydata[i * 3 + 1] = graydata[i * 3] = i;
	}
	grayramp = xil_lookup_create(state, XIL_BYTE, XIL_BYTE,
				     3, num_entries, 0,
				     graydata);
	init_cmap(grayramp, xwin->xdisplay, xwin->xwindow, 0);
      }
    }
    else {
      get_dither_info(state, imgproc->scale.rescale_values, 
		      imgproc->scale.offset_values,
		      &(imgproc->mask), &yuv_to_rgb, &(imgproc->colorcube));
      if (!xwin->nodisplay) {
	init_cmap(yuv_to_rgb, xwin->xdisplay, xwin->xwindow, -1);
	xil_lookup_set_offset(imgproc->colorcube, 
			      xil_lookup_get_offset(yuv_to_rgb));
      }
    }
  }
  else if (img[1].subimg.display_depth == 24) {
    if (imgproc->scale.grayscale) {
      int i, t;
      Xil_unsigned8 *data = (Xil_unsigned8 *) malloc(256 * 3);
        
      if (data == NULL) {
	fprintf(stderr, "Out of memory\n");
	return 0;
      }
      for (i = 0, t = 0; i < 256; i++, t += 3) {
	data[t] = i;
	data[t + 1] = i;
	data[t + 2] = i;
      }
      imgproc->graylookup = xil_lookup_create(state, XIL_BYTE, XIL_BYTE,
				     3, 256, 0, data);
    }
    else {
      cspace_ycc601 = xil_colorspace_get_by_name(state, "ycc601");
      cspace_rgb709 = xil_colorspace_get_by_name(state, "rgb709");
      if (imgproc->scale.doscale)
	xil_set_colorspace(img[3].handle, cspace_ycc601);
      else
	xil_set_colorspace(img[0].handle, cspace_ycc601);
      xil_set_colorspace(img[1].handle, cspace_rgb709);
    }
  }
  else {
    fprintf(stderr, "visual depth of %d is not supported\n", 
	    img[1].subimg.display_depth);
    return 0;
  }

  return 1;
}


/* ******************************* */
/* Selecting function of X events  */
/* ******************************* */
int x_event (struct xwin *xwin)
{
  if (!xwin->nodisplay) {
    while (XPending(xwin->xdisplay)) {
      XNextEvent(xwin->xdisplay, &(xwin->event));
	
      if ((xwin->event.xany.type == ClientMessage &&
	  ((XClientMessageEvent *) & 
	   (xwin->event))->data.l[0] == closedownAtom) ||
	  (xwin->event.xany.type == ButtonPress)) {
	return 0;
      }
      else if (xwin->event.xany.type != Expose) {
	fprintf(stderr, "event %d\n", xwin->event.xany.type);
      }
    }
  }
  return 1;
}


/* ************************************ */
/* Display image on Xwindow format.     */
/* it is controlled by image handle     */
/* ************************************ */
int display_by_handle (struct image *img, struct img_proc *imgproc,
		       struct xwin *xwin)
{
  float low[2];
  float high[2];
  float map[2];

  /* GRAYSCALE */
  int	GS_Threshold_min = 32;
  int	GS_Threshold_max = 253;

  low[0]  = 0.0;
  high[0] = GS_Threshold_min;
  map[0]  = GS_Threshold_min;
  low[1]  = GS_Threshold_max;
  high[1] = 255.0;
  map[1]  = GS_Threshold_max;


  /*    for (frames = 0; frames < total_frames; frames++) { */  
             /* for loop displays only the given num. of frames.  */
  while(1){  /* but, while loop displays with the infinite num. of frames */
    if (imgproc->scale.grayscale) {
      if (img[1].subimg.display_depth == 24) {
	/* even if imgproc->scale.scale_factor is 1, 
         * this combination of xil_scale()
	 * & xil_lookup() is faster than
	 * xil_lookup(img[4].handle,img[1].handle, imgproc->graylookup)
	 */
	xil_scale(img[4].handle, img[1].subimg.a, "nearest",
		  imgproc->scale.scale_factor, imgproc->scale.scale_factor);
	xil_threshold(img[1].subimg.a, img[1].subimg.b, &low[0],
		      &high[0], &map[0]); 
	xil_threshold(img[1].subimg.b, img[1].subimg.c, &low[1], 
		      &high[1], &map[1]);

	xil_lookup(img[1].subimg.c, img[1].handle, imgproc->graylookup);
	xil_toss(img[1].subimg.a);
	xil_toss(img[1].subimg.b);
	xil_toss(img[1].subimg.c);
      }
      else {		/* subimg.display_depth is 8 */
	/*
	 * even if scale->scale_factor is 1, xil_scale() is faster than
	 * xil_copy(img[4].handle,img[1].handle) for this case
	 */
	xil_scale(img[4].handle, img[1].subimg.a, "nearest",
		  imgproc->scale.scale_factor, imgproc->scale.scale_factor);
	xil_threshold(img[1].subimg.a, img[1].subimg.b, &low[0], 
		      &high[0], &map[0]);
	xil_threshold(img[1].subimg.b, img[1].subimg.c, &low[1], 
		      &high[1], &map[1]);
          
	xil_copy(img[1].subimg.c, img[1].handle);
	xil_toss(img[1].subimg.a);
	xil_toss(img[1].subimg.b);
	xil_toss(img[1].subimg.c);

	if (xwin->nodisplay)
	  xil_sync(img[1].handle);
      }
    }
    else {
      if (img[1].subimg.display_depth == 24) {
	if (imgproc->scale.doscale) {
	  xil_scale(img[0].handle, img[3].handle, "nearest",
		    imgproc->scale.scale_factor, imgproc->scale.scale_factor);
	  xil_color_convert(img[3].handle, img[1].handle);
	  xil_toss(img[3].handle);
	}
	else {
	  xil_color_convert(img[0].handle, img[1].handle);
	}
      }
      else {		/* subimg.display_depth is 8 */
	xil_rescale(img[0].handle, img[2].handle, 
		    imgproc->scale.rescale_values,
		    imgproc->scale.offset_values);
	if (imgproc->scale.doscale) {
	  xil_scale(img[2].handle, img[3].handle, "nearest",
		    imgproc->scale.scale_factor, imgproc->scale.scale_factor);
	  xil_ordered_dither(img[3].handle, img[1].handle, 
			     imgproc->colorcube, imgproc->mask);
	  xil_toss(img[3].handle);
	}
	else {
	  xil_ordered_dither(img[2].handle, img[1].handle,
			     imgproc->colorcube, imgproc->mask);
	}
	if (xwin->nodisplay)
	  xil_sync(img[1].handle);
	xil_toss(img[2].handle);
      }
    }

    /* X event selection to close window */
    if (x_event(xwin) == 0) 
      return 0;
  }

  return 1;
}
  

