/***************************************************************************** 
*                       cpc.c
*-----------------------------------------------------------------------------
*         All rights reserved by Bo-kyun Na (bkna@top.cis.syr.edu)
******************************************************************************
*  This file reads an image from MMACo1k and displays the image in a Xwin.
* It also saves the captured XIL compressed image seqence (CIS) into a file.
*****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <xil/xil.h>

#include "ddither.h"
#include "vdo.h"

/* - - - - - - - - - - - - - - - - - - - - - - */
main(int argc, char **argv)
{
  IMAGE cam;
  IMAGE compressed;
  CIS cis;
  VDOTYPE vdotype= {512, 512, 512, 512, 512, 100, -1, 0};
  XWIN xwin;
  IMGPROC imgproc;

  XilSystemState state;
  Xil_boolean error_out = FALSE;
  int format;
  int c;

  cam.subimg.child_width = 0;
  cam.subimg.child_height = 0;
  cam.display_depth = 0;
  cam.subimg.a = NULL;
  cam.subimg.b = NULL;
  cam.subimg.c = NULL;

  (void *) xwin.titlebar = "camera";
  xwin.winx = 0;
  xwin.winy = 0;
  xwin.xyflag = 0;
  xwin.port = 1;
  (void *) xwin.devname = "/dev/o1k0";
  xwin.nodisplay = FALSE; 

  imgproc.scale.doscale = 2;
  imgproc.scale.grayscale=0;

  (void *) cis.cisfile = NULL;
  cis.do_cis = JPEG;
  (void *) cis.cis_type = "Jpeg";

  /* Getting the option parameters */
  while ((c = getopt(argc, argv, GETOPTARGS)) != EOF)
    switch (c) {
    case 'B':
      vdotype.brightness = atoi(optarg);
      if (vdotype.brightness < -128)
	vdotype.brightness = -128;
      if (vdotype.brightness > 127)
	vdotype.brightness = 127;
      break;
    case 'C':
      vdotype.contrast = atoi(optarg);
      if (vdotype.contrast < 0)
	vdotype.contrast = 0;
      if (vdotype.contrast > 511)
	vdotype.contrast = 511;
      break;
    case 'D':
      xwin.devname = optarg;
      break;
    case 'E':
      error_out = TRUE;
      break;
    case 'H':
      vdotype.hue = atoi(optarg);
      if (vdotype.hue < -128)
	vdotype.hue = -128;
      if (vdotype.hue > 127)
	vdotype.hue = 127;
      break;
    case 'I':
      xwin.port = atoi(optarg);
      break;
    case 'Q':
      cis.mpeg.fixed_quant = cis.h261.mquant = atoi(optarg);
      break;
    case 'R':
      cis.mpeg.rate_control = atoi(optarg);
      break;
    case 'U':
      vdotype.gain_u = atoi(optarg);
      if (vdotype.gain_u < 0)
	vdotype.gain_u = 0;
      if (vdotype.gain_u > 511)
	vdotype.gain_u = 511;
      break;
    case 'V':
      vdotype.gain_v = atoi(optarg);
      if (vdotype.gain_v < 0)
	vdotype.gain_v = 0;
      if (vdotype.gain_v > 511)
	vdotype.gain_v = 511;
      break;
    case 'c':
      if (!strcasecmp(optarg, "CellB")) {
	cis.do_cis = CELLB;
	cis.cis_type = "CellB";
      }
      else if (!strcasecmp(optarg, "Jpeg")) {
	cis.do_cis = JPEG;
	cis.cis_type = "Jpeg";
      }
      else if (!strcasecmp(optarg, "H261")) {
	cis.do_cis = H261;
	cis.cis_type = "H261";
      }
      else if (!strcasecmp(optarg, "Mpeg1")) {
	cis.do_cis = MPEG;
	cis.cis_type = "Mpeg1";
      }
    case 'd':
      xwin.nodisplay = TRUE;	/* useful for timing raw capture speed */
      break;
    case 'f':
      vdotype.total_frames = atoi(optarg);
      break;
    case 'g':
      imgproc.scale.grayscale = 1;
      break;
    case 'h':
      cam.subimg.child_height = atoi(optarg);
      break;
    case 'i':
      vdotype.image_skip = atoi(optarg);
      break;
    case 'm':
      vdotype.max_buffers = atoi(optarg);
      break;
    case 'o':
      cis.cisfile = optarg;
      break;
    case 'r':
      cis.mpeg.bitrate = cis.h261.bitrate = atoi(optarg);
      break;
    case 's':
      imgproc.scale.doscale = atoi(optarg);
      break;
    case 'v':
      dither_version = atoi(optarg);
      break;
    case 'w':
      cam.subimg.child_width = atoi(optarg);
      break;
    case 'x':
      xwin.winx = atoi(optarg);
      xwin.xyflag = 1;
      break;
    case 'y':
      xwin.winy = atoi(optarg);
      xwin.xyflag = 1;
      break;
    case '?':
    default:
      usage(argv[0], NULL);
      break;
    }
  /* expect no args left over */
  if (argv[optind]) {
    usage(argv[0], argv[optind]);
  }

  /* Initialization of xil libraries */
  state = xil_open();
  if (state == NULL) {
    fprintf(stderr, "unable to open xil library\n");
    exit(1);
  }
    
  /* setup error-handler */
  if (error_out)
    xil_install_error_handler(state, error_handler);

  /* Initialize camera */
  if (init_cam(state, &cam, &vdotype, &xwin) == 0) {
    fprintf(stderr, "Unknown video format, exiting.\n");
    xil_close(state);
    exit(1);
  }

  if ((void *) cis.cisfile) {
    if (compressor(state, &cam, &cis, &vdotype, &imgproc) != 0) {
      fprintf(stderr, "Can't compress, exiting.\n");
      xil_close(state);
      exit(1);
    }
  }
  else {
    /* Initialization of Display */
    if (init_display(argc, argv, state, &cam, &imgproc, &xwin) == 0) {
      fprintf(stderr, "Can't initialize display, exiting.\n");
      xil_close(state);
      exit(1);
    }

    /* Display by frame or by handle */
    if (display_by_handle(&cam, &imgproc, &xwin) == 0) {
      if (!xwin.nodisplay) {	     /* It should exist in main(), also */
	xil_destroy(cam.display);   /* must destroy display image      */
	XCloseDisplay(xwin.xdisplay);  /* before calling XCloseDisplay.   */
      }
      else {
	fprintf(stderr, "Can't display, exiting.\n");
	xil_close(state);
	exit(1);
      }
    }
  }
  xil_close(state);
    
  return 0;
}
/* - - - - - - - - - - - - - - - - - - - - - - */

