#include "odo.h"

 
void main(int argc, char* argv[])
{
  int   i;
  int   rc;

  ODO odo;

  odo.aud_fd = -1;
  odo.audctl_fd = -1;
  odo.poll_enabled = 0;
  odo.buf_sz = 1000;

  odo.dev.aud_device = "/dev/o1k0";
  odo.dev.format = 0;
  odo.dev.rate = 8000;
  odo.dev.channels = 1;
  odo.dev.port = AUDIO_LINE_IN; 
 /*  odo.dev.port = 0; */
  odo.dev.igain = 0.125;
  odo.dev.mgain = 0.0;

  odo.rec.output_fd = -1;
  odo.rec.byte_count = 0;
  odo.rec.recording = 0;
  odo.rec.hash_amt = 0;
  odo.rec.hash_cnt = 0;


  strcpy(odo.info, "");
  
  for (i=1; i<argc; i++){
    switch (argv[i][1])
    {
      case 'D':
      {
        odo.dev.aud_device = argv[++i];
        break;
      }
      case 'f':
      {
        odo.dev.format = atoi(argv[++i]);
        break;
      }
      case 'r':
      {
        odo.dev.rate = atoi(argv[++i]);
        break;
      }
      case 'c':
      {
        odo.dev.channels = atoi(argv[++i]);
        break;
      }
      case 'p':
      {
        odo.dev.port = atoi(argv[++i]);
        break;
      }
      case 'g':
      {
        odo.dev.igain = atof(argv[++i]);
        printf("gain %f\n", odo.dev.igain);
        break;
      }
      case 'm':
      {
        odo.dev.mgain = atof(argv[++i]);
        break;
      }
      case 'i':
      {
        strncpy(odo.info, argv[++i],120);
        break;
      }
      case 'b':
      {
        odo.buf_sz = atoi(argv[++i]);
        break;
      }
      case 'h':
      {
        odo.rec.hash_amt = atoi(argv[++i]);
        odo.rec.hash_cnt = odo.rec.hash_amt;
        break;
      }
      case 'o':
      {
        strcpy(odo.filename, argv[++i]);
        odo.rec.output_fd = open(odo.filename, 
				 O_WRONLY | O_CREAT | O_TRUNC, 0666);
        if (odo.rec.output_fd < 0)
          fprintf(stderr,"could not create %s\n", odo.filename);
        break;
      }
      case '?':
      default:
      {
        usage(argv[0]);
        exit(0);
      }
    }
  }

  if (odo.rec.output_fd < 0){
    usage(argv[0]);
    exit(0);
  }

  /*
   * setup signal handlers
   */
  signal(SIGINT,  g_exit);
  signal(SIGPOLL, poll_handler);

  printf("initializing audio device...\n");

  /*
   * This call must be made before any other
   */
  if ((rc = oti_audio_init(NULL, NULL)) < 0){
    fprintf(stderr,"oti_audio_init failed %d\n",rc);
    g_exit(&odo, 0);
  }
fprintf (stderr, "1:AUD_LINE_IN %d\n", AUDIO_LINE_IN);
  
  /*
   * Open audio device and setup POLL signal
   */
  if ((odo.aud_fd = oti_open(odo.dev.aud_device, O_RDONLY | O_NDELAY))<0){
    fprintf(stderr,"could not open osprey audio device\n");
    exit(0);
  }
  if (oti_ioctl(odo.aud_fd, I_SETSIG, (void*)(S_INPUT | S_MSG)) < 0){
    perror("I_SETSIG ioctl");
    g_exit(&odo, 0);
  }
  
  /*
   * Open audio control device and setup POLL signal
   */
  get_audioctl_name(odo.dev.audctl_device, odo.dev.aud_device);
  if ((odo.audctl_fd = oti_open(odo.dev.audctl_device, O_RDWR))<0){
    fprintf(stderr,"could not open audio ctl device\n");
    g_exit(&odo, 0);
  }   
  if (oti_ioctl(odo.audctl_fd, I_SETSIG, (void*)(S_MSG)) < 0){
    oti_close(odo.audctl_fd);
    perror("I_SETSIG ioctl");
    g_exit(&odo, 0);
  }

  /*
   * Set the desired input port and gains
   */
  oti_audio_set_record_port(odo.aud_fd, &odo.dev.port);
  oti_audio_set_record_gain(odo.aud_fd, &odo.dev.igain);
  oti_audio_set_monitor_gain(odo.aud_fd, &odo.dev.mgain);  

  /*
   * Set up the desired format
   */
  switch (odo.dev.format)
  {
    case 0: odo.hdr.encoding = AUDIO_ENCODING_ULAW;   break;
    case 1: odo.hdr.encoding = AUDIO_ENCODING_ALAW;   break;
    case 2: odo.hdr.encoding = AUDIO_ENCODING_LINEAR; break;
    case 3: odo.hdr.encoding = AUDIO_ENCODING_LINEAR; break;
    case 4: odo.hdr.encoding = AUDIO_ENCODING_G722;   break;
    case 5: odo.hdr.encoding = AUDIO_ENCODING_G728;   break;
    default: odo.hdr.encoding = AUDIO_ENCODING_ULAW;
  }

  switch (odo.dev.rate)
  {
    case 8000:  odo.hdr.sample_rate =  8000; break;
    case 11025: odo.hdr.sample_rate = 11025; break;
    case 16000: odo.hdr.sample_rate = 16000; break;
    case 22050: odo.hdr.sample_rate = 22050; break;
    default:    odo.hdr.sample_rate =  8000; 
  }

  switch (odo.dev.channels)
  {
    case 1:  odo.hdr.channels = 1; break;
    case 2:  odo.hdr.channels = 2; break;
    default: odo.hdr.channels = 1; break;
  }

  switch(odo.hdr.encoding)
  {
    case AUDIO_ENCODING_ULAW:
    case AUDIO_ENCODING_ALAW:    
    {
      odo.hdr.samples_per_unit = 1;
      odo.hdr.bytes_per_unit = 1;
      break;
    }
    case AUDIO_ENCODING_LINEAR:
    {
      odo.hdr.samples_per_unit = 1;
      if (odo.dev.format == 2)
        odo.hdr.bytes_per_unit = 1; /* 8 bit PCM */
      else
        odo.hdr.bytes_per_unit = 2; /* 16 bit PCM */
      break;
    }
    case AUDIO_ENCODING_G722:
    {
      /* G722 fixed rate and channels */
      odo.hdr.sample_rate = 8000;
      odo.hdr.channels    = 1;
      odo.hdr.samples_per_unit = 1;
      odo.hdr.bytes_per_unit = 1;
      break;
    }
    case AUDIO_ENCODING_G728:
    {
      /* G728 fixed rate and channels */
      odo.hdr.sample_rate = 1600;
      odo.hdr.channels    = 1;
      odo.hdr.samples_per_unit = 8;
      odo.hdr.bytes_per_unit = 10;
      break;
    }
  }
  
  odo.hdr.data_size = AUDIO_UNKNOWN_SIZE;
  odo.hdr.endian = AUDIO_ENDIAN_BIG;

  /*
   * Set the desired buffer size
   */
  AUDIO_INITINFO(&odo.tmpinfo);
  odo.tmpinfo.record.buffer_size = odo.buf_sz;
  odo.tmpinfo.play.buffer_size = odo.buf_sz;
  oti_audio_setinfo(odo.aud_fd, &odo.tmpinfo);

  /*
   * Write audio format to file and then set this format
   */
  printf("Recording: encoding=%d sample_rate=%d samples_per_unit=%d\n",
         odo.hdr.encoding, odo.hdr.sample_rate, odo.hdr.samples_per_unit);
  printf("           bytes_per_unit=%d channels=%d endian=%d\n",
         odo.hdr.bytes_per_unit, odo.hdr.channels, odo.hdr.endian);
    
  if ((rc = audio_write_filehdr(odo.rec.output_fd, &odo.hdr, odo.info, 
                                ((unsigned)strlen(odo.info)))) != AUDIO_SUCCESS){
    fprintf(stderr,"unable to write audio header %d\n",rc);
    g_exit(&odo, 0);
  }

  oti_audio_set_record_config(odo.aud_fd, &odo.hdr);
       
  odo.rec.recording  = 1;

  /*
   * Pause the input and flush the read queue
   */
  (void) oti_audio_pause_record(odo.aud_fd);
  AUDIO_INITINFO(&odo.tmpinfo);
  odo.tmpinfo.record.pause = 0;
  odo.tmpinfo.record.samples = 0;
  odo.tmpinfo.record.error = 0;
  oti_audio_setinfo(odo.aud_fd, &odo.tmpinfo);
  (void) oti_audio_flush_record(odo.aud_fd);

  /*
   * Recording happens in the background via async POLL signals
   */
  odo.poll_enabled = 1;  
  while (1){
    poll(NULL,0,5);
  }
}
  
  

