
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <math.h>
#include <fcntl.h>
#include <stropts.h>
#include <sys/types.h>
#include <sys/param.h>
#include <sys/file.h>
#include <sys/filio.h>
#include <sys/stat.h>
#include <sys/signal.h>
#include <sys/ioctl.h>
#include <sys/audioio.h>
#include <multimedia/libaudio.h>
#include <multimedia/audio_device.h>
#include <multimedia/audio_encode.h>

#include "oti_libaudio.h"
#include "oti_audio_device.h"

#define MAX_BUFFER_SIZE (1024*4)

enum MODE {PLAY, REC};

/* object for odo playing variables  */
typedef struct {
  int   playing;
  int   input_fd;
  int   aud_amnt;
  char  *aud_ptr;
} ODO_PLAY;

/* object for odo recording variables */
typedef struct {
  int   output_fd;
  int   byte_count;
  int   recording;
  int   hash_amt;
  int   hash_cnt;
} ODO_REC;

/* object for odo device */
typedef struct {
  char *aud_device; 
  char  audctl_device[80];  
  int format;
  int rate;  
  int channels;
  unsigned int port;
  double igain;
  double mgain;
  double ogain;
} ODO_DEV;

/* object of audio */
typedef struct {
  enum MODE aud_mode;
  int   aud_fd;
  int   audctl_fd;
  int   poll_enabled;
  unsigned int buf_sz;
  char  info[120];
  char  filename[80];
  char  aud_data[MAX_BUFFER_SIZE];
  
  Audio_hdr hdr;
  Audio_info tmpinfo;

  ODO_DEV dev;
  ODO_REC rec;
  ODO_PLAY play;
} ODO;


/* Function declarations */
extern void g_exit(ODO *odo, int signo);
extern void record_service(ODO *odo);
extern void stop_record(ODO *odo);
extern void poll_handler(ODO *odo, int signo);
extern void get_audioctl_name(char* adevice_ctl_name_ptr, 
			      char* adevice_name_ptr);
extern int odo_record (ODO *odo);
extern void play_service(ODO *odo);
extern void stop_play(ODO *odo);
extern int odo_play (ODO *odo);

