
#include "odo.h"

void main (int argc, char* argv[])
{
  int rc;

  ODO odo;

  odo.aud_mode = PLAY;
  odo.aud_fd = -1;
  odo.audctl_fd = -1;
  odo.poll_enabled = 0;
  odo.buf_sz = 1000;

  (char *) odo.dev.aud_device = "/dev/o1k0";
  odo.dev.format = 0;
  odo.dev.rate = 8000;
  odo.dev.channels = 1;
  odo.dev.port = AUDIO_LINE_IN; 
 /*  odo.dev.port = 0; */
  odo.dev.igain = 0.125;
  odo.dev.mgain = 0.0;
  odo.dev.ogain = 1.0;

  odo.rec.output_fd = -1;
  odo.rec.byte_count = 0;
  odo.rec.recording = 0;
  odo.rec.hash_amt = 0;
  odo.rec.hash_cnt = 0;

  odo.play.playing = 0;
  odo.play.input_fd = -1;
  odo.play.aud_amnt = 0;
  (char *) odo.play.aud_ptr = odo.aud_data;


  strcpy(odo.info, "");

  /* Select optional parameters */
  if (select_param (argc, argv, &odo)) {
    fprintf (stderr, "Unable to read parameter options\n");
    exit (1);
  }

  /* setup signal handlers  */
  signal(SIGINT,  g_exit);
  signal(SIGPOLL, poll_handler);

  fprintf(stderr, "initializing audio device...\n");

  /* This call must be made before any other   */
  if ((rc = oti_audio_init(NULL, NULL)) < 0){
    fprintf(stderr,"oti_audio_init failed %d\n",rc);
    g_exit(&odo, 0);
  }

  if (odo.aud_mode == PLAY) {
    if (odo_play (&odo)) {
      fprintf (stderr, "Unable to play audio sound\n");
      exit (1);
    }
  } /* end of audio playing mode */

  else if (odo.aud_mode == REC) {
    /*    if (odo_record (&odo)) {
      fprintf (stderr, "Unable to record audio sound\n");
      exit (1);
    }
    */

    /* Open audio device and setup POLL signal  */
    if((odo.aud_fd = oti_open(odo.dev.aud_device, O_RDONLY | O_NDELAY)) < 0){
      fprintf(stderr,"could not open osprey audio device\n");
      exit(0);
    }
    if (oti_ioctl(odo.aud_fd, I_SETSIG, (void*)(S_INPUT | S_MSG)) < 0){
      perror("I_SETSIG ioctl");
      g_exit(&odo, 0);
    }
  
    /* Open audio control device and setup POLL signal  */
    get_audioctl_name(odo.dev.audctl_device, odo.dev.aud_device);
    if ((odo.audctl_fd = oti_open(odo.dev.audctl_device, O_RDWR))<0){
      fprintf(stderr,"could not open audio ctl device\n");
      g_exit(&odo, 0);
    }   
    if (oti_ioctl(odo.audctl_fd, I_SETSIG, (void*)(S_MSG)) < 0){
      oti_close(odo.audctl_fd);
      perror("I_SETSIG ioctl");
      g_exit(&odo, 0);
    }

fprintf(stderr,"1:\n");
    /* Set the desired input port and gains   */
    oti_audio_set_record_port(odo.aud_fd, &(odo.dev.port));
fprintf(stderr,"2:\n");
    oti_audio_set_record_gain(odo.aud_fd, &(odo.dev.igain));
fprintf(stderr,"3:\n");
    oti_audio_set_monitor_gain(odo.aud_fd, &(odo.dev.mgain));  
fprintf(stderr,"4:\n");

    if (odo_set_param(&odo)) {
      fprintf(stderr, "Unable to configure parameters\n");
      exit (1);
    }

    /* Write audio format to file and then set this format   */
    printf("Recording: encoding=%d sample_rate=%d samples_per_unit=%d\n",
	   odo.hdr.encoding, odo.hdr.sample_rate, odo.hdr.samples_per_unit);
    printf("           bytes_per_unit=%d channels=%d endian=%d\n",
	   odo.hdr.bytes_per_unit, odo.hdr.channels, odo.hdr.endian);
    
    if ((rc = audio_write_filehdr(odo.rec.output_fd, &odo.hdr, odo.info, 
			  ((unsigned)strlen(odo.info)))) != AUDIO_SUCCESS){
      fprintf(stderr,"unable to write audio header %d\n",rc);
      g_exit(&odo, 0);
    }

    oti_audio_set_record_config(odo.aud_fd, &odo.hdr);
       
    odo.rec.recording  = 1;

    /* Pause the input and flush the read queue  */
    (void) oti_audio_pause_record(odo.aud_fd);
    AUDIO_INITINFO(&odo.tmpinfo);
    odo.tmpinfo.record.pause = 0;
    odo.tmpinfo.record.samples = 0;
    odo.tmpinfo.record.error = 0;
    oti_audio_setinfo(odo.aud_fd, &odo.tmpinfo);
    (void) oti_audio_flush_record(odo.aud_fd);

    /* Recording happens in the background via async POLL signals   */
    odo.poll_enabled = 1;  
    while (1){
      poll(NULL,0,5);
    }
  } /* end of audio recording mode */
}
