/* ************************************************************************* 
 *                       vdo.h
 *--------------------------------------------------------------------------
 *         rights reserved by Bo-kyun Na (bkna@top.cis.syr.edu)
 ***************************************************************************
 *  This header file defines a set of objects using MMACo1k and saves the 
 *  captured XIL compressed image seqence (CIS) into a file.
 * *************************************************************************/

enum compressor_type { H261, JPEG, MPEG };
enum history_state {OFF,ON};
enum visual_type {PSEUDOCOLOR, TRUECOLOR};

#define GETOPTARGS "C:D:H:I:Q:R:U:V:b:c:f:h:i:m:o:r:s:v:w:x:y:egEdXz"

/* Definitions of objects for video processing */
struct scale {
  int doscale;
  float scale_factor;
  float rescale_values[3];
  float offset_values[3];
  int grayscale;
};

typedef struct img_proc {
  XilLookup colorcube;
  XilLookup graylookup;
  XilDitherMask mask;
  int  exit_on_done;

  struct scale scale;
} IMGPROC;

typedef struct xwin {
  char *titlebar;
  char *devname;
  char **argv;
  int *argc;
  int winx;
  int winy;
  int xyflag;
  int port;
  int screen_num;

  Xil_boolean nodisplay;
  Display *xdisplay;
  Window xwindow;
  XEvent event;
  Colormap x_cmap;
  Atom closedownAtom;

  Visual *visual;
  VisualID visualid;
  int matching_visuals;
  XVisualInfo vis_template;
  enum visual_type default_vis;
} XWIN;

typedef struct vdo_attr {
  int brightness;   /* 512 is not a valid value for any of the */
  int contrast;     /* following. If value is not 512 after    */
  int hue;          /* parsing cmd line args,                  */
  int gain_u;       /* then we know to set the desired value.  */
  int gain_v;

  int total_frames;
  int max_buffers;
  int image_skip;
} VDOTYPE;

struct subimage {
  int child_width;
  int child_height;

  XilImage a;
  XilImage b;
  XilImage c;
};

typedef struct image {
  XilImage handle;

  unsigned int width;
  unsigned int height;
  unsigned int nbands;
  XilDataType datatype; 
  XilImageType outputtype;
  int display_depth;

  XilImage grayscaled;
  XilImage rescaled;
  XilImage scaled;
  XilImage display;

  struct subimage subimg;
} IMAGE;

struct h261 {
  int bitrate;
  int mquant;
};

struct mpeg {
  int bitrate;
  int rate_control;
  int fixed_quant;
};

struct jpeg {

};

typedef struct CIS {
  XilCis cis;
  char *cis_type;
  enum compressor_type do_cis;
  char *cisfile;
  unsigned int rate;

  struct h261 h261;
  struct jpeg jpeg; 
  struct mpeg mpeg;
} CIS;
  

/* Function declarations */
extern Xil_boolean error_handler(XilError error);
extern int init_cam (XilSystemState state, IMAGE *img, VDOTYPE *vdo, 
		     XWIN *xwin);
extern int init_display (int argc, char **argv, XilSystemState state, 
			 IMAGE *img, IMGPROC *imgproc, XWIN *xwin);
extern int x_event (XWIN *xwin);
extern int display_by_handle (IMAGE *img, IMGPROC *imgproc, XWIN *xwin);
extern int compressor (XilSystemState state, IMAGE *img, CIS *cis, 
		       VDOTYPE *vdo, IMGPROC *imgproc);
extern XilLookup 
              create_cmap(XilSystemState State, XilCis cis, Display *display,
			  Window window, int screen, Colormap *rcmap,
			  char *type, XilIndexList *ilist, XilLookup yuvtorgb,
			  XilLookup colorcube);
extern void find_quit(Display *display, Atom quitAtom);
extern int xwin_titlebar (IMAGE *img, XWIN *xwin);
extern int dcompressor (XilSystemState state, IMAGE *img, IMGPROC *imgproc, 
			XWIN *xwin, CIS *cis);
