/* Copyright (c) 1997 MultiMedia Access Corporation - Osprey Systems Division. 
 * 1150 S.E. Maynard Rd., Suite 110, Cary, NC 27511 USA. All Rights Reserved.
 *
 * This product and related documentation is protected by copyright laws and
 * international treaties; however, the user of this software is granted a 
 * nonexclusive, royalty free license to modify, make derivitive works, use,
 * distribute, or otherwise incorporate this software in products which use
 * or incorporate the MMAC product.
 *
 * MMAC reserves the right to change any products herein at any time and 
 * without notice. MMAC does not assume any liability arising from the 
 * application or use of this product, except as expressly agreed to in 
 * writing by MMAC. Use and purchase of this product does not convey a license
 * under any patent rights, copyrights, trademark rights, or any other 
 * intellectual property rights of MMAC, Sun Microsystems, or others. 
 */

#ifndef XIL_PLAY_H
#define XIL_PLAY_H

#include  <pixrect/rasterfile.h>
#include  <xil/xil.h>

#define MAXCOLORS 256

#define GRAYSCALE_FORMAT      0
#define DITHERED8_FORMAT      1
#define RGB24BIT_FORMAT       2
#define RGB32BIT_FORMAT       3

Atom closedownAtom;

/*==================================================
 * Function:    setColorMap 
 * Description: Setting the colormap to the Xwindow.
 *==================================================*/
void setColorMap(Display *xdisplay, 
		 Window xwindow, 
		 int format,
		 XilLookup colormap, 
		 unsigned char *cmap[], 
		 int cmap_len);

/*=================================================================
 * Function:    readSunRasterHeaderInfo 
 * Description: Reads the sun rasterfile header info from the file.
 *              return -1 for failure.
 *              return 0 or positive integer for success.
 *=================================================================*/
int readSunRasterHeaderInfo(int fd, 
			    int* width, 
			    int* height, 
			    int* src_image_size, 
			    int* cmap_size);

/*===================================================================
 * Function:    readColorMapInfo 
 * Description: Reads the color map information from the opened file.
 *              return -1 for failure.
 *              return 0 or positive integer for success.
 *===================================================================*/
int readColorMapInfo(int fd, 
		     unsigned char* cmap[],
		     int cmap_len);

/*=========================================================
 * Function:    rateControl 
 * Description: Determines the number of frames to playback
 *              by waiting some amount of time before the 
 *              the next playback.
 *==========================================================*/
void rateControl(int rate);

/*==============================================
 * Function:    usage
 * Description: Prints the user input interface.
 *==============================================*/
void usage(char *name);

#endif /* XIL_PLAY_H */
