/* @(#)receive.c	1.7 93/10/27  */

#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>
#include <sys/types.h>
#include <stropts.h>
#include <sys/conf.h>
#include <sys/socket.h>
#include <netdb.h>
#include <sys/param.h>
#include <netinet/in.h>

#include <poll.h>

#include "packet.h"
#include "params.h"
#include "receive.h"


ReceiveState *
initialize_receive(Parameters * param)
{
    ReceiveState *receiver;
    int rsock;
    struct sockaddr_in *name;

    /*
     * .... create receiver state ....
     */
    receiver = (ReceiveState *) malloc(sizeof(ReceiveState));
    if (receiver == NULL)
	return (NULL);


    /*
     * .... open receiver socket and listen to anyone ....
     */
    if ((rsock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
	perror("socket");
	free(receiver);
	return (NULL);
    }
    name = &receiver->name;
    name->sin_family = AF_INET;
    name->sin_addr.s_addr = INADDR_ANY;
    name->sin_port = CHANNEL_BASE + param->channel;

    if (bind(rsock, (struct sockaddr *) name, sizeof(*name)) < 0) {
	perror("bind");
	close(rsock);
	free(receiver);
	return (NULL);
    }

    receiver->socket = rsock;

    {
	int value;

	ioctl(rsock, I_GRDOPT, &value);
	ioctl(rsock, I_SRDOPT, RMSGD);
    }

    /* for poll to tell us if there is a packet waiting for read */
    pollfds.fd = receiver->socket;;
    pollfds.events = POLLIN;
    pollfds.revents = 0;

    return (receiver);
}


int 
receive_packet(WindowState * ws, ReceiveState * receiver,
	       PacketType * packet)
{
    int len;
    int size;

    while ((pollfds.revents & POLLIN) == 0) {

	/*
	 * check for X events at least once, maybe more if the poll does not
	 * see a video packet.
	 */
	check_events(ws);

	/* wait 100 msecs for packet */
	if (poll(&pollfds, 1, 100) < 0) {
	    perror("poll failed");
	    close(receiver->socket);
	    free(receiver);
	    return 0;
	}
    }

    /*
     * reset, so next time we will have to check at least once.
     */
    pollfds.revents &= ~POLLIN;

    len = read(receiver->socket, (char *) packet, sizeof(PacketType));
    size = sizeof(PacketHeader) + packet->header.size;
    if (len != size)
	return 0;
    return 1;
}


void 
close_receive(ReceiveState * receiver)
{
    close(receiver->socket);
    free(receiver);
}
