/* $Id: play.cpp 1.1 1999/01/25 17:30:30 scl Exp $ */
/****************************************************************
 ** File: File Long Name ($RCSfile: play.cpp $)
 **
 ** ($$)
 **
 ** Abstract:
 **   Class implementation for the playback operations
 **   
 ** Copyright (c) 1998, MultiMedia Access Corp
 **                     Osprey Technologies Division
 ****************************************************************/
#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include "play.h"

/****************************************************************
 ** Method: PlayH263P  (CPlay::CPlay)
 **
 ** Abstract:
 **  Constructor
 **
 ** Error Conditions: None
 **
 ***************************************************************/
CPlay::CPlay()
{
  pipeline   = NULL;
  num_queued = 0;
  videofile  = NULL;
  window     = NULL;
  farview  = FALSE;

  // Open the OPI system.  
  opi = new OPISystem();
  
  // setDebugLevel will control the amount of debug information 
  // provided by OPI system. Currently, the only effect 
  // this has is disabling detection of streaming timeouts
  // if passed a non-zero value.
  opi->setDebugLevel(1);
}

/****************************************************************
 ** Method: PlayH263P  (CPlay::~CPlay)
 **
 ** Abstract:
 **  Destructor
 **
 ** Error Conditions: None
 **
 ***************************************************************/
CPlay::~CPlay()
{   
  if (opi)
    delete opi;
 }

/****************************************************************
 ** Method: PlayH263P  (CPlay::Playback)
 **
 ** Abstract:
 **   Opens the video file for reading.
 **   Creates the pipeline to convert the compressed H263P image to 
 **   uncompressed RGB565.
 **   Creates another branch of the pipeline to send the uncompressed
 **   RGB565 image to an opi window for display.
 **
 ** Error Conditions: 
 **   Could not open video file.
 **   Could not create pipeline.
 **   Could not create OPI window. 
 **   Could not map to pipeline.
 **
 ***************************************************************/
int CPlay::PlayBack()
{
  
  FILE* fd = NULL;
  FILE* fd_sizes = NULL;
  unsigned int size;
  unsigned int frame_count = 0;

  // Open the file for reading
  if (videofile) {
    char* fsize_name = new char[strlen(videofile) + 10];

    if ((fd = fopen(videofile, "rb")) == NULL) {
	  cerr<<"Could not open video file"<<endl;
	  return APP_ERROR;
	}
    
    strcpy(fsize_name, videofile);
    strcat(fsize_name, ".sz");
    if ((fd_sizes = fopen(fsize_name,"rb")) == NULL) {
	  cerr<<"Could not open video size file"<<endl;
	  return APP_ERROR;
	}
    printf("reading from %s and %s\n",videofile, fsize_name);

  } else {
    cerr << "video file is not specified" << endl;
    return APP_ERROR;
  }
  
  // Creating the pipeline
  if ((pipeline = new OPIPipeline(opi)) == NULL) {
    cerr << "Could not create pipeline\n";
    return APP_ERROR;
  }	
  
  // Set up the recorded format variables for the first stage of the pipeline.
  // In this example, the recorded format is Compressed H263P(IP64), 
  // the decompressed format is RGB565.
  OPIFormatVideo recordedFmt;
  recordedFmt.compression   = "H263P";
  
  // Set up the decompressed format variables for the first stage of the pipeline.
  OPIFormatVideo decompressedFmt;

  if (farview)
    decompressedFmt.compression  = "RGB565";
  else
    decompressedFmt.compression  = "YUV422";

  decompressedFmt.width        = VIDEO_WIDTH;
  decompressedFmt.height       = VIDEO_HEIGHT;
  decompressedFmt.depth        = 16;
  
  // Creating the first stage of the pipeline
  OPIStage* videoDecompressionStage;
  videoDecompressionStage = pipeline->firstStage(CAP_VID_DECOMPRESSION,&recordedFmt,&decompressedFmt);
  
  // Set up the window format variables for the 2nd stage of the pipeline.
  OPIFormatVideo windowFmt;
  windowFmt.width          = WINDOW_WIDTH;
  windowFmt.height         = WINDOW_HEIGHT;
  
  if (farview) {
    windowFmt.compression    = "WINDOW";
    windowFmt.externalOutput = FALSE;
    
    // Creating the OPI window for the image display.
    if ((window = new OPIWindow(0,0,windowFmt.width,windowFmt.height,"Output")) == NULL) {
      cerr << "Could not create OPI window\n" ;
      delete pipeline;
      return APP_ERROR;
    }
    windowFmt.param = (void *)window;
  } else {
    windowFmt.compression    = "NTSC";
    windowFmt.externalOutput = TRUE;
  }
  
  // Creating a branch of the pipeline for viewing the decompressed video.
  pipeline->addStage(videoDecompressionStage, CAP_VID_OUTPUT, &decompressedFmt, &windowFmt);
  
  // Assigning the actual hardware and software to each of 
  // the required actions in the pipeline. If it returns false 
  // then delete the pipeline and window.
  if (!pipeline->mapToDevices()) {  
    cerr << "Could not map to pipeline" << endl;
    delete pipeline;
    delete window;
    return APP_ERROR;
  }
  
  // Finishing the pipeline construction.
  pipeline->finishConstruction();
  
  // Setting up the call back for the input stream.
  OPIStream* inVidStream = NULL;
  inVidStream = pipeline->getInStream();
  inVidStream->setDataDoneCallback(&DoneBuffer, (void*)this);
  
  // Start the pipeline.
  pipeline->start();
  
  // Read data from a file and then send data to decompressor
  OPIBuffer* vidBuf;

  hrtime_t htime_cur_base = 0;
  hrtime_t htime_sav_base = 0;
  hrtime_t htime_next = 0;

  for(;;) {
    // Get an empty buffer that we can use.
    if ((vidBuf = inVidStream->getBuffer()) == NULL) {
	  OPISleep(10);
	  continue;
	}
      
    // read frame size
    if (fread(&size, 4, 1, fd_sizes) != 1) {
      printf("no more data from size file\n");
      break;
    }

    // read frame time stamp
    if (fread(&htime_next, sizeof(hrtime_t), 1, fd_sizes) != 1) {
      printf("no more data from size file\n");
      break;
    }
    
    // set base time
    if (htime_sav_base == 0){
      printf("init time\n");
      htime_sav_base = htime_next;
      htime_cur_base = gethrtime();
    }

    // wait until time
    while ((gethrtime() - htime_cur_base) < (htime_next - htime_sav_base)){
      OPISleep(1);
    }

    frame_count++;
    printf("reading frame[%d] bytes %d\n",frame_count, size);
    if((vidBuf->fsize = fread(vidBuf->data, 1, size, fd)) == 0)
      break;
    
    // Send data to pipeline.
    inVidStream->addData(vidBuf);
    OPISleep(10);
    num_queued++;
  }
  
  // Wait for buffers to empty.
  while (num_queued > 0) 
    OPISleep(50);
  
  // Stop the pipeline.
  pipeline->stop();

  // Delete the window and pipeline
  delete window;
  delete pipeline;

  fclose(fd);
  fclose(fd_sizes);
  
  return APP_OK;
}

/****************************************************************
 ** Method: PlayH263P  (CPlay::DoneBuffer)
 **
 ** Abstract:
 **   Returns the buffer and decrements the num_queued.
 **
 ** Error Conditions: None
 **
 ***************************************************************/
void CALLBACK CPlay::DoneBuffer(OPIBuffer* buffer, void* userData)
{
  CPlay* playPtr = (CPlay*) userData;
  playPtr->pipeline->getInStream()->returnBuffer(buffer);
  playPtr->num_queued--;
}

