//----------------------------------------------------------------------------
// Filename   : cpanel.cpp
// 
// Description: Implementation file
//----------------------------------------------------------------------------
#include <iostream.h>
#include <string.h>
#include "cpanel.h"

//-------------------------
// Method     : Cpanel()
// Description: Constructor
//-------------------------
 Cpanel::Cpanel()
{
   opisys = NULL;
   dp = NULL;
   devinfo = NULL;
   name = "o1c";
   num_of_instances = 0;
  
}

//-------------------------
// Method     :  ~Cpanel()
// Description: Destructor
//-------------------------
 Cpanel::~Cpanel()
{
   if (opisys)
   {
      delete opisys;
      opisys = NULL;
   }

}

//-----------------------------------------------------------------------------
// Method     : openOPI()
// Description: Opens the opi system. Checks for the type of devices
//              installed in the system. Also checks for the instances of
//              that particular type  of device found in the system. Storing
//              the device instance ptr and the name of the device instance
//              into a structure called devinfo.
//-----------------------------------------------------------------------------
int Cpanel::openOPI()
{
   if (opisys == NULL) 
   {
      opisys = new OPISystem();
      opisys->setDebugLevel(1);
   }
   else
   {
      cerr << "Could not Open OPI\n";
      return APP_ERROR;
   }

   dp = opisys->getDeviceByName(name);
      
   if (dp == NULL)
   {
      cerr << "No " << name << " devices installed\n";
      return APP_ERROR;
   }
   // Getting the number of instances of the device type.
	    
   num_of_instances = dp->instances.getNumItems();
   devinfo = new DeviceInfo[num_of_instances];
	
   // Storing the name of the instances and the dp_inst ptr 
   // to the devinfo structure.
	       
   char* name_of_instance;	    
   for (int j = 0; j < num_of_instances; j++)
   {
     name_of_instance = dp->instances.getItemName(j);
     strcpy(devinfo[j].inst_name, name_of_instance);
     devinfo[j].dp_inst = dp->getInstanceByName(name_of_instance);
   }
   return APP_OK;
}

//-----------------------------------------------------------------------------
// Method     :  setDeviceAttribute(int device_num,
//				    char* attr_name, long value)
// Description:  Setting the values to the device Attributes.
//-----------------------------------------------------------------------------
void Cpanel::setDeviceAttribute(int device_num, char* attr_name, long value)
{
   OPIAttribute val_set;
   if ((strcmp(attr_name, BRIGHT) == 0) || strcmp(attr_name, HUE) == 0)
      val_set.i = (int)value;
   else
      val_set.i = value;
   /* Set the attribute */
   devinfo[device_num].dp_inst->setAttribute(attr_name, val_set);
 
}
//-----------------------------------------------------------------------------
// Method     :  getDeviceAttribute(int device_num,
//				    char* attr_name,
//				    long & value)
// Description:  Get the values to the device Attributes.
//-----------------------------------------------------------------------------
void Cpanel::getDeviceAttribute(int device_num, char* attr_name, long& value)
{
   OPIAttribute opival;
   devinfo[device_num].dp_inst->getAttribute(attr_name, &opival);
   value = opival.i;
}



