//----------------------------------------------------------------------
// Filename     : main.cpp
// Description  : This is a command line driven control panel for the
//                Osprey-150 card.
//----------------------------------------------------------------------
#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cpanel.h"

#define OK   1
#define FAIL -1


int main(int argc, char* argv[])
{ 
   Cpanel* cpanel;
   cpanel = new Cpanel();
  
   long brightness = -26;
   long contrast = 270;
   int device_no = 0;   // defaults to the first instance of the device type
   long hue = 0;
   long video_port = 1;
   long video_format = 0;
   long gain_u = 254;
   long gain_v = 180;
  
   char* name_of_attr;
   long set_value;
  
   // Open the opi system.
   if(cpanel->openOPI() == FAIL)
   {
      cerr << " OpenOPI failed \n";
      delete cpanel;
      return FAIL;
   }
 
   // Checking for user inputs
   if(argc >= 2)
   {

      for (int i = 1; i < argc; i++)
      {

	 int exec = 1;   // execute update 

	 if (strcmp(argv[i], "-b") == 0)
	 {
	    brightness = atoi(argv[++i]);
	    if (brightness < -128)
	       brightness = -128;
	    if (brightness > 127)
	       brightness = 127;
	      
	    name_of_attr = BRIGHT;
	    set_value = brightness;
	 }
	 else if (strcmp(argv[i], "-c") == 0)
	 {
	    contrast = atoi(argv[++i]);
	    if (contrast < 0)
	       contrast = 0;
	    if (contrast > 511)
	       contrast = 511;
	    
	    name_of_attr = CONTRAST;
	    set_value = contrast;
	 }
	 else if (strcmp(argv[i], "-D") == 0)
	 {
            exec = 0;
	    device_no = atoi(argv[++i]);
            printf("device_no is %x \n", device_no);
	 }
	 else if (strcmp(argv[i], "-h") == 0)
	 {
	    hue = atoi(argv[++i]);
	    if (hue < -128)
	       hue = -128;
	    if (hue > 127)
	       hue = 127;
	    
	    name_of_attr = HUE;
	    set_value = hue;
	 }
	 else if (strcmp(argv[i], "-p") == 0)
	 {
	    video_port = atoi(argv[++i]);
	    
	    name_of_attr = PORT;
	    set_value = video_port;
	 }
	 else if (strcmp(argv[i], "-f") == 0)
	 {
	    video_format = atoi(argv[++i]);
	    if ((video_format == 0) || (video_format == 1))
            {
	      name_of_attr = FORMAT;
	      set_value = video_format;
	    }
	 }
	 else if (strcmp(argv[i], "-u") == 0)
	 {
	    gain_u = atoi(argv[i++]);
	    if (gain_u < 0)
	       gain_u = 0;
	    if (gain_u > 511)
	       gain_u = 511;
	    
	    name_of_attr = GAINU;
	    set_value = gain_u;
	 }
	 else if (strcmp(argv[i], "-v") == 0)
	 {
	    gain_v = atoi(argv[++i]);
	    if (gain_v < 0)
	       gain_v = 0;
	    if (gain_v > 511)
	       gain_v = 511;
	    
	    name_of_attr = GAINV;
	    set_value = gain_v;
	 }
	 else 
	 {
	    cout <<
	       "------------------------------------------------------------\n"
	       "                 o1c Control Panel                          \n"
	       "------------------------------------------------------------\n"
               "Legal Options   |    Description                  | Defaults\n"
	       "------------------------------------------------------------\n"
	       "-b              |    Brightness (-128 to 127)     |  50     \n"
	       "-c              |    Contrast   (0 to 511)        |  216    \n"
	       "-D              |    Device Number                |  0      \n"
	       "-h              |    Hue        (-128 to 127)     |  0      \n"
	       "-p              |    Video Port (0 = S_Video,     |  1      \n"
	       "                |               (1 = Composite1,  |         \n"
	       "                |               (2 = Composite2,  |         \n"
	       "-f              |    Video Format (0 = NTSC       |  0      \n"
	       "                |                 (1 = PAL)       |         \n"
	       "-u              |    Gain-U                       |  254    \n"
	       "-v              |    Gain-V                       |  180    \n"
	       "------------------------------------------------------------\n"
		  ;
	       return FAIL;
	    }

   
	    // Setting the attribute of the device based on user input.
            if ( exec)
	    {
	       //printf("execute setDeviceAttribute ( %x, %x, %x)\n",
	       //	      device_no, name_of_attr, set_value);

	       cpanel->setDeviceAttribute(device_no, name_of_attr, set_value);
	    }

	 }
      }
      else // When no arguments are given, just set it all to defaults.
      {
	 cpanel->setDeviceAttribute(device_no, BRIGHT, brightness);
	 cpanel->setDeviceAttribute(device_no, CONTRAST, contrast); 
	 cpanel->setDeviceAttribute(device_no, HUE, hue); 
	 cpanel->setDeviceAttribute(device_no, GAINU, gain_u); 
	 cpanel->setDeviceAttribute(device_no, GAINV, gain_v);
	 cpanel->setDeviceAttribute(device_no, PORT, video_port);
	 cpanel->setDeviceAttribute(device_no, FORMAT, video_format);
      }
  

   delete cpanel;
   return OK;
}





