/* $Id: main.cpp 1.6 1999/01/25 22:02:52 scl Release $ */
/*************************************************************
 ** File: File Long Name ($RCSfile: main.cpp $)
 **
 ** ($ Revision: $)
 **
 ** Abstract:
 **	This is an OPI playback example which demonstrates 
 **     the creation of a pipeline stage to playback an
 **     uncompressed image data to a window.
 ** 
 **     To execute the program, change directory to where
 **     playRaw.exe is located.
 **
 **     Enter "playRaw -f [filename] -c [color_format]
 **
 **     Usage:
 **     Legal Options |	    Description		| Defaults
 **     ---------------------------------------------------
 **	    -f            | name of file            | None
 **     -c            | color_format            | RGB565
 **
 ** Copyright (c) 1998, MultiMedia Access Corp
 **                     Osprey Technologies Division
 *************************************************************/
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "playraw.h"

/*****************************************************************************
 ** Function: Display Program Usage ( usage )
 **
 ** Abstract:
 **   Prints out the valid command line arguments and then exits.
 **
 ** Error Conditions: None
 **
 ****************************************************************************/
void usage(char* appName)
{
  fprintf(stderr, "usage: %s [options]\n",appName);
  fprintf(stderr, "Legal Options |       Description           | Defaults \n");
  fprintf(stderr, "--------------|-----------------------------|----------\n");
  fprintf(stderr, "-c format     | color format                |   RGB565  \n");
  fprintf(stderr, "-f filename   | output file name            |   video.raw\n");
  exit(0);
}

int main(int argc, char** argv)
{
  CPlayRaw  *play;
  char      *filename = NULL;
  char      *color_format = "RGB565";
  int        errflag = 0;
	
  try {
    play = new CPlayRaw();
  }
  catch(OPIInternalError) {
    cerr << " OPI Internal error." << endl;
    return APP_ERROR;
  }
	
  if (argc >= 2)
  {
		
    for(int i = 1; i < argc; i++)
    {
      if (strcmp(argv[i], "-f") == 0)
      {
        if (++i < argc)
          filename = strdup(argv[i]);
      }
      else if (strcmp(argv[i], "-c") == 0)
      {
        if (++i < argc)
          color_format = strdup(argv[i]);
      }
      else
        usage(argv[0]);
    }
  }
	
  play->setVideoFile(filename);
	
  if (play->PlayBack(color_format) == APP_ERROR) {
    cerr << " Playback operation failed." << endl;
    delete play;
    return APP_ERROR;
  }
	
  delete play;
  return APP_OK;
	
}
