/* $Id: playraw.cpp 1.8 1998/10/21 14:15:38 scl Initial_Build $ */
/****************************************************************
 ** File: File Long Name ($RCSfile: playraw.cpp $)
 **
 ** ($Revision: 1.8 $)
 **
 ** Abstract:
 **   This class is used to play video from a file to the display
 **   
 ** Copyright (c) 1998, MultiMedia Access Corp
 **                     Osprey Technologies Division
 ****************************************************************/
#include <iostream.h>
#include <stdio.h>
#include "playraw.h"


/****************************************************************
 ** Method: Raw Video Play  (CPlayRaw::CPlayRaw)
 **
 ** Abstract:
 **  Constructor
 **
 ** Error Conditions: None
 **
 ***************************************************************/
CPlayRaw::CPlayRaw()
{
  pipeline   = NULL;
  videofile  = NULL;
        
  // Open the OPI system.  
  opi = new OPISystem();
  
  // setDebugLevel will control the amount of debug information 
  // provided by OPI system. Currently, the only effect 
  // this has is disabling detection of streaming timeouts
  // if passed a non-zero value.
  opi->setDebugLevel(1);
}

/****************************************************************
 ** Method: Raw Video Play  (CPlayRaw::~CPlayRaw)
 **
 ** Abstract: 
 **   Destructor
 **
 ** Error Conditions: None
 **
 ***************************************************************/
CPlayRaw::~CPlayRaw()
{
  if (opi) 
    delete opi;
}

/****************************************************************
 ** Method: Play Back  (CPlayRaw::PlayBack)
 **
 ** Abstract: 
 **   It sets up the video formats.
 **   It creates an OPI Window.
 **   It creates a pipeline to playback the uncompressed image to a 
 **   window.
 **   Sets up the callback for the input stream.
 **
 ** Error Conditions: 
 **   Could not open video file.
 **   Could not create the pipeline.
 **   Could not create the window.
 **   Could not map the pipeline.
 **   No data in input file during priming.
 ***************************************************************/
int CPlayRaw::PlayBack(char* color_format)
{
  // Open the file for reading
  if (videofile) 
  {
    if ((fd = fopen(videofile, "rb")) == NULL) 
    {
      cerr<<"Could not open video file"<<endl;
      return APP_ERROR;
    }
  } 
  else {
    cerr << "video file is not specified" << endl;
    return APP_ERROR;
  }
        
  // Creating the pipeline
  if ((pipeline = new OPIPipeline(opi)) == NULL) 
  {
    cerr << "Could not create pipeline\n";
    return APP_ERROR;
  }       
        
  OPIFormatVideo recordedFmt;
  recordedFmt.compression   = color_format;
  recordedFmt.width         = WIDTH;
  recordedFmt.height        = HEIGHT;
  recordedFmt.depth         = DEPTH;
        
  OPIFormatVideo windowFmt;
  windowFmt.compression    = "WINDOW";
  windowFmt.width          = recordedFmt.width;
  windowFmt.height         = recordedFmt.height;
        
  // Creating the OPI window for the image display.
  OPIWindow* window = NULL;
  window = new OPIWindow(0,0,
			 windowFmt.width,windowFmt.height,
			 "Output");
  if (window  == NULL) 
  {
    cerr << "Could not create OPI window\n" ;
    delete pipeline;
    return APP_ERROR;
  }
  windowFmt.param = (void *)window;
        
  pipeline->firstStage(CAP_VID_OUTPUT, &recordedFmt,
		       &windowFmt);
        
  // Assigning the actual hardware and software to each 
  // of the required actions in the pipeline. If it 
  // returns false hen delete the pipeline and window.
  if (pipeline->mapToDevices() == FALSE) 
  { 
    cerr << "Unable to map to devices\n"; 
    delete window;
    delete pipeline;
    return APP_ERROR;
  }
        
  // Finishing the pipeline construction.
  pipeline->finishConstruction();
        
  // Setting up the call back for the input stream.
  OPIStream* inVidStream = NULL;
  inVidStream = pipeline->getInStream();
  inVidStream->setDataDoneCallback(&DoneBuffer,
				   (void*)this);
        
  //  Prime the pipeline by filling the initial buffers
  //  Once the pipeline is started, we can refill buffers
  //  in the callback
  primedBuffers = 0;
  OPIBuffer *vidBuf;
  while ((vidBuf = inVidStream->getBuffer()) != NULL)
  {
    vidBuf->fsize = fread(vidBuf->data,
			  1, vidBuf->bufferSize(),
			  fd);
    if(vidBuf->fsize  == 0)
    {
       cerr << "No data in input file during priming.\n";
       break;
    }
    primedBuffers++;
    inVidStream->addData(vidBuf);
  }

  // Start the pipeline.
  pipeline->start();
        
  // Wait for data to be played
  while (primedBuffers)
    OPISleep(50);

  // Stop the pipeline
  pipeline->stop();

  // Delete the window and then the pipeline
  delete window;
  delete pipeline;

  fclose(fd);
        
  return APP_OK;
}

/****************************************************************
 ** Method:  Done Buffer  (CPlayRaw::DoneBuffer)
 **
 ** Abstract: 
 **   Copy more data to buffer and add buffer to stream
 **
 ** Error Conditions: None
 **
 ***************************************************************/
void CALLBACK CPlayRaw::DoneBuffer(OPIBuffer* buffer,
				   void* userData)
{
  CPlayRaw* playPtr = (CPlayRaw*) userData;

  // Read more data from input file into buffer.  If there
  // is no more data, indicate that we have one fewer primed
  // buffers and return buffer to free queue.
  buffer->fsize = fread(buffer->data, 1,
			buffer->bufferSize(), playPtr->fd);
  if (buffer == 0)
  {
    playPtr->primedBuffers--;
    playPtr->pipeline->getInStream()->returnBuffer(buffer);
  }
  else {
    // Add buffer to full queue
    playPtr->pipeline->getInStream()->addData(buffer);
  }
}

