
/***********************************************************************
*
*              FILE: DIDAPP.C
*
*    Description - Example DID application
*
*  This example application waits for incoming DID calls, obtains the
*  called number, plays ring tone, answers the call, and speaks the
*  digits. If the called number is in the "invalid" (for this demo)
*  range 9XX, a fast busy is played.
*  If the called number is in the range 8XX, a slow busy is played.
*
*    Build instructions:
*    Microsoft 6.0:
*        CL /AL /c /Zi /Od /W3 /I\VBX\INCLUDE didapp.c
*        link /NOD /COD /PM:VIO didapp,,,os2+LLIBCEP+..\VBX\LIB\me2;
*
*    Borland C 1.0:
*        bcc -c -I\VBX\INCLUDE didapp.c
*        bcc -L\VBX\LIB didapp.obj me2.lib
*
*    IBM C Set/2:
*        icc /c /Gt+ /Ti /Kacoprx /Ss /Sp1 /I\vbx\include didapp.c
*        link386 /De /PM:VIO didapp.obj,didapp.exe,didapp.map,\vbx\lib\me2.lib;
*
*    UNIX:
*        cc -I /opt/nms/mex/include didapp.c /opt/nms/mex/lib/me2lib.a -o didapp -lx
*
*
* Rev 1: 12/27/90 Original
* Rev 2: 03/11/91 Renamed TCDIDT1 to TCPDID
* Rev 3: 06/04/91 Used defines for BOARD_TYPE and NUM_DID_DIGITS
* Rev 4: 10/16/91 Additional comments added
* Rev 5: 8-Sep-93 Allow signal_type to be set in me2.pro
* Rev 6: 5-Nov-93 Compatible with 32-bit compiler
* Rev 7: 7-Feb-94 Require signal_type to be set in command line or ME2.PRO
***********************************************************************/

#define NUM_DID_DIGITS 5    /*xx- Central office sends 3 digits in this example-xx*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "mixed.h"
#include "me2def.h"
#include "me2err.h"
#define FALSE 0
#define TRUE  1
#define REMOTE_ANSWER LPCURON
#define REMOTE_HANGUP LPCUROFF
#define MAXWAIT   600000L
#define FOREVER   (-1L)

/* Message Numbers */
#define MWELCOME        1
#define MGOODBYE        2
#define MMENU           3
#define MSEL1           4
#define MSEL2           5
#define MSEL3           6
#define MINVALID        7
#define MPMPT1          8
#define MPMPT2          9
#define MRECORD         1

void setparm();
void flushttqueue();
ME2INT vox_waitdone();
void doexit();
int  play();
int  recplay();
void docall();
void ogt_exit();
ME2INT waitdone() ;
ME2INT waittrunkevent() ;
ME2INT ogt_waittrunkevent();
void showevent();
void showtrunkevent();

/*xx- GLOBALS-xx*/
ME2INT Vmsg = -1;       /* Voice file handles.*/
ME2INT Vtst = -1;
ME2INT Answered = 0;        /*xx- set in waitdone-xx*/
ME2INT Hungup   = 0;        /*xx- set in waitdone-xx*/
ME2INT LastCallprogevent = 0;   /*xx- set in waitdone-xx*/
ME2INT Voice = 0;           /*xx- set in waitdone-xx*/
ME2INT Trunk; /* for tone detection (port #;0 for demo) */
short  trunk_type = 255; /* for tone and ogt (3 for T1) */
char   *digits; /* dest. dial # */
short  Maxrec = 10;	 /*  maximum voice recording sec. */
char Vmsgfil[20];    /* Name of voice file containing prompts used by demo.*/
char Vtstfil[80];    /* Name of voice file to record test message to. */


/*---------------------------------------------------------------------------
 * main
 *
 *--------------------------------------------------------------------------*/
void main(argc, argv)
  int argc; char **argv;
    {

    ME2INT i ;
    char   digit[NUM_DID_DIGITS+1];
    char   dummy_digit;
    char   buff[20]; 
    ME2INT prompt_handle ;
    ME2INT msglst[32];
    ME2INT msgcnt;
    ME2INT vh ;
    char  *parm ;
  short val ;
  short id=0 ;
  short cpflags[6] ;
  short linenum;
  char  *trunk_type_text ;

    ME2INT event ;
    int    remote_hangup_detected ;
    TRUNKTONETYPE tone ;
    ME2INT trunk;

    if (argc <= 1 || argc > 3 )
        {
        puts ("usage: didapp {port #} [{trunk_type}]");
        exit (0);
        }
    else
      {
        trunk = atoi(argv[1]) ;
        if (argc == 3)
          trunk_type = (short) atoi(argv[2]);
        else trunk_type = 255;
      }

    /*xx- Get ME/2 channel-xx*/
    Me2RequestChannel(1,"DIDAPP ME/2");

    /*xx- Perform initialization necessary to speak numbers-xx*/

    i = Me2PromptLoad( "AMERICAN", &prompt_handle );
    if ( i < SUCCESS )
    {
        printf( "Error loading prompt table AMERICAN.TBL: %d\n", i );
        exit(1);
    }
    i = Me2VoxOpen ("AMERICAN", &vh, PLAYONLY);
    if ( i < SUCCESS )
    {
       printf( "Error opening AMERICAN.VOX: %d\n", i );
       exit(1);
    }

    /*xx- category.-xx*/

    i=Me2ParmByName("TRUNK.overlay", "TCPDID", SETPARM) ;

    if (i != SUCCESS)
    {
        if (i==ME2_CATEGORY_NOT_FOUND)
            printf("Missing TRUNK parameter file\n");
        else
            printf("Error %d setting parameter TRUNK.Overlay\n", i) ;
        exit(1);
    }

    /*xx- by an entry in ME2.PRO.)-xx*/

    if (trunk_type != 255)
        {
        i=Me2ParmByName("TRUNK.SIGNAL_TYPE", &trunk_type, SETPARM) ;
        if (i != SUCCESS)
            {
            printf("Error %d setting parameter TRUNK.Signal_Type\n", i) ;
            exit(1);
            }
        }
    else
        {
        i=Me2ParmByName("TRUNK.SIGNAL_TYPE", &trunk_type, GETPARM) ;
        if (i != SUCCESS)
            {
            printf("Error %d getting parameter TRUNK.Signal_Type\n", i) ;
            exit(1);
            }
        }

    if (trunk_type == 255)
        {
        puts ("\nYou must specify trunk type either via an ME/2 parameter:");
        puts ("\n         TRUNK.SIGNAL_TYPE = <type>");
        puts ("\nor with a command line argument:");
        puts ("\n         didapp {port #} [{trunk_type}]");
        puts ("\n where {trunk_type} is one of");
        puts ("     1 - VBX 400  or AG-8 DID");
        puts ("     2 - VBX 400  or AG-8 E&M");
        puts ("     3 - VBX 1200 or AG Digital (connected to T1 via MVIP)");
        exit (1) ;
        }

    switch (trunk_type)
        {
        case 1: trunk_type_text = "DID" ; break ;
        case 2: trunk_type_text = "E&M" ; break ;
        case 3: trunk_type_text = "T1" ; break ;
        default: trunk_type_text = "???" ; break ;
        }
    printf ("Trunk type is %d (%s)\n", trunk_type, trunk_type_text);

    /*xx- Modify some of the default settings.-xx*/

    /*xx- Change default tone to Slow Busy.  This tone is played by the-xx*/
    /*xx- trunk control program if there is no response from the PC to an-xx*/
    /*xx- incoming call.-xx*/

    parm = "TCPDID.LOCAL_HANGUP" ;                          /* Rev D */
    val = 500 ;
    i=Me2ParmByName(parm, (void*)&val, SETPARM) ;

    parm = "TCPDID.DEFAULT_TONE" ;
    val = 2 ;
    i=Me2ParmByName(parm, &val, SETPARM) ;

    /* Change the timeout to 5 seconds (default is 10 secs). */
    if (i == SUCCESS)
        {
        parm = "TCPDID.PC_TIMEOUT" ;
        val = 500 ;    /* units are 1/100's secs */
        i=Me2ParmByName(parm, (void*)&val, SETPARM) ;
        }

    /*xx- Set the number of digits expected-xx*/
    if (i == SUCCESS)
        {
        parm = "TCPDID.NUM_DIGITS" ;
        val = NUM_DID_DIGITS ;
        i = Me2ParmByName(parm, &val, SETPARM) ;
        }

    if (i != SUCCESS)
        {
        if (i==ME2_CATEGORY_NOT_FOUND)
            printf("Missing TCPDID parameter file");
        else
            printf("Error %d setting DID parameter %s\n", i, parm) ;
        exit(1);
        }

    /*xx- Set the name of the Trunk Control overlay and associated parameter-xx*/
    /*xx- category.-xx*/
/*  setparm ("TRUNK.overlay", "TCPOGT" ) ; */

    /*xx- Set the parameter that defines hardware type, unless already set-xx*/
    /*xx- in ME2.PRO.-xx*/

  i = ME2PARMLOAD("DSP");		/* loading DSP.PRM */

  if( i != SUCCESS ) {	/* see if load DSP.PRO parameter file is success */
    puts("\tError processing .PRO file information.");
    doexit(i);		/* if not, process routine to provide error msg. */
  }
 i = ME2PARMBYNAME( "DSP.APPL_ID", &id, GETPARM );
 if( i != SUCCESS )     /* See if call to ME/2 function is successful */
    {
      puts( "\tError reading DSP.APPL_ID parameter." );
      printf("\t%d",id);
      doexit( i );       /* If not, process routine to provide error msg. */
    } 
  i = Me2ParmByName( "DSP.MAX_RECORD", &Maxrec, GETPARM);
  if( i != SUCCESS )     /* See if call to ME/2 function is successful */
    {
      puts( "\tError reading DSP.MAX_RECORD parameter." );
      doexit( i );       /* If not, process routine to provide error msg. */
    }

  i = Me2ParmByName( "DSP.LINENUM", &linenum, GETPARM);
  if( i != SUCCESS )     /* See if call to ME/2 function is successful */
    {
      puts( "\tError reading DSP.LINENUM parameter." );
      doexit( i );       /* If not, process routine to provide error msg. */
    }

  i = Me2ParmByName( "DSP.TRUNK", &Trunk, GETPARM);
  if( i != SUCCESS )     /* See if call to ME/2 function is successful */
    {
      puts( "\tError reading DSP.TRUNK parameter." );
      doexit( i );       /* If not, process routine to provide error msg. */
    }
printf("\tTrunk => %d\n", Trunk);

  i = Me2ParmByName( "DSP.TRUNK_TYPE", &trunk_type, GETPARM);
  if( i != SUCCESS )     /* See if call to ME/2 function is successful */
    {
      puts( "\tError reading DSP.TRUNK_TYPE parameter." );
      doexit( i );       /* If not, process routine to provide error msg. */
    }

  i = Me2ParmByName( "DSP.VOX_MSG_FILE", Vmsgfil, GETPARM);
  if( i != SUCCESS )     /* See if call to ME/2 function is successful */
    {
      puts( "\tError reading DSP.VOX_MSG_FILE parameter." );
      doexit( i );       /* If not, process routine to provide error msg. */
    }

  /*  Here we are appending our channel number to the end of the test
      voice file name in order to make sure it is unique from the test voice
      file used by any other channels.  It is because of this appending
      process that we only allow six characters in in the test voice file
      name, instead of eight (see parameter file).
    */
    i = Me2ParmByName( "DSP.VOX_REC_FILE", Vtstfil, GETPARM);

    if( i != SUCCESS )     /* See if call to ME/2 function is successful */
        {
        puts( "\tError reading DSP.VOX_REC_FILE parameter." );
        doexit( i );       /* If not, process routine to provide error msg. */
        }

    sprintf (buff, "%d", linenum );          /* Turn line # into ascii. */
    strcat( Vtstfil, buff );                 /* Append to end of name.       */

    /*
      Open the voice file to use for all messages played during the
      demo.  Then open the file that will be used for voice recordings
      during the demonstration.  The function Me2VoxOpen searches the path
      specified by the ME2LOCAL variable for a voice file that is opened for
      PLAYONLY.  (If a voice file is opened for PLAYREC and cannot be found
      through this path, it is created in the current, local subdirectory.)

      It is also recommended practice not to specify extensions on voice
      file names when creating the source code.  This allows the flexibility
      of using other voice modules to support another file format or
      compression algorithim.  The replacement module will append the
      appropriate extension for it, as ME/2 does with the '.VOX' extension.
      NOTE:  This also applies to .PRO files and DATABASES.
    */
    i = Me2VoxOpen(Vmsgfil, &Vmsg, PLAYONLY);
    if( i != SUCCESS )     /* See if call to ME/2 function is successful */
        {
        puts( "\tDEMO.vox file could not be found in the correct subdirectory!" );
        doexit( i );       /* If not, process routine to provide error msg. */
        }

    i = Me2VoxOpen( Vtstfil, &Vtst, PLAYREC);
    if( i != SUCCESS )     /* See if call to ME/2 function is successful */
        {
        puts( "\tError creating/opening the test file." );
        doexit( i );       /* If not, process routine to provide error msg. */
        }

restart:

#ifdef unix
    /* Grab a specific VBX port, initialize it for Trunk service. */
    if ((i = Me2GrabTrunkLine(trunk)) != SUCCESS)
        {
        printf ("Me2GrabTrunkLine returned error %d.\n", i) ;
        exit (1) ;
        }
#else

    /*xx- Now get a port. The LINETRUNK argument causes the trunk overlay to be-xx*/
    /*xx- loaded and initialized.-xx*/
    /*xx- Note: Group 0 is a pre-defined group that contains all VBX devices.-xx*/
    /*xx- You may define and use a different group.-xx*/

    if ( (i=Me2RequestLine( 0 /* group */, LINETRUNK, 0L )) < SUCCESS )
        {
        printf ("Me2RequestLine returned error %d\n", i) ;
        exit (1) ;
        }
    if ((event=waitdone()) == LINRQFAIL)
        {
        printf ("Line request failed. Me2GetLineError returned %d\n",
              Me2GetLineError());
        exit(1);
        }
    else if (event != LINRQDONE)
        {
        printf ("Unexpected completion event from line request: %d (0x%x)\n",
                   event, event);
        exit(0);
        }
    else
        printf ("Line request succeeded.\n");
#endif

    /*xx-  Start the Trunk Control Program at its initial entry point.-xx*/
    if ((i=Me2TrunkStart(0)) != SUCCESS)
        {
        printf ("Me2TrunkStart returned error %d\n", i) ;
        exit (1) ;
        }
    printf("Trunk Control Program started, %d digits are expected\n",
        NUM_DID_DIGITS );

    /*xx- Enter a loop to process calls.-xx*/

    /*xx- The line must be held continuously until the application exits,-xx*/
    /*xx- in order that the trunk protocol is not interrupted, and incoming-xx*/
    /*xx- digits are not missed.-xx*/
    /*xx- Note: when the application exits, the protocol will continue-xx*/
    /*xx- running autonomously on the board.-xx*/

    while (1)
        {
        /*xx- Wait for the trunk event that signifies an incoming-xx*/
        /*xx- call has arrived.   All other trunk events, including incomplete-xx*/
        /*xx- calls, can be ignored.  Note that the touch tone queue (digit-xx*/
        /*xx- queue) will be automatically flushed by the Trunk Manager-xx*/
        /*xx- between calls.-xx*/

        if ((event = ogt_waittrunkevent()) == TRUNKFAIL){
            printf("\tnext loop\n");
            goto port_error ;      /*xx- internal protocol error-xx*/
        }
        else if (event != TRUNKCALLRECD )
            continue;              /*xx- ignore partial calls and other events-xx*/

        /*xx- Got a call. Get the dialed number-xx*/
        for (i=0;i<NUM_DID_DIGITS;i++)
            {
            if (Me2GetTTC(&digit[i], QREMOVE) != SUCCESS)
                {
                printf ("(warning) Missing digit(s)\n");
                break ;
                }
            }
        digit [i] = '\0' ;    /*xx- form null-terminated string-xx*/
        printf ("Digits received: %s\n", digit);

        /*xx- If first digit is 8 or 9, for this demo, just play a tone-xx*/
        /*xx-  until the caller hangs up.-xx*/
        if (digit[0] == '8' || digit[0] == '9')
            {
            if (digit[0] == '8')
                {
                printf("First digit is 8... Playing REORDER tone\n");
                tone = TRUNKREORDER ;
                }
            else
                {
                printf("First digit is 9... Playing BUSY tone\n");
                tone = TRUNKBUSY ;
                }
            if ((i=Me2TrunkTone(tone)) != SUCCESS)
                {
                printf("Error %d sending tone request\n",i);
                goto port_error;
                }
            printf("Tone request successful\n");

            /*xx- Wait for hangup-xx*/
            /*xx- NOTE: If this application is very slow, the trunk control-xx*/
            /*xx- program will time out and start playing the default tone on its-xx*/
            /*xx- own. The TRUNKTIMEOUT event will be received as an indication.-xx*/

            while ((event = ogt_waittrunkevent()) != TRUNKHANGUP)
                {
                if (event == TRUNKFAIL)
                    goto port_error ;      /*xx- internal protocol error-xx*/
                }
                /*xx- go back to top of loop-xx*/
            }
        else
            {
        /*xx- Otherwise play ringing tone for a time based on the dialed number:-xx*/
        /*xx- 001 is 0.01 seconds, 799 is 7.99 seconds; then answer.-xx*/
        /*xx- The argument to Me2TrunkAnswer specifies the amount of time-xx*/
        /*xx- to play ringing tone, in 1/100 seconds.-xx*/

            if ((i=Me2TrunkAnswer(atoi(digit)) ) != SUCCESS)
                {
                printf("Error %d sending response to DID control program\n", i );
                goto port_error ;
                }
            printf ("Trunk Ring+Answer request successful\n");

            /*xx-   The normal case is that after the ring time period, the trunk-xx*/
            /*xx- manager answers the call and then exits.  A TRUNKDONE event-xx*/
            /*xx- will be received.-xx*/
            /*xx-   If the caller hangs up before the trunk manager answers, then the-xx*/
            /*xx- ring tone stops and a TRUNKHANGUP event will be received.  The-xx*/
            /*xx- trunk manager stays active.-xx*/

            /*xx- Wait for the trunk program to exit after performing the answer.-xx*/
            while ((event = ogt_waittrunkevent()) != TRUNKDONE)
                {
                if (event == TRUNKFAIL)
                    goto port_error ;      /*xx- internal protocol error-xx*/
                if (event == TRUNKHANGUP)
                    break ;
                }
            if (event == TRUNKHANGUP)
                {
                printf ("Caller abandoned\n");
                continue ;        /*xx- back to top of loop-xx*/
                }
            remote_hangup_detected = FALSE ;

            /*xx-  If "done" then the trunk overlay has-xx*/
            /*xx- stopped and hangup detection is now being done by the-xx*/
            /*xx- VMOS supervision detector.  From this point on, a remote-xx*/
            /*xx- hangup is reported with a "loop current off" event.-xx*/

            /*xx- Use entry '5' to the rules table to speak individual digits.-xx*/
            i = Me2PromptBuild( prompt_handle, 5, digit, msglst, &msgcnt );
            if ( i != SUCCESS)
                {
                printf( "Error in prompt build: %d\n", i );
                goto hangup ;
                }

            /*xx- Flush TT queue-xx*/
            while (Me2GetTTC(&dummy_digit, QREMOVE) == SUCCESS)
                {
                printf ("Extra digit ignored: %c\n", dummy_digit);
                }

            /*xx- Speak the digits.-xx*/

            i = Me2VoxPlayMsgLst(vh, msgcnt, msglst);
            if ( i != SUCCESS )
                {
                printf( "Error playing message list: %d\n", i );
                }
            if (waitdone() == LPCUROFF)
                {
                remote_hangup_detected = TRUE ;
                goto hangup ;
                }

            /*xx- ... more caller interaction could go here ...-xx*/

            hangup:

            /*xx- Pull all events from the event queue, to find any lurking loop-xx*/
            /*xx- current events.  This is needed to discover if the caller has-xx*/
            /*xx- released and also to ensure that a leftover event doesn't cause-xx*/
            /*xx- problems later.-xx*/

            while (Me2GetEvent(&event, QREMOVE) == SUCCESS)
                {
                if (event == LPCUROFF)
                    remote_hangup_detected = TRUE ;
                }

            /*xx-   Now start the trunk overlay program at a specified entry point-xx*/
            /*xx- to release the current call.-xx*/
            /*xx-   If the call has not been released by the caller, then to-xx*/
            /*xx- initiate the disconnect sequence and wait for caller to release,-xx*/
            /*xx- use entry point 1.-xx*/
            /*xx-   If the call has already been released, then treat an incoming-xx*/
            /*xx- off-hook as a new call (entry point 0).-xx*/

            /*xx-   This function will also disable the VMOS supervision-xx*/
            /*xx- signal monitor because signal detection will be taken over-xx*/
            /*xx- by the trunk control program.-xx*/

            if ((i=Me2TrunkStart(remote_hangup_detected ? 0 : 1)) != SUCCESS)
                {
                printf ("Me2TrunkStart(1) returned error %d\n", i) ;
                exit (1) ;
                }
            printf("Trunk Control Program restarted\n");
        }
        /*xx- Now go to top of loop and wait for another call.-xx*/

    } /*xx- End of call processing loop-xx*/

    /*xx- never get here!-xx*/

port_error:
    /*xx- If there was a protocol error, re-initialize the port.-xx*/
    Me2ReleaseLine() ;

    /*xx- flush event queue-xx*/
    while (Me2GetEvent(&event, QREMOVE) == SUCCESS)
        ;
    goto restart ;

    }

/*******************************************************************
* waitdone - local subroutine to wait for completion of a background
*   function.  If a hangup event is detected, this routine
*   will stop the background function, wait for the completion code,
*   and return a hangup indication.
******************************************************************/

ME2INT waitdone()
    {
    ME2INT ev, hangup_ev;
    ME2INT i ;

    hangup_ev = 0 ;
    while (1)
        {
        i = Me2WaitEvent( -1L );       /* Wait for an event to come through */
                                       /* the event queue.                  */
        if ( i != SUCCESS )
            {
            printf ("Me2WaitEvent returned error %d\n", i) ;
            exit (1) ;
            }

        if ((i=Me2GetEvent(&ev, QREMOVE)) != SUCCESS)
            {
            printf ("Me2GetEvent returned error %d\n", i) ;
            exit (1) ;
            }
        if (ev == LPCUROFF)                       /*xx- If caller hangs up, abort-xx*/
            {                                     /*xx- active function (if any)-xx*/
            printf("REMOTE HANGUP DETECTED\n") ;
            Me2Stop();
#ifdef unix
            return ev;
#else
            hangup_ev  = ev;
#endif
            }

          /*xx- Check for done code.-xx*/
        else if (EXITCODE <= (unsigned)ev && (unsigned)ev <= EXITCMAX )
            if (hangup_ev != 0)
                return hangup_ev ;
            else
                return ev ;

        }
    }

/*******************************************************************
* waittrunkevent - local subroutine to wait for an event from the
*   trunk handler thread.  This routine looks for events in the
*  TRUNKMIN - TRUNK MAX range and also for any exit code.
******************************************************************/

ME2INT waittrunkevent()
    {
    ME2INT i ;
    ME2INT ev;

    while (1)
        {
        i = Me2WaitEvent( 10000L );    /* Wait for an event to come through */
                                       /* the event queue.                  */
        if ( i == ME2_QUEUE_TIMEOUT)
            {
            puts ("still waiting for trunk event...");
            continue ;
            }

printf("\ti => %d, Me2Queue_empty = %d, Qtimeout = %d",i,ME2_QUEUE_EMPTY,ME2_QUEUE_TIMEOUT);

        if ( i != SUCCESS )
            {
            printf ("Me2WaitEvent returned error %d\n", i) ;
            exit (1) ;
            }

        i=Me2GetEvent(&ev, QREMOVE);
     printf("\n event = %d", ev);
        if (i != SUCCESS)
            {
            printf ("Me2GetEvent returned error %d\n", i) ;
            exit (1) ;
            }
         /*xx- Check for done code.-xx*/
        else if (EXITCODE <= (unsigned)ev && (unsigned)ev <= EXITCMAX )
            {
            if (ev == TRUNKFAIL)
                printf ("TRUNKFAIL received\n");
            else if (ev == TRUNKDONE)
                printf ("TRUNKDONE received\n");
            else if (ev == TRUNKSTOP)
                printf ("TRUNKSTOP received\n");

            return ev ;
            }
         /*xx- Check for trunk  code-xx*/
        else if (TRUNKCODEMIN <= (unsigned)ev && (unsigned)ev <= TRUNKCODEMAX)
            {
            showevent(ev);
            return ev ;
            }
        else
            continue ;
        }
    }


void showevent(event)
  ME2INT event;
{
char buff[80] ;
char *etext;
static char     *trunkevent[] = {
       "TRUNKBEG        ",
       "TRUNKCALLRECD   ",
       "TRUNKHANGUP     ",
       "TRUNKTIMEOUT    ",
       "TRUNKSEIZED     ",
       "TRUNKNODIGITS   ",
       "TRUNKPARTIALDIAL",
       "TRUNKPERMANENT  "};

    if (TRUNKCODEMIN <= event && event <= TRUNKCODEMAX)
        etext = trunkevent[event-TRUNKCODEMIN] ;
    else
        {
        sprintf (buff, "0x%x", event);
        etext = buff ;
        }
    printf ("Received event: %s\n", etext);
    return ;
    }
/* ***************************************** */
ME2INT ogt_waittrunkevent()
    {
    ME2INT i;
    ME2INT ev;

    while (1)
        {
printf("\tinit i = %d\n",i);
        i = ME2WAITEVENT( 10000L );    /* Wait for an event to come through */
                                       /* the event queue.                  */
        if ( i == ME2_QUEUE_EMPTY)     /*xx- DOS case-xx*/
            continue ;
printf("\ti => %d, Me2Queue_empty = %d, Qtimeout = %d\n",i,ME2_QUEUE_EMPTY,ME2_QUEUE_TIMEOUT);

        if ( i == ME2_QUEUE_TIMEOUT)
            {
            puts ("still waiting for trunk event...");
            continue ;
            }

        if ( i != SUCCESS )
            {
            printf ("Me2WaitEvent returned error %d\n", i) ;
            ogt_exit (1) ;
            }


        i=ME2GETEVENT(&ev, QREMOVE);
        printf("\tev = %d, exitcode=%d, exitcmax= %d\n, Trunkcodem= %d, trunkcodeM=%d\n",ev,EXITCODE,EXITCMAX,TRUNKCODEMIN,TRUNKCODEMAX);

        if (i != SUCCESS)
            {
            printf ("Me2GetEvent returned error %d\n", i) ;
            ogt_exit (1) ;
            }

        else if (EXITCODE <= ev && ev <= EXITCMAX ) /*xx- Check for done code.-xx*/
            {
            if (ev == TRUNKFAIL)
                printf ("TRUNKFAIL received\n");
            else if (ev == TRUNKDONE)
                printf ("TRUNKDONE received\n");
            else if (ev == TRUNKSTOP)
                printf ("TRUNKSTOP received\n");

            return ev ;
            }
        else if (TRUNKCODEMIN <= ev && ev <= TRUNKCODEMAX ) /*xx- Check for trunk-xx*/
            {
 printf("\nbefore showtrunkevent()\n");
            showtrunkevent(ev);
  printf("\nbefore return\n");
            return ev ; 
            }
        else
            {
            printf ("(waittrunkevent) Got event 0x%x\n", ev);
            continue ;
            }
 printf("\nev => %d\n", ev);
        }
    }
/*xx- waittrunkevent-xx*/


/***********************/

/********************************************************************/
void showtrunkevent(event)
   ME2INT event;
{
static char     *trunkevent [ (TRUNKCODEMAX-TRUNKCODEMIN) +1 ] = {
       "TRUNKBEG        ",
       "TRUNKCALLRECD   ",
       "TRUNKHANGUP     ",
       "TRUNKTIMEOUT    ",
       "TRUNKSEIZED     ",
       "TRUNKNODIGITS   ",
       "TRUNKPARTIALDIAL",
       "TRUNKPERMANENT  ",
       "TRUNKWINKTIMEOUT",
       "TRUNKSTUCKSENDER",
       "TRUNKENDOFDIAL" };

    printf ("Received event: %s\n", trunkevent[event-TRUNKCODEMIN]);
    return ;
}

/*********************************************************************
* ogt_exit - cleanup and exit
*********************************************************************/
void  ogt_exit (stat)
  int stat;
    {
    ME2RELEASELINE();
    ME2RELEASECHANNEL();
    exit (stat) ;
}

/*******************************************************************************
 DOCALL:
      This routine starts the interactive segments of the demonstration.

       Return Value:    None
******************************************************************************/
void docall()
 {
    char sel[15];     /* Character array for storing touchtones. */
    ME2INT result;    /* Return code from Me2GetTTS. */

    puts("\tSTART OF Prompt message");
    Me2Delay( 40L); 
/*    vox_waitdone( MAXWAIT ); */
    flushttqueue();                            /* Empty the touch-tone queue.*/
puts("\tUntil good3");
    if( play( Vmsg, MWELCOME ) ) return;     /* Play the WELCOME message   */
puts("\tUntil good");

    for( ;; )
        {
        if( play(Vmsg, MMENU ) ) return;     /* Play the main menu message.*/

        /* This function collects one touchtone digit */
        Me2GetTTS( "X", "", sel, &result, 1 );

        if( vox_waitdone( MAXWAIT ) ) return;    /* Abort if no touch-tones */
                                             /* entered.                */

        /* See what touchtone was entered */

        if ( result == 'T' || sel[0] == '0' )   /* If timeout occurred or */
            {                                   /* '0' entered, play      */
            play( Vmsg, MGOODBYE );             /* GOODBYE message        */
            return;
        }
        else if( sel[0]>='1' && sel[0] <= '3' ) /* If touchtones '1'-'3'  */
            {                                   /* entered, play MENU     */
            if( play( Vmsg, (sel[0]-'0')+3) )   /* selection message.     */
                return;
        }
        else if( sel[0] == '4' )                /* If touchtone '4' play */
            {                                   /* prompt for recording  */
            if( recplay() )                     /* and playing voice.    */
                return;
        }
        else if( play( Vmsg, MINVALID ) )       /* If anything other than */
            return;
                                                /* '1-4' or '0' play      */
                                                /* invalid selection msg. */

        flushttqueue();                        /* Empty the touch-tone queue.*/
    }
}

/*****************************************************************************
PLAY:
   This routine plays the specified message from a particular voice file.
   It waits for the completion of play before returning.  While playing special
   event processing occurs to detect local phone hangup, <ESC> key press, etc.

   Argument List:    ME2INT vsel    => Voice file handle.

                     ME2INT msg     => Message number to be played.

   Return Value:          value from waitdone()
*****************************************************************************/
ME2INT play( vsel, msg )
    ME2INT vsel, msg;  
 {
    ME2INT i;

    i = Me2VoxPlayMsg( vsel, msg );
    if( i != SUCCESS )                 /* Display error msg if play fails.   */
        {
        puts( "\tProblem with Callprocs voice file.");
        puts("\tReinstall callprocs to correct.\n");
        doexit( i );
        }
    return( vox_waitdone( MAXWAIT ) );
}

/****************************************************************************
RECPLAY:
   This function records a test message and plays it
   back when the record is done executing.


   Return Value:        value from waitdone()
*****************************************************************************/
ME2INT recplay()
    {
    ME2INT i ;

    if ((i=play( Vmsg, MPMPT1 )) != 0)  /* Play prompt for recording test msg */
        return i;

    Me2Beep( 1 );                          /* Beep tone for start of record. */
    if ((i=vox_waitdone( MAXWAIT )) != 0)
        return i;

    i = Me2VoxRecordMsg( Vtst, MRECORD, Maxrec ); /* Record up to Maxrec secs */
    if( i != SUCCESS )                            /* Display error message if */
        {                                         /*  record failed.          */
        puts( "\tError recording speech! Bad file handle or no available disk space." );
        doexit( i );
        }
    if ((i=vox_waitdone( MAXWAIT )) != 0)
        return i;

    flushttqueue();                       /*Empty the touch-tone queue.         */

    if ((i=play( Vmsg, MPMPT2 )) != 0)  /* Play prompt that prepares caller         */
        return i;                       /* to hear what was recorded.        */

    i = Me2VoxPlayMsg( Vtst, MRECORD ); /* Play msg that contains their voice.*/
    if( i != SUCCESS )                  /* Abort if problem with that voice file. */
        {
        puts( "\tError playing message back." );
        doexit( i );
        }
    return( vox_waitdone( MAXWAIT ) );
}

/* ********************************************************************************
*
*
*
********************************************************************************* */

ME2INT vox_waitdone( timeout )
     long timeout;
{
  ME2INT ev ;
  ME2INT i;

  while (1)
  {
      i = Me2WaitEvent( timeout );   /* Wait for an event to come through */
      /* the event queue.                  */
      if ( i == ME2_QUEUE_TIMEOUT )  /* Check and see if a timeout.       */
        {
         Me2Stop();                 /* If timeout, stop everything and   */
	 return( i );               /* abort.                            */
      }

      Me2GetEvent( &ev, QREMOVE );   /* Get the event off the queue*/
      if ( ev == LPCUROFF )          /* Check and see if a hangup occurred*/
	{
	  Me2Stop();                 /* If so, stop everything and   */
          return( 1 );               /* return.                       */
      }

          /* Check for done code.  */
      if( EXITCODE <= (unsigned)ev && (unsigned)ev <= EXITCMAX )
	return( 0 );
  }
}


/*******************************************************************************
DOEXIT:
   This function is used to abort cleanly from the application if a failure
   occurs.  Error descriptions can found in ME2ERR.H.
   Argument List:    ME2INT n       =>    Error code.
   Return Value:     No return
*******************************************************************************/
void doexit( n )
    ME2INT n;
{
    if( Vmsg >= 0 )                 /* Close both voice files if opened. */
        Me2VoxClose( Vmsg );
    if( Vtst >= 0 )
        Me2VoxClose( Vtst );

    Me2ReleaseLine();               /* Release the line                  */
    Me2ReleaseChannel();            /* and channel before exiting.       */

    strcat( Vtstfil, ".vox");
#ifdef __STDC__
    remove( Vtstfil );
#else
    unlink( Vtstfil );
#endif

    if( n != SUCCESS )
        {
        printf( "ME/2 error code: %d, process aborted.\n", n );
        printf( "Refer to 'me2err.h' and your manual for a list of errors.\n" );
    }

    exit( n );
}

/******************************************************************************
FLUSHTTQUEUE:
   This function removes any existing touchtones from the touch-tone queue.
   Return Value:     none
******************************************************************************/
void flushttqueue()
    {
    char c;

    while( Me2GetTTC( &c, QREMOVE ) == 0 )
        ;
    return ;
    }


/*******************************************************************
* setparm - subroutine to call Me2ParmByName and test the return code.
******************************************************************/
void setparm(parm, val)
  char *parm;
  void *val;
    {
    ME2INT i=ME2PARMBYNAME(parm, val, SETPARM) ;
    if (i != SUCCESS)
        {
        if (i==ME2_CATEGORY_NOT_FOUND)
            printf("Category not found for %s\n", parm);
        else
            printf("Error %d setting parameter %s\n", i, parm) ;
        ogt_exit(1);
        }
    return ;
    }


