/* Copyright (c) 1997 MultiMedia Access Corporation - Osprey Systems Division. 
 * 1150 S.E. Maynard Rd., Suite 110, Cary, NC 27511 USA. All Rights Reserved.
 *
 * This product and related documentation is protected by copyright laws and
 * international treaties; however, the user of this software is granted a 
 * nonexclusive, royalty free license to modify, make derivitive works, use,
 * distribute, or otherwise incorporate this software in products which use
 * or incorporate the MMAC product.
 *
 * MMAC reserves the right to change any products herein at any time and 
 * without notice. MMAC does not assume any liability arising from the 
 * application or use of this product, except as expressly agreed to in 
 * writing by MMAC. Use and purchase of this product does not convey a license
 * under any patent rights, copyrights, trademark rights, or any other 
 * intellectual property rights of MMAC, Sun Microsystems, or others. 
 */

#ifndef XIL_GRAB_H
#define XIL_GRAB_H

#include    <pixrect/rasterfile.h>
#include    <xil/xil.h>

/* default device name and device type */
#ifndef DEVNAME
#define DEVNAME "/dev/o1k0"
#endif

#ifndef DEVTYPE
#define DEVTYPE "MMACo1k"
#endif

/* color map */
#define MAX_CMAP_LEN		256
#define CMAP_SIZE		(MAX_CMAP_LEN - 2)

/* file output types */
#define GRAYSCALE_FORMAT          0
#define DITHERED8_FORMAT          1
#define RGB24BIT_FORMAT           2
#define RGB32BIT_FORMAT           3

#define FAILURE  -1
#define SUCCESS  0

/*==========================================================
 * Function:    initSunRasterHeaderInfo
 * Description: Initializing the sun raster file information
 *              for the selected format. 
 *==========================================================*/
void initSunRasterHeaderInfo(struct rasterfile* ras_hdr,
			     int frame_width, 
			     int frame_height,
			     int frame_size, 
			     int cmap_len, 
			     int format);

/*==============================================================
 * Function:    writeSunRasterHeaderInfo 
 * Description: Writes the sun raster header info into the file.
 *              return -1 for failure.
 *              return 0 or positive integer for success.
 *==============================================================*/
int writeSunRasterHeaderInfo(int fd, 
			     struct rasterfile* ras_hdr);

/*=======================================================
 * Function:    writeColorMapInfo 
 * Description: Writes the color map info to the file.
 *              return -1 for failure.
 *              return 0 or positive integer for success.
 *=======================================================*/
int writeColorMapInfo(int fd, 
		      unsigned char* cmap[],
		      int cmap_len);

/*=============================================
 * Function:    createGrayScaleCMap 
 * Description: Creates the grayscale colormap.
 *              return -1 for failure.
 *              return 0 for success.
 *=============================================*/
int createGrayScaleCMap(unsigned char* cmap[],
			int cmap_len, 
			float gamma);

/*==========================================================
 * Function:    createDitheredCMap 
 * Description: Creating the color map for Dithered8 format.
 *              return -1 for failure.
 *              return 0 for failure.
 *==========================================================*/
int createDitheredCMap(XilLookup colorcube855, 
		       XilLookup yuv_to_rgb,
		       unsigned char* cmap[],
		       int cmap_len);

/*==============================================
 * Function:    usage
 * Description: Prints the user input interface.
 *==============================================*/
void usage(char *progName);

#endif /* XIL_GRAB_H */
