// $Id: play.cpp 1.6 1999/01/25 21:36:02 scl Release $ 
//****************************************************************************
// File: File Long Name ($RCSfile: play.cpp $)
//
// ($Revision: 1.6 $)
//
// Abstract:
//   Implementation for the playback operations.
//
// Revision History:
//   $Log: play.cpp $
//   Revision 1.6  1999/01/25 21:36:02  scl
//   added error message on pipeline map fail
//   Revision 1.5  1998/10/14 14:18:32  scl
//   added stdio.h
//   Revision 1.4  1998/07/08 19:37:09  kimng
//   coding stds
//
// Copyright (c) 1998, MultiMedia Access Corp. - Osprey Systems Division
//****************************************************************************
#include <iostream.h>
#include <stdio.h>
#include "play.h"

//****************************************************************************
// Method: CPlay()
//
// Abstract:
//   Constructor
//
// Error Conditions: None
//
//****************************************************************************
CPlay::CPlay()
{
  pipeline   = NULL;
  num_queued = 0;
  audiofile  = NULL;
  
  // Open the OPI system.  
  opi = new OPISystem();
  
  // setDebugLevel will control the amount of debug information 
  // provided by OPI system. Currently, the only effect 
  // this has is disabling detection of streaming timeouts
  // if passed a non-zero value.
  opi->setDebugLevel(1);
}

//****************************************************************************
// Method: ~CPlay()
//
// Abstract:
//   Destructor
//
// Error Conditions: None
//
//****************************************************************************
CPlay::~CPlay()
{
  if (pipeline)
    {
      pipeline->stop();
      delete pipeline;
      pipeline = NULL;
    }
  
  if (opi)
    {
      delete opi;
    }
}

//****************************************************************************
// Method: PlayBack()
//
// Abstract:
//   Opens the file for reading.
//   Creates the pipeline to convert the compressed data to 
//   uncompressed.
//   Creates another branch of the pipeline to send the uncompressed
//   data for output.
//
// Error Conditions: 
//   Could not open audio file.
//   Could not create pipeline.
//   Could not map to pipeline.
//
//****************************************************************************
int CPlay::PlayBack()
{
  
  FILE* fd = NULL;
  
  // Open the file for reading
  if (audiofile) 
    {
      if ((fd = fopen(audiofile, "rb")) == NULL)
	{
	  cerr<<"Could not open audio file"<<endl;
	  return APP_ERROR;
	}
    }
  else
    {
      cerr << "audio file is not specified" << endl;
      return APP_ERROR;
    }
  
  // Creating the pipeline
  if ((pipeline = new OPIPipeline(opi)) == NULL)
    {
      cerr << "Could not create pipeline\n";
      return APP_ERROR;
    }	
  
  // Set up the recorded format variables for the first stage of the pipeline.
  OPIFormatAudio compressedFmt;
  compressedFmt.compression   = "G728";
  compressedFmt.externalInput = FALSE;
  
  // Set up the decompressed format variables for the first stage of the pipeline.
  OPIFormatAudio decompressedFmt;
  decompressedFmt.compression  = "PCM";
  
  // Set up the output format variables
  OPIFormatAudio outputFmt;
  outputFmt.compression = "ANALOG";
  
  // Creating the first stage of the pipeline to do decompression
  OPIStage* audioDecompressionStage;
  audioDecompressionStage = pipeline->firstStage(CAP_AUD_DECOMPRESSION, &compressedFmt,&decompressedFmt);
  
  // Creating second stage of pipeline for output
  OPIStage *audioOutputStage;
  audioOutputStage = pipeline->addStage(audioDecompressionStage, CAP_AUD_OUTPUT, &decompressedFmt, &outputFmt);
  
  // Assigning the actual hardware and software to each of 
  // the required actions in the pipeline. If it returns false 
  // then delete the pipeline and window.
  if (!pipeline->mapToDevices())    
    {  
      cerr<<"Could not map pipeline "<<endl; 
      delete pipeline;
      pipeline = NULL;
      return APP_ERROR;
    }
  
  // Finishing the pipeline construction.
  pipeline->finishConstruction();
  
  // Setting up the call back for the input stream.
  OPIStream* inAudStream = NULL;
  inAudStream = pipeline->getInStream();
  inAudStream->setDataDoneCallback(&DoneBuffer, (void*)this);
  
  // Start the pipeline.
  pipeline->start();
  
  // Read data from a file and then send data to decompressor
  OPIBuffer* audBuf;
  for(;;)
    {
      // Get an empty buffer that we can use.
      if ((audBuf = inAudStream->getBuffer()) == NULL)
	{
	  OPISleep(10);
	  continue;
	}
      
      // Read data from file and place it into buffer.
      if((audBuf->fsize = fread(audBuf->data, 1, audBuf->bufferSize(), fd)) == 0)
	break;
      
      // Send data to pipeline.
      inAudStream->addData(audBuf);
      OPISleep(10);
      num_queued++;
    }
  
  // Wait for buffers to empty.
  while (num_queued > 0) 
    OPISleep(50);
  
  fclose(fd);
  
  return APP_OK;
}

//****************************************************************************
// Method: DoneBuffer()
//
// Abstract:
//   Returns the buffer and decrements the num_queued.
//
// Error Conditions: None
//
//****************************************************************************
void CALLBACK CPlay::DoneBuffer(OPIBuffer* buffer, void* userData)
{
  CPlay* playPtr = (CPlay*) userData;
  playPtr->pipeline->getInStream()->returnBuffer(buffer);
  playPtr->num_queued--;
}

