// $Id: record.cpp 1.5 1999/01/25 21:36:17 scl Release $ 
//****************************************************************************
// File: File Long Name ($RCSfile: record.cpp $)
//
// ($Revision: 1.5 $)
//
// Abstract:
//   Implementations for the record operations.
//
// Revision History:
//   $Log: record.cpp $
//   Revision 1.5  1999/01/25 21:36:17  scl
//   added error message on pipeline map fail
//   Revision 1.4  1998/10/14 14:16:59  scl
//   added stdio.h
//   Revision 1.3  1998/07/08 19:50:10  kimng
//   coding stds
//
// Copyright (c) 1998, MultiMedia Access Corp. - Osprey Systems Division
//****************************************************************************
#include <iostream.h>
#include <stdio.h>
#include "record.h"

//****************************************************************************
// Method: CRecord()
//
// Abstract:
//   Constructor
//
// Error Conditions: None
//
//****************************************************************************
CRecord::CRecord()
{
  pipeline      = NULL;
  fd            = NULL;
  audiofile     = NULL;
  buffers_saved = 0;
  
  // Open the OPI system.
  opi = new OPISystem();
  
  // setDebugLevel will control the amount of debug information 
  // provided by OPI system. Currently, the only effect 
  // this has is disabling detection of streaming timeouts
  // if passed a non-zero value.
  opi->setDebugLevel(1);	
}

//****************************************************************************
// Method: ~CRecord()
//
// Abstract:
//   Destructor
//
// Error Conditions: None
//
//****************************************************************************
CRecord::~CRecord()
{
  if (pipeline)
    {
      pipeline->stop();
      delete pipeline;
      pipeline = NULL;
    }
  
  if (opi)
    {
      delete opi;
      opi = NULL;
    }
  
  if (fd)
    {
      fclose(fd);
      fd = NULL;
    }
  
}

//****************************************************************************
// Method: Record()
//
// Abstract:
//   Opens an audio file for writing.
//   Create a pipeline.
//   Map pipelin to devices.  
//
// Error Conditions: 
//   Could not create a pipeline.
//   Fail to map to pipeline.
//
//****************************************************************************
int CRecord::Record()
{
  if (audiofile) 
    fd = fopen(audiofile,"wb");
  
  // Set up the recorded format variables for the first stage of the pipeline.
  OPIFormatAudio captureFmt;
  captureFmt.compression   = "ANALOG";
  
  // Set up the decompressed format variables for the first stage of the pipeline.
  OPIFormatAudio decompressedFmt;
  decompressedFmt.compression   = "PCM";
  
  // Set up the compressed format variables
  OPIFormatAudio compressedFmt;
  compressedFmt.compression = "G728";
  
  // Creating the pipeline.
  if ((pipeline = new OPIPipeline(opi)) == NULL)
    {
      cerr << "Could not Create pipeline\n";
      return APP_ERROR;
    }	
  
  // Create the first stage of the pipeline.
  OPIStage* captureStage;
  captureStage = pipeline->firstStage(CAP_AUD_CAPTURE, &captureFmt, &decompressedFmt);
  
  // Create the second (compression) stage of the pipeline
  OPIStage *compressionStage;
  compressionStage = pipeline->addStage(captureStage, CAP_AUD_COMPRESSION, &decompressedFmt, &compressedFmt);

  // Assigning the actual hardware and software to
  // each of the required actions in the pipeline. 
  // If it returns false then delete the pipeline
  if (!pipeline->mapToDevices()) 
    {  
      cerr<<"Could not map pipeline "<<endl; 
      delete pipeline;
      pipeline = NULL;
      return APP_ERROR;
    }
  //TODO:  appropriate calls for capture device
  // Finish the pipeline construction
  pipeline->finishConstruction();
  
  // Set up the callback on the output stream.
  pipeline->getOutStream()->setDataReadyCallback(&SaveBuffer, (void*)this);
  
  // Start the pipeline.
  pipeline->start();
  
  return APP_OK;
}

//****************************************************************************
// Method: SaveBuffer()
//
// Abstract:
//   Writes the image data to the file.
//   Increments the variable buffers_saved and returns the buffer.
//
// Error Conditions: None
//
//****************************************************************************
void CALLBACK CRecord::SaveBuffer(OPIBuffer* buffer, void* userData)
{ 
  CRecord* recordPtr = (CRecord*) userData;
  fwrite(buffer->data, 1, buffer->fsize, recordPtr->fd);
  recordPtr->buffers_saved++;
  recordPtr->pipeline->getOutStream()->returnBuffer(buffer);
} 
