/* $Id: main.cpp 1.7 1998/10/20 15:39:06 scl Exp $ */
/****************************************************************
 ** File: File Long Name ($RCSfile: main.cpp $)
 **
 ** ($Revision: 1.7 $)
 **
 ** Abstract:
 **   This is an OPI loopback example which demonstrates the use
 **   of the first instance of the device for video capture &
 **   video compression and the use of the last instance of the
 **   device for decompression.
 **
 **   To execute the program, change directory to where loopback.exe
 **   is located.
 **   Please enter: "loopback" at your prompt.
 **
 ** Copyright (c) 1998, MultiMedia Access Corp
 **                     Osprey Technologies Division
 ****************************************************************/
#include <iostream.h>
#include <stdio.h>
#include <stropts.h>
#include <poll.h>
#include <unistd.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <stdlib.h>
#include <string.h>
#include "loopback.h"

#define OK    0
#define FAIL -1

#define PAL_FORMAT  "PAL"
#define NTSC_FORMAT "NTSC"

int   VIDEO_WIDTH  = 352;
int   VIDEO_HEIGHT = 288;
char* VIDEO_FORMAT = NTSC_FORMAT;

int   WINDOW_WIDTH  = 352;
int   WINDOW_HEIGHT = 288;

void usage(char* s)
{
  cout << " Legal Options  |    Description          |  Defaults\n";
  cout << " ------------------------------------------------------\n";
  cout << "  -v format     | video format (PAL/NTSC) |  NTSC\n";
  cout << "  -qcif         | qcif capture size       |  FALSE\n";
  cout << "  -cif          |  cif capture size       |  TRUE\n";
  cout << "  -rescale      | rescale NTSC height     |  FALSE\n";
}

int main(int argc, char** argv)
{
  CLoopback* loopback;
  int i;
  int cif = 1;
  int rescale = 0;

  for(i = 1; i < argc; i++) {
    if (strcmp(argv[i], "-rescale") == 0) {
      rescale = 1;
    } else if (strcmp(argv[i], "-v") == 0) {
      i++;
      if (strcmp(argv[i], PAL_FORMAT) == 0)
        VIDEO_FORMAT = PAL_FORMAT;
      else if (strcmp(argv[i], NTSC_FORMAT) == 0)
        VIDEO_FORMAT = NTSC_FORMAT;
      else {
        cout << " Invalid Video Format " << argv[i] << "\n";
        usage(argv[0]);
        exit(0);
      }
    } else if (strcmp(argv[i], "-qcif") == 0) {
      cif = 0;
    } else if (strcmp(argv[i], "-cif") == 0) {
      cif = 1;
    } else {
      usage(argv[0]);
      exit(0);
    }
  }

  // set width and height for compression
  if (cif) {
    VIDEO_WIDTH  = 352;
    VIDEO_HEIGHT = 288;
  } else {
    VIDEO_WIDTH  = 176;
    VIDEO_HEIGHT = 144;
  }
  WINDOW_WIDTH  = VIDEO_WIDTH;
  WINDOW_HEIGHT = VIDEO_HEIGHT;

  // for NTSC, window decompression can rescale height back to NTSC
  if (rescale) {
    if (strcmp(VIDEO_FORMAT, NTSC_FORMAT) == 0){
      if (cif)
        WINDOW_HEIGHT = 240;
      else
        WINDOW_HEIGHT = 120;
    } else {
      cerr << "Can not rescale PAL video" << endl;
      return FAIL;
    }
  }
  
  printf("Size %s Width %d Height %d Format %s Display Width %d Height %d\n",
         ((cif) ? "CIF" : "QCIF"),
         VIDEO_WIDTH, VIDEO_HEIGHT, VIDEO_FORMAT, 
         WINDOW_WIDTH, WINDOW_HEIGHT);

  try
    {
      loopback = new CLoopback();
    }
  catch (OPIInternalError)
    {
      cerr << " OPIInternal error." << endl;
      return FAIL;
    }
 
  if (loopback->Loopback() == APP_ERROR)
    {
      cerr << " Loopback operation failed. " << endl;
      delete loopback;
      return FAIL;
    }
  
  while (1)
    OPISleep(50);
  
  delete loopback;

  return OK;
}





