/* $Id: play.cpp 1.7 1998/10/21 14:05:41 scl Initial_Build $ */
/****************************************************************
 ** File: File Long Name ($RCSfile: play.cpp $)
 **
 ** ($$)
 **
 ** Abstract:
 **   Class implementation for the playback operations
 **   
 ** Copyright (c) 1998, MultiMedia Access Corp
 **                     Osprey Technologies Division
 ****************************************************************/
#include <iostream.h>
#include <stdio.h>
#include "play.h"

/****************************************************************
 ** Method: PlayH261  (CPlay::CPlay)
 **
 ** Abstract:
 **  Constructor
 **
 ** Error Conditions: None
 **
 ***************************************************************/
CPlay::CPlay()
{
  pipeline   = NULL;
  num_queued = 0;
  videofile  = NULL;
  window     = NULL;
    
  // Open the OPI system.  
  opi = new OPISystem();
  
  // setDebugLevel will control the amount of debug information 
  // provided by OPI system. Currently, the only effect 
  // this has is disabling detection of streaming timeouts
  // if passed a non-zero value.
  opi->setDebugLevel(1);
}

/****************************************************************
 ** Method: PlayH261  (CPlay::~CPlay)
 **
 ** Abstract:
 **  Destructor
 **
 ** Error Conditions: None
 **
 ***************************************************************/
CPlay::~CPlay()
{   
  if (opi)
    delete opi;
 }

/****************************************************************
 ** Method: PlayH261  (CPlay::Playback)
 **
 ** Abstract:
 **   Opens the video file for reading.
 **   Creates the pipeline to convert the compressed IP64 image to 
 **   uncompressed RGB565.
 **   Creates another branch of the pipeline to send the uncompressed
 **   RGB565 image to an opi window for display.
 **
 ** Error Conditions: 
 **   Could not open video file.
 **   Could not create pipeline.
 **   Could not create OPI window. 
 **   Could not map to pipeline.
 **
 ***************************************************************/
int CPlay::PlayBack()
{
  
  FILE* fd = NULL;
  
  // Open the file for reading
  if (videofile) 
    {
      if ((fd = fopen(videofile, "rb")) == NULL)
	{
	  cerr<<"Could not open video file"<<endl;
	  return APP_ERROR;
	}
    }
  else
    {
      cerr << "video file is not specified" << endl;
      return APP_ERROR;
    }
  
  // Creating the pipeline
  if ((pipeline = new OPIPipeline(opi)) == NULL)
    {
      cerr << "Could not create pipeline\n";
      return APP_ERROR;
    }	
  
  // Set up the recorded format variables for the first stage of the pipeline.
  // In this example, the recorded format is Compressed H261(IP64), 
  // the decompressed format is RGB565.
  OPIFormatVideo recordedFmt;
  recordedFmt.compression   = "IP64";
  
  // Set up the decompressed format variables for the first stage of the pipeline.
  OPIFormatVideo decompressedFmt;
  decompressedFmt.compression  = "RGB565";
  decompressedFmt.width        = 160;
  decompressedFmt.height       = 120;
  decompressedFmt.depth        = 16;
  
  // Creating the first stage of the pipeline
  OPIStage* videoDecompressionStage;
  videoDecompressionStage = pipeline->firstStage(CAP_VID_DECOMPRESSION,&recordedFmt,&decompressedFmt);
  
  // Set up the window format variables for the 2nd stage of the pipeline.
  OPIFormatVideo windowFmt;
  windowFmt.compression    = "WINDOW";
  windowFmt.externalOutput = FALSE;
  windowFmt.width          = decompressedFmt.width;
  windowFmt.height         = decompressedFmt.height;
  
  // Creating the OPI window for the image display.
  if ((window = new OPIWindow(0,0,windowFmt.width,windowFmt.height,"Output")) == NULL)
    {
      cerr << "Could not create OPI window\n" ;
      delete pipeline;
      return APP_ERROR;
    }
  windowFmt.param = (void *)window;
  
  // Creating a branch of the pipeline for viewing the decompressed video.
  pipeline->addStage(videoDecompressionStage, CAP_VID_OUTPUT, &decompressedFmt, &windowFmt);
  
  // Assigning the actual hardware and software to each of 
  // the required actions in the pipeline. If it returns false 
  // then delete the pipeline and window.
  if (!pipeline->mapToDevices())    
    {  
      cerr << "Could not map to pipeline" << endl;
      delete pipeline;
      delete window;
      return APP_ERROR;
    }
  
  // Finishing the pipeline construction.
  pipeline->finishConstruction();
  
  // Setting up the call back for the input stream.
  OPIStream* inVidStream = NULL;
  inVidStream = pipeline->getInStream();
  inVidStream->setDataDoneCallback(&DoneBuffer, (void*)this);
  
  // Start the pipeline.
  pipeline->start();
  
  // Read data from a file and then send data to decompressor
  OPIBuffer* vidBuf;
  for(;;)
    {
      // Get an empty buffer that we can use.
      if ((vidBuf = inVidStream->getBuffer()) == NULL)
	{
	  OPISleep(10);
	  continue;
	}
      
      // Read data from file and place it into buffer.
      if((vidBuf->fsize = fread(vidBuf->data, 1, vidBuf->bufferSize(), fd)) == 0)
	break;
      
      // Send data to pipeline.
      inVidStream->addData(vidBuf);
      OPISleep(10);
      num_queued++;
    }
  
  // Wait for buffers to empty.
  while (num_queued > 0) 
    OPISleep(50);

  // Stop the pipeline.
  pipeline->stop();

  // Delete the window and pipeline
  delete window;
  delete pipeline;

  fclose(fd);
  
  return APP_OK;
}

/****************************************************************
 ** Method: PlayH261  (CPlay::DoneBuffer)
 **
 ** Abstract:
 **   Returns the buffer and decrements the num_queued.
 **
 ** Error Conditions: None
 **
 ***************************************************************/
void CALLBACK CPlay::DoneBuffer(OPIBuffer* buffer, void* userData)
{
  CPlay* playPtr = (CPlay*) userData;
  playPtr->pipeline->getInStream()->returnBuffer(buffer);
  playPtr->num_queued--;
}

