/* $Id: record.cpp 1.1 1999/01/25 17:30:15 scl Exp $ */
/*************************************************************
 ** File: File Long Name ($RCSfile: record.cpp $)
 **
 ** ($ Revision: $)
 **
 ** Abstract:
 **    Class Implementation for the record operations.
 ** 
 **   Copyright (c) 1998, MultiMedia Access Corp
 **                     Osprey Technologies Division
 *************************************************************/
#include <iostream.h>
#include <stdio.h>
#include <string.h>
#include "record.h"

/****************************************************************
 ** Method: Record H263P  (CRecord::CRecord)
 **
 ** Abstract:
 **  Constructor
 **
 ** Error Conditions: None
 **
 ***************************************************************/
CRecord::CRecord()
{
  pipeline      = NULL;
  fd            = NULL;
  fd_sizes      = NULL;
  videofile     = NULL;
  window        = NULL;
  buffers_saved = 0;
  localview     = FALSE;
  
  // Open the OPI system.
  opi = new OPISystem();
  
  // setDebugLevel will control the amount of debug information 
  // provided by OPI system. Currently, the only effect 
  // this has is disabling detection of streaming timeouts
  // if passed a non-zero value.
  opi->setDebugLevel(1);	
}

/****************************************************************
 ** Method: Record H263P  (CRecord::~CRecord)
 **
 ** Abstract:
 **  Destructor
 **
 ** Error Conditions: None
 **
 ***************************************************************/
CRecord::~CRecord()
{
  if (opi)
    delete opi;    
}
/****************************************************************
 ** Method: Record H263P  (CRecord::Record)
 **
 ** Abstract:
 **   Opens a file for writing.
 **   Creates a pipeline to captures video and converts it into RGB565.
 **   Creates another pipeline to convert RGB565 to compressed H263P.
 **   Creates a branch of pipeline to display the local view to an
 **   opi window while the video is being captured to a file.
 **
 ** Error Conditions: 
 **   Could not create pipeline.
 **  Could not create opi window.
 **  Could not map to pipeline.
 **
 ***************************************************************/
int CRecord::Record(int framecount)
{
  // Capture H.261 to a file
  if (videofile) {
    char* fsize_name = new char[strlen(videofile) + 10];
    fd = fopen(videofile,"wb");
    strcpy(fsize_name, videofile);
    strcat(fsize_name, ".sz");
    fd_sizes = fopen(fsize_name,"wb");
    printf("writing to %s and %s\n",videofile, fsize_name);
  }
  
  // Set up the external format variables for the first stage of the pipeline.
  OPIFormatVideo externalFmt;
  externalFmt.compression   = VIDEO_FORMAT;
    
  // Set up the captured format variables for the first stage of the pipeline.
  OPIFormatVideo capturedFmt;
  capturedFmt.width         = VIDEO_WIDTH;
  capturedFmt.height        = VIDEO_HEIGHT;
  capturedFmt.depth         = 16;
  if (localview == TRUE) 
    capturedFmt.compression   = "RGB565"; /* video in a window */
  else
    capturedFmt.compression   = "YUV422"; /* video composite out */
    
  // Creating the pipeline.
  if ((pipeline = new OPIPipeline(opi)) == NULL) {
    cerr << "Could not Create pipeline\n";
    delete pipeline;
    return APP_ERROR;
  }	
  
  // Create the first stage of the pipeline.
  OPIStage* captureStage;
  captureStage = pipeline->firstStage(CAP_VID_CAPTURE, &externalFmt, &capturedFmt);
  
  // Set up the compressed format variables for the second stage of the pipeline.
  OPIFormatVideo compressedFmt;
  compressedFmt.compression   = "H263P";
  compressedFmt.width         = VIDEO_WIDTH;
  compressedFmt.height        = VIDEO_HEIGHT;
  compressedFmt.depth         = 16;
    
  // Creating the 2nd stage of the pipeline.
  OPIStage* videoCompressionStage;
  videoCompressionStage = pipeline->addStage(captureStage, CAP_VID_COMPRESSION, &capturedFmt, &compressedFmt);
  
  if(localview == TRUE) {
    // Setting the window format variables that are needed for the
    // creation of the opi window for the local view while
    // video is being captured to the file.
    OPIFormatVideo windowFmt;
    windowFmt.compression    = "WINDOW";
    windowFmt.externalOutput = FALSE;
    windowFmt.width          = WINDOW_WIDTH;
    windowFmt.height         = WINDOW_HEIGHT;
    windowFmt.depth          = 16;
    
    // Creating the OPI window.
    if((window = new OPIWindow(10,0,windowFmt.width, windowFmt.height, "Local View"))== NULL) {
      cerr << "Could not create OPI window.\n";
      return APP_ERROR;
    }
    windowFmt.param = (void*) window;
    
    // Creating a branch of pipeline for the viewing of the captured video.
    pipeline->addStage(captureStage,CAP_VID_OUTPUT, &capturedFmt, &windowFmt);
  }
  
  // Assigning the actual hardware and software to
  // each of the required actions in the pipeline. 
  // If it returns false then delete the pipeline and window.
  if (!pipeline->mapToDevices()) {  
    cerr <<" Could not map to pipeline\n";
    if (window)
      delete window;
    delete pipeline;
    return APP_ERROR;
  } 


  // Getting the specific compression device and the capture device.
  // Setting some of the attributes of the compression device and 
  // capture device.
  OPICompressionDeviceH263 *cdH263;
  cdH263 = (OPICompressionDeviceH263 *)pipeline->getDeviceFromHandle(videoCompressionStage);
  cdH263->setQuality(max_q);
  cdH263->setBitRate(bitrate);
  
  OPICaptureDeviceVideo *cap = (OPICaptureDeviceVideo *)pipeline->getDeviceFromHandle(captureStage);
  cap->setFrameRate(33);    // in milliseconds per frame
  cap->setOverlayRate(33); // set the frame rate for local view.
  
  // Finish the pipeline construction
  pipeline->finishConstruction();
  
  // Set up the callback on the output stream.
  pipeline->getOutStream()->setDataReadyCallback(&SaveBuffer, (void*)this);
  // Start the pipeline.
  pipeline->start();
  
  while (framecount > buffers_saved)
    OPISleep(10);

  // Stop the pipeline
  pipeline->stop();

  // Delete the window and then the pipeline
  if (window)
    delete window;
  delete pipeline;

  if (fd){
    fclose(fd);
    fclose(fd_sizes);
  }

  return APP_OK;
}

// THIS IS THE H261 RTP PACKET STRUCTURE
typedef struct {
  unsigned long int offset  : 20;
  unsigned long int m       :  4;
  unsigned long int tempref :  8;
  unsigned long int vmvd    :  5;
  unsigned long int hmvd    :  5;
  unsigned long int quant   :  5;
  unsigned long int mbap    :  5;
  unsigned long int gobn    :  4;
  unsigned long int v       :  1;
  unsigned long int i       :  1;
  unsigned long int ebit    :  3;
  unsigned long int sbit    :  3;
} H263PacketInfoFrameType;

/****************************************************************
 ** Method: Record H263P  (CRecord::SaveBuffer)
 **
 ** Abstract:
 **   Writes the image data to the file.
 **   Increments the variable buffers_saved and returns the buffer.
 **
 ** Error Conditions: None
 **
 ***************************************************************/
void CALLBACK CRecord::SaveBuffer(OPIBuffer* buffer, void* userData)
{   
  CRecord* recordPtr = (CRecord*) userData;

  hrtime_t htime = gethrtime();

  // data
  unsigned int size = buffer->fsize;  
  fwrite(buffer->data, 1, buffer->fsize, recordPtr->fd);

  // size
  fwrite(&size, 1, 4, recordPtr->fd_sizes);

  // stamp
  fwrite(&htime, 1, sizeof(hrtime_t), recordPtr->fd_sizes);
  
  recordPtr->buffers_saved++;
  printf("wrote frame[%d] bytes %d\n", recordPtr->buffers_saved, size);
  recordPtr->pipeline->getOutStream()->returnBuffer(buffer);

  fflush(recordPtr->fd_sizes);
  fflush(recordPtr->fd);
} 
