/* $Id: example.cpp 1.6 1998/07/10 18:48:33 brettb Initial_Build $ */
/*******************************************************************
 ** File: File Long Name ($RCSfile: example.cpp $)
 **
 ** ($Revision: 1.6 $)
 **
 ** Abstract:
 **   This example program demonstrates video display.
 **
 **   A capture pipeline is created and is then started.
 **   The pipeline captures video and displays this video 
 **   in a window.
 **
 **   To execute this program, please enter: example [option]
 **   Legal Option   |   Description   |  Defaults
 **   -------------------------------------------------------
 **       -c         |  video format   |  RGB565
 **
 ** Copyright (c) 1998, MultiMedia Access Corp 
 **                     Osprey Tech Division
 ******************************************************************/

#include <iostream.h>
#include <string.h>
#include "opi.h"

#define APP_OK      0
#define APP_FAIL   -1
#define WIDTH     320
#define HEIGHT    240
#define DEPTH      16


/*******************************************************************
 ** Function: Raw Display (main)
 **
 ** Abstract:
 **   Initialize OPI, and display video to a window.
 **
 ** Error Conditions: None
 **
 ******************************************************************/
int main(int argc, char** argv)
{
  int nframes = 100;
  char* video_format = "RGB565";
  
  if (argc >= 2) {
    if (strcmp( argv[1], "-c") == 0)
      video_format = strdup( argv[2]);
    else {
      cout << "Please enter rawdisplay -c [video format] "<< endl;
      return APP_FAIL;
    }
  }
  
  // Initializing the OPI System 
  OPISystem* opiSys = NULL;
  try
  {
    opiSys = new OPISystem();
    opiSys->setDebugLevel( 1);
  }
  catch( OPIInternalError)
  {
    cerr << "An OPI Internal Error has occurred.\n";
    return APP_FAIL;
  }
  
  // Creating the OPI pipeline
  OPIPipeline* capturePipeline = NULL;  
  if((capturePipeline = new OPIPipeline( opiSys)) == NULL)
  {
    cerr << "Could not create pipeline\n";
    delete capturePipeline;
    delete opiSys;
    return APP_FAIL;
  }
  
  // Set up the input format variables for the first stage of
  // the pipeline.
  OPIFormatVideo externalFmt;
  externalFmt.compression = "NTSC";
	
  // Set up the output format variables for the first stage of
  // the pipeline.
  OPIFormatVideo captureFmt;
  captureFmt.width  = WIDTH;
  captureFmt.height = HEIGHT;
  captureFmt.depth  = DEPTH;
  captureFmt.compression = video_format;
	
  // Set up the window format variables for the window stage of
  // the pipeline.
  OPIFormatVideo windowFmt;
  windowFmt.compression    = "WINDOW";
  windowFmt.externalOutput = FALSE;
  windowFmt.width  = WIDTH;
  windowFmt.height = HEIGHT;
  windowFmt.depth  = DEPTH;
  
  // Create the capture stage.
  OPIStage* captureStage;
  captureStage = capturePipeline->firstStage(CAP_VID_CAPTURE,
                                             &externalFmt,
                                             &captureFmt);
  
  // Create the display stage.
  OPIStage* displayStage;
  displayStage = capturePipeline->addStage(captureStage,
                                           CAP_VID_OUTPUT,
                                           &captureFmt,
                                           &windowFmt);
  
  // Mapping the pipeline.
  if(capturePipeline->mapToDevices() == FALSE)
  {
    cerr << "Could not map to pipeline.\n";
    delete capturePipeline;
    delete opiSys;
    return APP_FAIL;
  }
  
  // Finish the pipeline construction.
  capturePipeline->finishConstruction();
  
  // Start the pipeline
  capturePipeline->start();
  
  // Putting a delay immediately after starting the frame capture 
  // to let the user view the captured frames on the display. 
  // This is just a simple example. In an actual application, 
  // the developer may implement a "QUIT" Button here to wait 
  // for a user's response to terminate the application.	
  OPISleep(33*nframes);

  // Delete the OPI pipeline, OPI system and OPI window.  
  capturePipeline->stop();
  delete capturePipeline;
  delete opiSys;
  return APP_OK;
}

