/* $Id: main.cpp 1.5 1998/10/14 14:11:35 scl Exp $ */
/*****************************************************************************
 ** File: File Long Name ($RCSfile: main.cpp $)
 **
 ** ($Revision: 1.5 $)
 **
 ** Abstract:
 **   Main implementation file.
 **   This is an OPI playback example which demonstrates the creation of 
 **   pipeline stages to play back (display) raw (uncompressed) video 
 **   data from a file. 
 **
 **   To execute the program, change directories to where opi_play
 **   is located.  Then enter opi_play -o at your prompt.  Observe the 
 **   command line options.  Execute again, this time choosing 
 **   whatever options are necessary for playback of the desired file.
 **
 ** Revision History:
 **   $Log: main.cpp $
 **   Revision 1.5  1998/10/14 14:11:35  scl
 **   added stdio.h
 **   Revision 1.4  1998/07/27 18:54:14  ott
 **   Minor coding standard change.
 **
 ** Copyright (c) 1998, MultiMedia Access Corp. - Osprey Systems Division
 ****************************************************************************/
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "play.h"

#define OK    1
#define FAIL -1

/*****************************************************************************
 ** Function: Display Program Usage ( usage )
 **
 ** Abstract:
 **   Prints out the valid command line arguments and then exits.
 **
 ** Error Conditions: None
 **
 ****************************************************************************/
void usage(char* appName)
{
  fprintf(stderr, "usage: %s [options]\n",appName);
  fprintf(stderr, "Legal Options |       Description           | Defaults \n");
  fprintf(stderr, "--------------|-----------------------------|----------\n");
  fprintf(stderr, "-c format     | color format                |   RGB32  \n");
  fprintf(stderr, "-z depth      | color depth                 |   16     \n");
  fprintf(stderr, "-w width      | width                       |   320    \n");
  fprintf(stderr, "-h height     | height                      |   240    \n");
  fprintf(stderr, "-s sizediv    | size divisor                |   2      \n");
  fprintf(stderr, "-f filename   | file name                   |   video.raw\n");
  fprintf(stderr, "-o options    | list command line options   |          \n");
  exit(FAIL);
}


/*****************************************************************************
 ** Function: Start Video capture and display ( main)
 **
 ** Abstract:
 **   This program demonstrates playback of video data from a file using OPI.
 **
 ** Error Conditions: None
 **   StartCodec failed. Closing down
 **
 ****************************************************************************/
int main(int argc, char** argv)
{

  // Set default values 
  char* colorFormat  = "RGB32";
  int   colorDepth   = 16;
  int   maxWidth     = 640;
  int   maxHeight    = 480;
  int   sizeDivisor  = 2;
  char* filename     = "video.raw";

  CPlay* play;
  int c;

  try {
    play = new CPlay();
  }
  catch(OPIInternalError) {
    cerr << " OPI Internal error." << endl;
    return FAIL;
  }
  
  while ((c = getopt(argc, argv, "c:z:w:h:s:f:o")) != EOF) {
    switch (c) {
      case 'c': colorFormat  = optarg;                              break;
      case 'z': colorDepth   = atoi(optarg);                        break; 
      case 'w': maxWidth     = atoi(optarg);    sizeDivisor = 1;    break;
      case 'h': maxHeight    = atoi(optarg);    sizeDivisor = 1;    break;
      case 's': sizeDivisor  = atoi(optarg);                        break;
      case 'f': filename     = optarg;                              break;
      case 'o': usage(argv[0]);
      default: usage(argv[0]);
      break;
    }
  }

  maxWidth = maxWidth/sizeDivisor;
  maxHeight = maxHeight/sizeDivisor;
  
  play->setVideoFile(filename);
  
  if (!play->PlayBack(colorFormat,colorDepth,maxWidth,maxHeight)) {
    cerr << " Playback operation failed." << endl;
    delete play;
    return FAIL;
  }
  
  delete play;
  
  return OK;
  
}
