
#include <stdio.h>
#include <xil/xil.h>
 
#include "CamPlayer.h"

/* ************************************ */
/* Initialization of xil display image  */
/* ************************************ */
int init_display (int argc, char **argv, XilSystemState state, XilImage *img,
		  struct subimage *subimg, struct img_proc *imgproc, 
		  struct xwin *xwin)
{
  unsigned int width, height, nbands;
  XilDataType datatype; 
  XilLookup yuv_to_rgb;
  XilColorspace cspace_ycc601;
  XilColorspace cspace_rgb709;

  xil_get_info(img[0], &width, &height, &nbands, &datatype);
  
  /* check for width or height too big */
  if (subimg->child_width > width || subimg->child_height > height) {
    fprintf(stderr, "Maximum width, height = %d,%d\n", width, height);
    xil_close(state);
    exit(1);
  }
    
  if ((subimg->child_width) || (subimg->child_height)) {
    if (!subimg->child_width)
      subimg->child_width = width;
    if (!subimg->child_height)
      subimg->child_height = height;
    if (!(img[0] = xil_create_child(img[0],
					  (width - subimg->child_width) / 2,
					  (height - subimg->child_height) / 2,
					  subimg->child_width, 
					  subimg->child_height, 0, nbands))) { 
      fprintf(stderr, "failed to open child image on MMACo1k\n");
      xil_close(state);
      exit(1);
    }
      
    xil_get_info(img[0], &width, &height, &nbands, &datatype);
  }
    
  if (imgproc->scale.grayscale) {
    subimg->handle = xil_create_child(img[0], 0, 0, width, height, 0, 1);      
  }
  else {
    img[2] = xil_create(state, width, height, nbands, XIL_BYTE);
  }
    
  if (imgproc->scale.doscale) {
    imgproc->scale.scale_factor = 1.0 / imgproc->scale.doscale;
    width = (int) (((float) width * imgproc->scale.scale_factor) + 0.5);
    height = (int) (((float) height * imgproc->scale.scale_factor) + 0.5);
    if (!imgproc->scale.grayscale) {
      img[3] = xil_create(state, width, height, nbands, XIL_BYTE);
    }
  }
  else {
    imgproc->scale.scale_factor = 1.0;
  }
  if (imgproc->scale.grayscale) {
    /* create a 1-band images with the scaled dimensions 
       for capture threshold */
    subimg->image8a = xil_create(state, width, height, 1, XIL_BYTE);
    subimg->image8b = xil_create(state, width, height, 1, XIL_BYTE);
    subimg->image8c = xil_create(state, width, height, 1, XIL_BYTE);
  }
    
  if (xwin->nodisplay) {
    /* for simplicity's sake, the 'xwin->nodisplay' option assumes a 1 band */
    /* XIL_BYTE destination image */
    img[1] = xil_create(state, width, height, 1, XIL_BYTE);
    subimg->display_depth = 8;
  }
  else {
    XSizeHints hints;
    int port;
    char *format;
    char titlebar[1024];
      
    /* xlib window creation */
    xwin->xdisplay = XOpenDisplay(NULL);
    if (!xwin->xdisplay) {
      fprintf(stderr, "Unable to connect to server\n");
      xil_close(state);
      exit(1);
    }
    subimg->display_depth = DefaultDepth(xwin->xdisplay, 
					 DefaultScreen(xwin->xdisplay));
    xwin->xwindow = XCreateSimpleWindow(xwin->xdisplay, 
				  DefaultRootWindow(xwin->xdisplay),
				  xwin->winx, xwin->winy, width, height,
				  0, 0, 0);
    if (!xwin->xwindow) {
      fprintf(stderr, "Unable to create window\n");
      xil_close(state);
      exit(1);
    }
    if (xwin->xyflag) {
      hints.flags = USPosition;
      hints.x = xwin->winx;
      hints.y = xwin->winy;
    }
    else {
      hints.flags = 0;
    }
    xil_get_device_attribute(img[0], "FORMAT", (void **) &format);
    xil_get_device_attribute(img[0], "PORT_V", (void **) &(xwin->port));
    sprintf(titlebar, "%s: Port %d [%s]",
	    (xwin->devname ? xwin->devname : "/dev/o1k0"), xwin->port, format);
    
    XSetStandardProperties(xwin->xdisplay, xwin->xwindow, titlebar, 
			   "o1k_display", 0, argv, argc, &hints);
    
    /* if the user tries to shut down the window using the menu */
    if (closedownAtom = XInternAtom(xwin->xdisplay, "WM_DELETE_WINDOW", False))
      XSetWMProtocols(xwin->xdisplay, xwin->xwindow, &closedownAtom, 1);
      
    /* list window events used */
    XSelectInput(xwin->xdisplay, xwin->xwindow, 
		 ExposureMask | ButtonPressMask);
    
    /* make the window visible */
    XMapWindow(xwin->xdisplay, xwin->xwindow);
      
    /* wait for the window to be mapped (an Expose event) */
    do
      XNextEvent(xwin->xdisplay, &(xwin->event));
    while (xwin->event.xany.type != Expose);
      
    img[1] = xil_create_from_window(state, xwin->xdisplay, xwin->xwindow);
  }
    
  if (subimg->display_depth == 8) {
    if (imgproc->scale.grayscale) {
      if (!xwin->nodisplay) {
	XilLookup grayramp;
	int num_entries = 256;
	int i;
          
	Xil_unsigned8 *graydata = (Xil_unsigned8 *) malloc (3 * num_entries); 

	for (i = 0; i < num_entries; i++) {
	  graydata[i * 3 + 2] = graydata[i * 3 + 1] = graydata[i * 3] = i;
	}
	grayramp = xil_lookup_create(state, XIL_BYTE, XIL_BYTE,
				     3, num_entries, 0,
				     graydata);
	init_cmap(grayramp, xwin->xdisplay, xwin->xwindow, 0);
      }
    }
    else {
      get_dither_info(state, imgproc->scale.rescale_values, 
		      imgproc->scale.offset_values,
		      &(imgproc->mask), &yuv_to_rgb, &(imgproc->colorcube));
      if (!xwin->nodisplay) {
	init_cmap(yuv_to_rgb, xwin->xdisplay, xwin->xwindow, -1);
	xil_lookup_set_offset(imgproc->colorcube, 
			      xil_lookup_get_offset(yuv_to_rgb));
      }
    }
  }
  else if (subimg->display_depth == 24) {
    if (imgproc->scale.grayscale) {
      int i, t;
      Xil_unsigned8 *data = (Xil_unsigned8 *) malloc(256 * 3);
        
      if (data == NULL) {
	fprintf(stderr, "Out of memory\n");
	return 0;
      }
      for (i = 0, t = 0; i < 256; i++, t += 3) {
	data[t] = i;
	data[t + 1] = i;
	data[t + 2] = i;
      }
      imgproc->graylookup = xil_lookup_create(state, XIL_BYTE, XIL_BYTE,
				     3, 256, 0, data);
    }
    else {
      cspace_ycc601 = xil_colorspace_get_by_name(state, "ycc601");
      cspace_rgb709 = xil_colorspace_get_by_name(state, "rgb709");
      if (imgproc->scale.doscale)
	xil_set_colorspace(img[3], cspace_ycc601);
      else
	xil_set_colorspace(img[0], cspace_ycc601);
      xil_set_colorspace(img[1], cspace_rgb709);
    }
  }
  else {
    fprintf(stderr, "visual depth of %d is not supported\n", 
	    subimg->display_depth);
    return 0;
  }

  return 1;
}


/* ******************************* */
/* Selecting function of X events  */
/* ******************************* */
int x_event (struct xwin *xwin)
{
  if (!xwin->nodisplay) {
    while (XPending(xwin->xdisplay)) {
      XNextEvent(xwin->xdisplay, &(xwin->event));
	
      if ((xwin->event.xany.type == ClientMessage &&
	  ((XClientMessageEvent *) & 
	   (xwin->event))->data.l[0] == closedownAtom) ||
	  (xwin->event.xany.type == ButtonPress)) {
	return 0;
      }
      else if (xwin->event.xany.type != Expose) {
	fprintf(stderr, "event %d\n", xwin->event.xany.type);
      }
    }
  }
  return 1;
}


/* ************************************ */
/* Display image on Xwindow format.     */
/* it is controlled by image handle     */
/* ************************************ */
int display_by_handle (XilImage *img, 
		       struct subimage *subimg, struct img_proc *imgproc,
		       struct xwin *xwin)
{
  float low[2];
  float high[2];
  float map[2];

  /* GRAYSCALE */
  int	GS_Threshold_min = 32;
  int	GS_Threshold_max = 253;

  low[0]  = 0.0;
  high[0] = GS_Threshold_min;
  map[0]  = GS_Threshold_min;
  low[1]  = GS_Threshold_max;
  high[1] = 255.0;
  map[1]  = GS_Threshold_max;


  /*    for (frames = 0; frames < total_frames; frames++) { */  
  /* for loop  */
  /* displays only the given num. of frames  */
  while(1){		/* but, while loop */
                        /* displays with the infinite num. of frames */
    if (imgproc->scale.grayscale) {
      if (subimg->display_depth == 24) {
	/* even if imgproc->scale.scale_factor is 1, 
         * this combination of xil_scale()
	 * & xil_lookup() is faster than
	 * xil_lookup(subimg->handle,img[1], imgproc->graylookup)
	 */
	xil_scale(subimg->handle, subimg->image8a, "nearest",
		  imgproc->scale.scale_factor, imgproc->scale.scale_factor);
	xil_threshold(subimg->image8a,  subimg->image8b, &low[0],
		      &high[0], &map[0]); 
	xil_threshold(subimg->image8b, subimg->image8c, &low[1], 
		      &high[1], &map[1]);

	xil_lookup(subimg->image8c, img[1], imgproc->graylookup);
	xil_toss(subimg->image8a);
	xil_toss(subimg->image8b);
	xil_toss(subimg->image8c);
      }
      else {		/* subimg->display_depth is 8 */
	/*
	 * even if scale->scale_factor is 1, xil_scale() is faster than
	 * xil_copy(subimg->handle,img[1]) for this case
	 */
	xil_scale(subimg->handle, subimg->image8a, "nearest",
		  imgproc->scale.scale_factor, imgproc->scale.scale_factor);
	xil_threshold(subimg->image8a, subimg->image8b, &low[0], 
		      &high[0], &map[0]);
	xil_threshold(subimg->image8b, subimg->image8c, &low[1], 
		      &high[1], &map[1]);
          
	xil_copy(subimg->image8c, img[1]);
	xil_toss(subimg->image8a);
	xil_toss(subimg->image8b);
	xil_toss(subimg->image8c);

	if (xwin->nodisplay)
	  xil_sync(img[1]);
      }
    }
    else {
      if (subimg->display_depth == 24) {
	if (imgproc->scale.doscale) {
	  xil_scale(img[0], img[3], "nearest",
		    imgproc->scale.scale_factor, imgproc->scale.scale_factor);
	  xil_color_convert(img[3], img[1]);
	  xil_toss(img[3]);
	}
	else {
	  xil_color_convert(img[0], img[1]);
	}
      }
      else {		/* subimg->display_depth is 8 */
	xil_rescale(img[0], img[2], imgproc->scale.rescale_values,
		    imgproc->scale.offset_values);
	if (imgproc->scale.doscale) {
	  xil_scale(img[2], img[3], "nearest",
		    imgproc->scale.scale_factor, imgproc->scale.scale_factor);
	  xil_ordered_dither(img[3], img[1], 
			     imgproc->colorcube, imgproc->mask);
	  xil_toss(img[3]);
	}
	else {
	  xil_ordered_dither(img[2], img[1],
			     imgproc->colorcube, imgproc->mask);
	}
	if (xwin->nodisplay)
	  xil_sync(img[1]);
	xil_toss(img[2]);
      }
    }

    /* X event selection to close window */
    if (x_event(xwin) == 0) 
      return 0;
  }

  return 1;
}
  
