#include "callprocs.h"

/*xx- GLOBALS-xx*/
ME2INT Vmsg = -1;           /* Voice file handles.*/
ME2INT Vrec = -1;
char   digits[NUM_DEST_DIGITS];    /* dest. dial # */
short  Maxrec = 10;	    /*  maximum voice recording sec. */
char Vmsgfil[20];           /* Name of voice file containing prompts */
char Vrecfil[80];           /* Name of voice file to record usr message to. */

/*******************************************************************
* setparm - subroutine to call Me2ParmByName and test the return code.
******************************************************************/
void setparm(parm, val)
     char *parm;
     void *val;
{
  ME2INT i=ME2PARMBYNAME(parm, val, SETPARM) ;
  if (i != SUCCESS) {
    if (i==ME2_CATEGORY_NOT_FOUND)
      printf("Category not found for %s\n", parm);
    else
      printf("Error %d setting parameter %s\n", i, parm) ;
    ME2RELEASELINE();
    ME2RELEASECHANNEL();
    exit (1) ;
  }
  return ;
}




/*******************************************************************************
DOEXIT:
   This function is used to abort cleanly from the application if a failure
   occurs.  Error descriptions can found in ME2ERR.H.
   Argument List:    ME2INT n       =>    Error code.
   Return Value:     No return
*******************************************************************************/
void doexit( n )
    ME2INT n;
{
  if( Vmsg >= 0 )            /* Close both voice files if opened. */
    ME2VOXCLOSE( Vmsg );
  if( Vrec >= 0 )
    ME2VOXCLOSE( Vrec );

  ME2RELEASELINE();          /* Release the line                  */
  ME2RELEASECHANNEL();       /* and channel before exiting.       */

  strcat( Vrecfil, ".vox");
#ifdef __STDC__
  remove( Vrecfil );
#else
  unlink( Vrecfil );
#endif

  if( n != SUCCESS ) {
    printf( "ME/2 error code: %d, process aborted.\n", n );
    printf( "Refer to 'me2err.h' and your manual for a list of errors.\n" );
  }

  exit( n );
}


/*******************************************************************
* dsp_setparm - set ME2 parameters
*
* For this example, parameters are set by the program.  The same result
* could be obtained by adding the settings to ME2.PRO, or to a user profile
* that is loaded with Me2ParmLoad.  Note that setting of trunk-related
* parameters must be done before getting a port with Me2RequestLine.
********************************************************************/
void dsp_setparms()
{
  short val ;
  char  *parm ;
  ME2INT prompt_handle ;
  ME2INT vh,
         buff[20] ;
  short id=0 ;
  short cpflags[6] ;
  short linenum;
  char  *trunk_type_text ;
  ME2INT i ;


    /*xx- Set the parameter that defines hardware type, unless already set-xx*/
    /*xx- in ME2.PRO.-xx*/

  i = ME2PARMLOAD("DSP");		/* loading DSP.PRM */

  if( i != SUCCESS ) {	/* see if load DSP.PRO parameter file is success */
    puts("\tError processing .PRO file information.");
    doexit(i);		/* if not, process routine to provide error msg. */
  }
 i = ME2PARMBYNAME( "DSP.APPL_ID", &id, GETPARM );
 if(i != SUCCESS) { /* See if call to ME/2 function is successful */
   puts( "\tError reading DSP.APPL_ID parameter." );
   printf("\t%d",id);
   doexit( i );  /* If not, process routine to provide error msg. */
 } 
 i = ME2PARMBYNAME( "DSP.MAX_RECORD", &Maxrec, GETPARM);
 if(i != SUCCESS){ /* See if call to ME/2 function is successful */
    puts( "\tError reading DSP.MAX_RECORD parameter." );
    doexit(i);   /* If not, process routine to provide error msg. */
 }

 i = ME2PARMBYNAME( "DSP.VOX_MSG_FILE", Vmsgfil, GETPARM);
 if(i != SUCCESS){ /* See if call to ME/2 function is successful */
    puts( "\tError reading DSP.VOX_MSG_FILE parameter." );
    doexit(i);   /* If not, process routine to provide error msg. */
 }

  /*  Here we are appending our channel number to the end of the test
      voice file name in order to make sure it is unique from the test voice
      file used by any other channels.  It is because of this appending
      process that we only allow six characters in in the test voice file
      name, instead of eight (see parameter file).
    */
 i = ME2PARMBYNAME( "DSP.VOX_REC_FILE", Vrecfil, GETPARM);

 if(i != SUCCESS){ /* See if call to ME/2 function is successful */
    puts( "\tError reading DSP.VOX_REC_FILE parameter." );
    doexit(i);   /* If not, process routine to provide error msg. */
 }

 sprintf (buff, "%d", linenum);       /* Turn line # into ascii. */
 strcat( Vrecfil, buff );             /* Append to end of name.  */

 i = ME2VOXOPEN(Vmsgfil, &Vmsg, PLAYONLY);
 if(i != SUCCESS){ /* See if call to ME/2 function is successful */
    puts( "\tDSP.vox file could not be found in the correct subdirectory!" );
    doexit(i);  /* If not, process routine to provide error msg. */
 }

 i = ME2VOXOPEN( Vrecfil, &Vrec, PLAYREC);
 if(i != SUCCESS){ /* See if call to ME/2 function is successful */
    puts( "\tError creating/opening the test file." );
    doexit(i);   /* If not, process routine to provide error msg. */
 }
}


/*****************************************************************************
VOX_WAITDONE:
   This function waits for a 'DONE' code to appear on the event queue
   while checking any other events coming in to see if they need special
   handling. All other events are thrown away.  Note that even though Touch-
   Tone events are thrown away here, ME/2 maintains a copy of them in the TT
   queue for Me2GetTTS() and Me2GetTTC() to process.

   All ME/2 functions which take more than a fraction of a second to
   complete are designed to return immediately upon startup and then
   report a 'DONE' event code when the action is totally finished.  A
   function of this type generally has three valid 'DONE' codes to
   indicate one of the following:  finished normally, stopped by
   Me2Stop(), or the function failed and was aborted.

   All events which indicate the termination of a ME/2 function have been
   placed in a special range of values which can be easily checked for.
   vox_waitdone () returns when an event code in the 'DONE' code value range
   appears on the event queue.

   Return Value:        1   A hangup has been detected. Stop currently
                            active function and return.
                        0   No special event has been found.
                       <0   Timeout
*******************************************************************************/
ME2INT vox_waitdone (timeout)
  long timeout;
{
  ME2INT ev ;
  ME2INT i;

  while (1) {
    i = ME2WAITEVENT (timeout);   /* Wait for an event to come through */
    /* the event queue.                  */
    if (i == ME2_QUEUE_TIMEOUT){  /* Check and see if a timeout.  */
      ME2STOP();                  /* If timeout, stop everything  */
      return( i );                /* and abort.                   */
    }

    ME2GETEVENT( &ev, QREMOVE );   /* Get the event off the queue*/
    if (ev == LPCUROFF){     /* Check and see if a hangup occurred*/
      ME2STOP();                 /* If so, stop everything and   */
      return( 1 );               /* return.                       */
    }

          /* Check for done code.  */
    if( EXITCODE <= (unsigned)ev && (unsigned)ev <= EXITCMAX )
      return( 0 );
  }
}



/******************************************************************************
FLUSHTTQUEUE:
   This function removes any existing touchtones from the touch-tone queue.
   Return Value:     none
******************************************************************************/
void flushttqueue()
{
  char c;

  while( ME2GETTTC( &c, QREMOVE ) == 0 ) ;
  return ;
}


/*******************************************************************************
 get_string ():
      This routine has got the expected digits from T1

******************************************************************************/
void collect_digits(digits,length)
     char *digits;  /* Character array for storing touchtones. */
     int length;
{
  ME2INT ttstat;    /* Return code from Me2GetTTS. */
  int i;
  char ndigits[15];
  do {
    ME2GETTTS( "X", "#", ndigits, &ttstat, length);
    vox_waitdone (6000L);
    flushttqueue ();
  } while (ttstat != 77);
/*  for (i=0; i<length; i++)
    ndigits[i] -= 48; */
  strcpy(digits,ndigits);
/*  flushttqueue (); */
}


/*****************************************************************************
PLAY:
   This routine plays the specified message from a particular voice file.
   It waits for the completion of play before returning.  While playing special
   event processing occurs to detect local phone hangup, <ESC> key press, etc.

   Argument List:    ME2INT vsel    => Voice file handle.
                     ME2INT msg     => Message number to be played.

   Return Value:          value from waitdone()
*****************************************************************************/
ME2INT play( vsel, msg )
     ME2INT vsel, msg;  
{
  ME2INT i;

  i = ME2VOXPLAYMSG( vsel, msg );
  if( i != SUCCESS ) {      /* Display error msg if play fails.   */
    puts( "\tProblem with Callprocs voice file.");
    doexit( i );
  }
  vox_waitdone(MAXWAIT); 
  return ;
}


/****************************************************************************
RECPLAY:
   This function records a test message and plays it
   back when the record is done executing.

   Return Value:        value from waitdone()
*****************************************************************************/
ME2INT recplay()
{
  ME2INT i ;
  
  if ((i=play( Vmsg, MPMPT1 )) != 0)  /* Play prompt for recording test msg */
    return i;

  ME2BEEP( 1 );                          /* Beep tone for start of record. */
  if ((i=vox_waitdone( MAXWAIT )) != 0)
    return i;

  i = ME2VOXRECORDMSG(Vrec, MRECORD, Maxrec); /* Record up to Maxrec secs */
  if( i != SUCCESS ){                /* Display error message if */
                                     /*  record failed.          */
    puts("\tError recording speech! Bad file handle or no available disk space." );
    doexit( i );
  }
  if ((i=vox_waitdone( MAXWAIT )) != 0)
    return i;

  flushttqueue();              /* Empty the touch-tone queue.  */

  if ((i=play( Vmsg, MPMPT2 )) != 0)  /* Play prompt that prepares caller         */
    return i;                /* to hear what was recorded.   */

  i = ME2VOXPLAYMSG( Vrec, MRECORD ); /* Play msg that contains their voice.*/
  if( i != SUCCESS ){  /* Abort if problem with that voice file. */
    puts( "\tError playing message back." );
    doexit( i );
  }
  return( vox_waitdone( MAXWAIT ) );
}


/***************************************************************
*   caller_digits ()
*
*       for detect caller digits
*****************************************************************/
void caller_digits (digits,length) 
     char digits[21];
     int length;
{
  int i;

  for (i=0; i<length; i++){
    if (Me2GetTTC (&digits[i], QREMOVE) != SUCCESS){
      printf ("Missing digits\n");
      break;
    }
  }
  digits[i] = '\0';
}
