/* ************************************************************************* 
 *                       vdo.h
 *--------------------------------------------------------------------------
 *         rights reserved by Bo-kyun Na (bkna@top.cis.syr.edu)
 ***************************************************************************
 *  This header file defines a set of objects using MMACo1k and saves the 
 *  captured XIL compressed image seqence (CIS) into a file.
 * *************************************************************************/

Atom closedownAtom;

enum compressor_type {
  JPEG, CELLB, H261, MPEG
};

#define GETOPTARGS "C:D:E:H:I:Q:R:U:V:b:c:f:h:i:m:o:r:s:v:w:x:y:gEdX"

/* Definitions of objects for video processing */
struct scale {
  int doscale;
  float scale_factor;
  float rescale_values[3];
  float offset_values[3];
  int grayscale;
};

typedef struct img_proc {
  XilLookup colorcube;
  XilLookup graylookup;
  XilDitherMask mask;

  struct scale scale;
} IMGPROC;

typedef struct xwin {
  char *titlebar;
  int winx;
  int winy;
  int xyflag;
  int port;
  char *devname;
  Xil_boolean nodisplay;
  Display *xdisplay;
  Window xwindow;
  XEvent event;
} XWIN;

typedef struct vdo_attr {
  int brightness;   /* 512 is not a valid value for any of the */
  int contrast;     /* following. If value is not 512 after    */
  int hue;          /* parsing cmd line args,                  */
  int gain_u;       /* then we know to set the desired value.  */
  int gain_v;

  int total_frames;
  int max_buffers;
  int image_skip;
} VDOTYPE;

struct subimage {
  int child_width;
  int child_height;

  XilImage a;
  XilImage b;
  XilImage c;
};

typedef struct image {
  XilImage handle;

  unsigned int width;
  unsigned int height;
  unsigned int nbands;
  XilDataType datatype; 
  int display_depth;

  XilImage grayscaled;
  XilImage rescaled;
  XilImage scaled;
  XilImage display;

  struct subimage subimg;
} IMAGE;

struct h261 {
  int bitrate;
  int mquant;
};

struct mpeg {
  int bitrate;
  int rate_control;
  int fixed_quant;
};

struct jpeg {

};

typedef struct CIS {
  XilCis cis;
  char *cis_type;
  enum compressor_type do_cis;
  char *cisfile;
  
  struct h261 h261;
  struct jpeg jpeg; 
  struct mpeg mpeg;
} CIS;
  

/* Function declarations */
extern Xil_boolean error_handler(XilError error);
extern int init_cam (XilSystemState state, IMAGE *img, VDOTYPE *vdo, 
		     XWIN *xwin);
extern int init_display (int argc, char **argv, XilSystemState state, 
			 IMAGE *img, IMGPROC *imgproc, XWIN *xwin);
extern int x_event (XWIN *xwin);
extern int display_by_handle (IMAGE *img, IMGPROC *imgproc, XWIN *xwin);
extern int compressor (XilSystemState state, IMAGE *img, CIS *cis, 
		       VDOTYPE *vdo, IMGPROC *imgproc);
extern int decompressor (XilSystemState state, IMAGE *img, CIS *cis, 
			 VDOTYPE *vdo);
