/*  init_camera.c */
/* by Bokyun Na bkna@npac.syr.edu */

#include <stdio.h>
#include <stdlib.h>
#include <xil/xil.h>

#include "vdo.h"

/* Function for initialization of camera */
int init_cam (XilSystemState state, struct image *cam,
	      struct vdo_attr *vdo, struct xwin *xwin)
{
  XilDevice device;
  int format;

  if (!(device = xil_device_create(state, "MMACo1k"))) {
    fprintf(stderr, "Unable to create a device object\n");
    xil_close(state);
    exit(1);
  }
      
  xil_device_set_value(device, "DEVICE_NAME", (void *) xwin->devname);
  xil_device_set_value(device, "PORT_V", (void *) xwin->port);
  if (vdo->image_skip)
    xil_device_set_value(device, "IMAGE_SKIP", (void *) vdo->image_skip);
  if (vdo->max_buffers >= 0)
    xil_device_set_value(device, "MAX_BUFFERS", (void *) vdo->max_buffers);
  
  /*
   * Before the device is created, we can set these values
   * via xil_device_set_value(). After the device is created
   * with xil_create_from_device(), you would use the call
   * to xil_set_device_attribute() on the device image to
   * set these values.
   */
  if (vdo->brightness != 512)
    xil_device_set_value(device, "BRIGHTNESS", (void *) vdo->brightness);
  if (vdo->contrast != 512)
    xil_device_set_value(device, "CONTRAST", (void *) vdo->contrast);
  if (vdo->hue != 512)
    xil_device_set_value(device, "HUE", (void *) vdo->hue);
  if (vdo->gain_u != 512)
    xil_device_set_value(device, "CHROMA_GAIN_U", (void *) vdo->gain_u);
  if (vdo->gain_v != 512)
    xil_device_set_value(device, "CHROMA_GAIN_V", (void *) vdo->gain_v);

  if (!(cam->handle = xil_create_from_device(state, "MMACo1k", device))) {
    fprintf(stderr, "failed to open MMACo1k device\n");
    xil_close(state);
    exit(1);
  }
  xil_device_destroy(device);

  xil_get_device_attribute(cam->handle, "FORMAT_V", (void **) &format);

  return format;
}    
