/*  init_camera.c */

#include <stdio.h>
#include <stdlib.h>
#include <xil/xil.h>

#include "player.h"

/* Function for initialization of camera */
int init_cam ()
{
  XilDevice device;
  
  if (!(device = xil_device_create(state, "MMACo1k"))) {
    fprintf(stderr, "Unable to create a device object\n");
    xil_close(state);
    exit(1);
  }
      
  xil_device_set_value(device, "DEVICE_NAME", (void *) devname);
  xil_device_set_value(device, "PORT_V", (void *) port);
  if (image_skip)
    xil_device_set_value(device, "IMAGE_SKIP", (void *) image_skip);
  if (max_buffers >= 0)
    xil_device_set_value(device, "MAX_BUFFERS", (void *) max_buffers);
  
  /*
   * Before the device is created, we can set these values
   * via xil_device_set_value(). After the device is created
   * with xil_create_from_device(), you would use the call
   * to xil_set_device_attribute() on the device image to
   * set these values.
   */
  if (brightness != 512)
    xil_device_set_value(device, "BRIGHTNESS", (void *) brightness);
  if (contrast != 512)
    xil_device_set_value(device, "CONTRAST", (void *) contrast);
  if (hue != 512)
    xil_device_set_value(device, "HUE", (void *) hue);
  if (gain_u != 512)
    xil_device_set_value(device, "CHROMA_GAIN_U", (void *) gain_u);
  if (gain_v != 512)
    xil_device_set_value(device, "CHROMA_GAIN_V", (void *) gain_v);

  if (!(o1k_image = xil_create_from_device(state, "MMACo1k", device))) {
    fprintf(stderr, "failed to open MMACo1k device\n");
    xil_close(state);
    exit(1);
  }
  xil_device_destroy(device);

  xil_get_device_attribute(o1k_image, "FORMAT_V", (void **) &format);

  return format;
}    
