

unsigned char *clp1;

/* Data for ConvertYUVtoRGB*/
long int crv_tab[256];
long int cbu_tab[256];
long int cgu_tab[256];

long int cgv_tab[256];
long int tab_76309[256];

void init_dither_tab()
{
  long int crv,cbu,cgu,cgv;
  int i;   
  	
  clp1 = (unsigned char *)malloc(1024);
  clp1 += 384;
	
  for(i=-384; i<640;  i++)
	  clp1[i] = (i < 0) ? 0 : ((i > 255) ? 255 : i);
  
  crv = 104597; cbu = 132201;  /* fra matrise i global.h */
  cgu = 25675;  cgv = 53279;
  
  for (i = 0; i < 256; i++) {
    crv_tab[i] = (i-128) * crv;
    cbu_tab[i] = (i-128) * cbu;
    cgu_tab[i] = (i-128) * cgu;
    cgv_tab[i] = (i-128) * cgv;
    tab_76309[i] = 76309*(i-16);
  }
}



            
/**********************************************************************
 *
 *	Name:	         ConvertYUVtoRGB32	
 *	Description:     Converts YUV image to RGB 32
 *	
 *	Input:	         pointer to source luma, Cr, Cb, destination,
 *                       image width and height
 *	Returns:       
 *	Side effects:
 *
 *	Date: 951208	Author: Karl.Lillevold@nta.no
 *					Modified 991104 Tom Stachowiak
 *
 ***********************************************************************/



void ConvertYUVtoRGB32(unsigned char *src0,unsigned char *src1,unsigned char *src2,
					 unsigned char *dst_ori,int width,int height)
{       
  extern long int crv_tab[];
  extern long int cbu_tab[];
  extern long int cgu_tab[];

  extern long int cgv_tab[];
  extern long int tab_76309[];

  int y11,y21;
int y12,y22;
  int y13,y23;
	int y14,y24;
  int u,v; 
  int i,j;
	int c11, c21, c31, c41;
	int c12, c22, c32, c42;
	unsigned int DW;
	unsigned int *id1, *id2;
  unsigned char *py1,*py2,*pu,*pv;
  unsigned char *d1, *d2;
  
  d1 = dst_ori;
  d1 += width*height*4 - width*4;
	d2 = d1 - width*4;
  
  py1 = src0; pu = src1; pv = src2;
	py2 = py1 + width;
 
	id1 = (unsigned int *)d1;
	id2 = (unsigned int *)d2;

  for (j = 0; j < height; j += 2) { 
    /* line j + 0 */
    for (i = 0; i < width; i += 4) {
      u = *pu++;
      v = *pv++;
      c11 = crv_tab[v];
      c21 = cgu_tab[u];
      c31 = cgv_tab[v];
      c41 = cbu_tab[u];
      u = *pu++;
      v = *pv++;
      c12 = crv_tab[v];
      c22 = cgu_tab[u];
      c32 = cgv_tab[v];
      c42 = cbu_tab[u];

      y11 = tab_76309[*py1++]; /* (255/219)*65536 */
      y12 = tab_76309[*py1++];
      y13 = tab_76309[*py1++]; /* (255/219)*65536 */
      y14 = tab_76309[*py1++];

      y21 = tab_76309[*py2++];
      y22 = tab_76309[*py2++];
      y23 = tab_76309[*py2++];
      y24 = tab_76309[*py2++];

      /* alpha */
	  *id1++ = 0;

	  /* RGBR*/
      DW = ((clp1[(y11 + c41)>>16])) |
           ((clp1[(y11 - c21 - c31)>>16])<<8) |
           ((clp1[(y11 + c11)>>16])<<16) |  
           ((clp1[(y12 + c41)>>16])<<24);
      *id1++ = DW;

      /* GBRG*/
      DW = ((clp1[(y12 - c21 - c31)>>16])) |
           ((clp1[(y12 + c11)>>16])<<8) |  
           ((clp1[(y13 + c42)>>16])<<16) |
           ((clp1[(y13 - c22 - c32)>>16])<<24);
      *id1++ = DW;

      /* BRGB*/
      DW = ((clp1[(y13 + c12)>>16])) |  
           ((clp1[(y14 + c42)>>16])<<8) |
           ((clp1[(y14 - c22 - c32)>>16])<<16) |
           ((clp1[(y14 + c12)>>16])<<24);  
      *id1++ = DW;

      /* alpha */
	  *id2++ = 0;

	  /* RGBR*/
      DW = ((clp1[(y21 + c41)>>16])) |
           ((clp1[(y21 - c21 - c31)>>16])<<8) |
           ((clp1[(y21 + c11)>>16])<<16) |  
           ((clp1[(y22 + c41)>>16])<<24);
      *id2++ = DW;

      /* GBRG*/
      DW = ((clp1[(y22 - c21 - c31)>>16])) |
           ((clp1[(y22 + c11)>>16])<<8) |  
           ((clp1[(y23 + c42)>>16])<<16) |
           ((clp1[(y23 - c22 - c32)>>16])<<24);
      *id2++ = DW;

      /* BRGB*/
      DW = ((clp1[(y23 + c12)>>16])) |  
           ((clp1[(y24 + c42)>>16])<<8) |
           ((clp1[(y24 - c22 - c32)>>16])<<16) |
           ((clp1[(y24 + c12)>>16])<<24);  
      *id2++ = DW;
    }
    id1 -= (12 * width)>>2;
    id2 -= (12 * width)>>2;
    py1 += width;
    py2 += width;
  }           
}  



