
#include "odo.h"

void main (int argc, char* argv[])
{
  enum MODE {PLAY, REC};

  enum MODE aud_mode = PLAY;

  int i;
  int rc;

  ODO odo;

  odo.aud_fd = -1;
  odo.audctl_fd = -1;
  odo.poll_enabled = 0;
  odo.buf_sz = 1000;

  (char *) odo.dev.aud_device = "/dev/o1k0";
  odo.dev.format = 0;
  odo.dev.rate = 8000;
  odo.dev.channels = 1;
  odo.dev.port = AUDIO_LINE_IN; 
 /*  odo.dev.port = 0; */
  odo.dev.igain = 0.125;
  odo.dev.mgain = 0.0;
  odo.dev.ogain = 1.0;

  odo.rec.output_fd = -1;
  odo.rec.byte_count = 0;
  odo.rec.recording = 0;
  odo.rec.hash_amt = 0;
  odo.rec.hash_cnt = 0;

  odo.play.playing = 0;
  odo.play.input_fd = -1;
  odo.play.aud_amnt = 0;
  (char *) odo.play.aud_ptr = odo.aud_data;


  strcpy(odo.info, "");

  for (i=1; i<argc; i++){
    switch (argv[i][1])
      {
      case 'D':
	{
	  odo.dev.aud_device = argv[++i];
	  break;
	}
      case 'f':
	{
	  odo.dev.format = atoi(argv[++i]);
	  break;
	}
      case 'r':
	{
	  odo.dev.rate = atoi(argv[++i]);
	  break;
	}
      case 'c':
	{
	  odo.dev.channels = atoi(argv[++i]);
	  break;
	}
      case 'p':
	{
	  odo.dev.port = atoi(argv[++i]);
	  break;
	}
      case 'g':
	{
	  odo.dev.igain = atof(argv[++i]);
	  fprintf(stderr, "Mic gain %f\n", odo.dev.igain);
	  break;
	}
      case 'm':
	{
	  odo.dev.mgain = atof(argv[++i]);
	  break;
	}
      case 's':
	{
	  strncpy(odo.info, argv[++i],120);
	  break;
	}
      case 'b':
	{
	  odo.buf_sz = atoi(argv[++i]);
	  if (odo.buf_sz > MAX_BUFFER_SIZE){
	    fprintf(stderr,"Desired buffer size %d too large. Using %d\n",
		    odo.buf_sz, MAX_BUFFER_SIZE);  
	    odo.buf_sz = MAX_BUFFER_SIZE;
	  }
	  break;
	}
      case 'h':
	{
	  odo.rec.hash_amt = atoi(argv[++i]);
	  odo.rec.hash_cnt = odo.rec.hash_amt;
	  break;
	}
      case 'i':
	{
	  aud_mode = PLAY;
	  strcpy(odo.filename, argv[++i]);
	  if (audio_isaudiofile(odo.filename)) {
	    odo.play.input_fd = open(odo.filename, O_RDONLY, 0666);
	    if (odo.play.input_fd < 0)
	      fprintf(stderr,"could not open %s\n", odo.filename);
	  } 
	  else {
	    fprintf(stderr,"file %s is not an audio file\n", odo.filename);
	  }
	  break;
	}
      case 'o':
	{
	  aud_mode = REC;
	  strcpy(odo.filename, argv[++i]);
	  odo.rec.output_fd = open(odo.filename, 
				   O_WRONLY | O_CREAT | O_TRUNC, 0666);
	  if (odo.rec.output_fd < 0)
	    fprintf(stderr,"could not create %s\n", odo.filename);
	  break;
	}
      case 'v':
	{
	  odo.dev.ogain = atof(argv[++i]);
	  break;
	}
      case '?':
      default:
	{
	  usage(argv[0]);
	  exit(0);
	}
      }
  }

  if ((odo.rec.output_fd || odo.play.input_fd) < 0){
    usage(argv[0]);
    exit(0);
  }

  /* setup signal handlers  */
  signal(SIGINT,  g_exit);
  signal(SIGPOLL, poll_handler);

  printf("initializing audio device...\n");

  /* This call must be made before any other   */
  if ((rc = oti_audio_init(NULL, NULL)) < 0){
    fprintf(stderr,"oti_audio_init failed %d\n",rc);
    g_exit(&odo, 0);
  }

  if (aud_mode == PLAY) {
    if (odo_play (&odo)) {
      fprintf (stderr, "Unable to play audio sound\n");
      exit (1);
    }
  }
  else if (aud_mode == REC) {
    if (odo_record (&odo)) {
      fprintf (stderr, "Unable to record audio sound\n");
      exit (1);
    }
  }

}
