/***************************************************************************
 * VideoConferencing Tool 
 *      on Linux X Window (with v4l2 driver)
 *      all rights reserved 
 *      by Bo-kyun Na (bkna@npac.syr.edu), IntellActive Research Inc.
 ***************************************************************************
 *	gcc -o vdo -L/usr/X11R6/lib/ -lXt -lXaw -Wall vdo.c
 ***************************************************************************/

#define STREAMBUFS	4

#include "vdo.h"

VIMAGE	vimage[STREAMBUFS];


/* *************************************************************************
 * Function : int main (int argc, char *argv[])
 *
 * ************************************************************************* */
void main (int argc, char **argv)
{
  Widget viewport;
  XtAppContext	app;
  V4L2 v4l2;
  char	my_device[64];
  int	vid;
  int	x_depth = 0;
  int	argi;

  /* Put in the device node name */
  strcpy(my_device, VDO_DEVICE);
  for (argi = 1; argi < argc; ++argi) {
    if (strcmp(argv[argi], "-b") == 0) {
      x_depth = 32;
      continue;
    }
    strcpy(my_device, argv[argi]);
  }

  /* Initialization of video device */
  vid = open(my_device, O_RDONLY);
  if (vid < 0) {
    printf("No video device \"%s\"\n", my_device);
    exit (0);
  }

  /* print info of video device */
  if (ioctl(vid, VIDIOC_QUERYCAP, &v4l2.cap)) {
    printf("QUERYCAP returned error %d\n", errno);
    exit (0);
  }
  printf("Device node:  %s\n", my_device);
  printf("Device:       %s\n", v4l2.cap.name);
  printf("Capabilities:");
  if (v4l2.cap.flags & V4L2_FLAG_READ)       printf(" capture via read();");
  if (v4l2.cap.flags & V4L2_FLAG_STREAMING)  printf(" streaming capture;");
  if (v4l2.cap.flags & V4L2_FLAG_PREVIEW)    printf(" automatic preview;");
  if (v4l2.cap.flags & V4L2_FLAG_SELECT)     printf(" select() call;");
  if (v4l2.cap.flags & V4L2_FLAG_TUNER)      printf(" tuner;");
  if (v4l2.cap.flags & V4L2_FLAG_MONOCHROME) printf(" monochrome only;");
  printf("\n");
  if (v4l2.cap.type != V4L2_TYPE_CAPTURE) {
    printf("Not a capture device.\n");
    exit (0);
  }

  /* Function: to get the video from my camera */
  if (BV_get_vdo (vid, &v4l2)) {
    printf ("Fail: Camera image capturing\n");
    exit (0);
  }

  /* Function: to initialize Xwindow */
  if (BV_init_xwin (vid, &app, &x_depth, &viewport, &v4l2, argc, argv)) {
    printf ("Fail: initialization of xwindow\n");
    exit (0);
  }

  v4l2.req.count = 1;		/* Num of requested window displays */

 /* X win setting  */
  if(BV_set_xwin (vid, &x_depth, &viewport, vimage, &app, &v4l2)) {
    printf ("Fail: set up xwindow\n");
    exit (0);
  }

}





