

#include "vdo.h"


/* *************************************************************************
 * int init_vidcap ()
 *
 *
 * ************************************************************************* */
int BV_init_vidcap (char *my_device, V4L2 *v4l2)
{
  int vid;

  /* Initialization of video device */
  vid = open(my_device, O_RDONLY);
  if (vid < 0) {
    printf("No video device \"%s\"\n", my_device);
    exit (0);
  }

  /* print info of video device */
  if (ioctl(vid, VIDIOC_QUERYCAP, &v4l2->cap)) {
    printf("QUERYCAP returned error %d\n", errno);
    exit (0);
  }
  printf("Device node:  %s\n", my_device);
  printf("Device:       %s\n", v4l2->cap.name);
  printf("Capabilities:");
  if (v4l2->cap.flags & V4L2_FLAG_READ)       printf(" capture via read();");
  if (v4l2->cap.flags & V4L2_FLAG_STREAMING)  printf(" streaming capture;");
  if (v4l2->cap.flags & V4L2_FLAG_PREVIEW)    printf(" automatic preview;");
  if (v4l2->cap.flags & V4L2_FLAG_SELECT)     printf(" select() call;");
  if (v4l2->cap.flags & V4L2_FLAG_TUNER)      printf(" tuner;");
  if (v4l2->cap.flags & V4L2_FLAG_MONOCHROME) printf(" monochrome only;");
  printf("\n");
  if (v4l2->cap.type != V4L2_TYPE_CAPTURE) {
    printf("Not a capture device.\n");
    exit (0);
  }
  return vid;
}


/* *************************************************************************
 * int BV_close_vid ()
 *
 *
 * ************************************************************************* */
int BV_close_vid (int fd)
{
  int rt;

  rt = close (fd);
  if (rt == -1) {
    perror ("Error closing /dev/video");
    exit (1);
  }

  return rt;
}

/* *************************************************************************
 * void printctrl (struct v4l2_queryctrl *qc)
 *
 *
 * ************************************************************************* */
void printctrl (struct v4l2_queryctrl *qc)
{	
  printf("  %-12s %-6s %8d %8d %7d %8d  %s\n", qc->name,
	 (qc->type == V4L2_CTRL_TYPE_MENU) ? "(menu)" : 
	 (qc->type == V4L2_CTRL_TYPE_BOOLEAN) ? "(bool)" : "(int)",
	 qc->minimum, qc->maximum, qc->step, qc->default_value, qc->catname);
}

/* *************************************************************************
 * int BV_get_vdo ()
 *
 *
 * ************************************************************************* */
int BV_get_vdo (int vid,  V4L2 *v4l2)
{
  int i;
  int input;
  int err;
  struct v4l2_queryctrl qc;

  printf("Video inputs on this device:\n");
  for (i = 0, err = 0; err == 0; ++i) {	
    struct v4l2_input inp;
    inp.index = i;
    err = ioctl(vid, VIDIOC_ENUMINPUT, &inp);
    if (!err)
      printf("  %d: \"%s\" is a%s, %s audio\n", i, inp.name,
	     (inp.type == V4L2_INPUT_TYPE_TUNER) ?
	     " TV tuner" : "n analog input",
	     (inp.capability & V4L2_INPUT_CAP_AUDIO) ? "has" : "no");
  }

  input = VDO_INPUT;
  if (ioctl(vid, VIDIOC_S_INPUT, input))
    printf("S_INPUT returned error %d\n",errno);

  printf("Video standards supported:");
  for (i = 0, err = 0; err == 0; ++i) {
    struct v4l2_enumstd	estd;
    estd.index = i;
    err = ioctl(vid, VIDIOC_ENUMSTD, &estd);
    if (!err) {
      printf("%c %s", i?',':' ', estd.std.name);
      if (strcmp(VDO_STD, estd.std.name) == 0)
	v4l2->std = estd.std;
    }
  }
  printf("\n");

  printf("Video capture image formats supported:\n");
  for (i = 0, err = 0; err == 0; ++i) {
    struct v4l2_fmtdesc	fmtd;
    fmtd.index = i;
    err = ioctl(vid, VIDIOC_ENUM_CAPFMT, &fmtd);
    if (!err)
      printf("  %d: %s  (%s)\n", i, fmtd.description,
	     (fmtd.flags & V4L2_FMT_FLAG_COMPRESSED) ?
	     "compressed" : "uncompressed");
  }

  printf("Controls supported:\n");
  printf("  Label        type    Minimum  Maximum    Step  Default"
	 " Category\n");
  qc.id = V4L2_CID_BRIGHTNESS;
  if (ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0)
    printctrl(&qc);
  qc.id = V4L2_CID_CONTRAST;
  if (ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0)
    printctrl(&qc);
  qc.id = V4L2_CID_SATURATION;
  if (ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0)
    printctrl(&qc);
  qc.id = V4L2_CID_HUE;
  if (ioctl(vid, VIDIOC_QUERYCTRL, &qc) == 0)
    printctrl(&qc);

  return 0;
}


/* *************************************************************************
 *
 *
 *
 * ************************************************************************* */
