/*	X Window Video for Linux Two Control Panel Tool
 *
 *	For testing a Video for Linux Two driver
 *
 *	This program was written by Bill Dirks.
 *	This program is in the public domain.
 *
 *	gcc -o vidpanel -L/usr/X11R6/lib/ -lXt -lXaw3d -Wall vidpanel.c
 */

/*  Set these according to your set-up */
#define MY_DEVICE "/dev/video0"


#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/mman.h>
#include <errno.h>

/* These are needed to use the V4L2 driver */
#include <linux/fs.h>
#include <linux/kernel.h>
#include <linux/videodev.h>  /* Video for Linux Two */

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw3d/Command.h>
#include <X11/Xaw3d/Box.h>
#include <X11/Xaw3d/Scrollbar.h>
#include <X11/Xaw3d/MenuButton.h>
#include <X11/Xaw3d/SimpleMenu.h>
#include <X11/Xaw3d/SmeBSB.h>


typedef struct
{
	struct v4l2_queryctrl	qc;
	Widget			box;
	Widget			label;
	Widget			scroll;
}	ToneScroll;

typedef struct
{
	Widget			entry;
	Widget			menu;
	Widget			menubutton;
	struct v4l2_input	vi;
} VideoInput;


int			vid;
ToneScroll		tone[4];
VideoInput		input[8];

int	tone_control_id[] = 
{
	V4L2_CID_BRIGHTNESS, V4L2_CID_CONTRAST,
	V4L2_CID_SATURATION, V4L2_CID_HUE,
};


void quit(Widget w, XtPointer client, XtPointer call)
{
	exit(0);
}


#define SLIDERLEN	160

void
jumpproc(Widget w, XtPointer client, XtPointer call)
{
	struct v4l2_control	vc;
	ToneScroll		*tone = (ToneScroll *)client;
	float			percent = *(float *)call;

	vc.id = tone->qc.id;
	vc.value = tone->qc.minimum
		 + (int)(percent * (tone->qc.maximum - tone->qc.minimum + 1));
	if (vc.value > tone->qc.maximum)
		vc.value = tone->qc.maximum;
	ioctl(vid, VIDIOC_S_CTRL, &vc);
}

void
scrollproc(Widget w, XtPointer client, XtPointer call)
{
	Arg		arg[1];
	XtArgVal	*p;
	int		d = (int)call;
	static float	f;

	XtVaGetValues(w, XtNtopOfThumb, &f, NULL);
	f += (float)d / (float)SLIDERLEN;
	if (f < 0) f = 0;
	if (f > 1) f = 1;
	
	p = (XtArgVal *)&f;
	if (sizeof(float) > sizeof(XtArgVal))
		XtSetArg(arg[0], XtNtopOfThumb, &f);
	else
		XtSetArg(arg[0], XtNtopOfThumb, *p);
	XtSetValues(w, arg, 1);
	jumpproc(w, client, &f);/* to change the control */
}

void
init_tone_control(ToneScroll *tone)
{
	struct v4l2_control	vc;
	int			d;
	static float		f;
	Arg			arg[1];
	XtArgVal		*p;

	/*  Initialize the thumb position  */
	vc.id = tone->qc.id;
	if (ioctl(vid, VIDIOC_G_CTRL, &vc))
		return;
	d = (SLIDERLEN * (vc.value - tone->qc.minimum)
	     + ((tone->qc.maximum - tone->qc.minimum) >> 1))
		/ (tone->qc.maximum - tone->qc.minimum);

	f = 0;
	p = (XtArgVal *)&f;
	if (sizeof(float) > sizeof(XtArgVal))
		XtSetArg(arg[0], XtNtopOfThumb, &f);
	else
		XtSetArg(arg[0], XtNtopOfThumb, *p);
	XtSetValues(tone->scroll, arg, 1);
	scrollproc(tone->scroll, tone, (XtPointer)d);
}

void
add_tone_control(ToneScroll *tone, Widget parent)
{
	tone->box =
		XtVaCreateManagedWidget("tonebox",
					boxWidgetClass, parent,
					XtNorientation, XtorientHorizontal,
					XtNhSpace, 2,
					XtNvSpace, 2,
					XtNborderWidth, 0,
					NULL);
	tone->scroll =
		XtVaCreateManagedWidget("scroll",
					scrollbarWidgetClass,
					tone->box,
					XtNorientation, XtorientHorizontal,
					XtNlength, SLIDERLEN,
					XtNminimumThumb, 14,
					XtNthickness, 18,
					NULL);
	tone->label = 
		XtVaCreateManagedWidget("label",
					labelWidgetClass, tone->box,
					XtNlabel, tone->qc.name,
					XtNborderWidth, 0,
					NULL);
	XtAddCallback(tone->scroll, XtNscrollProc, scrollproc, tone);
	XtAddCallback(tone->scroll, XtNjumpProc, jumpproc, tone);
	init_tone_control(tone);
}



Widget
new_menu(Widget parent, char *name)
{
	Widget	menu;
	menu =
		XtVaCreatePopupShell(name,
				     simpleMenuWidgetClass, parent,
				     NULL);
	return menu;
}

void
inputproc(Widget w, XtPointer client, XtPointer call)
{
	VideoInput	*vi = (VideoInput *)client;
	int		input;
	int		i;

	ioctl(vid, VIDIOC_G_INPUT, &input);
	if (input != vi->vi.index)
	{
		input = vi->vi.index;
		ioctl(vid, VIDIOC_S_INPUT, input);
	}

	for (i = 0; i < sizeof(tone)/sizeof(tone[0]); ++i)
		init_tone_control(&tone[i]);
}

void
add_input_menu_entry(VideoInput *vi, Widget menu)
{
	vi->menu = menu;
	vi->entry = 
		XtVaCreateManagedWidget(vi->vi.name,
					smeBSBObjectClass, menu,
					XtNlabel, vi->vi.name,
					NULL);
	XtAddCallback(vi->entry, XtNcallback, inputproc, vi);
}



int
main(int argc, char *argv[])
{
	XtAppContext		app;
	Widget			toplevel;
	Widget			w_mainbox;
	Widget			w_menubox;
	Widget			w_tonebox;
	Widget			w_inputmb;
	Widget			w_inputmenu;
	Widget			w_quit;
	
	char			my_device[64];
	int			i;


/*->	Put in the device node name */
	strcpy(my_device, MY_DEVICE);
	if (argc >= 2)
		strcpy(my_device, argv[1]);
	vid = open(my_device, O_NONCAP);
	if (vid < 0)
	{
		printf("No video device \"%s\"\n", my_device);
		return 1;
	}

	toplevel = XtAppInitialize(&app, "V4L2 Control Panel", NULL, 0,
				   &argc, argv, NULL, NULL, 0);

	w_mainbox = XtVaCreateManagedWidget("mainbox",
					    boxWidgetClass, toplevel,
					    XtNorientation, XtorientVertical,
					    XtNhSpace, 0,
					    XtNvSpace, 0,
					    NULL);

	w_menubox = XtVaCreateManagedWidget("menubox",
					    boxWidgetClass, w_mainbox,
					    XtNorientation, XtorientHorizontal,
					    XtNhSpace, 7,
					    XtNvSpace, 2,
					    XtNborderWidth, 0,
					    NULL);

	w_inputmb = XtVaCreateManagedWidget("inputmenubutton",
					    menuButtonWidgetClass, w_menubox,
					    XtNlabel, "Video In",
					    XtNmenuName, "inputmenu",
					    XtNborderWidth, 0,
					    NULL);
	w_inputmenu = new_menu(w_inputmb, "inputmenu");
	for (i = 0; i < sizeof(input)/sizeof(input[0]); ++i)
	{
		input[i].vi.index = i;
		if (ioctl(vid, VIDIOC_ENUMINPUT, &input[i].vi) == 0)
			add_input_menu_entry(&input[i], w_inputmenu);
	}
	w_quit = XtVaCreateManagedWidget("Quit!",
					 commandWidgetClass, w_menubox,
					 XtNborderWidth, 0,
					 NULL);
	XtAddCallback(w_quit, XtNcallback, quit, NULL);



	w_tonebox = XtVaCreateManagedWidget("Tone Controls",
					    boxWidgetClass, w_mainbox,
					    XtNorientation, XtorientVertical,
					    XtNvSpace, 5,
					    NULL);
	for (i = 0; i < sizeof(tone_control_id)/sizeof(int); ++i)
	{
		tone[i].qc.id = tone_control_id[i];
		if (ioctl(vid, VIDIOC_QUERYCTRL, &tone[i].qc) == 0)
			add_tone_control(&tone[i], w_tonebox);
	}


	XtRealizeWidget(toplevel);
	XtAppMainLoop(app);
	return 0;
}
