
#include "vdo.h"

#define X_DEPTH  32


/* *************************************************************************
 * __u32 frame_buffer_format (Display *d, Window w)
 *
 *
 * ************************************************************************* */
__u32 frame_buffer_format (Display *d, Window w, int *x_depth)
{
  __u32	fbf = V4L2_PIX_FMT_RGB565;

  if (*x_depth == 0) {
#ifdef X_DEPTH
    *x_depth = X_DEPTH;
#else
    XWindowAttributes wa;
    XGetWindowAttributes(d, w, &wa);
    x_depth = &wa.depth;
#endif
  }
  if (*x_depth == 15)      fbf = V4L2_PIX_FMT_RGB555;
  else if (*x_depth == 16) fbf = V4L2_PIX_FMT_RGB565;
  else if (*x_depth == 24) fbf = V4L2_PIX_FMT_BGR24;
  else if (*x_depth == 32) fbf = V4L2_PIX_FMT_BGR32;
  else {
    printf("Unrecognized display depth. You may get an X Windows error.\n");
  }
  return fbf;
}

/* *************************************************************************
 * void quit (Widget w, XtPointer client, XtPointer call)
 *
 *
 * ************************************************************************* */
void quit (int vid, Widget w, XtPointer client, XtPointer call)
{
  struct v4l2_performance	perf;
  
  if (vid >= 0) {
    if (ioctl(vid, VIDIOC_G_PERF, &perf) != 0)
      printf("G_PERF returned error code %d\n",errno);
    else
      printf("Captured %d frames\n", perf.frames);
  }
  close (vid);
  exit(0);
}

/* *************************************************************************
 * int BV_init_xwin ()
 *
 *
 * ************************************************************************* */
int BV_init_xwin (int vid, XtAppContext *app, int *x_depth, Widget *viewport,
		  V4L2 *v4l2, int argc, char **argv) 
{
  Widget toplevel;
  Widget topbox;
  Widget menubox;
  Widget quit_widget;

  struct v4l2_captureparm parm;
  int err;
  
  if (!(v4l2->cap.flags & V4L2_FLAG_READ)) {
    printf("Device does not support the read() call.\n");
    return 1;
  }

  ioctl(vid, VIDIOC_G_PARM, &parm);
  if (ioctl(vid, VIDIOC_S_PARM, &parm))
    printf("S_PARM returned error %d\n",errno);

  if (ioctl(vid, VIDIOC_S_STD, &v4l2->std))
    printf("S_STD returned error %d\n",errno);

  ioctl(vid, VIDIOC_G_FMT, &v4l2->fmt);
  v4l2->fmt.width = VDO_WIDTH;
  v4l2->fmt.height = VDO_HEIGHT;

#ifdef MY_PIXELFORMAT
  v4l2->fmt.pixelformat = MY_PIXELFORMAT;
#endif
  if (ioctl(vid, VIDIOC_S_FMT, &v4l2->fmt)) {
    printf("S_FMT returned error %d\n",errno);
    return 1;
  }

  toplevel = XtAppInitialize(app, "xvideum", NULL, 0,
			       &argc, argv, NULL, NULL, 0);
  topbox = XtVaCreateManagedWidget("topbox", boxWidgetClass, toplevel,
			    XtNorientation, XtorientVertical, XtNborderWidth,
			    0, XtNhSpace, 0, XtNvSpace, 0, NULL);
  menubox = XtVaCreateManagedWidget("menubox", boxWidgetClass, topbox,
			    XtNorientation, XtorientHorizontal, XtNborderWidth,
			    0, XtNhSpace, 4, XtNvSpace, 1, NULL);
  quit_widget = XtVaCreateManagedWidget("Quit!", commandWidgetClass, menubox,
			    XtNborderWidth, 0, NULL);
  XtAddCallback(quit_widget, XtNcallback, (XtCallbackProc) quit, NULL);
  *viewport = 
    XtVaCreateManagedWidget("port", portholeWidgetClass, topbox, XtNwidth,
			   v4l2->fmt.width, XtNheight, v4l2->fmt.height, NULL);
  XtRealizeWidget(toplevel);

#ifndef MY_PIXELFORMAT
  ioctl(vid, VIDIOC_G_FMT, &v4l2->fmt);
  v4l2->fmt.pixelformat = 
    frame_buffer_format(XtDisplay(*viewport),
			DefaultRootWindow(XtDisplay(*viewport)), x_depth);
  err = ioctl(vid, VIDIOC_S_FMT, &v4l2->fmt);
  if (err) {
    printf("S_FMT returned error %d\n", errno);
    return 1;
  }
#endif

  return 0;
}


/* *************************************************************************
 * int BV_vdo_display ()
 *
 *
 * ************************************************************************* */
int BV_vdo_display (int vid, int *x_depth, Widget *viewport, VIMAGE *vimage, 
		  XtAppContext *app, V4L2 *v4l2)
{
  int i;
  GC imagegc;

  vimage->data = malloc(v4l2->fmt.sizeimage);
  if (vimage->data == NULL) {
    printf("malloc(%d) failed\n", v4l2->fmt.sizeimage);
    return 1;
  }

  imagegc = XCreateGC(XtDisplay(*viewport), XtWindow(*viewport), 0, NULL);
  if (imagegc == NULL) {
    printf("XCreateGC failed.\n");
    return 1;
  }

  for (i = 0; i < v4l2->req.count; ++i) {
    vimage->ximage = XCreateImage(XtDisplay(*viewport),
				    DefaultVisual(XtDisplay(*viewport), 0),
				    (*x_depth == 32) ? 24 : *x_depth, ZPixmap,
				    0, vimage->data, v4l2->fmt.width, 
				    v4l2->fmt.height, 16, 0);
    if (vimage->ximage == NULL) {
      printf("No XImage\n");
      return 1;
    }
  }

  printf("Video : %dx%dx%d \"%4.4s\" images\n", v4l2->fmt.width, 
	 v4l2->fmt.height, v4l2->fmt.depth, (char *)&v4l2->fmt.pixelformat);
  printf("        Images are %d bytes each\n", v4l2->fmt.sizeimage);

  for (;;) {
    XEvent event;

    if (XtIsRealized(*viewport)) {
      read(vid, vimage->data, v4l2->fmt.sizeimage);
      XPutImage(XtDisplay(*viewport), XtWindow(*viewport), imagegc,
		vimage->ximage, 0, 0, 0, 0, v4l2->fmt.width, v4l2->fmt.height);
    }
    else {
      XtAppPeekEvent(*app, &event);
    }

    while (XtAppPending(*app)) {
      XtAppNextEvent(*app, &event);
      XtDispatchEvent(&event);
    }
  }
  
  return 0;
}


#ifdef CODEC
/* *************************************************************************
 * char *BV_vdo_code (int codec_type, char *input)
 *
 *
 * ************************************************************************* */
char *BV_vdo_code (int codec_type, char *input)
{
  static int first = 1;
  int size;
  char buf[1000];

  switch (codec_type) {
  case H261:
    if (first) {
      size = H261codeIntraFrame (input, buf);
      first = 0;
    } 
    else 
      size = H261codeFrame (&input, buf);
    return buf;

    break;
  case H263: {
    if (first) {
      size = H263codeIntraFrame (input, buf);
      first = 0;
    } 
    else 
      size = H263codeFrame (&input, buf);
    return buf;
  }
  case MPEG:
    break;
  default:
    break;
  }

}


/* *************************************************************************
 * char *BV_vdo_decode (int codec_type, char *input, int size)
 *
 *
 * ************************************************************************* */
char *BV_vdo_decode (int codec_type, char *input, int size)
{
  char *frame;

  switch (codec_type) {
  case H261:
    break;
  case H263: 
    frame = (char *) H263decodeFrame (0, input, size);
    return frame;
  case MPEG:
    break;
  default:
    break;
  }

}

#endif
