
#include "vdo.h"

#define X_DEPTH  32


/* *************************************************************************
 * __u32 frame_buffer_format (Display *d, Window w)
 *
 *
 * ************************************************************************* */
__u32 frame_buffer_format (Display *d, Window w, int *x_depth)
{
  __u32	fbf = V4L2_PIX_FMT_RGB565;

  if (*x_depth == 0) {
#ifdef X_DEPTH
    *x_depth = X_DEPTH;
#else
    XWindowAttributes wa;
    XGetWindowAttributes(d, w, &wa);
    x_depth = &wa.depth;
#endif
  }
  if (*x_depth == 15)      fbf = V4L2_PIX_FMT_RGB555;
  else if (*x_depth == 16) fbf = V4L2_PIX_FMT_RGB565;
  else if (*x_depth == 24) fbf = V4L2_PIX_FMT_BGR24;
  else if (*x_depth == 32) fbf = V4L2_PIX_FMT_BGR32;
  else {
    printf("Unrecognized display depth. You may get an X Windows error.\n");
  }
  return fbf;
}

/* *************************************************************************
 * void quit (Widget w, XtPointer client, XtPointer call)
 *
 *
 * ************************************************************************* */
void quit (int vid, Widget w, XtPointer client, XtPointer call)
{
  struct v4l2_performance	perf;
  
  if (vid >= 0) {
    if (ioctl(vid, VIDIOC_G_PERF, &perf) != 0)
      printf("G_PERF returned error code %d\n",errno);
    else
      printf("Captured %d frames\n", perf.frames);
  }
  exit(0);
}

/* *************************************************************************
 * int BV_init_xwin ()
 *
 *
 * ************************************************************************* */
int BV_init_xwin (int vid, XtAppContext *app, int *x_depth, Widget *w_viewport,
		  V4L2 *v4l2, int argc, char **argv) 
{
  Widget w_toplevel;
  Widget w_topbox;
  Widget w_menubox;
  Widget w_quit;

  struct v4l2_captureparm parm;
  int err;
  
  if (!(v4l2->cap.flags & V4L2_FLAG_READ)) {
    printf("Device does not support the read() call.\n");
    return 1;
  }

  ioctl(vid, VIDIOC_G_PARM, &parm);
  if (ioctl(vid, VIDIOC_S_PARM, &parm))
    printf("S_PARM returned error %d\n",errno);

  if (ioctl(vid, VIDIOC_S_STD, &v4l2->std))
    printf("S_STD returned error %d\n",errno);

  ioctl(vid, VIDIOC_G_FMT, &v4l2->fmt);
  v4l2->fmt.width = VDO_WIDTH;
  v4l2->fmt.height = VDO_HEIGHT;

#ifdef MY_PIXELFORMAT
  v4l2->fmt.pixelformat = MY_PIXELFORMAT;
#endif
  if (ioctl(vid, VIDIOC_S_FMT, &v4l2->fmt)) {
    printf("S_FMT returned error %d\n",errno);
    return 1;
  }

  w_toplevel = XtAppInitialize(app, "xvideum", NULL, 0,
			       &argc, argv, NULL, NULL, 0);
  w_topbox =
    XtVaCreateManagedWidget("topbox", boxWidgetClass, w_toplevel,
			    XtNorientation, XtorientVertical, XtNborderWidth,
			    0, XtNhSpace, 0, XtNvSpace, 0, NULL);
  w_menubox =
    XtVaCreateManagedWidget("menubox", boxWidgetClass, w_topbox,
			    XtNorientation, XtorientHorizontal, XtNborderWidth,
			    0, XtNhSpace, 4, XtNvSpace, 1, NULL);
  w_quit =
    XtVaCreateManagedWidget("Quit!", commandWidgetClass, w_menubox,
			    XtNborderWidth, 0, NULL);
  XtAddCallback(w_quit, XtNcallback, quit, NULL);
  *w_viewport =
    XtVaCreateManagedWidget("port", portholeWidgetClass, w_topbox, XtNwidth,
			    v4l2->fmt.width, XtNheight, v4l2->fmt.height, NULL);
  XtRealizeWidget(w_toplevel);

#ifndef MY_PIXELFORMAT
  ioctl(vid, VIDIOC_G_FMT, &v4l2->fmt);
  v4l2->fmt.pixelformat = 
    frame_buffer_format(XtDisplay(*w_viewport),
			DefaultRootWindow(XtDisplay(*w_viewport)), x_depth);
  err = ioctl(vid, VIDIOC_S_FMT, &v4l2->fmt);
  if (err) {
    printf("S_FMT returned error %d\n", errno);
    return 1;
  }
#endif

  return 0;
}


/* *************************************************************************
 * int BV_set_xwin ()
 *
 *
 * ************************************************************************* */
int BV_set_xwin (int *x_depth, Widget *w_viewport, VIMAGE *vimage, GC *imagegc, V4L2 *v4l2)
{
  int i;

  vimage[0].data = malloc(v4l2->fmt.sizeimage);
  if (vimage[0].data == NULL) {
    printf("malloc(%d) failed\n", v4l2->fmt.sizeimage);
    return 1;
  }

  *imagegc = XCreateGC(XtDisplay(*w_viewport), XtWindow(*w_viewport), 0, NULL);
  if (*imagegc == NULL) {
    printf("XCreateGC failed.\n");
    return 1;
  }

  for (i = 0; i < v4l2->req.count; ++i) {
    vimage[i].ximage = XCreateImage(XtDisplay(*w_viewport),
				    DefaultVisual(XtDisplay(*w_viewport), 0),
				    (*x_depth == 32) ? 24 : *x_depth, ZPixmap,
				    0, vimage[i].data, v4l2->fmt.width, 
				    v4l2->fmt.height, 16, 0);
    if (vimage[i].ximage == NULL) {
      printf("No XImage\n");
      return 1;
    }
  }
  
  return 0;
}


/* *************************************************************************
 *
 *
 *
 * ************************************************************************* */
